/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __DISPLAY_H__

    #define __DISPLAY_H__

    //----------
    // INCLUDES
    //----------
    #include "application.h"
//    #include "wct_lib_cfg.h"
	#include "wct_hal_cfg.h"
    #include "wct_lib.h"

    //---------
    // DEFINES
    //---------
    #define NUM_LED_BLINK_TYPES_SUPPORTED           3       // Support distinct blink rates for MOD Fault, System Fault and Operational states

    #if USE_ONE_LED_CHANNEL_PER_DEVICE
    #define NUM_LED_CHANNELS                NUM_DEVICES
    #else
    #define NUM_LED_CHANNELS                NUM_COILS
    #endif

    // The following definitions are for the LED control bitmasks
    #define INITIALIZING_BITMASK            0x0001
    #define STANDBY_BITMASK                 0x0002
    #define POWER_TRANSFER_BITMASK          0x0004
    #define DEVICE_CHARGED_BITMASK          0x0008
    #define MOD_FAULT_BITMASK               0x0010
    #define DEVICE_FAULT_BITMASK            0x0020
    #define SYSTEM_FAULT_BITMASK            0x0040
    #define NVM_MEMORY_FAULT_BITMASK        0x0080
    #define POWER_LIMIT_BITMASK             0x0100
    #define DIAG_LED_ON_BITMASK             0x0200
    #define DIAG_LED_OFF_BITMASK            0x0400

    #ifndef CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    #define CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED    FALSE
    #endif

    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    // This module utilizes duty cycle to indicate charge status when charge indication is enabled
    #define DISPLAY_MODULE_SUPPORTS_DUTY_CYCLE      TRUE
    #define DISPLAY_MODULE_SUPPORTS_CHARGE_STATE    TRUE
    #endif

    #ifndef LED_DIMMING_ENABLED
    #define LED_DIMMING_ENABLED             0
    #endif
    
    #if LED_DIMMING_ENABLED
    
    #define LED_DIMMING_CONTROL_OFF         0
    #define LED_DIMMING_CONTROL_VIA_APP     1
    #define LED_DIMMING_CONTROL_VIA_PIN     2
    
    #define LED_DIMMING_INTENSITY_OFF       -1
    #define LED_DIMMING_INTENSITY_MIN       0
    #define LED_DIMMING_INTENSITY_MAX       1024
    
    #endif

    //----------
    // TYPEDEFS
    //----------


    // LED Operating Parameters
    typedef enum
    {
        LED_OFF = 0,
        LED_ON,
#if SIMPLIFY_DISPLAY_CONTROL
        LED_BLINK_SLOW,
        LED_BLINK_FAST,
        LED_LOOP_BLINK_SLOW,
        LED_LOOP_BLINK_FAST,
#else
        LED_BLINK,
#endif
        LED_UNINITIALIZED,

    } LED_STATES;

    typedef enum
    {
        BLINK_TYPE_MOD_FAULT = 0,
        BLINK_TYPE_FAULT,
        BLINK_TYPE_OP_STATE

    } BLINK_TYPE;

    typedef struct
    {
        WORD        wLedStatus;
        #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
        BLINK_TYPE  eBlinkType;
        WORD        wDutyCycleMs;
        #else
        BYTE        byLedState;
        WORD        wBlinkTimeMs;
        WORD        wAccumTimeMs;
        #endif

    } PACKING_ATTRIBUTE   LED_CONTROL;


    typedef struct
    {
        WORD                wPwmDutyCycleMin;
        WORD                wPwmDutyCycleMax;

    } LED_DIMMING_PARAMS;

    //-----------
    // VARIABLES
    //-----------
    extern LED_CONTROL LedControl[ NUM_LED_CHANNELS ][ NUM_LEDS_PER_CHANNEL ];
    
#if LED_DIMMING_ENABLED
    extern SWORD LedIntensity[ NUM_LEDS_PER_CHANNEL ];
#endif


    //-----------
    // FUNCTIONS
    //-----------

    void DisplayInit( void );
    void DisplayInitLedControl( void );
    #if LOW_POWER_MODE_SUPPORTED
    WORD DisplayProcessTimeSlice( WORD wTimePassedMs );
    #else
    void DisplayProcessTimeSlice( WORD wTimePassedMs );
    #endif
    //void DisplayDiagnosticCoilLed( BYTE byLedChannel, BYTE byLedIndex, BOOL bEnable );
    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    void DisplayUpdateLedStatus( BYTE byLedChannel, DEVICE_STATE eDeviceState, BYTE byChargeStatus );
    #else
    void DisplayUpdateLedStatus( BYTE byLedChannel, DEVICE_STATE eDeviceState );
    #endif

    #if SIMPLIFY_DISPLAY_CONTROL
    void Led_Loop_Control(BYTE byLedChannel, WORD wTimePassedMs);
    #endif
    void ResetLedGpio( BYTE byLedChannel );
    void DisplayUpdateLedsInLowPowerMode( WORD wStopTimeMs,WORD wTimePassedMs);
#endif
