/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "systemStatusWPC.h"
#include "display.h"
#if SUPERVISOR_MICRO_SUPPORTED
#include "supervisoryComm.h"
#endif
#if SOUND_SUPPORTED
#include "sound.h"
#endif


//-----------------
// PRIVATE DEFINES
//-----------------


//-------------------
// PRIVATE VARIABLES
//-------------------


//-------------------
// PRIVATE FUNCTIONS
//-------------------


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

/*
//------------------------------------------------------------------------------
void InitDeviceStatus( BYTE byDeviceId, BYTE byActiveCoilIndex, DEVICE_STATE eDeviceState )
{
    #if !USE_ONE_LED_CHANNEL_PER_DEVICE
    BYTE byCoilId;
    BYTE byDeviceCoil;
    #endif

    if( eDeviceState == UNCHANGED )
    {
        // ERROR!  Default to INITIALIZING
        eDeviceState = INITIALIZING;
    }

    RunTimeParams[ byDeviceId ].DeviceStatus = eDeviceState;


    #if USE_ONE_LED_CHANNEL_PER_DEVICE
        #if DISPLAY_MODULE_SUPPORTS_CHARGE_STATE
            DisplayUpdateLedStatus( byDeviceId, eDeviceState, RunTimeParams[ byDeviceId ].byPercentCharged );
        #else
            DisplayUpdateLedStatus( byDeviceId, eDeviceState );
        #endif
    #else

    byCoilId = (BYTE)( NUM_COILS_PER_DEVICE * byDeviceId );

    for( byDeviceCoil = 0; byDeviceCoil < NUM_COILS_PER_DEVICE; byDeviceCoil++, byCoilId++ )
    {
        if( byDeviceCoil == byActiveCoilIndex )
        {
            #if DISPLAY_MODULE_SUPPORTS_CHARGE_STATE
                DisplayUpdateLedStatus( byCoilId, eDeviceState, RunTimeParams[ byDeviceId ].byPercentCharged );
            #else
                DisplayUpdateLedStatus( byCoilId, eDeviceState );
            #endif
        }
        else
        {
            #if DISPLAY_MODULE_SUPPORTS_CHARGE_STATE
                DisplayUpdateLedStatus( byCoilId, NO_DEVICE_PRESENT, 0 );
            #else
                DisplayUpdateLedStatus( byCoilId, NO_DEVICE_PRESENT );
            #endif
        }
    }

    #endif
}
*/

#if MUTE_FOR_SHORT_RESET
WORD wTimeToLastPwrOff;
#endif
//------------------------------------------------------------------------------
void WCT_OnUpdateDeviceStatus( BYTE byDeviceId, DEVICE_STATE eDeviceState )
{
    RUNTIME_PARAMS *pDeviceRunTimeParams = &RunTimeParams[ byDeviceId ];
    static BYTE bTheFirstTimePowerOnFlag = TRUE;

    #if SOUND_SUPPORTED
    SYSTEM_PARAMS *pSystemParams = &NvmParams.SystemParams;

    // Process sound transducer - this output should provide a burst at the beginning of power xfer
    if( ( !pDeviceRunTimeParams->btDeviceControlFlags.bInitialBeepIssued ) && 
        ( pDeviceRunTimeParams->eOpState == OP_STATE_POWER_XFER_INIT ) &&
        ( eDeviceState == DEVICE_CHARGING ) )
    {
        // We are initializing our first power transfer
        //WORD wTimeToLastPwrOff;

        #if MUTE_FOR_SHORT_RESET
        if(bTheFirstTimePowerOnFlag)
        {
            bTheFirstTimePowerOnFlag = FALSE;
            wTimeToLastPwrOff = MUTE_TIME_FOR_SHORT_RESET;
        }
        else
        {
            wTimeToLastPwrOff = WCT_TimerGetElapsedTime(pDeviceRunTimeParams->wLastPwrOffTime);
        }
        #endif

        pDeviceRunTimeParams->btDeviceControlFlags.bInitialBeepIssued = TRUE;
        #if MUTE_FOR_SHORT_RESET
        if( pSystemParams->OpStateParams.WpcDiagnostics.btEnableStartPowerXferTone 
            && wTimeToLastPwrOff >= MUTE_TIME_FOR_SHORT_RESET)
        #else
        if( pSystemParams->OpStateParams.WpcDiagnostics.btEnableStartPowerXferTone )
        #endif
        {
            SoundQueueUpdate( SOUND_POWER_ON );
        }
    }
    else if( ( pDeviceRunTimeParams->DeviceStatus != NO_DEVICE_PRESENT ) && 
        ( pDeviceRunTimeParams->DeviceStatus != INITIALIZING ) &&
        ( eDeviceState == NO_DEVICE_PRESENT ) &&
        ( pDeviceRunTimeParams->btDeviceControlFlags.bInitialBeepIssued == TRUE ) )
    {
        // Device was present and has been removed
        pDeviceRunTimeParams->btDeviceControlFlags.bDevicePresent = FALSE;
        pDeviceRunTimeParams->btDeviceControlFlags.bInitialBeepIssued = FALSE;
        if( pSystemParams->OpStateParams.WpcDiagnostics.btEnableStopPowerXferTone )
        {
            SoundQueueUpdate( SOUND_POWER_OFF );
        }
    }
    else if( ( eDeviceState == DEVICE_CHARGING ) ||
            ( eDeviceState == NO_DEVICE_PRESENT ) )
    {
        // Do not interrupt any START or STOP tones in progress
    }
    else
    {
        // Abort any sound in progress
        SoundQueueUpdate( SOUND_NONE );
    }
    #endif

    if( eDeviceState != UNCHANGED )
    {
        // If status has changed, update run-time parameter and LEDs
        pDeviceRunTimeParams->DeviceStatus = eDeviceState;

        #if USE_ONE_LED_CHANNEL_PER_DEVICE
            #if DISPLAY_MODULE_SUPPORTS_CHARGE_STATE
                DisplayUpdateLedStatus( byDeviceId, eDeviceState, pDeviceRunTimeParams->byPercentCharged );
            #else
                DisplayUpdateLedStatus( byDeviceId, eDeviceState );
            #endif
        #else
            #if DISPLAY_MODULE_SUPPORTS_CHARGE_STATE
                DisplayUpdateLedStatus( pDeviceRunTimeParams->byActiveCoilId, eDeviceState, pDeviceRunTimeParams->byPercentCharged )
            #else
                DisplayUpdateLedStatus( pDeviceRunTimeParams->byActiveCoilId, eDeviceState );
            #endif
        #endif
    }

    #if SUPERVISOR_MICRO_SUPPORTED
    // NOTE: ALWAYS update the supervisor (even if UNCHANGED)
    //  (We may need to return to low-power mode)
    UpdateSupervisoryStatus( byDeviceId, pDeviceRunTimeParams->byCoilIdIndex, eDeviceState );
    #endif

}


//------------------------------------------------------------------------------
DEVICE_STATE GetDeviceStatus( BYTE byDeviceId )
{
    return RunTimeParams[ byDeviceId ].DeviceStatus;
}

//------------------------------------------------------------------------------
void GetDeviceStatusStructure( BYTE byDeviceId, DEVICE_STATUS* pDeviceStatus )
{
    RUNTIME_PARAMS* pRunTimeParams = &RunTimeParams[ byDeviceId ];

    pDeviceStatus->MajorVersion         = pRunTimeParams->DeviceIdentification.MajorVersion;
    pDeviceStatus->MinorVersion         = pRunTimeParams->DeviceIdentification.MinorVersion;
    pDeviceStatus->byUsesExtendedId     = pRunTimeParams->DeviceIdentification.byUsesExtendedId;
    pDeviceStatus->wManufacturerId      = pRunTimeParams->DeviceIdentification.wManufacturerId;
    pDeviceStatus->wDeviceIdMsw         = pRunTimeParams->DeviceIdentification.wDeviceIdMsw;
    pDeviceStatus->wDeviceIdLsw         = pRunTimeParams->DeviceIdentification.wDeviceIdLsw;
    //pDeviceStatus->ExtDeviceId          = pRunTimeParams->DeviceIdentification.ExtDeviceId;

    pDeviceStatus->bySignalStrength     = pRunTimeParams->bySignalStrength;

    pDeviceStatus->wMaxHalfWatts        = pRunTimeParams->wMaxHalfWatts;
    pDeviceStatus->byPercentCharged     = pRunTimeParams->byPercentCharged;
    pDeviceStatus->byPercentPowerUsed   = pRunTimeParams->byPercentPowerUsed;

    pDeviceStatus->dwCalculatedInputPower = pRunTimeParams->dwCalculatedInputPower;
    pDeviceStatus->dwRxReportedPower    = pRunTimeParams->dwRxReportedPower;

    pDeviceStatus->wDeviceFaultStatus   = pRunTimeParams->wDeviceFaultStatus;

    pDeviceStatus->byEptReasonCode      = pRunTimeParams->byEptReasonCode;
    pDeviceStatus->DeviceState          = pRunTimeParams->DeviceStatus;

}







