## ###################################################################
##
##     Copyright Freescale Semiconductor, Inc. 2012
##
##     Processor : MWCT1000
##
##     Abstract  :
##
##     This file is used by the linker. It describes files to be linked,
##     memory ranges, stack size, etc. For detailed description about linker
##     command files see CodeWarrior documentation.
##
## ###################################################################

MEMORY 
{

    # Program Memory space 
    # 0x000000 - 0x007FFF   32KW Primary program/data flash -  Flash memory 64KB
    .p_interrupts_ROM     (RX)  : ORIGIN = 0x0000,   LENGTH = 0x0200   # reserved for interrupts
    .p_flash_config_DATA  (RX)  : ORIGIN = 0x0200,   LENGTH = 0x0008   # 16 bytes reserved for Flash Configuration Fields
    .p_flash_ROM          (RX)  : ORIGIN = 0x0208,   LENGTH = 0x3BF8   # reserved for program code
    .p_dflash_DATA        (RX)  : ORIGIN = 0x3E00,   LENGTH = 0x0200   # reserved for EEPROM emulation (2 1024 byte sectors)
    
    # code running in XRAM 
    .p_flash_ROM_code	  (RX)  : ORIGIN = 0xF200,   LENGTH = 0x0380
    
    # DDM buffer
    .DDM_RAM        (RW)  : ORIGIN = 0x000000, LENGTH = 0x000200 
    
    # RAM - 3.0KB
    .p_flash_ROM_data     (RX)  : ORIGIN = 0x0580,   LENGTH = 0x0680   
 

    # Data Memory space 
    # 0x000600 - 0x000C00   RAM 3.0KB
    .x_internal_RAM        (RW)  : ORIGIN = 0x000580, LENGTH = 0x000680 
    #.reserved_1           (RW)  : ORIGIN = 0x001000, LENGTH = 0x003000 # unused
    .x_platform            (RW)  : ORIGIN = 0x00C000, LENGTH = 0x002000 # on-platform peripheral address space
    .x_peripherals         (RW)  : ORIGIN = 0x00E000, LENGTH = 0x002000 # off-platform perihperal address space
    #.reserved_2           (RW)  : ORIGIN = 0x010000, LENGTH = 0x010000
    #.reserved_3           (RW)  : ORIGIN = 0x020000, LENGTH = 0x020000
    #.x_external           (RW)  : ORIGIN = 0x040000, LENGTH = 0x040000 # 256k words not used in this device
    .x_EOnCE               (RW)  : ORIGIN = 0xFFFF00, LENGTH = 0x000100 # debugger and other uses at end of memory space (24 bit)
}

KEEP_SECTION { interrupt_vectors.text }
KEEP_SECTION { dflash.text }
KEEP_SECTION { flash_config.text }

SECTIONS 
{
        .interrupt_vectors :
        {
          * (interrupt_vectors.text)
        } > .p_interrupts_ROM

        .dflash :
        {
          * (dflash.text)
        } > .p_dflash_DATA

        .flash_config :
        {
          * (flash_config.text)
        } > .p_flash_config_DATA
        
        .ApplicationCode :
        {
              F_Pcode_start_addr = .;
              # .text sections
              * (.text)
              * (rtlib.text)
              * (startup.text)
              * (fp_engine.text)
              * (ll_engine.text)
              * (user.text)
              * (.data.pmem)

              F_Pcode_end_addr = .;
              # save address where for the data start in pROM
              . = ALIGN(2);
              __pROM_code_start = .;
        } >.p_flash_ROM

        # AT sets the download address
        # the download stashes the code just after the program code
        
        .code_in_p_flash_ROM : AT(__pROM_code_start)
        {
        	__xRAM_code_start = .;
        	* (.code_in_ram)
        	. = ALIGN(2);
            __xRAM_code_end = .;
            __code_size = __xRAM_code_end - __xRAM_code_start;
            __pROM_data_start = __pROM_code_start + __code_size;
        } >.p_flash_ROM_code

        .data_in_p_flash_ROM : AT(__pROM_data_start)
        {
              # the data sections flashed to pROM
              # save data start so we can copy data later to xRAM

              __xRAM_data_start = .;

              # .data sections
              * (.const.data.char)  # used if "Emit Separate Char Data Section" enabled
              * (.const.data)

              * (fp_state.data)
              * (rtlib.data)
              * (.data.char)        # used if "Emit Separate Char Data Section" enabled
              * (.data)

              # save data end and calculate data block size
              . = ALIGN(2);
              __xRAM_data_end = .;
              __data_size = __xRAM_data_end - __xRAM_data_start;

        } > .p_flash_ROM_data    # this section is designated as p-memory
                                 # with X flag in the memory map
                                 # the start address and length map to
                                 # actual internal xRAM

	    .DDMData :
	    {
	        * (.dma_data)
	        _END_BSS2   = .;
	    } > .DDM_RAM

        .ApplicationData :
        {
              # save space for the pROM data copy
              

              . =  __data_size + .;

              # .bss sections
              * (rtlib.bss.lo)
              * (rtlib.bss)
              . = ALIGN(4);
              F_Xbss_start_addr = .;
              _START_BSS = .;
              * (.bss.char)         # used if "Emit Separate Char Data Section" enabled
              * (.bss)
              . = ALIGN(2);         # used to ensure proper functionality of the zeroBSS hardware loop utility
              _END_BSS   = .;
              F_Xbss_length = _END_BSS - _START_BSS;

              /* Setup the HEAP address */
              . = ALIGN(4);
              _HEAP_ADDR = .;
              _HEAP_SIZE = 0x00000010;
              _HEAP_END = _HEAP_ADDR + _HEAP_SIZE;
              . = _HEAP_END;

              /* SETUP the STACK address */
              _min_stack_size = 0x00000100;
              _stack_addr = _HEAP_END;
              _stack_end  = _stack_addr + _min_stack_size;
              . = _stack_end;

              /* EXPORT HEAP and STACK runtime to libraries */
              F_heap_addr   = _HEAP_ADDR;
              F_heap_end    = _HEAP_END;
              F_Lstack_addr = _HEAP_END;
              F_StackAddr = _HEAP_END;
              F_StackEndAddr = _stack_end - 1;

              # runtime code __init_sections uses these globals:

              F_Ldata_size     = __data_size;
              F_Ldata_RAM_addr = __xRAM_data_start;
              F_Ldata_ROM_addr = __pROM_data_start;
              
              F_Lcode_size     = __code_size;
              F_Lcode_RAM_addr = __xRAM_code_start - 0xF000;
              F_Lcode_ROM_addr = __pROM_code_start;

              F_pROM_to_pRAM   = 0x0001;
              F_pROM_to_xRAM   = 0x0001;

              F_start_bss   = _START_BSS;
              F_end_bss     = _END_BSS;

              __DATA_END=.;

    } > .x_internal_RAM

}
