/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __APPLICATION_CFG_H
#define __APPLICATION_CFG_H     1

#include "defines.h"
//---------------------------- Temperature Defines -----------------------------
#define ROOM_TEMPERATURE_CENTIGRADE 25
#define BOARD_TEMPERATURE_CENTIGRADE_LIMIT 60

//---------------------------- Touch Defines -----------------------------
#define CUSTOMER_TOUCH 					FALSE   //It's FALSE for FSL EVM
#define COIL_AS_TOUCH_ELECTRODE_SUPPORT  FALSE  // Use Tx coil as touch sensor electrode, and a relay is used to swith coil to touch sensor or main power.
#define TOUCH_SENSOR_INTERRUPT                        void MWCT10xx_ISR_GPIOC( void )

#define TIME_OUT_SECOND_FROM_IDLE_TO_SLEEP  20

//----------------------- Display module defines -------------------------
// NOTE: Initially configured for single-channel operation

// General LED definitions
#define USE_ONE_LED_CHANNEL_PER_DEVICE          TRUE
#define CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED    FALSE
#define NUM_LEDS_PER_CHANNEL                    2
#define DIAGNOSTIC_MODE_LED_PATTERN             0xF     // All LEDs are illuminated when entering Diag mode
#define NUM_LED_PIN_ASSIGNMENT_OPTIONS          1       // No alternate pin assignment per LED

// The LEDs are connected to the following ports:
// LED1_C1 pin: GPIOC9  ( Control Reg 0xE221, Bit 0x0200 )
// LED2_C1 pin: GPIOF2  ( Control Reg 0xE251, Bit 0x0004 )
// LED1_C2 pin: GPIOB3  ( Control Reg 0xF151, Bit 0x0008 )
// LED2_C2 pin: GPIOB1  ( Control Reg 0xF151, Bit 0x0002 )
// LED1_C3 pin: GPIOF4  ( Control Reg 0xF191, Bit 0x0010 )
// LED2_C3 pin: GPIOF5  ( Control Reg 0xF191, Bit 0x0020 )
// LED1_C4 pin: GPIOF7  ( Control Reg 0xF191, Bit 0x0080 )
// LED2_C4 pin: GPIOF6  ( Control Reg 0xF191, Bit 0x0040 )
// NOTE: We are implementing a single-device platform utilizing hardware CH4

// NOTE: There are no alternate LED pin assignments
// NOTE: Set the bit mask to zero for LEDs not implemented
#define LED_PORTS                               { { { (WORD*) &GPIOC_DR, (WORD*) &GPIOF_DR } } }
#define LED_BIT_MASKS                           { { { 0x0010, 0x0004 } } }

// Define the polarity for each pin assignment option:
//  (Standard pin assignment is active LOW, Alternate pin assignment is active HIGH)
#define LED_PIN_ASSIGNMENT_ACTIVE_HIGH          { 0 }      // '1' = Active HIGH, '0' = Active LOW

#define NUM_LED_BINS                            1

// Define which LED bins use the alternate pin assignment:
//  (Set to '0' for STANDARD, set to '1' for ALTERNATE)
#define LED_PIN_ASSIGNMENTS                     { 0 }

// Define which LED bins use NVM to define functionality
//  (Set to '0' for ROM Table Defined, set to '1' for NVM Defined)
#define LED_CONTROL_VIA_NVM_INIT                { 1 }

//  LED1_ON, LED1_Blink, LED2_ON, LED2_Blink, Fault Blink Rate, MOD Fault Blink Rate, Op Blink Rate, Pwr-Up Delay
#define NVM_LED_MATRIX                          { 0x0000, 0x0000, 0x0000, 0x0000, 0, 0, 0, 0 }          // Set to zeroes as place-holder

// NOTE: Bins '0' is reserved for use with NVM
#define LED_MATRIX_INITIALIZATION               { NVM_LED_MATRIX }

// Simplified display control
#define SIMPLIFY_DISPLAY_CONTROL FALSE
#define LED_LOOP_BLINK           FALSE             //LED blink in turn supported
#define LED_LOOP_BLINK_COUNTS    6
#define CUSTOMER_LED_PIN_ASSIGNMENT_ACTIVE_HIGH 1   // '1' = Active HIGH, '0' = Active LOW

#define CUSTOMER_LED_PORTS                    { { (WORD*) &GPIOC_DR, (WORD*) &GPIOF_DR } }
#define CUSTOMER_LED_BIT_MASKS                { { 0x0010, 0x0004 } }

#define CUSTOMER_NUM_LEDS_PER_CHANNEL  2

//Quick blinking mode in Low Power mode 
#define LOW_POWER_DISPLAY_MODE        			FALSE


//----------------------- Timer module defines -------------------------
#define TICK_TIMER_USE_PIT  TRUE

//----------------------- Analog sensing module defines -------------------------
#define CURRENT_SENSING_TRANSFORMER  FALSE


//----------------------- LOW POWER mode defines -----------------------
// NOTE: Uses TimerA0 to wake from LOW POWER (STOP) mode
#define LOW_POWER_MODE_SUPPORTED                TRUE   //To enter low power mode, it should be true.
#define LOW_POWER_MODE_BY_TOUCH                 TRUE
#define LOW_POWER_MODE_BY_ANALOG_PING           FALSE   //Touch mode & analog ping mode, only one can work at a time.   	
#define ENABLE_DSC_LOW_POWER_MODE               TRUE   // LPSTOP mode
#define ENABLE_ROSC_8M_STANDBY_MODE             FALSE  //Setting ROSC_8M standby. Only select one between ROSC_8M_STANDBY_MODE and ROSC_200K!!
#define ENABLE_ROSC_200K                        TRUE  //FALSE

#define DEBUG_LOW_POWER_MODE                    FALSE
#define STOP_TIME_OVERHEAD_MS                   (0U) // if idle time is larger than this value, MCU will enter into STOP mode
#define VCCH_RESTORE_TIME_MS                    (50U)//50ms for restoring Power
//defines for led status in low power mode
#define LED_PORT_BLINK_IN_LOW_POWER_MODE        (1U) //0---the first element in LED_PORT, 1---the second element in LED_PORT,
#define LED_ON_TIME_IN_LOW_POWER_MODE_MS        (400U) //recommended value: Multiples of analog ping interval
#define LED_OFF_TIME_IN_LOW_POWER_MODE_MS       (5200U) //recommended value: Multiples of analog ping interval

// NOTE: Leave rail supply enabled during low-power mode (keep caps charged)
#define PRE_LOW_POWER_HANDLER
#define POST_LOW_POWER_HANDLER

#define ENTER_LOW_POWER_MODE_MS( x )            uControllerEnterStopModeMs( x )
#define ENTER_SHUTDOWN_MODE                     uControllerEnterStopMode()

//macros check
#if LOW_POWER_MODE_SUPPORTED

#if(LOW_POWER_MODE_BY_ANALOG_PING && LOW_POWER_MODE_BY_TOUCH)
#error please select only one approach for low power mode  
#endif

#if(ENABLE_ROSC_8M_STANDBY_MODE & ENABLE_ROSC_200K )
#error please select only one clock source for low power mode 
#endif
#endif
//----------------------------- Sound defines ---------------------------- 
// The sound output can be generated using an PWM output. 
#define SOUND_SUPPORTED                      FALSE 
#define DC_SOUND_ENABLED                    FALSE
#define AC_SOUND_ENABLED                    TRUE
#define SOUND_PWM_DUTYCYCLE_PERCENT_DEFAULT 5000            // This represents 50% duty cycle output

#define PWM0_USED_FOR_SOUND_CONTROL      FALSE
#define PWM1_USED_FOR_SOUND_CONTROL      FALSE
#define PWM2_USED_FOR_SOUND_CONTROL      TRUE
#define PWM3_USED_FOR_SOUND_CONTROL      FALSE

//----------------------------- Power limit defines ---------------------------- 
//

//----------------------------- Faults protection defines -----------------------
#define OVER_TEMPERATURE_PROTECTION_SUPPORT TRUE

//----------------------- debug module defines -------------------------
#if _DEBUG
#define DEBUG_GPIO FALSE
#define DEBUG_CLKO FALSE
#define DEBUG_PWM_TRIG0 FALSE
#endif

/*GPIO measure selection.*/
    #define MEASURE_NONE                     0
    #define MEASURE_TICK_ISR                 1
    #define MEASURE_TMRA0_ISR                2
    #define MEASURE_DDM_FILTER               3
    #define MEASURE_Timer1msTickHandler      4
    #define MEASURE_AnalogPing               5
    #define MEASURE_ADC_ISR                  6
    #define MEASURE_TimerProcess             7
    #define MEASURE_ProcessPowerXferState    8
    #define MEASURE_PIDExec                  9
    #define MEASURE_AfterPIDExec             10
    #define MEASURE_byCommPacketTypeReceived 11
    #define MEASURE_DMA_RESET                12
    #define MEASURE_T_NEXT_TIMEOUT           13
    #define MEASURE_PACKET_WAITING_FOR_START 14

    #define GPIO_DBG1_MEASURE MEASURE_NONE
    #define GPIO_DBG2_MEASURE MEASURE_NONE  

// Debug pin mapping
#define SET_COMM_DATA_DEBUG_PIN_LOW             CLEAR_TP1
#define SET_COMM_DATA_DEBUG_PIN_HIGH            SET_TP1
#define TOGGLE_COMM_DATA_DEBUG_PIN              TOGGLE_TP1
#define SET_LOW_POWER_MODE_DEBUG_PIN_HIGH       SET_TP2
#define SET_LOW_POWER_MODE_DEBUG_PIN_LOW        CLEAR_TP2
#define TOGGLE_LOW_POWER_MODE_DEBUG_PIN         TOGGLE_TP2

//----------------------- FOD module defines -------------------------
#define FOD_ENABLE TRUE

//----------------------- Customer message print defines -------------------------
#if _DEBUG
#define PRINT_CUSTOMER_MESSAGE FALSE
#endif

//----------------------- FreeMASTER Support -------------------------
#define FREEMASTER_SUPPORTED TRUE

#endif  //__APPLICATION_CFG_H
