/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//----------
// INCLUDES
//----------
#include <stdio.h>
#include <stdlib.h>

#include "defines.h" /* include peripheral declarations */
#include "peripheralInit.h"
#include "application.h"
//#include "wct_lib_cfg.h"
#include "wct_hal_cfg.h"
#include "wct_hal.h"
#include "wct_lib.h"
#include "freemaster.h"
#include "display.h"
#include "flash.h"
#include "touch.h"


//-----------
// FUNCTIONS
//-----------

void WCT_Init(void);


/*
 * This list describes all TSA tables which should be exported to the 
 * FreeMASTER application.
 */
#if FMSTR_USE_TSA
FMSTR_TSA_TABLE_LIST_BEGIN()
    FMSTR_TSA_TABLE(runtimeParams)
FMSTR_TSA_TABLE_LIST_END()
#endif


// Initialization routine for Witeless charging library
void WCT_Init(void)
{
    //init system Errors
	WCT_InitError();

    // Initialize RAM from NVM 
    // - MUST occur prior to Coil Init for deadtime specification
    // - MUST occur prior to FOD config to allow for override
    WCT_InitSystemParams((WORD*) SYSTEM_PARAMS_FLASH_BASE_ADDRESS, (WORD*)&NvmParams.SystemParams, sizeof( SYSTEM_PARAMS ));
    WCT_InitOpParams((WORD*) OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE, (WORD*)&NvmParams.OpParams, sizeof( OP_PARAMS ));
    WCT_InitCalParams((WORD*) CAL_PARAMS_FLASH_BASE_ADDRESS, (WORD*)&NvmParams.CalParams, sizeof( CAL_PARAMS ));

    if( !( WCT_GetSystemErrorCode() & ERROR_CODE_CORRUPT_E2_SYSTEM_PARAMS ) )
    {
        // Check for FOD configuration override
        if( NvmParams.SystemParams.OpStateParams.WpcProtections.btFodHardwareBinOverride )
        {
            HwConfigOptions.byPowerLossDetectionSelection = NvmParams.SystemParams.OpStateParams.WpcProtections.btFodOverrideBinSelection;
        }
    }

    // Check for Power Loss Detection activation
#if FOD_ENABLE
    if( HwConfigOptions.byPowerLossDetectionSelection < 13 )
        HwConfigOptions.bPowerLossDetectionActive = TRUE;
    else
#endif
        HwConfigOptions.bPowerLossDetectionActive = FALSE;

    //enable the power
    SWITCH_AUX_VCCH_ON;   //todo, replaced by low power mode
    
    ENABLE_FET_DRIVER;   //todo, replaced by low power mode
    
    //Run the WCT state machine
    WCT_Run();

    
    //Enable communication interrupt
    //ENABLE_COMM_INTERRUPTS;
    //todo IPR9: PWMA_RELOAD0=3, IPR2 ADC_CC0=1, IPR12: GPIOB=2,GPIOC=3, INTC_IPR0: RX_REG=2,TX_REG=2

#if DEBUG_GPIO
    dbg1_select_flag=GPIO_DBG1_MEASURE;
    dbg2_select_flag=GPIO_DBG2_MEASURE;
#endif

}


int main(void)
{
    DISABLE_WATCHDOG();
    // Initialise the Hardware Layer
	HwInit();

#if LOW_POWER_MODE_BY_TOUCH
    TouchSensorInit();
#endif
	
	// Initialise the Application Layer in library
    WCT_Init();

    
    // Initialize LED control
    DisplayInit();

#if 1 // !_DEBUG
    // Initialise the WDT, allow WDT to reset the processor
    WatchdogTimerInit();
#endif

#if FREEMASTER_SUPPORTED    
    //Initialise FreeMASTER tool
    FMSTR_Init();
#endif

    
#if FREEMASTER_SUPPORTED    
    //Enable interrupts of FreeMASTER tool 
#if FMSTR_USE_SCI
#if FMSTR_SCI_BASE == 0xE080
#if DEBUG_CONSOLE_QSCI0
#error debug console uses same intrface as FreeMASTER, Please disable consolle by DEBUG_CONSOLE_QSCI0 macro.
#endif
    INTC_IPR5 = ( INTC_IPR5 & ~INTC_IPR5_QSCI0_RCV ) | ( ( (DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL) + 1) << 6 );
    INTC_IPR5 = ( INTC_IPR5 & ~INTC_IPR5_QSCI0_TDRE ) | ( ( (DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL) + 1) << 10 );
    QSCI0_CTRL2 = 0; //disable fifo
#else
#if DEBUG_CONSOLE_QSCI1
#error debug console uses same intrface as FreeMASTER, Please disable consolle by DEBUG_CONSOLE_QSCI1 macro.
#endif
    INTC_IPR4 = ( INTC_IPR4 & ~INTC_IPR4_QSCI1_RCV ) | ( ( (DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL) + 1) << 14 );
    INTC_IPR5 = ( INTC_IPR5 & ~INTC_IPR5_QSCI1_TDRE ) | ( ( (DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL) + 1) << 2 );
    QSCI1_CTRL2 = 0; //disable fifo
#endif    
#elif FMSTR_USE_JTAG
    // enable JTAG EOnCE ISR for FMSTR
    INTC_IPR0 |= INTC_IPR0_RX_REG_0 |INTC_IPR0_TX_REG_0; //set "EOnCE Transmit Register Empty"  and "EOnCE Receive Register Full"
#endif
#endif

    while( 1 )
    {
        WCT_Poll();
#if FREEMASTER_SUPPORTED    
        FMSTR_Poll();
#endif
    }
    return(0);

}


