/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/


#include "lowPowerMode.h"
#include "defines.h"
#include "touch.h"
#include "display.h"
#include "qsci.h"
#include "application.h"

void TimerAddTimerTicks(WORD);
//-----------------
// PRIVATE DEFINES
//-----------------

// Calculate the clock configuration required for the wake timer to ensure adequate resolution
//  (target clock period between 20us and 50us to yield MAX wake times from 1 to 3 seconds using 16-bit timer )
/* Global variables */
//#define LOW_POWER_CLOCK_PRESCALER   (OCCS_DIVBY_COD_0)  // /2
//#define ENABLE_ROSC_8M_STANDBY_MODE   1
#define WAKE_TIMER_CLOCK_SOURCE_SEL (PIT0_CTRL_CLKSEL_1|PIT0_CTRL_CLKSEL_0) //alternate clock 3  
#define WAKE_TIMER_CLOCK_SOURCE_HZ  (200000) //ROSC_200KHz 
#define WAKE_TIMER_CLOCK_PRESCALER  (PIT0_CTRL_PRESCALER_1)  //4
#define WAKE_TIMER_TICKS_PER_MS     ( WORD )( ( 0.001 * ( WAKE_TIMER_CLOCK_SOURCE_HZ / (1<<(WAKE_TIMER_CLOCK_PRESCALER >> 3)) ) ) + 0.5 )


static void system_sleep_by_analog_ping( WORD wStopTimeMs );
static void system_sleep_by_touch( WORD wStopTimeMs );

//------------------------------------------------------------------------------
// uControllerEnterStopModeMs
//
// Puts the processor into STOP Mode. This is the operating mode
// with the lowest power consumption for the device.
//
// Stop mode disables the DSC Core clock and peripheral clocks. The SIM_SDn registers
// indicate whether a peripheral clock will continue to run when the processor enters
// STOP mode or not (default is all peripheral clocks are off).
//
// The micro can be awakened by an interrupt from any active peripheral, low voltage
// interrupt, debug command (JTAG), or any reset.
//
// NOTE: The watchdog timer is not included in the DIM_SDn registers however,
//       the watchdog peripheral register sets allow the option to continue in
//       STOP mode or not. A watchdog timeout causes a reset.
//
// input:   n/a
// output:  n/a
// modify:
// return:  void
//
//------------------------------------------------------------------------------
void uControllerEnterStopMode( void )
{
    // Go to STOP mode of operation
    asm( STOP );
}

void uControllerEnterStopModeMs( WORD wStopTimeMs )
{
#if (LOW_POWER_MODE_BY_ANALOG_PING)
	system_sleep_by_analog_ping(wStopTimeMs);
#elif (LOW_POWER_MODE_BY_TOUCH)	
	system_sleep_by_touch(wStopTimeMs);
#endif
}

#if LOW_POWER_MODE_SUPPORTED
/*
** ===================================================================
**     Method      :  system_sleep_by_analog_ping
**
**     Description : system enter into low power mode when analog ping is used
**        
**         
**     Parameters  : wStopTimeMs
**     Returns     : Nothing
** ===================================================================
*/
#if LOW_POWER_MODE_BY_ANALOG_PING
static void system_sleep_by_analog_ping( WORD wStopTimeMs )
{
    WORD wStopTicks,wAdjustStopTimeMs;
    WORD wCOPcontrolRegValue;
    WORD wOccsDivbySetting;
    WORD wPit0CtrlBackup,wPit0ModBackup;
    BOOL bTurnOffVcch = 0;
    
    // Adjust WAIT time by the specified overhead value
    // NOTE: The overhead value for the system should be specified in the mapping.h file!
    if( wStopTimeMs > (STOP_TIME_OVERHEAD_MS + VCCH_RESTORE_TIME_MS) )
    {
    	wAdjustStopTimeMs = wStopTimeMs - VCCH_RESTORE_TIME_MS; 
        bTurnOffVcch = 1;
    }
    else if (wStopTimeMs > STOP_TIME_OVERHEAD_MS)
    {
    	wAdjustStopTimeMs = wStopTimeMs;
    	bTurnOffVcch = 0;
    }
    else
    {
    	bTurnOffVcch = 0;
    	wAdjustStopTimeMs = 0;
    }

    // If WAIT time is 0, bypass function
    if( wAdjustStopTimeMs != 0 )
    {
    	 DISABLE_FET_DRIVER; //disable FET driver
    	 SWITCH_AUX_POWER_OFF; //turn off AVCC
  
        //GPIOF_PER &= ~(BIT2|BIT3);
        //GPIOF_DDR &= ~BIT3;
        //GPIOF_DR |= BIT2;
        //GPIOC_DR |= BIT4;
    	// power down analog part
    	ADC_PWR |= (ADC_PWR_PD0 | ADC_PWR_PD1);//power down ADC
 
    	if(bTurnOffVcch)
    	{
    		SWITCH_AUX_VCCH_OFF;//turn off VCCH
    	}  
        // Make sure interrupts are disabled globally
        DISABLE_CPU_INTERRUPTS;

        // Save the original setting for the Watchdog Timer configuration...
        wCOPcontrolRegValue = COP_CTRL;

        // Ensure the WDT is disabled, so that it doesn't wake micro from STOP mode.
        // This is done by clearing the COP Wait mode enable bit. The COP resumes
        // operation automatically after exiting STOP mode.
        // NOTE: This assumes the COP_CTRL register is writable... If the user configures
        // the COP to protect the register, the watchdog should be configured to stop
        // in STOP mode.
        COP_CTRL &= ~COP_CTRL_CSEN;

       
        //WCT_DebugPutString("\r\n");
        //DebugPutDoubleWordValue(wStopTicks);
        
        wPit0CtrlBackup = PIT0_CTRL;
        wPit0ModBackup = PIT0_MOD;
		//wPit1CtrlBackup = PIT1_CTRL; 
		//STOP time ticks
        // Change PIT timer length
        PIT0_CTRL &= ~(PIT0_CTRL_PRESCALER+PIT0_CTRL_CNT_EN);  
        // PIT clock= 400K (ROSC_8M_Standby)/4
        PIT0_CTRL |= (WAKE_TIMER_CLOCK_SOURCE_SEL | WAKE_TIMER_CLOCK_PRESCALER|PIT0_CTRL_PRIE);
        // Convert milliseconds to Wake Timer ticks
        wStopTicks = wAdjustStopTimeMs * WAKE_TIMER_TICKS_PER_MS;
        PIT0_MOD = wStopTicks;
        INTC_IPR10 |= (INTC_IPR10_PIT0_ROLLOVR_0);
        // Set board power off, DSC modules off, PLL off
        Cpu_SetLowSpeed_LP();
        ENABLE_CPU_INTERRUPTS;
        
        // start the timer
        PIT0_CTRL |= PIT0_CTRL_CNT_EN;      /* Run PIT */ 
        // Allow PIT0 to stay alive in stop mode
        SIM_SD2|=SIM_SD2_PIT0;    // set in init code.
        
        // Go to STOP mode of operation
        // uControllerEnterStopMode();
         asm(STOP);
        // NOTE: Code execution will resume here after Wake Timer interrupts micro

        // Disable global and wake interrupts
        DISABLE_CPU_INTERRUPTS;
		SWITCH_AUX_VCCH_ON;//turn on VCCH 
		if(bTurnOffVcch)
		{
			wStopTicks = VCCH_RESTORE_TIME_MS * WAKE_TIMER_TICKS_PER_MS;
			PIT0_MOD = wStopTicks;
			ENABLE_CPU_INTERRUPTS;
			// start the timer
			PIT0_CTRL |= PIT0_CTRL_CNT_EN;      /* Run PIT */ 
		   
			asm(STOP); //continue STOP 
			DISABLE_CPU_INTERRUPTS;
		}
        ADC_PWR &= ~(ADC_PWR_PD0 | ADC_PWR_PD1); //power on ADC
        ENABLE_FET_DRIVER;
	    SWITCH_AUX_POWER_ON;
        // Return clock to original setting
        Cpu_SetHighSpeed_LP();
        if( wCOPcontrolRegValue & COP_CTRL_CEN )
        {
            KICK_THE_WATCHDOG();
        }
        COP_CTRL = wCOPcontrolRegValue;
        //system_wakeup();
        //restore time ticks
        TimerAddTimerTicks(wStopTimeMs);
        TICK_TIMER_INTERRUPT_ENABLE;
        PIT0_MOD = wPit0ModBackup;
        PIT0_CTRL = wPit0CtrlBackup;
        // Re-enable the global interrupts
        ENABLE_CPU_INTERRUPTS;
    }

}
#endif
/*
** ===================================================================
**     Method      :  system_sleep_by_touch
**
**     Description : system enter into low power mode when touch is enabled
**        
**         
**     Parameters  : wStopTimeMs
**     Returns     : Nothing
** ===================================================================
*/
#if LOW_POWER_MODE_BY_TOUCH
static void system_sleep_by_touch( WORD wStopTimeMs )
{
    DISABLE_FET_DRIVER;
    SWITCH_AUX_POWER_OFF;

    /* Disable IO clock to save power */
    DISABLE_SIM_PCE0_CLOCK;
    DISABLE_SIM_PCE1_CLOCK;
    DISABLE_SIM_PCE2_CLOCK;
    DISABLE_SIM_PCE3_CLOCK;

    /* Change IO clock due to master clock changed. */
    #if QSCI0_ENABLED
    QSCI0_RATE = QSCI0_RATE_CONFIGURATION_OF_LOW_POWER_MODE;
    #endif


    if(wTxIdleTime<gStaticConf.wTimeOutForSleep)
    {
        Cpu_SetLowSpeed();
        WCT_TimerDelayMs(wStopTimeMs);
		Cpu_SetHighSpeed(); 	/* Set CPU full speed mode. */
    }
    else
    {
        SWITCH_AUX_VCCH_OFF;
		ADC_PWR |= (ADC_PWR_PD0 | ADC_PWR_PD1);//power down ADC
        touch_enable();
        #if CUSTOMER_TOUCH
        Cpu_SetLowSpeed();    // At customer touch mode, to run LED, the pll is till on, otherwise a crash may happen when GPIO IRQ asserted.
        {
            WORD lastTick,currTick;
           
            lastTick= WCT_TimerGetTimerTicks();
            // Led loop control in sleep state untill touch assert.
            //while(GPIOC_DR&BIT6)
            while((get_touch_status()==TOUCH_ON))
            {   
                currTick=WCT_TimerGetTimerTicks();
                if(lastTick!=currTick)
                {
                    lastTick=currTick;
                    #if SIMPLIFY_DISPLAY_CONTROL
                    Led_Loop_Control((BYTE)0,(WORD)1);
                    #endif
                    WCT_KickTheWatchdog();
                }
            }
        }
		Cpu_SetHighSpeed(); 	/* Set CPU full speed mode. */
        #else
        TICK_TIMER_INTERRUPT_DISABLE;   /* Disable TICK interrupt */
		ENABLE_CPU_INTERRUPTS;
        Cpu_SetLowSpeed_LP();
        asm(STOP);
		DISABLE_CPU_INTERRUPTS;
		Cpu_SetHighSpeed_LP();
        #endif
        /* When system waked up by touch IRQ, the program will go here */
		 // Disable global and wake interrupts

		ADC_PWR &= ~(ADC_PWR_PD0 | ADC_PWR_PD1);
        SWITCH_AUX_VCCH_ON;
    }

	
	ENABLE_FET_DRIVER;
	SWITCH_AUX_POWER_ON;
	/* Enable IO clock */
	ENABLE_SIM_PCE0_CLOCK;
	ENABLE_SIM_PCE1_CLOCK;
	ENABLE_SIM_PCE2_CLOCK;
	ENABLE_SIM_PCE3_CLOCK;
	
		/* Change back IO clock due to master clock changed. */
    #if QSCI0_ENABLED
		QSCI0_RATE = SCI_0_SCIBR_INIT;
    #endif
#if (LOW_POWER_MODE_BY_TOUCH & COIL_AS_TOUCH_ELECTRODE_SUPPORT)
		relay_switch(SWITCH_KEEP);	/* Delay several ms to set relay to keep state */
#endif
	TICK_TIMER_INTERRUPT_ENABLE;
	ENABLE_CPU_INTERRUPTS;

}
#endif
#endif
/*
** ===================================================================
**     Method      :  Cpu_SetHighSpeed (component WCT1000)
**
**     Description :
**         Sets the high speed mode. The method is enabled only if
**         low or slow speed mode is enabled in the bean as well.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void Cpu_SetHighSpeed(void)
{
    EnterCritical();                   /* If yes then disable global interrupts */
    OCCS_OSCTL1|=OCCS_OSCTL1_CLK_MODE; /* Select an external clock bypass mode */
    OCCS_CTRL&=~OCCS_CTRL_PRECS; /* Select an internal 8MHz clock source for the CPU core */
    OCCS_CTRL&=~OCCS_CTRL_PLLPD;
    OCCS_CTRL|=OCCS_CTRL_LCKON;
    /* OCCS_DIVBY: LORTP=2,COD=1,??=0,??=0,PLLDB=0x1D */
    OCCS_DIVBY=OCCS_DIVBY_INIT;     /* Set the clock prescalers */ 
    while(!(OCCS_STAT&OCCS_STAT_LCK0)){} /* Wait for PLL lock */
    OCCS_CTRL|=OCCS_CTRL_ZSRC; /* Enable PLL, LCKON and select clock source from prescaler */
    ExitCritical();                    /* Enable global interrupts */
}


/*
** ===================================================================
**     Method      :  Cpu_SetLowSpeed (component WCT1000)
**
**     Description :
**         Sets the low speed mode. The method is enabled only if
**         low speed mode is enabled in the bean.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void Cpu_SetLowSpeed(void)
{
    EnterCritical();                   /* If yes then disable global interrupts */
    OCCS_OSCTL1|=OCCS_OSCTL1_CLK_MODE; /* Select an external clock bypass mode */
    OCCS_CTRL&=~OCCS_CTRL_PRECS;        /* Select an internal 8MHz clock source for the CPU core */
    OCCS_CTRL&=~OCCS_CTRL_ZSRC;        /* Select clock source from MSTR_OSC */
    OCCS_CTRL|=OCCS_CTRL_PLLPD;       /* Disable PLL */
    OCCS_DIVBY&=~OCCS_DIVBY_COD;    /* Set the clock prescalers */
    ExitCritical();                    /* Enable global interrupts */
}


/*
** ===================================================================
**     Method      :  Cpu_SetHighSpeed_LP (component WCT1000)
**
**     Description :
**         Sets the high speed mode. The method is enabled only if
**         low or slow speed mode is enabled in the bean as well.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void Cpu_SetHighSpeed_LP(void)
{
    EnterCritical();                   /* If yes then disable global interrupts */
    OCCS_OSCTL1|=OCCS_OSCTL1_CLK_MODE; /* Select an external clock bypass mode */

#if ENABLE_DSC_LOW_POWER_MODE    
      SIM_PWRMODE&=~SIM_PWRMODE_LPMODE;   /* Exit low power mode */
#endif

#if ENABLE_ROSC_200K
    OCCS_OSCTL1&=~OCCS_OSCTL1_ROPD;       /* Power up ROSC 8MHz. */
    OCCS_CTRL&=~OCCS_CTRL_PRECS; /* Select ROSC */
    OCCS_OSCTL2|=OCCS_OSCTL2_ROPD200K;    /* Power down 200KHz */   
#else
#if ENABLE_ROSC_8M_STANDBY_MODE
    OCCS_OSCTL1&=~OCCS_OSCTL1_ROSB;       /* Set ROSC normal mode 8MHz. */
#endif
    OCCS_CTRL&=~OCCS_CTRL_PRECS; /* Select an internal 8MHz clock source for the CPU core */
#endif

    OCCS_CTRL&=~OCCS_CTRL_PLLPD; /* Enable PLL, LCKON and select clock source from prescaler */
    OCCS_CTRL|=OCCS_CTRL_LCKON;  
    
    /* OCCS_DIVBY: LORTP=2,COD=1,??=0,??=0,PLLDB=0x1D */
    OCCS_DIVBY=OCCS_DIVBY_INIT;     /* Set the clock prescalers */ 
    while(!(OCCS_STAT&OCCS_STAT_LCK0)){} /* Wait for PLL lock */
    OCCS_CTRL|=OCCS_CTRL_ZSRC; /* Enable PLL, LCKON and select clock source from prescaler */
    ExitCritical();                    /* Enable global interrupts */
}


/*
** ===================================================================
**     Method      :  Cpu_SetLowSpeed_LP (component WCT1000)
**
**     Description :
**         Sets the low speed mode. Set low power mode. The method is enabled only if
**         low speed mode is enabled in the bean.
**     Parameters  : None
**     Returns     : Nothing
** ===================================================================
*/
void Cpu_SetLowSpeed_LP(void)
{
    EnterCritical();                   /* If yes then disable global interrupts */
    OCCS_OSCTL1|=OCCS_OSCTL1_CLK_MODE; /* Select an external clock bypass mode */

    OCCS_CTRL&=~OCCS_CTRL_ZSRC;        /* Select clock source from MSTR_OSC */
    OCCS_CTRL|=OCCS_CTRL_PLLPD;       /* Disable PLL */

#if ENABLE_ROSC_200K
    OCCS_OSCTL2&=~OCCS_OSCTL2_ROPD200K;    /* Power up 200KHz */   
    OCCS_CTRL=(OCCS_CTRL&~OCCS_CTRL_PRECS)|(2<<2); /* Select internal 200KHz clock */
    {   /* Wait 6 NOPs for resynchronization */
        WORD i;
        for(i=0;i<6;i++) asm(nop);
    }
    OCCS_OSCTL1|=OCCS_OSCTL1_ROPD;       /* Power down ROSC 8MHz. */
#else
    OCCS_CTRL&=~OCCS_CTRL_PRECS; /* Select an internal 8MHz clock source for the CPU core */
#if ENABLE_ROSC_8M_STANDBY_MODE
    OCCS_OSCTL1|=OCCS_OSCTL1_ROSB;       /* Set ROSC standby mode 400KHz. */
#endif
#endif

#if ENABLE_DSC_LOW_POWER_MODE    
#if !ENABLE_ROSC_200K
    OCCS_DIVBY&=~OCCS_DIVBY_COD; /* Set the clock prescalers for sys_clk_2x<4MHz for low power mode */
    OCCS_DIVBY|=2<<8;
#endif    
    SIM_PWRMODE|=SIM_PWRMODE_LPMODE;   /* Enter low power mode */
#else
    OCCS_DIVBY&=~OCCS_DIVBY_COD; /* Set the clock prescalers */
#endif

    ExitCritical();                    /* Enable global interrupts */
}

