/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include DEBUG_CONSOLE_HEADER_FILE

#if PRINT_CUSTOMER_MESSAGE && QSCI0_ENABLED
//-----------------
// PRIVATE DEFINES
//-----------------

// Print limited counts of message every 1sec, otherwise the print will be lost due to high priority task. 
typedef enum
{
    MESSAGE_PRINT_PART1=0,
    MESSAGE_PRINT_PART2,
    MESSAGE_PRINT_PART3,
    MESSAGE_PRINT_PARTS,
}CUSTOMER_MESSAGE_PRINT;

//-------------------
// PRIVATE VARIABLES
//-------------------
WORD wMessagePrintCount[NUM_DEVICES];

void message_print(BYTE byDevice, DEVICE_OP_STATE state)
{
    if(state!=OP_STATE_POWER_XFER)  
    {   //When no charging, print a short message.
        WCT_DebugPutString( "Input current:" );
        DebugPutDoubleWordValue(WCT_AnalogIoGetInputCurrent(byDevice,CURRENT_REPORT_RMS_MILLIAMPS));
        WCT_DebugPutString( "\r\n" );
        return;
    }
        
    switch(wMessagePrintCount[byDevice])
    {   // To prevent print loss, divide long message to several short message to print out.  
        case MESSAGE_PRINT_PART1:
        WCT_DebugPutString( "Input voltage:" );
        DebugPutDoubleWordValue(RunTimeParams[byDevice].wAveragedInputVoltage);
        WCT_DebugPutString( "\r\n" );
    
        WCT_DebugPutString( "Input current:" );
        DebugPutDoubleWordValue(RunTimeParams[byDevice].dwMeasuredInputCurrent);
        WCT_DebugPutString( "\r\n" );
        break;
    
        case MESSAGE_PRINT_PART2:
        WCT_DebugPutString( "Input power:" );
        DebugPutDoubleWordValue(RunTimeParams[byDevice].dwCalculatedInputPower);
        WCT_DebugPutString( "\r\n" );
    
        WCT_DebugPutString( "PWM freq:" );
        DebugPutDoubleWordValue(RunTimeParams[byDevice].dwActiveFreq);
        WCT_DebugPutString( "\r\n" );
        break;
    
        case MESSAGE_PRINT_PART3:
        WCT_DebugPutString( "Rx power:" );
        DebugPutDoubleWordValue(RunTimeParams[byDevice].dwRxReportedPower);
        WCT_DebugPutString( "\r\n" );
    
        WCT_DebugPutString( "NPLoss:" );
        DebugPutSignedDecWord(RunTimeParams[byDevice].swNormalizedParasiticLoss);
        WCT_DebugPutString( "\r\n" );
        break;
    }
    
    wMessagePrintCount[byDevice]++;
    if(wMessagePrintCount[byDevice]>=MESSAGE_PRINT_PARTS)
        wMessagePrintCount[byDevice]=0;
}
#endif
