/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"
#include "adc.h"

// Temperature sensor source code

/* Temperature Sense Circuits:
 * Voltage on temperature sensor, Vt= ADC_REFERENCE_VOLTAGE*Rb/(Rt+Rb)=ADC_REFERENCE_VOLTAGE*adc/MAX_ADC_COUNTS,
 * , so  Rb/(Rt+Rb) = adc/MAX_ADC_COUNTS = > Rt=(4095*Rs-Rs*adc)/adc=4095*Rs/adc-Rs
 * OTP 60 degree protect, 2.74V
 */
#define BOARD_TEMPERATURE_BOTTOM_R_VALUE 100 //100Kohm


typedef struct {
    SBYTE tempe;         //Temperature, centigrade.
    WORD wResistance;   //Resistance, kohm
}T_TEMPERATURE_RESISTANCE_TABLE;

#define TEMPE_TABLE_SIZE 21
#define ROOM_TEMPE_25_CENTIGRADE_OFFSET 11

//Thermistor: NCP15WL104E03RC, large Adc sample gets small resistor, then gets high temperature value.
T_TEMPERATURE_RESISTANCE_TABLE tempeResisTable[TEMPE_TABLE_SIZE] = {
        {80 , 9,   },  //0
        {75 , 11,  },  //1
        {70 , 14,  },  //2
        {65 , 17,  },  //3
        {60 , 20,  },  //4
        {55 , 25,  },  //5
        {50 , 31,  },  //6
        {45 , 39,  },  //7
        {40 , 49,  },  //8
        {35 , 61,  },  //9
        {30 , 78,  },  //10
        {25 , 100, },  //11
        {20 , 128, },  //12
        {15 , 166, },  //13
        {10 , 217, },  //14
        {5  , 285, },  //15
        {0  , 378, },  //16
        {-5 , 505, },  //17
        {-10, 679, },  //18
        {-15, 921, },  //19
        {-20, 1262,},  //20
};

WORD wTempeResistance;

/* Convert ADC value to temperature centigrade */
SBYTE WCT_ConvertAdcToTempe(DWORD adc)
{
    SBYTE tempe;
    WORD i, resis;

    if( !adc )
    {
        resis = tempeResisTable[ROOM_TEMPE_25_CENTIGRADE_OFFSET].wResistance;   // At startup, the adc history buffer is zero.
    }
    else
    {
        resis = (WORD)(( DWORD )MAX_ADC_COUNTS*BOARD_TEMPERATURE_BOTTOM_R_VALUE / adc);
        if(resis >= BOARD_TEMPERATURE_BOTTOM_R_VALUE)
            resis -= BOARD_TEMPERATURE_BOTTOM_R_VALUE;
        else
            resis = 0;
    }

    wTempeResistance = resis;

    // Find the temperature range.
    tempe = ROOM_TEMPERATURE_CENTIGRADE;
    for(i=0;i<TEMPE_TABLE_SIZE;i++)
    {   
        if( resis <= tempeResisTable[i].wResistance )
        {
            tempe = tempeResisTable[i].tempe;
            break;
        }
    }

    return tempe;
}

