/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __TOUCH_H__
#define __TOUCH_H__

    //----------
    // INCLUDES
    //----------
    #include "application.h"
    #include "peripheralinit.h"
    #include "wct_hal_cfg.h"
    #include "wct_lib.h"
    #include "wct_hal.h"


    //---------
    // DEFINES
    //---------

    #define PIN_GPIO 0
    #define PIN_I2C 1

    /* Relay states */
    #define SWITCH_KEEP 0
    #define SWITCH_TOUCH 1
    #define SWITCH_POWER 2

    #define TOUCH_OFF 0
    #define TOUCH_ON 1
    #define MPR121_I2C_ADDR 0x5a

    #define DISABLE_IIC_CLOCK (SIM_PCE1&=~SIM_PCE1_IIC0)
    #define ENABLE_TOUCH_IRQ (GPIOC_IENR|=BIT6)
    #define DISABLE_TOUCH_IRQ (GPIOC_IENR&=~BIT6)
    #define CLEAR_TOUCH_IRQ_FLAG (GPIOC_IESR|=BIT6)

    #define SET_RELAY_SET_PIN GPIOE_DR |= BIT4 //D4
    #define SET_RELAY_CLR_PIN GPIOE_DR |= BIT5 //D5
    #define CLR_RELAY_SET_CLR_PIN GPIOE_DR &= ~(BIT4+BIT5)
    #define SET_GPIO_TO_SWITCH_RELAY_TO_TOUCH SET_RELAY_SET_PIN
    #define SET_GPIO_TO_SWITCH_RELAY_TO_POWER SET_RELAY_CLR_PIN
    #define CLR_GPIO_TO_SWITCH_RELAY_TO_KEEP CLR_RELAY_SET_CLR_PIN
    #define TOUCH_GPIO_INTC_IPR_BITS_OFFSET 0    // GPIOC
    #define TOUCH_IRQ_PRIORITY_LEVEL                2
    #define SET_TOUCH_INTERRUPT_PRIORITY           ( INTC_IPR12 = ( INTC_IPR12 & ~INTC_IPR12_GPIOC ) | ( ((TOUCH_IRQ_PRIORITY_LEVEL) + 1) << TOUCH_GPIO_INTC_IPR_BITS_OFFSET ) )


    // MPR121 register definition
    #define MPR121_REGISTER_SIZE 128 
    #define MPR121_TOUCH_STATUS_ADDR 0x00 
    #define MPR121_TOUCH_THRESHOLD_BASE_ADDR 0x41 
    #define MPR121_TOUCH_THRESHOLD_ELE7_ADDR 0x4F 
    #define MPR121_ECR_ADDR 0x5e 
    #define ECR_CL_BIT_MASK 0xC0 
    #define ECR_ELEPROX_EN_BIT_MASK 0x30 
    #define ECR_ELE_EN_BIT_MASK 0x0F 
    #define MPR121_ECR_ENABLE 0x88    //ELE0~ ELE7 for electrode detection enabled
    #define MPR121_ECR_DISABLE 0    //ELE0~ ELE7 for electrode detection disabled

    #define TouchThre 4          //15//30//10
    #define ReleaThre 2          //8//25//8

    #define Prox_TouchThre 15    //6      
    #define Prox_ReleaThre 8     //4


    //----------
    // TYPEDEFS
    //----------

typedef struct{
    BYTE addr;
    BYTE value;
}T_MPR121_SETTING_STRUCT;

    //-----------
    // VARIABLES
    //-----------
 

    //-----------
    // FUNCTIONS
    //-----------
void TouchSensorInit(void);
void switch_GPIO_or_I2C(BYTE i2c_on);
void MPR121_onOff(BYTE onOff);
//void write_MPR121(BYTE reg, BYTE value);
void read_MPR121(BYTE reg, BYTE* value);
void relay_switch(BYTE touch_on);
void touch_enable(void);
void touch_disable(void);
BYTE get_touch_status(void);



#endif // __TOUCH_H__

