/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __DELAY_H__

    #define __DELAY_H__

    //----------
    // INCLUDES
    //----------
    #include "wct_callbacks.h"
	#include "wct_hal.h"
	#include "wct_hal_cfg.h"
    //----------
    // TYPEDEFS
    //----------


    //---------
    // DEFINES
    //---------

    #define NUM_INSTRUCTIONS_PER_DELAY_CALL         16
    #define NUM_INSTRUCTIONS_PER_EMPTY_LOOP         12

    // Delay definitions for operating frequency
    #define INSTRUCTION_TIME_IN_NS                  ( ( 1.00 * 1e9 ) / CPU_CLOCK_FREQ )
    #define FUNC_CALL_OVERHEAD_IN_NS                ( NUM_INSTRUCTIONS_PER_DELAY_CALL * INSTRUCTION_TIME_IN_NS )
    #define EMPTY_LOOP_OVERHEAD_IN_NS               ( NUM_INSTRUCTIONS_PER_EMPTY_LOOP * INSTRUCTION_TIME_IN_NS )

    // Delay definitions for default startup clock
    #define INSTRUCTION_TIME_STARTUP_CLK_IN_NS      ( ( 1.00 * 1e9 ) / ( RELAXATION_OSC_NORMAL_FREQUENCY / 2 ) )
    #define FUNC_CALL_OVERHEAD_STARTUP_CLK_IN_NS    ( NUM_INSTRUCTIONS_PER_DELAY_CALL * INSTRUCTION_TIME_STARTUP_CLK_IN_NS )
    #define EMPTY_LOOP_OVERHEAD_STARTUP_CLK_IN_NS   ( NUM_INSTRUCTIONS_PER_EMPTY_LOOP * INSTRUCTION_TIME_STARTUP_CLK_IN_NS )

    // Macro defintion for conversion of desired microseconds into a terminal count for the DelayUs() function
    // WARNING: Due to optimization, the DelayUs function may not accurately provide a 1us delay!
    // With a system clock of 60MHz the maximum possible us delay is 12500.
    #define CALC_TERMINAL_COUNT( x )                ( ( ( ( ( DWORD ) x * 1000 ) - FUNC_CALL_OVERHEAD_IN_NS ) / EMPTY_LOOP_OVERHEAD_IN_NS ) )
    // Maximum of 196000us possible for this function:
    #define CALC_TERMINAL_COUNT_STARTUP_CLK( x )    ( ( ( ( ( DWORD ) x * 1000 ) - FUNC_CALL_OVERHEAD_STARTUP_CLK_IN_NS ) / EMPTY_LOOP_OVERHEAD_STARTUP_CLK_IN_NS ) )

    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void DelayUs( WORD wTerminalCount );


#endif
