/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __QSCIX_H__

    #define __QSCIX_H__

    //----------
    // INCLUDES
    //----------
	#include "defines.h"


    //----------
    // TYPEDEFS
    //----------


    //---------
    // DEFINES
    //---------

    #define INVALID_HEX_VALUE                       0xFF

    #define ENABLE_QSCI0_RX                         ( QSCI0_CTRL1 |= 0x0004 )
    #define DISABLE_QSCI0_RX                        ( QSCI0_CTRL1 &= ~0x0004 )
    #define ENABLE_QSCI0_TX                         ( QSCI0_CTRL1 |= 0x0008 )
    #define DISABLE_QSCI0_TX                        ( QSCI0_CTRL1 &= ~0x0008 )

    #define ENABLE_QSCI1_RX                         ( QSCI1_CTRL1 |= 0x0004 )
    #define DISABLE_QSCI1_RX                        ( QSCI1_CTRL1 &= ~0x0004 )
    #define ENABLE_QSCI1_TX                         ( QSCI1_CTRL1 |= 0x0008 )
    #define DISABLE_QSCI1_TX                        ( QSCI1_CTRL1 &= ~0x0008 )



    //------------------------ Baud rate defines -------------------------
    // NOTE: UART_BAUD_RATE should be defined in the mapping.h file.
    // NOTE: Formula for calculating the baud rate as given in the data sheet:
    //          BaudRate = clock / ( 16 * ( SBR + ( FRAC_SBR / 8 ) ) )
    // UART_CLOCK_BAUD_DIVIDER = ( SBR + ( FRAC_SBR / 8 ) )
    #define QSCI0_BAUD_RATE                         19200   // Baud rate register value is calculated precompile time in the QSCI module

    #define QSCI0_CLOCK_BAUD_DIVIDER                ( IP_BUS_CLK / ( QSCI0_BAUD_RATE * 16.0 ) )
    #define QSCI0_SBR_VALUE                         ( WORD )QSCI0_CLOCK_BAUD_DIVIDER
    // Calculate the fractional portion of the baud rate, add 0.5 for rounding
    #define QSCI0_FRAC_SBR_VALUE                    ( WORD )( ( ( QSCI0_CLOCK_BAUD_DIVIDER - QSCI0_SBR_VALUE ) * 8.0 ) + 0.5 )
    #define QSCI0_RATE_CONFIGURATION                ( ( QSCI0_SBR_VALUE << 3 ) + ( QSCI0_FRAC_SBR_VALUE & 0x07 ) )

    #define QSCI0_CLOCK_BAUD_DIVIDER_OF_LOW_POWER_MODE   ( IP_BUS_CLK_OF_LOW_POWER_MODE / ( QSCI0_BAUD_RATE * 16.0 ) )
    #define QSCI0_SBR_VALUE_OF_LOW_POWER_MODE                    ( WORD )QSCI0_CLOCK_BAUD_DIVIDER_OF_LOW_POWER_MODE
    // Calculate the fractional portion of the baud rate, add 0.5 for rounding
    #define QSCI0_FRAC_SBR_VALUE_OF_LOW_POWER_MODE                    ( WORD )( ( ( QSCI0_CLOCK_BAUD_DIVIDER_OF_LOW_POWER_MODE - QSCI0_SBR_VALUE_OF_LOW_POWER_MODE ) * 8.0 ) + 0.5 )
    #define QSCI0_RATE_CONFIGURATION_OF_LOW_POWER_MODE                ( ( QSCI0_SBR_VALUE_OF_LOW_POWER_MODE << 3 ) + ( QSCI0_FRAC_SBR_VALUE_OF_LOW_POWER_MODE & 0x07 ) )

    #define QSCI1_UART_BAUD_DIVIDER                 ( IP_BUS_CLK / ( QSCI1_BAUD_RATE * 16.0 ) )
    #define QSCI1_SBR_VALUE                         ( WORD )QSCI1_UART_BAUD_DIVIDER
    // Calculate the fractional portion of the baud rate, add 0.5 for rounding
    #define QSCI1_FRAC_SBR_VALUE                    ( WORD )( ( ( QSCI1_UART_BAUD_DIVIDER - QSCI1_SBR_VALUE ) * 8.0 ) + 0.5 )
    #define QSCI1_RATE_CONFIGURATION                ( ( QSCI1_SBR_VALUE << 3 ) + ( QSCI1_FRAC_SBR_VALUE & 0x07 ) )


	//------------------- IrDA MAC reference clock defines -------------------
	// The IrDA 16X reference clock is generated using the first 16-bit Timer (TMRA0).
	#define IRDA_16X_CLOCK_RATE                     ( QSCI0_BAUD_RATE * 16.0 )
	#define IRDA_16X_CLOCK_PERIOD                   ( 1.0 / IRDA_16X_CLOCK_RATE )
	#define IRDA_TIMER_CLK_DIVISOR                  ( WORD ) ( TMRA0_TIMER_CLOCK * IRDA_16X_CLOCK_PERIOD )

#if defined (SCI_0_SCIBR_INIT) || \
    defined (SCI_0_SCICR2_INIT) || \
    defined (SCI_0_SCICR3_INIT) || \
    defined (SCI_0_SCICR_INIT)
#define QSCI0_ENABLED TRUE
#else
#define QSCI0_ENABLED FALSE
#endif

#if defined (SCI_1_SCIBR_INIT) || \
    defined (SCI_1_SCICR2_INIT) || \
    defined (SCI_1_SCICR3_INIT) || \
    defined (SCI_1_SCICR_INIT)
#define QSCI1_ENABLED TRUE
#else
#define QSCI1_ENABLED FALSE
#endif


    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void Qsci0Init( void );
    void Qsci0PutChar( BYTE byChar );               // Transmit a character
    BYTE Qsci0GetChar( void );                      // Gets the next character from the UART receive buffer
    BYTE Qsci0GetRxBytes( void );                   // Return the number of bytes in the Rx buffer
    void Qsci0FlushBuffer( void );
    WORD Qsci0GetStatus( void );
    
    void Qsci1Init( void );
    void Qsci1PutChar( BYTE byChar );               // Transmit a character
    BYTE Qsci1GetChar( void );                      // Gets the next character from the UART receive buffer
    BYTE Qsci1GetRxBytes( void );                   // Return the number of bytes in the Rx buffer
    void Qsci1FlushBuffer( void );
    WORD Qsci1GetStatus( void );

#endif
