/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------



//----------------------
// APPLICATION INCLUDES
//----------------------
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"

#include "adc.h"

#define COIL_CURRENT_AVERAGE  0


//-----------------
// PRIVATE DEFINES
//-----------------

#define ADC_ROLLING_AVG_SAMPLE_SIZE_IN_BITS         ADC_ROLLING_AVG_SAMPLE_CONFIG
#define ADC_ROLLING_AVG_SAMPLE_SIZE                 ( 1 << ADC_ROLLING_AVG_SAMPLE_SIZE_IN_BITS )

#define DEBUG_ADC                                   FALSE 

ADC_CONVERSION_COMPLETE_INTERRUPT;
//-------------------
// PRIVATE VARIABLES
//-------------------

volatile WORD wAdcBuffer[ ADC_CHANNEL_MAX ][ ADC_ROLLING_AVG_SAMPLE_SIZE ];
volatile WORD wBufferPtr = 0;

volatile WORD wCoilCurrentPtr = 0;

//-------------------
// PRIVATE FUNCTIONS
//-------------------

// ADC Interrupt Service Routine
// This routine assumes the sequencing of the channels is configured to start
//  at Result0 and progress through the list of ADC_NUM_CHANS in the proper order.
//------------------------------------------------------------------------------
#if CODE_WARRIOR
#pragma interrupt saveall
#endif
ADC_CONVERSION_COMPLETE_INTERRUPT
{
    WORD wCurrentChannel;
    WORD *pwADCptr;

    // NOTE: This ISR assumes the ADC is configured for SCANNING mode... stopping after 1 scan.
    //  The ADC device will interrupt when the buffer is filled with samples
    //  from the scan.  The application should restart the scanning when appropriate.

    // Clear the main interrupt flag (cleared by writing a one to the flag)
    ADC_STAT |= 0x0800;

    wAdcBuffer[ ADC_CHANNEL_VIN_SENSE ] [ wBufferPtr ] = ADC_RSLT0>>3;
    wAdcBuffer[ ADC_CHANNEL_INPUT_CURR_SENSE_C1 ] [ wBufferPtr ] = ADC_RSLT2>>3;
    wAdcBuffer[ ADC_CHANNEL_BOARD_TEMPERATURE ] [ wBufferPtr ] = ADC_RSLT9>>3;

    // Increment the buffer pointer, check for roll-over
    wBufferPtr++;

    if( wBufferPtr == ADC_ROLLING_AVG_SAMPLE_SIZE )
    {
        wBufferPtr = 0;
    }
}
#if CODE_WARRIOR
#pragma interrupt off
#endif


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

//------------------------------------------------------------------------------
void AdcInit( void )
{
    int i;
    int j;

    // This connects the system clock to the ADC peripheral
    //enable clock in SYS module
    //    ENABLE_ADC_IPBUS_CLK;

    // Configure the A/D as described in "mapping.h"
  #ifdef ADC_ADCR1_INIT
  ADC_CTRL1   = ADC_ADCR1_INIT;
  #endif
  #ifdef ADC_ADCR2_INIT
  ADC_CTRL2   = ADC_ADCR2_INIT;
  #endif
//    ADC_ZXCTRL  = ADC_ZXCTRL_INIT;
  #ifdef ADC_ADLLMT0_INIT
  ADC_LOLIM0  = ADC_ADLLMT0_INIT;
  #endif
  #ifdef ADC_ADLLMT1_INIT
  ADC_LOLIM1  = ADC_ADLLMT1_INIT;
  #endif
  #ifdef ADC_ADLLMT2_INIT
  ADC_LOLIM2  = ADC_ADLLMT2_INIT;
  #endif
  #ifdef ADC_ADLLMT3_INIT
  ADC_LOLIM3  = ADC_ADLLMT3_INIT;
  #endif
  #ifdef ADC_ADLLMT4_INIT
  ADC_LOLIM4  = ADC_ADLLMT4_INIT;
  #endif
  #ifdef ADC_ADLLMT5_INIT
  ADC_LOLIM5  = ADC_ADLLMT5_INIT;
  #endif
  #ifdef ADC_ADLLMT6_INIT
  ADC_LOLIM6  = ADC_ADLLMT6_INIT;
  #endif
  #ifdef ADC_ADLLMT7_INIT
  ADC_LOLIM7  = ADC_ADLLMT7_INIT;
  #endif
  #ifdef ADC_ADHLMT0_INIT
  ADC_HILIM0  = ADC_ADHLMT0_INIT;
  #endif
  #ifdef ADC_ADHLMT1_INIT
  ADC_HILIM1  = ADC_ADHLMT1_INIT;
  #endif
  #ifdef ADC_ADHLMT2_INIT
  ADC_HILIM2  = ADC_ADHLMT2_INIT;
  #endif
  #ifdef ADC_ADHLMT3_INIT
  ADC_HILIM3  = ADC_ADHLMT3_INIT;
  #endif
  #ifdef ADC_ADHLMT4_INIT
  ADC_HILIM4  = ADC_ADHLMT4_INIT;
  #endif
  #ifdef ADC_ADHLMT5_INIT
  ADC_HILIM5  = ADC_ADHLMT5_INIT;
  #endif
  #ifdef ADC_ADHLMT6_INIT
  ADC_HILIM6  = ADC_ADHLMT6_INIT;
  #endif
  #ifdef ADC_ADHLMT7_INIT
  ADC_HILIM7  = ADC_ADHLMT7_INIT;
  #endif
    // Offsets are zero by default (ADC_OFFSTn registers)
    // If an offset is required the application must set the offsets
    // apart from the AdcInit() function
  #ifdef ADC_ADCPWR2_INIT
  ADC_PWR2    = ADC_ADCPWR2_INIT;
  #endif
  #ifdef ADC_ADCCAL_INIT
  ADC_CAL     = ADC_ADCCAL_INIT;
  #endif
  #ifdef ADC_ADCGC1_INIT
  ADC_GC1     = ADC_ADCGC1_INIT;
  #endif
  #ifdef ADC_ADCGC2_INIT
  ADC_GC2     = ADC_ADCGC2_INIT;
  #endif
  #ifdef ADC_ADCSCTRL_INIT
  ADC_SCTRL   = ADC_ADCSCTRL_INIT;
  #endif

    // Configure the ADC sequence as described in "mapping.h"
  #ifdef ADC_ADLST1_INIT
  ADC_CLIST1  = ADC_ADLST1_INIT;
  #endif
  #ifdef ADC_ADLST2_INIT
  ADC_CLIST2  = ADC_ADLST2_INIT;
  #endif
  #ifdef ADC_ADLST3_INIT
  ADC_CLIST3  = ADC_ADLST3_INIT;
  #endif
  #ifdef ADC_ADLST4_INIT
  ADC_CLIST4  = ADC_ADLST4_INIT;
  #endif
    //ADC_SDIS    = ( 0xFFFF << ADC_NUM_CHANS );  // Clear bits for enabled channels
    ADC_SDIS    = ADC_ADSDIS_INIT; //We used both ADCA and ADCB, so we can not use channel mask as (0xFFFF << ADC_NUM_CHANS)

    // Configure ADC Comparator channels?
    // place code here

    // Enable the channels specified in "mapping.h"?
    // place code here

    // Configure the ADC interrupt
    ADC_SET_INT_PRIORITY(0); //set ADC conv complete to 0
    ADC_STAT  |= ADC_STAT_EOSI1;     // clear interrupt flags by writting a 1 to them
    ADC_STAT  |= ADC_STAT_EOSI0;
    ADC_CTRL1 |= ADC_CTRL1_EOSIE0;     // Enable end of scan interrupts

    // Powerup the ADC module
  #ifdef ADC_ADCPOWER_INIT
  ADC_PWR = ADC_ADCPOWER_INIT;
  #endif

    // Wait for power-up to complete
    if(!(ADC_ADCPOWER_INIT&ADC_PWR_APD))    //Only when auto powerdown is not active, wait power-up
        while( ADC_PWR & ( ADC_PWR_PSTS0 | ADC_PWR_PSTS1) ){};  

    // NOTE: The ADC should be software triggered using the ENABLE_ADC_CONVERSIONS
    //  definition when required by the application

    // Initialize the ADC buffer values
    for( i = 0; i < ADC_CHANNEL_MAX; i++ )
    {
        for( j = 0; j < ADC_ROLLING_AVG_SAMPLE_SIZE; j++ )
        {
            wAdcBuffer[ i ][ j ] = 0;
        }
    }
}


//------------------------------------------------------------------------------
WORD AdcRead( ADC_CHANNEL_ID eAdcChannel )
{   
    // Read a value from the ADC local buffer

    DWORD dwAdcTemp;
    BOOL bIntSetting = FALSE;
    int i;

    // Verify valid channel request
    if( eAdcChannel < ADC_CHANNEL_MAX )
    {
#if !COIL_CURRENT_AVERAGE
    	if(eAdcChannel == ADC_CHANNEL_COIL_CURR_SENSE_C1)
    	{
    		dwAdcTemp = wAdcBuffer[ eAdcChannel ][ 0 ];
    	}
    	else
    	{
#endif
        if( ADC_EOS_INT_SETTING )
        {
            bIntSetting = TRUE;
        }
        DISABLE_ADC_EOS_INTERRUPTS;

        // Calculate the rolling average
        for( i = 0, dwAdcTemp = 0; i < ADC_ROLLING_AVG_SAMPLE_SIZE; i++ )
        {
            dwAdcTemp += wAdcBuffer[ eAdcChannel ][ i ];
        }

        dwAdcTemp >>= ADC_ROLLING_AVG_SAMPLE_SIZE_IN_BITS;

        // Restore interrupt enable state
        if( bIntSetting )
        {
            ENABLE_ADC_EOS_INTERRUPTS;
        }
#if !COIL_CURRENT_AVERAGE        
    	}
#endif
        return (WORD) dwAdcTemp;
    }
    else
    {
        return RET_VALUE_INVALID_PARAM;
    }
}

void AdcWriteChannel(BYTE byChannel, WORD wData)
{
	if( byChannel ==  ADC_CHANNEL_COIL_CURR_SENSE_C1 )
	{
#if !COIL_CURRENT_AVERAGE 
		wAdcBuffer[ byChannel ][ 0 ] = wData;
#else
		wAdcBuffer[ byChannel ][ wCoilCurrentPtr ] = wData;
		wCoilCurrentPtr++;
		if( wBufferPtr >= ADC_ROLLING_AVG_SAMPLE_SIZE )
			wBufferPtr = 0;
#endif
	}
}

