/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __ADC_H__

    #define __ADC_H__

    //----------
    // INCLUDES
    //----------
	#include "wct_callbacks.h"
	#include "wct_hal.h"
	#include "wct_hal_cfg.h"
    //----------
    // TYPEDEFS
    //----------
    // A/D readings are digitally filtered using a rolling average
    typedef enum
    {
        ADC_ROLLING_AVG_EQUAL_1_SAMPLE = 0,
        ADC_ROLLING_AVG_EQUAL_2_SAMPLES,
        ADC_ROLLING_AVG_EQUAL_4_SAMPLES,
        ADC_ROLLING_AVG_EQUAL_8_SAMPLES,
        ADC_ROLLING_AVG_EQUAL_16_SAMPLES,
        ADC_ROLLING_AVG_EQUAL_32_SAMPLES
    } ADC_ROLLING_AVG_SAMPLES;



    //---------
    // DEFINES
    //---------
    
    #define MAX_ADC_COUNTS                              0x0FFF
    #define ADC_MAX_COUNTS                              0x0FFF             // for backward compatibility
    #define ADC_BITS                                    12
    #define ADC_COUNTS_PER_VOLT                         ( ( int ) ( MAX_ADC_COUNTS / ADC_REFERENCE_VOLTAGE ) )
    #define MICROVOLTS_PER_ADC_COUNT                    ( WORD )( ( ADC_REFERENCE_VOLTAGE / MAX_ADC_COUNTS ) * 1000000 )
    #define MICROVOLTS_PER_ADC_COUNT_FLOAT              ( ( ADC_REFERENCE_VOLTAGE / MAX_ADC_COUNTS ) * 1000000.0 )

    // Time to capture one ADC12 reading, in nanoseconds.  (= 3840 ns for OSC_H = 250 MHz)
    // Each conversion takes 60 ADC clock ticks.  The ADC12 clock runs at OSC_H/16 (=250/16 MHz).
    // The 1000 and 1000000 term in this formula keep the integer math well-behaved.
    #define ADC12_SAMPLE_TIME_NS                        ( ( 1000L * 60 * 16 ) / ( INTOSC_FREQ / 1000000 ) )

 

    //------------ General Register definitions ---------------
    // These definitions are used by the timer module to trigger the ADC
    #define ENABLE_ADCA_CONVERSIONS                      ( ADC_CTRL1 |= ADC_CTRL1_START0 )
    #define DISABLE_ADCA_CONVERSIONS                     ( ADC_CTRL1 |= ADC_CTRL1_STOP0 )

    #define ENABLE_ADCB_CONVERSIONS                      ( ADC_CTRL2 |= ADC_CTRL2_START1 )
    #define DISABLE_ADCB_CONVERSIONS                     ( ADC_CTRL2 |= ADC_CTRL2_STOP1 )

    #define ENABLE_ADCB_HW_TRIGGER                      ( ADC_CTRL2 |= ADC_CTRL2_DMAEN1+ADC_CTRL2_SYNC1 )
    #define DISABLE_ADCB_HW_TRIGGER                     ( ADC_CTRL2 &= ~(ADC_CTRL2_DMAEN1+ADC_CTRL2_SYNC1) )

    //  WARNING: Clearing INT_ENA at the peripheral will clear the interrupt flag as a side effect, if it was set.
    //   For proper operation, control the ADC interrupts at the CIM rather than the ADCCTRL1 peripheral register.
    #define DISABLE_ADC_EOS_INTERRUPTS                  ( ADC_CTRL1 &= ~ADC_CTRL1_EOSIE0 )
    #define ENABLE_ADC_EOS_INTERRUPTS                   ( ADC_CTRL1 |= ADC_CTRL1_EOSIE0 )

    #define ADC_EOS_INT_SETTING                         ( ADC_CTRL1 & ADC_CTRL1_EOSIE0 )
    #define CLR_ADC_EOS0_INTERRUPTS_STS                 (ADC_STAT |= ADC_STAT_EOSI0)

    //----------- ADC_CLISTn Register definitions ------------
    // NOTE: These are all the options to choose from for each
    //       ADC sample (There are up to 16 samples possible)
    //       Place these definitions into the proper nibble of
    //       the appropriate register.
    // ADC_CLIST1: SAMPLE3 SAMPLE2 SAMPLE1 SAMPLE0 (from MSb to LSb)
    // ADC_CLIST2: SAMPLE7 SAMPLE6 SAMPLE5 SAMPLE4
    // ADC_CLIST3: SAMPLE11 SAMPLE10 SAMPLE9 SAMPLE8
    // ADC_CLIST4: SAMPLE15 SAMPLE14 SAMPLE13 SAMPLE12
    #define SINGLE_ENDED_ANA0_DIFF_ANA0PLUS_ANA1MINUS   0x00
    #define SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS   0x01
    #define SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS   0x02
    #define SINGLE_ENDED_ANA3_DIFF_ANA2PLUS_ANA3MINUS   0x03
    #define SINGLE_ENDED_ANB0_DIFF_ANA0PLUS_ANA1MINUS   0x04
    #define SINGLE_ENDED_ANB1_DIFF_ANA0PLUS_ANA1MINUS   0x05
    #define SINGLE_ENDED_ANB2_DIFF_ANA2PLUS_ANA3MINUS   0x06
    #define SINGLE_ENDED_ANB3_DIFF_ANA2PLUS_ANA3MINUS   0x07
    #define SINGLE_ENDED_ANA4_DIFF_ANA4PLUS_ANA5MINUS   0x08
    #define SINGLE_ENDED_ANA5_DIFF_ANA4PLUS_ANA5MINUS   0x09
    #define SINGLE_ENDED_ANA6_DIFF_ANA6PLUS_ANA7MINUS   0x0A
    #define SINGLE_ENDED_ANA7_DIFF_ANA6PLUS_ANA7MINUS   0x0B
    #define SINGLE_ENDED_ANB4_DIFF_ANA4PLUS_ANA5MINUS   0x0C
    #define SINGLE_ENDED_ANB5_DIFF_ANA4PLUS_ANA5MINUS   0x0D
    #define SINGLE_ENDED_ANB6_DIFF_ANA6PLUS_ANA7MINUS   0x0E
    #define SINGLE_ENDED_ANB7_DIFF_ANA6PLUS_ANA7MINUS   0x0F

    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void AdcInit( void );
    WORD AdcRead( ADC_CHANNEL_ID eAdcChannel );


#endif
