/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __COMPARATOR_H__
    #define __COMPARATOR_H__

    //----------
    // INCLUDES
    //----------
	#include "wct_callbacks.h"
	#include "wct_hal.h"
	#include "wct_hal_cfg.h"
    //----------
    // TYPEDEFS
    //----------

    //---------
    // DEFINES
    //---------

    #define COMP_DAC_MIN_VOLTAGE_MV                     0
    #define COMP_DAC_MAX_VOLTAGE_MV                     ( ( WORD ) ( COMP_DAC_REFERENCE_VOLTAGE * 1000 ) )

    #define MAX_COMP_DAC_COUNTS                         0x003F
    #define COMP_DAC_BITS                               6
    #define COMP_DAC_COUNTS_PER_VOLT                    ( MAX_COMP_DAC_COUNTS / COMP_DAC_REFERENCE_VOLTAGE )

    #define COMPC_DAC_SETTING_COUNTS                    ( ( WORD ) ( COMPC_DAC_SETTING_V * COMP_DAC_COUNTS_PER_VOLT ) )

    //------------ General Register definitions ---------------
    #define ENABLE_COMPA                                ( CMPA_CR1 |= COMP_ENABLED )
    #define ENABLE_COMPB                                ( CMPB_CR1 |= COMP_ENABLED )
    #define ENABLE_COMPC                                ( CMPC_CR1 |= COMP_ENABLED )
    #define ENABLE_COMPD                                ( CMPD_CR1 |= COMP_ENABLED )

    #define DISABLE_COMPA                               ( CMPA_CR1 &= ~COMP_DISABLED )
    #define DISABLE_COMPB                               ( CMPB_CR1 &= ~COMP_DISABLED )
    #define DISABLE_COMPC                               ( CMPC_CR1 &= ~COMP_DISABLED )
    #define DISABLE_COMPD                               ( CMPD_CR1 &= ~COMP_DISABLED )


    //------------ CMPx_CR0 Register definitions ------------
    #define COMP_FILTER_IS_DISABLED                     0x0000
    #define COMP_FILTER_SAMPLE_COUNT_IS_1               0x0010
    #define COMP_FILTER_SAMPLE_COUNT_IS_2               0x0020
    #define COMP_FILTER_SAMPLE_COUNT_IS_3               0x0030
    #define COMP_FILTER_SAMPLE_COUNT_IS_4               0x0040
    #define COMP_FILTER_SAMPLE_COUNT_IS_5               0x0050
    #define COMP_FILTER_SAMPLE_COUNT_IS_6               0x0060
    #define COMP_FILTER_SAMPLE_COUNT_IS_7               0x0070
    #define COMP_HYSTERESIS_LEVEL_0                     0x0000
    #define COMP_HYSTERESIS_LEVEL_1                     0x0001
    #define COMP_HYSTERESIS_LEVEL_2                     0x0002
    #define COMP_HYSTERESIS_LEVEL_3                     0x0003

    //------------ CMPx_CR1 Register definitions ------------
    #define COMP_SAMPLING_MODE_ENABLED                  0x0080
    #define COMP_SAMPLING_MODE_DISABLED                 0x0000
    #define COMP_WINDOWING_MODE_ENABLED                 0x0040
    #define COMP_WINDOWING_MODE_DISABLED                0x0000
    #define COMP_LOW_SPEED_MODE                         0x0000
    #define COMP_HIGH_SPEED_MODE                        0x0010
    #define COMP_OUTPUT_NOT_INVERTED                    0x0000
    #define COMP_OUTPUT_INVERTED                        0x0008
    #define COMP_OUTPUT_FILTERED                        0x0000
    #define COMP_OUTPUT_NOT_FILTERED                    0x0004
    #define COMP_OUTPUT_PIN_ENABLED                     0x0002
    #define COMP_OUTPUT_PIN_DISABLED                    0x0000
    #define COMP_ENABLED                                0x0001
    #define COMP_DISABLED                               0x0000

    //------------ CMPx_CFPR Register definitions ------------
    #define COMP_FILTER_SAMPLE_PERIOD_BITMASK           0x00FF

    //------------ CMPx_SCR Register definitions ------------
    #define COMP_RISING_EDGE_INTERRUPT_ENABLED          0x0010
    #define COMP_RISING_EDGE_INTERRUPT_DISABLED         0x0000
    #define COMP_FALLING_EDGE_INTERRUPT_ENABLED         0x0008
    #define COMP_FALLING_EDGE_INTERRUPT_DISABLED        0x0000
    #define COMP_RISING_EDGE_INTERRUPT_FLAG_BITMASK     0x0004
    #define COMP_FALLING_EDGE_INTERRUPT_FLAG_BITMASK    0x0002
    #define COMP_OUTPUT_BITMASK                         0x0001

    //------------ CMPx_DACCR Register definitions ------------
    #define COMP_DAC_ENABLED                            0x0080
    #define COMP_DAC_DISABLED                           0x0000
    #define COMP_DAC_REFERENCE_IS_VIN1                  0x0000
    #define COMP_DAC_REFERENCE_IS_VIN2                  0x0040
    #define COMP_DAC_VOLTAGE_SELECT_BITMASK             0x003F

    //------------ CMPx_MUXCR Register definitions ------------
    #define COMP_PASS_THROUGH_MODE_ENABLED              0x0080
    #define COMP_PASS_THROUGH_MODE_DISABLED             0x0000
    #define COMP_PLUS_INPUT_IS_IN0                      0x0000
    #define COMP_PLUS_INPUT_IS_IN1                      0x0008
    #define COMP_PLUS_INPUT_IS_IN2                      0x0010
    #define COMP_PLUS_INPUT_IS_IN3                      0x0018
    #define COMP_PLUS_INPUT_IS_IN4                      0x0020
    #define COMP_PLUS_INPUT_IS_IN5                      0x0028
    #define COMP_PLUS_INPUT_IS_IN6                      0x0030
    #define COMP_PLUS_INPUT_IS_IN7                      0x0038
    #define COMP_MINUS_INPUT_IS_IN0                     0x0000
    #define COMP_MINUS_INPUT_IS_IN1                     0x0001
    #define COMP_MINUS_INPUT_IS_IN2                     0x0002
    #define COMP_MINUS_INPUT_IS_IN3                     0x0003
    #define COMP_MINUS_INPUT_IS_IN4                     0x0004
    #define COMP_MINUS_INPUT_IS_IN5                     0x0005
    #define COMP_MINUS_INPUT_IS_IN6                     0x0006
    #define COMP_MINUS_INPUT_IS_IN7                     0x0007


#if defined (HSCMP_A_CR0_INIT) || \
    defined (HSCMP_A_CR1_INIT) || \
    defined (HSCMP_A_FPR_INIT) || \
    defined (HSCMP_A_SCR_INIT) || \
    defined (HSCMP_A_DACCR_INIT) || \
    defined (HSCMP_A_MUXCR_INIT) || \
    defined (HSCMP_A_PCR_INIT)
#define COMPARATORA_ENABLED 1
#endif

#if defined (HSCMP_B_CR0_INIT) || \
    defined (HSCMP_B_CR1_INIT) || \
    defined (HSCMP_B_FPR_INIT) || \
    defined (HSCMP_B_SCR_INIT) || \
    defined (HSCMP_B_DACCR_INIT) || \
    defined (HSCMP_B_MUXCR_INIT) || \
    defined (HSCMP_B_PCR_INIT)
#define COMPARATORB_ENABLED 1
#endif

#if defined (HSCMP_C_CR0_INIT) || \
    defined (HSCMP_C_CR1_INIT) || \
    defined (HSCMP_C_FPR_INIT) || \
    defined (HSCMP_C_SCR_INIT) || \
    defined (HSCMP_C_DACCR_INIT) || \
    defined (HSCMP_C_MUXCR_INIT) || \
    defined (HSCMP_C_PCR_INIT)
#define COMPARATORC_ENABLED 1
#endif

#if defined (HSCMP_D_CR0_INIT) || \
    defined (HSCMP_D_CR1_INIT) || \
    defined (HSCMP_D_FPR_INIT) || \
    defined (HSCMP_D_SCR_INIT) || \
    defined (HSCMP_D_DACCR_INIT) || \
    defined (HSCMP_D_MUXCR_INIT) || \
    defined (HSCMP_D_PCR_INIT)
#define COMPARATORD_ENABLED 1
#endif

    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void CompInit( void );

#endif // __COMPARATOR_H__
