/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* o Redistributions of source code must retain the above copyright notice, this list
*   of conditions and the following disclaimer.
*
* o Redistributions in binary form must reproduce the above copyright notice, this
*   list of conditions and the following disclaimer in the documentation and/or
*   other materials provided with the distribution.
*
* o Neither the name of Freescale Semiconductor, Inc. nor the names of its
*   contributors may be used to endorse or promote products derived from this
*   software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "peripheralInit.h"
#include "wct_callbacks.h"
#include "dac.h"

//-----------------
// PRIVATE DEFINES
//-----------------

// This is the worst case amount of time it takes for the DAC's output to settle. The glitch 
// filter should be configured to to wait at least this long before applying a new value
// to the output.
#define DAC_SETTLING_TIME_NS                        240
#define DAC_FILT_CNT_SETTING                        ( WORD )( 240 / CPU_CLOCK_PERIOD_NS )


//-------------------
// PRIVATE VARIABLES
//-------------------


//-------------------
// PRIVATE FUNCTIONS
//-------------------


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

//-----------------------------------------------------------------------------
void DacAInit( void )
{
//    ENABLE_DACA_IPBUS_CLK; enabled in appl

#ifdef DAC_A_CTRL1_INIT
	DACA_CTRL1 = DAC_A_CTRL1_INIT;
#endif
#ifdef DAC_A_CTRL_INIT
    DACA_CTRL0 = DAC_A_CTRL_INIT;
#endif
    ENABLE_DACA;
}


//-----------------------------------------------------------------------------
BYTE DacASetCounts( BYTE byChannel, WORD wCounts )
{
    WORD wData;
    BYTE byRetValue = RET_VALUE_GOOD;

    // If requested count is too high, set to MAX
    if( MAX_DAC_COUNTS < wCounts )
    {
        wCounts = MAX_DAC_COUNTS;
    }

    if( byChannel == 0 )
    {
       DACA_DATAREG_FMT0 = wCounts;
    }
    else
    {
        byRetValue = RET_VALUE_BAD;
    }

    return byRetValue;
}


//-----------------------------------------------------------------------------
WORD DacAGetCounts( BYTE byChannel )
{
    WORD wI2cData;
    WORD wDacData = 0;

    if( byChannel == 0 )
    {
        wDacData = DACA_DATAREG_FMT0;
    }

    return wDacData;
}


//-----------------------------------------------------------------------------
BYTE DacASetVoltage( BYTE byChannel, WORD wNewVoltage_mV )
{
    DWORD dwDacCount;

    // If requested voltage is too high, set to MAX
    if( DAC_MAX_VOLTAGE_MV < wNewVoltage_mV )
    {
        wNewVoltage_mV = DAC_MAX_VOLTAGE_MV;
    }

    dwDacCount = wNewVoltage_mV;
    dwDacCount *= DAC_COUNTS_PER_VOLT;
    dwDacCount /= 1000;
    dwDacCount &= MAX_DAC_COUNTS;

    return ( DacASetCounts( byChannel, (WORD)dwDacCount ) );
    
}


//-----------------------------------------------------------------------------
WORD DacAGetVoltage( BYTE byChannel )
{
    DWORD dwDacVoltageMv = 0;
    WORD wDacData;

    wDacData = DacAGetCounts( byChannel );

    // Calculate the mV setting
    dwDacVoltageMv = wDacData;
    dwDacVoltageMv *= MICROVOLTS_PER_DAC_COUNT;
    dwDacVoltageMv /= 1000;
    
    // Return result in mV
    return( (WORD) dwDacVoltageMv );
}
