/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* o Redistributions of source code must retain the above copyright notice, this list
*   of conditions and the following disclaimer.
*
* o Redistributions in binary form must reproduce the above copyright notice, this
*   list of conditions and the following disclaimer in the documentation and/or
*   other materials provided with the distribution.
*
* o Neither the name of Freescale Semiconductor, Inc. nor the names of its
*   contributors may be used to endorse or promote products derived from this
*   software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
* ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

#ifndef __DAC_H__
    #define __DAC_H__

    //----------
    // INCLUDES
    //----------
    #include "defines.h"

    //----------
    // TYPEDEFS
    //----------

    //---------
    // DEFINES
    //---------

    #define DAC_MIN_VOLTAGE_MV                          0
    #define DAC_MAX_VOLTAGE_MV                          ( ( WORD ) ( DAC_REFERENCE_VOLTAGE * 1000 ) )

    #define MAX_DAC_COUNTS                              0x0FFFUL
    #define DAC_BITS                                    12
    #define DAC_COUNTS_PER_VOLT                         ( ( WORD ) ( MAX_DAC_COUNTS / DAC_REFERENCE_VOLTAGE ) )
    #define MICROVOLTS_PER_DAC_COUNT                    ( WORD )( ( DAC_REFERENCE_VOLTAGE / MAX_DAC_COUNTS ) * 1000000 )
    #define MICROVOLTS_PER_DAC_COUNT_FLOAT              ( ( DAC_REFERENCE_VOLTAGE / MAX_DAC_COUNTS ) * 1000000.0 )


    //------------ General Register definitions ---------------
    #define ENABLE_DACA                                 ( DACA_CTRL0 &= ~ENABLE_DAC )
    #define ENABLE_DACB                                 ( DACB_CTRL0 &= ~ENABLE_DAC )


    // NOTE: All register definitions are valid for both DACA and DACB

    //------------ DACx_CTRL0 Register definitions ------------
    #define DAC_ENABLE_GLITCH_FILTER                    0x1000
    #define DAC_DISABLE_GLITCH_FILTER                   0x0000
    #define DAC_WATERMARK_LEVEL_IS_0                    0x0000
    #define DAC_WATERMARK_LEVEL_IS_2                    0x0100
    #define DAC_WATERMARK_LEVEL_IS_4                    0x0200
    #define DAC_WATERMARK_LEVEL_IS_6                    0x0300
    #define DAC_ENABLE_DMA_SUPPORT                      0x0080
    #define DAC_DISABLE_DMA_SUPPORT                     0x0000
    #define DAC_HIGH_SPEED_MODE                         0x0000
    #define DAC_LOW_SPEED_MODE                          0x0040
    #define DAC_ENABLE_UP_COUNTING                      0x0020
    #define DAC_DISABLE_UP_COUNTING                     0x0000
    #define DAC_ENABLE_DOWN_COUNTING                    0x0010
    #define DAC_DISABLE_DOWN_COUNTING                   0x0000
    #define DAC_DISABLE_AUTOMATIC_MODE                  0x0000
    #define DAC_ENABLE_AUTOMATIC_MODE                   0x0008
    #define DAC_SYNCHRONOUS_MODE                        0x0004
    #define DAC_ASYNCHRONOUS_MODE                       0x0000
    #define DAC_DATA_RIGHT_JUSTIFIED                    0x0000
    #define DAC_DATA_LEFT_JUSTIFIED                     0x0002
    #define DISABLE_DAC                                 0x0001
    #define ENABLE_DAC                                  0x0000

    //------------ DACx_STATUS Register definitions ------------
    #define DAC_FIFO_IS_FULL_BIT                        0x0002
    #define DAC_FIFO_IS_EMPTY_BIT                       0x0001

    //------------ DACx_CTRL1 Register definitions ------------
    // BITS[5:0] of this register contain the glitch filter count value (FILT_CNT).

    // From the reference manual:
    // This field represents the number of IP Bus clock cycles for which the DAC output is 
    // held unchanged after new data is presented to the analog DAC's inputs. The number of clock cycles 
    // for which DAC output is held unchanged is equal to the value of FILT_CNT. Approximately 240 ns 
    // is needed for worst-case settling of the DAC output, so a value of 29 should be used for 125 MHz 
    // operation and a value of 33 should be used for 140 MHz operation. The default value is 29 (can 
    // be used for 125 MHz operation)

    // NOTE: When using the glitch filter, ensure that the filter count is less than the update count. Otherwise,
    // the DAC output will never be updated.

    #define DAC_GLITCH_FILTER_COUNT_BITMASK             0x003F

    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void DacAInit( void );
    BYTE DacASetCounts( BYTE byChannel, WORD wCounts );
    WORD DacAGetCounts( BYTE byChannel );
    BYTE DacASetVoltage( BYTE byChannel, WORD wNewVoltage_mV );
    WORD DacAGetVoltage( BYTE byChannel );

#endif // __DAC_H__
