/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __DEFS_H__
    #define __DEFS_H__

    //----------
    // INCLUDES
    //----------
    #include "cpu.h"


    //----------
    // TYPEDEFS
    //----------
#if 0
    // For now, leave these for Freescale to #define.
    typedef signed char             int8;
    typedef char                    Uint8;
    typedef int                     int16;
    typedef unsigned int            Uint16;
    typedef long                    int32;
    typedef unsigned long           Uint32;
    typedef float                   float32;
#endif   
    typedef unsigned char           BYTE;
    typedef char                    SBYTE;
    typedef unsigned int            WORD;
    typedef int                     SWORD;
    typedef unsigned long           DWORD;
    typedef long                    SDWORD;

    typedef BYTE                    BOOL;



    // NOTE: Enumerations are not understood by the preprocessor!
    // Boolean logic states
    enum
    {
        NO=0,  LOW=0,  INACTIVE=0, INVALID=0,
        YES=1, HIGH=1, ACTIVE=1,   VALID=1
    };
    
/*    #define TRUE                       1
    #define FALSE                      0
*/

    //---------
    // DEFINES
    //---------

    #define WCT1000_PROCESSOR        TRUE
    #define CODE_WARRIOR                TRUE

    #define BOOTLOADER_INIT_TIME_MS     10          // This is the time it takes the bootloader to initialize and enter the application code

    #ifndef NULL
        #define NULL                    ((void*)0)
    #endif /* NULL */

    #define MAX_BYTE_VALUE              0xFF
    #define MAX_WORD_VALUE              0xFFFF
    #define MAX_DWORD_VALUE             0xFFFFFFFF

    #define RET_VALUE_GOOD              0
    #define RET_VALUE_BAD               1
    #define RET_VALUE_INVALID_PARAM     0xFF

    // Masks for extracting bytes from a 32-bit integer
    #define BYTE0_MASK                  0x000000FF
    #define BYTE1_MASK                  0x0000FF00
    #define BYTE2_MASK                  0x00FF0000
    #define BYTE3_MASK                  0xFF000000

    #define BYTE_ACCESS_NOT_SUPPORTED   FALSE
    #define STRUCT_POINTERS_USE_WORD_ADDRESSING   TRUE
    #define STANDARD_REGISTER_SIZE      WORD
    #define NVM_BYTE_POINTER            __pmem BYTE *


    // No pack directive available on Freescale compiler
    #define PACKING_SUPPORTED           FALSE
    #define PACKING_ATTRIBUTE

    // We're ARM-Big-Endian, so we cannot directly pack EEPROM structs  (must be transformed from EE into RAM)
    #define BIG_ENDIAN                  FALSE

    // The GPIO interrupt capability on this processor is limited to selecting RISING or FALLING edge (not BOTH)
    #define SINGLE_INTERRUPT_FLAG_FOR_ALL_GPIO_CHANNELS                     FALSE
    #define PROCESSOR_ALWAYS_GENERATES_GPIO_INTERRUPTS_ON_BOTH_EDGES        FALSE
    #define PROCESSOR_SUPPORTS_CONFIG_OF_GPIO_INTERRUPTS_ON_BOTH_EDGES      FALSE
    #define PROCESSOR_INTERRUPTS_REQUIRE_ACKNOWLEDGE                        FALSE

    #define BITS_PER_INT8           8
    #define BITS_PER_INT16          16

    #define SIGN_EXTEND_8_TO_16(x)  ( ( (x) << BITS_PER_INT8 ) >> BITS_PER_INT8 )
    #define SIGN_EXTEND_16_TO_32(x) ( ( (x) << BITS_PER_INT16 ) >> BITS_PER_INT16 )

    //---------
    // MACROS
    //---------
    // These macros are defined to handle character arrays as their argument of the intended size,
    // such that structs of character arrays can be used as a trick to pack and unpack structs
    // in the absense of a pack-enabled compiler like GCC.
    #define le16_to_cpu( x )    ( ( x[ 1 ] << 8 ) | x[ 0 ] )
    #define le32_to_cpu( x )    ( ( x[ 3 ] << 24 ) | ( x[ 2 ] << 16 ) | ( x[ 1 ] << 8 ) | x[ 0 ] )

    #define __EI0() { asm(bfclr  #0x0300,SR); } /* Enable interrupts of level 0,1,2,3 */
    #define __EI1() { asm(bfset  #0x0100,SR); asm(bfclr  #0x0200,SR);} /* Enable interrupts of level 1,2,3 */
    #define __EI2() { asm(bfset  #0x0200,SR); asm(bfclr  #0x0100,SR);} /* Enable interrupts of level 2,3 */
    #define __EI3() { asm(bfset  #0x0300,SR); } /* Enable interrupts of level 3 */
    #define __EI(level)  __EI##level()     /* Enable interrupts of the given level */
    #define __DI()  __EI3()                /* Disable interrupts, only level 3 is allowed */
	#define DISABLE_CPU_INTERRUPTS                  ( INTC_CTRL |= INTC_CTRL_INT_DIS )
	#define ENABLE_CPU_INTERRUPTS                   ( INTC_CTRL &= ~INTC_CTRL_INT_DIS )

    //---------------------
    // PUBLIC DECLARATIONS
    //---------------------


    //-------------------
    // PUBLIC PROTOTYPES
    //-------------------


#endif // __DEFS_H__



