/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------

//----------------------
// APPLICATION INCLUDES
//----------------------
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"

#include "adc.h"
#include "dma.h"

//-----------------
// PRIVATE DEFINES
//-----------------
DDM_DMA_INTERRUPT;

//-------------------
// PRIVATE VARIABLES
//-------------------

//-------------------
// PRIVATE FUNCTIONS
//-------------------

//DMA0 is allocated to DEVICE 0
#if CODE_WARRIOR
#pragma interrupt saveall
#endif
DDM_DMA_INTERRUPT
{
	//We should not reach here, restart DDM to in case of error
	
	DMA_DSR_BCR0 |= DMA_DSR_BCR0_DONE; //Clear DMA0 interrupt flag
	DDM_Stop(0);
	DDM_Start(0);
}

void DMA_Config(BYTE byDevice)
{
	DMA_SetSourceAddress(byDevice, 0xE516UL<<1);  //ADC result 8 register addr
	DMA_SetDestinationAddress(byDevice, ((DWORD)(&gDDMRecordBuf[byDevice][0]))<<1);  //Address should be byte address
	DMA_SetDestinationCircularBufferSize(byDevice, RECORD_LEN_POWER_OF_TWO);
	DMA_SetDMALenth(byDevice, 0xFFFFFEUL);  //DMA should be set max transfer length
}

void DMA_Start(BYTE byDevice)
{
	switch(byDevice)
	{
	case 0:
		DMA_DCR0 |= DMA_DCR0_ERQ;
		break;
	}
}

void DMA_Start_ForAdcRead(BYTE byDevice)
{
    DMA_Config(byDevice);
    DMA_Start(byDevice);
}

void DMA_Stop(BYTE byDevice)
{	
	switch(byDevice)
	{
    	case 0:
		DMA_DCR0 &= ~DMA_DCR0_ERQ;
		DMA_DSR_BCR0 |= DMA_DSR_BCR0_DONE;
		break;
	}
}

void DMA_SetSourceAddress(BYTE byDevice, DWORD srcAddr)
{
	switch(byDevice)
	{
	case 0:
		DMA_SAR0  = srcAddr;
		break;
	}
}

void DMA_SetDestinationAddress(BYTE byDevice, DWORD dstAddr)
{
	switch(byDevice)
	{
	case 0:
		DMA_DAR0  = dstAddr;
		break;
	}
}

#define DMA_DSR_BCR_BCR_SHIFT                    0
void DMA_SetDMALenth(BYTE byDevice, DWORD len)
{
	switch(byDevice)
	{
	case 0:
		DMA_DSR_BCR0 &= ~DMA_DSR_BCR0_BCR;
		DMA_DSR_BCR0 |= len;
		break;
	}
}

//The DMA circular buffer size (power) should be the power of two
#define DMA_DCR_DMOD_SHIFT                       8
void DMA_SetDestinationCircularBufferSize(BYTE byDevice, DWORD power)
{
	switch(byDevice)
	{
	case 0:
		DMA_DCR0 &= ~DMA_DCR0_DMOD;
		DMA_DCR0 |= (power-3) << DMA_DCR_DMOD_SHIFT;
		break;
	}
}

