/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __FLASH_H__

    #define __FLASH_H__

    //----------
    // INCLUDES
    //----------


    //----------
    // TYPEDEFS
    //----------


    //---------
    // DEFINES
    //---------

    //       Actual program flash is from 0x0000 to 0x7FFF
    #define PROGRAM_FLASH_SIZE                      63468       // Bytes
    #define FLASH_SECTOR_SIZE                       1024        // Bytes
    #define FLASH_SECTOR_SIZE_IN_WORDS              ( FLASH_SECTOR_SIZE >> 1 )
    #define NUM_FLASH_SECTORS                       32

    // NOTE: The location and size of Data flash should be defined in mapping.h!
    //       The remainder of flash will be used for Program space.
    #define DATA_FLASH_BASE_WORD_ADDRESS            ( FLASH_SECTOR_SIZE_IN_WORDS * DATA_FLASH_BASE_SECTOR_NUMBER )
    #define DATA_FLASH_SIZE_IN_WORDS                ( NUM_DATA_FLASH_SECTORS * FLASH_SECTOR_SIZE_IN_WORDS )
    #define DATA_FLASH_SIZE                         ( NUM_DATA_FLASH_SECTORS * FLASH_SECTOR_SIZE )
    #define NUM_PROGRAM_FLASH_SECTORS               ( NUM_FLASH_SECTORS - NUM_DATA_FLASH_SECTORS )

    // NOTE: Diagnostics use BYTE addressing - BYTE addressing differs from default WORD addressing!
    #define DATA_FLASH_BASE_ADDRESS                 ( DATA_FLASH_BASE_WORD_ADDRESS << 1 )


    //---------------- Control Register definitions ----------------
    // FTFA_FSTAT, flash module status register definitions
    #define COMMAND_COMPLETE_INT_FLAG               ( FTFA_FSTAT & 0x0080 )
    #define START_COMMAND_SEQUENCE                  ( FTFA_FSTAT |= 0x0080 )
    #define READ_COLLISION_ERROR_FLAG               ( FTFA_FSTAT & 0x0040 )
    #define CLEAR_READ_COLLISION_ERROR_FLAG         ( FTFA_FSTAT |= 0x0040 )
    #define ACCESS_ERROR_FLAG                       ( FTFA_FSTAT & 0x0020 )
    #define CLEAR_ACCESS_ERROR_FLAG                 ( FTFA_FSTAT |= 0x0020 )
    #define PROTECTION_VIOLATION_FLAG               ( FTFA_FSTAT & 0x0010 )
    #define CLEAR_PROTECTION_VIOLATION_FLAG         ( FTFA_FSTAT |= 0x0010 )
    // NOTE: This bit is read-only
    #define MEM_CONTROLLER_COMMAND_COMPLETE_FLAG    ( FTFA_FSTAT |= 0x0001 )

    // FTFA_FCNFG, flash module configuration register definitions
    #define ENABLE_COMMAND_COMPLETE_INTERRUPT       0x0080
    #define DISABLE_COMMAND_COMPLETE_INTERRUPT      0x0000
    #define ENABLE_COLLISION_ERROR_INTERRUPT        0x0040
    #define DISABLE_COLLISION_ERROR_INTERRUPT       0x0000
    #define FLASH_ERASE_ALL_REQUEST                 0x0020
    #define FLASH_ERASE_SUSPEND                     0x0010


    // FTFA_FSEC, flash module security register definitions
    #define ENABLE_BACKDOOR_KEY_ACCESS              0x0080
    #define DISABLE_BACKDOOR_KEY_ACCESS             0x0040
    #define ENABLE_MASS_ERASE                       0x0010
    #define DISABLE_MASS_ERASE                      0x0020
    #define ENABLE_FLASH_SECURITY                   0x0001
    #define DISABLE_FLASH_SECURITY                  0x0020

    // NOTE: Program flash is split into 32 flash regions

    // FTFA_FPROT0, region protection enable register definitions
    // NOTE: This register must not be written to while a flash
    // command is active (CCIF = 0)
    #define REGIONS_24_TO_31_ARE_NOT_PROTECTED      0xFFFF
    #define REGION_24_IS_NOT_PROTECTED              0x0001
    #define REGION_25_IS_NOT_PROTECTED              0x0002
    #define REGION_26_IS_NOT_PROTECTED              0x0004
    #define REGION_27_IS_NOT_PROTECTED              0x0008
    #define REGION_28_IS_NOT_PROTECTED              0x0010
    #define REGION_29_IS_NOT_PROTECTED              0x0020
    #define REGION_30_IS_NOT_PROTECTED              0x0040
    #define REGION_31_IS_NOT_PROTECTED              0x0080

    // FTFA_FPROT1, region protection enable register definitions
    // NOTE: This register must not be written to while a flash
    // command is active (CCIF = 0)
    #define REGIONS_16_TO_23_ARE_NOT_PROTECTED      0xFFFF
    #define REGION_23_IS_NOT_PROTECTED              0x0001
    #define REGION_22_IS_NOT_PROTECTED              0x0002
    #define REGION_21_IS_NOT_PROTECTED              0x0004
    #define REGION_20_IS_NOT_PROTECTED              0x0008
    #define REGION_19_IS_NOT_PROTECTED              0x0010
    #define REGION_18_IS_NOT_PROTECTED              0x0020
    #define REGION_17_IS_NOT_PROTECTED              0x0040
    #define REGION_16_IS_NOT_PROTECTED              0x0080

    // FTFA_FPROT2, region protection enable register definitions
    // NOTE: This register must not be written to while a flash
    // command is active (CCIF = 0)
    #define REGIONS_8_TO_15_ARE_NOT_PROTECTED       0xFFFF
    #define REGION_15_IS_NOT_PROTECTED              0x0001
    #define REGION_14_IS_NOT_PROTECTED              0x0002
    #define REGION_13_IS_NOT_PROTECTED              0x0004
    #define REGION_12_IS_NOT_PROTECTED              0x0008
    #define REGION_11_IS_NOT_PROTECTED              0x0010
    #define REGION_10_IS_NOT_PROTECTED              0x0020
    #define REGION_9_IS_NOT_PROTECTED               0x0040
    #define REGION_8_IS_NOT_PROTECTED               0x0080

    // FTFA_FPROT3, region protection enable register definitions
    // NOTE: This register must not be written to while a flash
    // command is active (CCIF = 0)
    #define REGIONS_0_TO_7_ARE_NOT_PROTECTED        0xFFFF
    #define REGION_7_IS_NOT_PROTECTED               0x0001
    #define REGION_6_IS_NOT_PROTECTED               0x0002
    #define REGION_5_IS_NOT_PROTECTED               0x0004
    #define REGION_4_IS_NOT_PROTECTED               0x0008
    #define REGION_3_IS_NOT_PROTECTED               0x0010
    #define REGION_2_IS_NOT_PROTECTED               0x0020
    #define REGION_1_IS_NOT_PROTECTED               0x0040
    #define REGION_0_IS_NOT_PROTECTED               0x0080


    // Flash memory commands
    #define READ_1S_SECTION                         0x01
    #define PROGRAM_CHECK                           0x02
    #define READ_RESOURCE                           0x03
    #define PROGRAM_LONGWORD                        0x06
    #define ERASE_FLASH_SECTOR                      0x09
    #define READ_1S_ALL_BLOCKS                      0x40
    #define READ_ONCE                               0x41
    #define PROGRAM_ONCE                            0x43
    #define ERASE_ALL_BLOCKS                        0x44
    #define VERIFY_BACKDOOR_ACCESS_KEY              0x45


    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------

    void FlashControlInitialization();
    void EraseProgramFlash( void );

    void EraseDataFlash( void );
    BOOL EraseDataFlashSector( BYTE bySector );

    BOOL WriteDataFlashLocation( WORD wAddress, DWORD dwData );
    WORD ReadDataFlashLocation( WORD wAddress );


#endif
