/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
/** ###################################################################
**     THIS COMPONENT MODULE IS GENERATED BY THE TOOL. DO NOT MODIFY IT.
**     Filename    : i2c.c
**     Project     : ProcessorExpert
**     Processor   : WCT1000
**     Component   : InternalI2C
**     Version     : Component 01.282, Driver 02.04, CPU db: 3.50.001
**     Compiler    : Metrowerks DSP C Compiler
**     Date/Time   : 2012-10-22, 13:38, # CodeGen: 131
**     Abstract    :
**          This component encapsulates the internal I2C communication 
**          interface. The implementation of the interface is based 
**          on the Philips I2C-bus specification version 2.0. 
**          Interface features:
**          MASTER mode
**            - Multi master communication
**            - The combined format of communication possible 
**              (see "Automatic stop condition" property)
**            - 7-bit slave addressing (10-bit addressing can be made as well)
**            - Acknowledge polling provided
**            - No wait state initiated when a slave device holds the SCL line low
**            - Holding of the SCL line low by slave device recognized as 'not available bus'
**            - Invalid start/stop condition detection provided
**          SLAVE mode
**            - 7-bit slave addressing
**            - General call address detection provided
**     Settings    :
**         Serial channel              : I2C0
**
**         Protocol
**             Mode                    : MASTER
**             Auto stop condition     : yes
**             SCL frequency           : 187.5 kHz
**
**         Initialization
**
**             Target slave address    : 90
**             Component init          : Enabled
**             Events                  : Enabled
**
**         Registers
**             Input buffer            : I2C_D    [E0F4]
**             Output buffer           : I2C_D    [E0F4]
**             Control register        : I2C_C1   [E0F2]
**             Status register         : I2C_S    [E0F3]
**             Baud setting reg.       : I2C_F    [E0F1]
**             Address register        : I2C_A1   [E0F0]
**             Glitch filter register  : I2C_FLT  [E0F6]
**
**
**         Used pins                   :
**       ----------------------------------------------------------
**            Function    | On package |    Name
**       ----------------------------------------------------------
**              SDA       |     30     |  GPIOC12/SDA1/RXD1
**              SCL       |     29     |  GPIOC11/SCL1/TXD1
**       ----------------------------------------------------------
**     Contents    :
**         Enable      - BYTE i2c_Enable(void);
**         Disable     - BYTE i2c_Disable(void);
**         SendChar    - BYTE i2c_SendChar(BYTE Chr);
**         RecvChar    - BYTE i2c_RecvChar(BYTE *Chr);
**         SendBlock   - BYTE i2c_SendBlock(void* Ptr, WORD Siz, WORD *Snt);
**         RecvBlock   - BYTE i2c_RecvBlock(void* Ptr, WORD Siz, WORD *Rcv);
**         SelectSlave - BYTE i2c_SelectSlave(BYTE Slv);
**         CheckBus    - BYTE i2c_CheckBus(void);
**
**     Copyright : 1997 - 2012 Freescale Semiconductor, Inc. All Rights Reserved.
**     
**     http      : www.freescale.com
**     mail      : support@freescale.com
** ###################################################################*/


/* MODULE i2c. */


#include "i2c.h"

#define OVERRUN_ERR      0x01U         /* Overrun error flag bit   */
#define WAIT_RX_CHAR     0x02U         /* Wait for received char. flag bit (Master)  */
#define CHAR_IN_TX       0x04U         /* Char is in TX buffer (Master)    */
#define CHAR_IN_RX       0x08U         /* Char is in RX buffer     */
#define FULL_TX          0x10U         /* Full transmit buffer     */
#define IN_PROGRES       0x20U         /* Communication is in progress (Master) */
#define FULL_RX          0x40U         /* Full receive buffer      */
#define MSxSL            0x80U         /* Master x Slave flag bit  */

static bool EnUser;                    /* Enable/Disable device */
static bool EnMode;                    /* Enable/Disable device in speed mode */
static BYTE i2c_SlaveAddr;            /* Variable for Slave address */
static WORD InpLenM;                   /* Length of input bufer's content */
static BYTE *InpPtrM;                  /* Pointer to input buffer for Master mode */
static WORD OutLenM;                   /* Length of output bufer's content */
static BYTE *OutPtrM;                  /* Pointer to output buffer for Master mode */
volatile WORD i2c_SndRcvTemp;         /* Temporary variable for SendChar (RecvChar) when they call SendBlock (RecvBlock) */
static WORD *PtrSndRcv;                /* Pointer to Snd/Rcv counter for SendBlock/RecvBlock */
static BYTE ChrTemp;                   /* Temporary variable for SendChar method */
static volatile BYTE i2c_SerFlag;     /* Flags for serial communication */
                                       /* Bits: 0 - OverRun error */
                                       /*       1 - Wait for received char. flag bit (Master) */
                                       /*       2 - Char is in TX buffer (Master) */
                                       /*       3 - Char in RX buffer */
                                       /*       4 - Full TX buffer */
                                       /*       5 - Running int from TX */
                                       /*       6 - Full RX buffer */
                                       /*       7 - Master x Slave */

/*
** ===================================================================
**     Method      :  HWEnDi (component InternalI2C)
**
**     Description :
**         Enables or disables the peripheral(s) associated with the bean.
**         The method is called automatically as a part of the Enable and 
**         Disable methods and several internal methods.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static void HWEnDi(void)
{
  if (EnMode && EnUser) {              /* Enable device? */
    i2c_SerFlag &= (BYTE)~(0x7FU);    /* Clear the status variable */
    I2C_C1|=I2C_C1_IICEN;          /* Enable device */
  } else {
    /* I2C_C1: IICEN=0,IICIE=0,MST=0,TX=0,TXAK=0,RSTA=0,WUEN=0,DMAEN=0 */
    I2C_C1=0;                 /* Disable device */
  }
}

/*
** ===================================================================
**     Method      :  i2c_SetHigh (component InternalI2C)
**
**     Description :
**         The method reconfigures the bean and its selected peripheral(s)
**         when the CPU is switched to the High speed mode. The method is 
**         called automatically as s part of the CPU SetHighSpeed method.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void i2c_SetHigh(void)
{
  EnMode = TRUE;                       /* Set the flag "device enabled" in the actual speed CPU mode */
  HWEnDi();                            /* Enable/disable device according to the status flags */
}

/*
** ===================================================================
**     Method      :  i2c_SetLow (component InternalI2C)
**
**     Description :
**         The method reconfigures the bean and its selected peripheral(s)
**         when the CPU is switched to the Low speed mode. The method is 
**         called automatically as a part of the CPU SetLowSpeed method.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void i2c_SetLow(void)
{
  EnMode = FALSE;                      /* Set the flag "device disabled" in the actual speed CPU mode */
  HWEnDi();                            /* Enable/disable device according to the status flags */
}

/*
** ===================================================================
**     Method      :  i2c_SetSlow (component InternalI2C)
**
**     Description :
**         The method reconfigures the bean and its selected peripheral(s)
**         when the CPU is switched to the Slow speed mode. The method is 
**         called automatically as a part of the CPU SetSlowSpeed method.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void i2c_SetSlow(void)
{
  EnMode = FALSE;                      /* Set the flag "device disabled" in the actual speed CPU mode */
  HWEnDi();                            /* Enable/disable device according to the status flags */
}

/*
** ===================================================================
**     Method      :  MainComm (component InternalI2C)
**
**     Description :
**         The method services the interrupt flags if Interrupt service 
**         is disabled.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define RXAK 0x01U
#define SRW  0x04U
#define IBAL 0x10U
#define IAAS 0x40U
static BYTE MainComm(bool isRb)
{
  register WORD Tr;
  register WORD Status;                /* Temporary variable for saving status register */

  *PtrSndRcv = 0U;                     /* Clear Snd/Rcv counter */
  for (Tr=0U; Tr<0x07D0U;Tr++) {
    if ((I2C_S&I2C_S_IICIF) != 0x00U) {
      Status = I2C_S;         /* Save status register */
      I2C_S|=I2C_S_IICIF;         /* Clear interrupt flag */
      Tr = (WORD)-1;                   /* Clear trials */
      if ((I2C_C1&I2C_C1_MST) != 0x00U) { /* Is device in master mode? */
        if ((I2C_C1&I2C_C1_TX) != 0x00U) { /* Is device in Tx mode? */
          if ((Status & I2C_S_RXAK) != 0x00U) { /* NACK received? */
            I2C_C1&=~I2C_C1_MST;    /* Switch device to slave mode (stop signal sent) */
            I2C_C1&=~I2C_C1_TX;     /* Switch to Rx mode */
            OutLenM = 0U;              /* Any character is not for sent */
            InpLenM = 0U;              /* Any character is not for reception */
            i2c_SerFlag &= (BYTE)~((BYTE)(CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)); /* Any character is not for sent or reception*/
            return ERR_BUSY;           /* Return with error */
          } else {
            if (OutLenM != 0x00U) {    /* Is any char. for transmitting? */
              OutLenM--;               /* Decrease number of chars for the transmit */
              I2C_D=*(OutPtrM)++; /* Send character */
              (*PtrSndRcv)++;          /* Increment Snd counter */
            }
            else {
              if (InpLenM != 0x00U) {  /* Is any char. for reception? */
                if (InpLenM == 1U) {   /* If only one char to receive */
                  I2C_C1|=I2C_C1_TXAK; /* then transmit ACK disable */
                } else {
                  I2C_C1&=~I2C_C1_TXAK; /* else transmit ACK enable */
                }
                I2C_C1&=~I2C_C1_TX; /* Switch to Rx mode */
                I2C_D;        /* Dummy read character */
              }
              else {
                if(!isRb)
                {
                    I2C_C1&=~I2C_C1_MST; /* Switch device to slave mode (stop signal sent) */

                    I2C_C1&=~I2C_C1_TX; /* Switch to Rx mode */
                    for (Tr=0x07D0U;Tr!=0U;Tr--) {
                      if ((I2C_S&I2C_S_BUSY) == 0x00U) { /* Bus is busy? */
                        return ERR_OK;     /* Return without error */
                      }
                    }
                    return ERR_BUSOFF;     /* Return with error */
                }
                else
                {
                    return ERR_OK;     /* Return without error */
                }
              }
            }
          }
        }
        else {
          InpLenM--;                   /* Decrease number of chars for the receive */
          if (InpLenM != 0x00U) {      /* Is any char. for reception? */
            if (InpLenM == 1U) {
              I2C_C1|=I2C_C1_TXAK; /* Transmit ACK disable */
            }
          } else {
            I2C_C1&=~I2C_C1_MST;    /* If no, switch device to slave mode (stop signal sent) */
          }
          *(InpPtrM)++ = (BYTE)I2C_D; /* Receive character */
          (*PtrSndRcv)++;              /* Increment Rcv counter */
          if (InpLenM == 0x00U) {      /* Is any char. for reception? */
            for (Tr=0x07D0U;Tr!=0U;Tr--) {
              if ((I2C_S&I2C_S_BUSY) == 0x00U) { /* Bus is busy? */
                return ERR_OK;         /* Return without error */
              }
            }
            return ERR_BUSOFF;         /* Return with error */
          }
        }
      } else {
        if ((Status & I2C_S_ARBL) != 0x00U) { /* Arbitration lost? */
          OutLenM = 0U;                /* Any character is not for sent */
          InpLenM = 0U;                /* Any character is not for reception */
          i2c_SerFlag &= (BYTE)~((BYTE)(CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)); /* Any character is not for sent or reception*/
          I2C_C1&=~I2C_C1_TX;       /* Switch to Rx mode */
          return ERR_ARBITR;           /* Return with error */
        }
      }
    }
  }
  return ERR_BUSOFF;                   /* Return with error */
}

/*
** ===================================================================
**     Method      :  i2c_errCheck (component InternalI2C)
**
**     Description :
**         I2C bus check 
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static BYTE i2c_errCheck(WORD size, WORD *cnt)
{
  if (!EnMode) {                       /* Is the device disabled in the actual speed CPU mode? */
    return ERR_SPEED;                  /* If yes then error */
  }
  if (size == 0x00U) {                  /* Test variable Size on zero */
    *cnt = 0U;
    return ERR_OK;                     /* If zero then OK */
  }
  if (!EnUser) {                       /* Is the device disabled by user? */
    return ERR_DISABLED;               /* If yes then error */
  }
  if ((I2C_S&I2C_S_BUSY) != 0x00U) { /* Is the bus busy? */
    return ERR_BUSOFF;                 /* If yes then error */
  }
  if (InpLenM != 0x00U) {              /* Are any data to receiving? */
    return ERR_BUSOFF;                 /* If yes then error */
  }
  if ((i2c_SerFlag & (BYTE)(CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)) != 0x00U) { /* Is the bus busy? */
    return ERR_BUSOFF;                 /* If yes then error */
  }

  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  i2c_PrepareMainComm (component InternalI2C)
**
**     Description :
**         Prepare for I2C comm
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
static BYTE i2c_PrepareMainComm(void* Ptr,WORD Siz,WORD *Cnt,BYTE rw)
{
  BYTE result;
  
  result = i2c_errCheck(Siz,Cnt);
  if(result!=ERR_OK)
    return result;

  if(rw==I2C_WRITE)  
  {
      PtrSndRcv = Cnt;                     /* Safe Snd pointer */
      OutLenM = Siz;                       /* Set lenght of data */
      OutPtrM = (BYTE *)Ptr;               /* Save pointer to data for transmitting */
  }
  else
  {
      PtrSndRcv = Cnt;                     /* Safe Rcv pointer */
      InpLenM = Siz;                       /* Set lenght of data */
      InpPtrM = (BYTE *)Ptr;               /* Save pointer to data for reception */
  }

  I2C_C1|=I2C_C1_TX;               /* Set TX mode */
  if ((I2C_C1&I2C_C1_MST) != 0x00U) { /* Is device in master mode? */
    I2C_C1|=I2C_C1_RSTA;           /* If yes then repeat start cycle generated */
  } else {
    I2C_C1|=I2C_C1_MST;            /* If no then start signal generated */
  }

  if(rw==I2C_WRITE)  
    I2C_D=i2c_SlaveAddr;       /* Send slave address for write operation*/
  else
    I2C_D=(WORD)((WORD)i2c_SlaveAddr+1U); /* Send slave address */

  return ERR_OK;
}

/*
** ===================================================================
**     Method      :  i2c_Enable (component InternalI2C)
**
**     Description :
**         Enables I2C component. Events may be generated
**         ("DisableEvent"/"EnableEvent").
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
BYTE i2c_Enable(void)
{
  if (!EnMode) {                       /* Is the device disabled in the actual speed CPU mode? */
    return ERR_SPEED;                  /* If yes then error */
  }
  if (!EnUser) {                       /* Is the device disabled by user? */
    EnUser = TRUE;                     /* If yes then set the flag "device enabled" */
    HWEnDi();                          /* Enable the device */
  }
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  i2c_Disable (component InternalI2C)
**
**     Description :
**         Disables I2C component. No events will be generated.
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
BYTE i2c_Disable(void)
{
  if (!EnMode) {                       /* Is the device disabled in the actual speed CPU mode? */
    return ERR_SPEED;                  /* If yes then error */
  }
  if (EnUser) {                        /* Is the device enabled by user? */
    EnUser = FALSE;                    /* If yes then set the flag "device disabled" */
    HWEnDi();                          /* Disable the device */
  }
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  i2c_SendChar (component InternalI2C)
**
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 0 to the I2C bus and then writes one
**         character (BYTE) to the bus. The slave address must be
**         specified before, by the "SelectSlave" or "SelectSlave10"
**         method or in the component initialization section, "Target
**         slave address init" property. If interrupt service is
**         enabled and the method returns ERR_OK, it doesn't mean that
**         transmission was successful. The state of transmission is
**         obtainable from (OnTransmitData, OnError or OnArbitLost)
**         events. 
**         When working as a SLAVE, this method writes a character to
**         the internal output slave buffer and, after the master
**         starts the communication, to the I2C bus. If no character is
**         ready for a transmission (internal output slave buffer is
**         empty), the Empty character will be sent (see "Empty
**         character" property).
**     Parameters  :
**         NAME            - DESCRIPTION
**         Chr             - Character to send.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by an acknowledge (only in
**                           master mode and when interrupt service is
**                           disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot transmit data
**                           ERR_TXFULL - Transmitter is full (slave
**                           mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
** ===================================================================
*/
BYTE i2c_SendChar(BYTE Chr)
{
  if ((I2C_S&I2C_S_BUSY) != 0x00U) { /* Is the bus busy? */
    return ERR_BUSOFF;                 /* If yes then error */
  }
  if (InpLenM != 0x00U) {              /* Are any data to receiving? */
    return ERR_BUSOFF;                 /* If yes then error */
  }
  if ((i2c_SerFlag & (BYTE)(CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)) != 0x00U) { /* Is the bus busy? */
    return ERR_BUSOFF;                 /* If yes then error */
  }
  ChrTemp = Chr;                       /* Save character */
  return (i2c_SendBlock(&ChrTemp, 1U, ((void *)&i2c_SndRcvTemp))); /* Send character and return */
}

/*
** ===================================================================
**     Method      :  i2c_RecvChar (component InternalI2C)
**
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 1 to the I2C bus, then reads one
**         character (BYTE) from the bus and then sends the stop
**         condition. The slave address must be specified before, by
**         the "SelectSlave" or "SelectSlave10" method or in component
**         initialization section, property "Target slave address init".
**         If interrupt service is enabled and the method returns
**         ERR_OK, it doesn't mean that transmission was finished
**         successfully. The state of transmission must be tested by
**         means of events (OnReceiveData, OnError or OnArbitLost). In
**         case of successful transmission, received data is ready
**         after OnReceiveData event is called. 
**         When working as a SLAVE, this method reads a character from
**         the input slave buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Chr             - Received character.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by the acknowledge (only
**                           in master mode and when interrupt service
**                           is disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot receive data
**                           ERR_RXEMPTY - No data in receiver (slave
**                           mode only)
**                           ERR_OVERRUN - Overrun error was detected
**                           from the last character or block received
**                           (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
**                           ERR_NOTAVAIL - Method is not available in
**                           current mode - see the comment in the
**                           generated code
** ===================================================================
*/
/*
BYTE i2c_RecvChar(BYTE *Chr)

**  This method is implemented as a macro. See i2c.h file.  **
*/

/*
** ===================================================================
**     Method      :  i2c_SendBlock (component InternalI2C)
**
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 0 to the I2C bus and then writes the
**         block of characters to the bus. The slave address must be
**         specified before, by the "SelectSlave" or "SlaveSelect10"
**         method or in component initialization section, "Target slave
**         address init" property. If interrupt service is enabled and
**         the method returns ERR_OK, it doesn't mean that transmission
**         was successful. The state of transmission is detectable by
**         means of events (OnTransmitData, OnError or OnArbitLost).
**         Data to be send is not copied to an internal buffer and
**         remains in the original location. Therefore the content of
**         the buffer should not be changed until the transmission is
**         complete. Event OnTransmitData can be used to detect the end
**         of the transmission.
**         When working as a SLAVE, this method writes a block of
**         characters to the internal output slave buffer and then,
**         after the master starts the communication, to the I2C bus.
**         If no character is ready for a transmission (internal output
**         slave buffer is empty), the "Empty character" will be sent
**         (see "Empty character" property). In SLAVE mode the data are
**         copied to an internal buffer, if specified by "Output buffer
**         size" property.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Ptr             - Pointer to the block of data to send.
**         Siz             - Size of the block.
**       * Snt             - Amount of data sent (moved to a buffer).
**                           In master mode, if interrupt support is
**                           enabled, the parameter always returns the
**                           same value as the parameter 'Siz' of this
**                           method.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by the acknowledge (only
**                           in master mode and when interrupt service
**                           is disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot transmit data
**                           ERR_TXFULL - Transmitter is full. Some data
**                           has not been sent. (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
** ===================================================================
*/
BYTE i2c_SendBlock(void* Ptr,WORD Siz,WORD *Snt)
{
  BYTE result;

  result=i2c_PrepareMainComm(Ptr,Siz,Snt,I2C_WRITE);
  if(result!=ERR_OK)
    return result;
  
  return (MainComm(FALSE));                 /* Call main communication method and return */
}

/*
** ===================================================================
**     Method      :  i2c_RecvBlock (component InternalI2C)
**
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 1 to the I2C bus, then reads the
**         block of characters from the bus and then sends the stop
**         condition. The slave address must be specified before, by
**         the "SelectSlave" or "SelectSlave10" method or in component
**         initialization section, "Target slave address init" property.
**         If interrupt service is enabled and the method returns
**         ERR_OK, it doesn't mean that transmission was finished
**         successfully. The state of transmission must be tested by
**         means of events (OnReceiveData, OnError or OnArbitLost). In
**         case of successful transmission, received data is ready
**         after OnReceiveData event is called. 
**         When working as a SLAVE, this method reads a block of
**         characters from the input slave buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Ptr             - A pointer to the block space for received
**                           data.
**         Siz             - The size of the block.
**       * Rcv             - Amount of received data. In master mode,
**                           if interrupt support is enabled, the
**                           parameter always returns the same value as
**                           the parameter 'Siz' of this method.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by an acknowledge (only in
**                           master mode and when interrupt service is
**                           disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot receive data
**                           ERR_RXEMPTY - The receive buffer didn't
**                           contain the requested number of data. Only
**                           available data (or no data) has been
**                           returned  (slave mode only).
**                           ERR_OVERRUN - Overrun error was detected
**                           from last character or block receiving
**                           (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
**                           ERR_NOTAVAIL - Method is not available in
**                           current mode - see the comment in the
**                           generated code.
** ===================================================================
*/
BYTE i2c_RecvBlock(void* Ptr,WORD Siz,WORD *Rcv)
{
  BYTE result;

  result=i2c_PrepareMainComm(Ptr,Siz,Rcv,I2C_READ);
  if(result!=ERR_OK)
    return result;
  
  return (MainComm(TRUE));                 /* Call main communication method and return */
}

/*
** ===================================================================
**      Method      :  i2c_WriteAndReadBlock (component InternalI2C)
**     Description :
**          Write command for value read back.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * wPtr            - Pointer to the block of data to send.
**         wSiz             - Size of the block.
**       * Snt             - Amount of data sent (moved to a buffer).
**                           In master mode, if interrupt support is
**                           enabled, the parameter always returns the
**                           same value as the parameter 'Siz' of this
**                           method.
**       * rPtr             - A pointer to the block space for received
**                           data.
**         rSiz             - The size of the block.
**       * Rcv             - Amount of received data. In master mode,
**                           if interrupt support is enabled, the
**                           parameter always returns the same value as
**                           the parameter 'Siz' of this method.
**     Returns     :
**         ---             - Error code:
** ===================================================================
*/
BYTE i2c_WriteAndReadBlock(void* wPtr, WORD wSiz, WORD *Snt, void* rPtr, WORD rSiz, WORD *Rcv)
{
  BYTE result;
  
  result=i2c_PrepareMainComm(wPtr,wSiz,Snt,I2C_WRITE);
  if(result!=ERR_OK)
    return result;
    
  result = MainComm(TRUE);                 /* Call main communication method and return */
  if(result!=ERR_OK)
    return result;

  PtrSndRcv = Rcv;                     /* Safe Rcv pointer */
  InpLenM = rSiz;                       /* Set lenght of data */
  InpPtrM = (BYTE *)rPtr;               /* Save pointer to data for reception */
  I2C_C1|=I2C_C1_TX;               /* Set TX mode */
  
  if ((I2C_C1&I2C_C1_MST) != 0x00U) { /* Is device in master mode? */
    I2C_C1|=I2C_C1_RSTA;           /* If yes then repeat start cycle generated */
  } else {
    I2C_C1|=I2C_C1_MST;            /* If no then start signal generated */
  }

  I2C_D=(WORD)((WORD)i2c_SlaveAddr+1U); /* Send slave address */
  
  return (MainComm(FALSE));                 /* Call main communication method and return */
}
/*
** ===================================================================
**     Method      :  i2c_SelectSlave (component InternalI2C)
**
**     Description :
**         This method selects a new slave for communication by its
**         7-bit slave address value. Any send or receive method
**         directs to or from selected device, until a new slave device
**         is selected by this method. This method is not available for
**         the SLAVE mode.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Slv             - 7-bit slave address value.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_BUSY - The device is busy, wait until
**                           the current operation is finished.
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  The device is disabled
** ===================================================================
*/
BYTE i2c_SelectSlave(BYTE Slv)
{
  if (!EnMode) {                       /* Is the device disabled in the actual speed CPU mode? */
    return ERR_SPEED;                  /* If yes then error */
  }
  if (!EnUser) {                       /* Is the device disabled by user? */
    return ERR_DISABLED;               /* If yes then error */
  }
  if ((I2C_C1&I2C_C1_MST) != 0x00U) { /* Is the device in the active state? */
    return ERR_BUSY;                   /* If yes then error */
  }
  i2c_SlaveAddr = (BYTE)(Slv << 1);   /* Set slave address */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  i2c_Init (component InternalI2C)
**
**     Description :
**         Initializes the associated peripheral(s) and the beans 
**         internal variables. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void i2c_Init(void)
{
  /* I2C_C1: IICEN=0,IICIE=0,MST=0,TX=0,TXAK=0,RSTA=0,WUEN=0,DMAEN=0 */
  I2C_C1=0U;                  /* Clear control register */
  i2c_SerFlag = 0x80U;                /* Reset all flags */
  EnUser = TRUE;                       /* Enable device */
  i2c_SlaveAddr = 0x5AU;              /* Set variable for slave address */
  /* I2C_FLT: ??=0,??=0,??=0,??=0,??=0,??=0,??=0,??=0,SHEN=0,??=0,??=0,FLT=0 */
  I2C_FLT=0x00U;              /* Set glitch filter register */
  /* I2C_F: MULT=1,ICR=0x18 */
  I2C_F=0x58U;                /* Set prescaler bits */
  /* I2C_SMB: FACK=0,ALERTEN=0,SIICAEN=0,TCKSEL=0,SLTF=1,SHTF1=0,SHTF2=0,SHTF2IE=0 */
  I2C_SMB=0x08U;
  i2c_SetHigh();                      /* Initial speed CPU mode is high */
}

/*
** ===================================================================
**     Method      :  i2c_CheckBus (component InternalI2C)
**
**     Description :
**         This method returns the status of the bus. If the START
**         condition has been detected, the method returns
**         'ComponentName'_BUSY. If the STOP condition has been
**         detected, the method returns 'ComponentName'_IDLE.
**     Parameters  : None
**     Returns     :
**         ---             - Status of the bus.
** ===================================================================
*/
/*
BYTE i2c_CheckBus(void)

**  This method is implemented as a macro. See i2c.h file.  **
*/

/*
** Write I2C byte
** Input: Register address, value.
** Output: None
*/
void IIC_ByteWrite(BYTE reg, BYTE val)
{
    BYTE buf[2];
    WORD sizeRet;

    buf[0]=reg;
    buf[1]=val;

    while(i2c_CheckBus()) {}
    i2c_SendBlock(buf,2,&sizeRet); 
}

/*
** Read I2C byte
** Input: Register address
** Output: Value read back.
*/
BYTE IIC_ByteRead(BYTE reg)
{
    BYTE value;
    WORD wSizeRet,rSizeRet;

    i2c_WriteAndReadBlock(&reg,1,&wSizeRet,&value,1,&rSizeRet);
    return value;
}

/* END i2c. */

/*
** ###################################################################
**
**     This file was created by Processor Expert 10.0 [05.02]
**     for the Freescale 56800 series of microcontrollers.
**
** ###################################################################
*/

