/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "peripheralInit.h"
#include "application.h"
#include "occs.h"

//-----------------
// PRIVATE DEFINES
//-----------------

#define OSCTL_TEMP (OCCS_OSCTL1_INIT & 0x3fff) /* keep internal oscillator enabled */
#define OSCTL_TEMP2 (OCCS_OSCTL1_INIT & 0xC000) /* shut down internal oscillator if required */

#if OCCS_USE_FACTORY_TRIM_TEMP && OCCS_USE_FACTORY_TRIM_32K
#define TRIM_TEMP_MASK (OCCS_OSCTL2_FREQ_TRIM200K|OCCS_OSCTL2_TEMP_TRIM8M)
#elif OCCS_USE_FACTORY_TRIM_TEMP
#define TRIM_TEMP_MASK (OCCS_OSCTL2_TEMP_TRIM8M)
#elif OCCS_USE_FACTORY_TRIM_32K
#define TRIM_TEMP_MASK (OCCS_OSCTL2_FREQ_TRIM200K)
#else
#define TRIM_TEMP_MASK (0)
#endif


//-------------------
// PRIVATE VARIABLES
//-------------------


//-------------------
// PRIVATE FUNCTIONS
//-------------------


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------

//-----------------------------------------------------------------------------
void OccsInit( void )
{
    WORD wtemp;
/* configure external oscillator and clock mode */
#ifdef OCCS_OSCTL1_INIT
    OCCS_OSCTL1 = OSCTL_TEMP;      /* OSCTL, even if PLL is not used */
#endif


/* load factory trimming value of the internal relaxation oscillator? */
#if OCCS_USE_FACTORY_TRIM
/* re-trim rxosc frequency? */
#if defined(OCCS_RXOSC_RETRIM_CLK)
#if (OCCS_RXOSC_RETRIM_CLK) != 8000000
    wtemp = SIM_NVMOPT2H&0x3ff;

    #warning Retrim clock is not supported
#endif /* OCCS_RXOSC_RETRIM_CLK != 8000000 */
#endif /* OCCS_RXOSC_RETRIM_CLK defined */

    wtemp = OCCS_OSCTL1;
    wtemp &=  ~OCCS_OSCTL1_FREQ_TRIM8M;

    //if (((SIM_NVMOPT2H) & SIM_NVMOPT2H_ROSC_8M_FTRIM)  != 0x03ff)      // temporary for non-trimmed samples, will be removed!!!
        wtemp |= SIM_NVMOPT2H&SIM_NVMOPT2H_ROSC_8M_FTRIM;
    //else
        //wtemp |= ROSC_8M_MANUAL_TRIM_VALUE;

    OCCS_OSCTL1 = wtemp;
#endif /* OCCS_USE_FACTORY_TRIM */

#if OCCS_USE_FACTORY_TRIM_TEMP || OCCS_USE_FACTORY_TRIM_32K
    wtemp = OCCS_OSCTL2;
    #if OCCS_OSCTL2_INIT
    //clean both trim value
    wtemp &= ~(OCCS_OSCTL2_FREQ_TRIM200K|OCCS_OSCTL2_TEMP_TRIM8M);
    //copy trim value from INIT macro, when Factory trim is disabled
    wtemp |= (OCCS_OSCTL2_INIT & (~TRIM_TEMP_MASK&(OCCS_OSCTL2_FREQ_TRIM200K|OCCS_OSCTL2_TEMP_TRIM8M)));
    #endif

    //if (((SIM_NVMOPT2H) & SIM_NVMOPT2H_ROSC_8M_TTRIM)  != 0x3C00)   // temporary for non-trimmed samples, will be removed!!!
    {
        //clean trim value, which must be initialized
        wtemp &= ~TRIM_TEMP_MASK;
        //copy factory trim
        wtemp |= (SIM_NVMOPT2H&SIM_NVMOPT2H_ROSC_8M_TTRIM | SIM_NVMOPT2L&SIM_NVMOPT2L_ROSC_200K_FTRIM)&TRIM_TEMP_MASK;
    }

    //Save initialization
    OCCS_OSCTL2 = wtemp;
#elif defined(OCCS_OSCTL2_INIT)
    OCCS_OSCTL2 = OCCS_OSCTL2_INIT;
#endif

#ifdef OCCS_CTRL_INIT
#if (OCCS_CTRL_INIT & 0x4) 
    /* switch to external clock source (set PRESC=1) */
    OCCS_CTRL |= OCCS_CTRL_PRECS_0;

    /* shut down internal oscillator if required */
    OCCS_OSCTL1 |= OSCTL_TEMP2;
#endif /* (OCCS_CTRL_INIT & 0x4) */


    // Below enable DSC fast mode.
    if(SIM_RSTAT & SIM_RSTAT_SWR)  //already software reset
    {
        if(SIM_MISC0 & SIM_MISC0_MODE_STAT)  // in fast mode, set the value
        {
            OCCS_DIVBY = OCCS_DIVBY_INIT;
        }
        else  // in normal mode, switch to fast mode
        {
            SIM_MISC0 |= SIM_MISC0_FAST_MODE;
            SIM_CTRL |= SIM_CTRL_SWRst;
        }
    }
    else  // none software reset
    {
        if(SIM_MISC0 & SIM_MISC0_MODE_STAT)  // in fast mode, switch to normal mode
        {
    	    SIM_MISC0 &= ~SIM_MISC0_FAST_MODE;
    	    SIM_CTRL |= SIM_CTRL_SWRst;
        }
        else  // in normal mode, switch to fast mode
        {
            SIM_MISC0 |= SIM_MISC0_FAST_MODE;
            SIM_CTRL |= SIM_CTRL_SWRst;
        }
    }

/* setup the PLL according to appconfig.h values */
#ifdef OCCS_DIVBY_INIT
    OCCS_DIVBY = OCCS_DIVBY_INIT;  /* PLLDB, even if PLL is not used */
#endif

#if (OCCS_CTRL_INIT & 1) /* PLL active ? */
    /* interrupts off and PLL bypassed */
    OCCS_CTRL = OCCS_CTRL_INIT & 0xfe;

    // wait for PLL lock
    while( !( ( OCCS_STAT & OCCS_STAT_LCK1 ) && ( OCCS_STAT & OCCS_STAT_LCK0 ) ) ){};

    /* PLL is locked: final PLL setup */
    OCCS_CTRL =  OCCS_CTRL_INIT;

    // Wait for clock transition to complete
    while( ( OCCS_STAT & OCCS_STAT_ZSRCS ) != OCCS_STAT_ZSRCS_0 ){};

#else
    /* PLL is locked: final PLL setup */
    OCCS_CTRL =  OCCS_CTRL_INIT;

#endif

#endif
#ifdef OCCS_CLKCHK_INIT
    move.w  OCCS_CLKCHK_INIT,ArchIO.Pll.clkchk
    nop
    nop
#endif

#ifdef OCCS_CLKCHKR_INIT
    OCCS_CLKCHKR = OCCS_CLKCHKR_INIT;
#endif

}




