/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __OCCS_H__
#define __OCCS_H__

//----------
// INCLUDES
//----------
#include "defines.h"
#include "peripheralInit.h"

//----------
// TYPEDEFS
//----------


//---------
// DEFINES
//---------
#define OCCS_HAS_RXOSC 1
#if OCCS_HAS_RXOSC
    #define OCCS_PLLCR_PRECS            0x0004
#endif
#define OCCS_OSCTL_EXT_SEL_CLKIN    0x0400
#define OCCS_OSCTL_EXT_SEL_EXTAL    0x0000

/* first decide master clock (MSTR_OSC) */
#ifndef OCCS_MSTROSC_FREQ
    #if OCCS_HAS_RXOSC
        #if !defined(OCCS_CTRL_INIT) || !((OCCS_CTRL_INIT) & (OCCS_PLLCR_PRECS))
            #if 0 //not supported OCCS_USE_FACTORY_TRIM && defined(OCCS_RXOSC_RETRIM_CLK) not supported
            #define OCCS_MSTROSC_FREQ (OCCS_RXOSC_RETRIM_CLK)  /* Re-trimmed RX OSC used */
            #elif (defined(OCCS_OSCTL1_INIT) && (OCCS_OSCTL1_INIT&OCCS_OSCTL1_ROSB))
            #define OCCS_MSTROSC_FREQ 400000L  /* Standby RX OSC used */
            #else
            #define OCCS_MSTROSC_FREQ 8000000L  /* Nominal RX OSC used */
            #endif
        #else
            #if (defined(OCCS_OSCTL1_INIT) && !((OCCS_OSCTL1_INIT) & (OCCS_OSCTL_EXT_SEL_CLKIN)))
                #if defined(EXTAL) 
                    #define OCCS_MSTROSC_FREQ (EXTAL)  /* Crystal clock used */
                #endif
             #elif defined(OCCS_CTRL1_INIT) && (OCCS_CTRL1_INIT&OCCS_CTRL_PRECS_1)
                 #define OCCS_MSTROSC_FREQ 200000L  /* 200kHz RX OSC used */
            #elif defined(EXTCLK)
                #define OCCS_MSTROSC_FREQ (EXTCLK)  /* External clock used */
            #endif
        #endif
    #else
        #if (defined(EXTCLK))
            #if (defined(OCCS_OSCTL1_INIT) && !(OCCS_OSCTL1_INIT&OCCS_OSCTL_EXT_SEL_CLKIN))
                #if defined(EXTAL) 
                    #define OCCS_MSTROSC_FREQ (EXTAL)  /* Crystal clock used */
                #endif
             #elif defined(OCCS_CTRL1_INIT) && (OCCS_CTRL1_INIT&OCCS_CTRL_PRECS_1)
                 #define OCCS_MSTROSC_FREQ 200000L  /* 200kHz RX OSC used */
            #elif defined(EXTCLK)
                #define OCCS_MSTROSC_FREQ (EXTCLK)  /* External clock used */
            #endif
        #endif
    #endif
#endif /* OCCS_MSTROSC_FREQ */

#if !defined(OCCS_CORE_FREQ) && defined(OCCS_MSTROSC_FREQ)
    /* when DIVBY register INIT value is not defined, we will use its reset value */
    #ifdef OCCS_DIVBY_INIT
    #define __OCCS_DIVBY_INIT OCCS_DIVBY_INIT
    #else
    #define __OCCS_DIVBY_INIT 0x2031
    #endif

    /* postscaler */
    #if (1<<(((__OCCS_DIVBY_INIT)>>8)&0xf)) <= 256
    #define __OCCS_POSTSCALER (1<<(((__OCCS_DIVBY_INIT)>>8)&0xf))
    #else
    #define __OCCS_POSTSCALER 256
    #endif

    /* PLL active */
    #if defined(OCCS_CTRL_INIT) && ((OCCS_CTRL_INIT & 1) == 1)    // changed from ...  ((OCCS_CTRL_INIT & 3) == 2)

        /* PLL Divide By */
        #define __OCCS_DIVIDE_BY (__OCCS_DIVBY_INIT&0x3f)  // changed from ... (__OCCS_DIVBY_INIT&0x7f)
        #define OCCS_CORE_FREQ (((OCCS_MSTROSC_FREQ)*(__OCCS_DIVIDE_BY+1)) / __OCCS_POSTSCALER / 4)
            
    /* PLL not active */
    #else
        #define OCCS_CORE_FREQ ((OCCS_MSTROSC_FREQ / __OCCS_POSTSCALER) / 2)
            
    #endif
#endif /* !OCCS_CORE_FREQ && OCCS_MSTROSC_FREQ */

//-----------
// VARIABLES
//-----------


//-----------
// FUNCTIONS
//-----------


#endif
