/*******************************************************************************
*
* Copyright 2012-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//---------------------------
// STANDARD LIBRARY INCLUDES
//---------------------------


//----------------------
// APPLICATION INCLUDES
//----------------------
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"
#include "wct_libconst.h"

#include "pwm.h"


//-----------------
// PRIVATE DEFINES
//-----------------
// General DPWM definitions
#define PWM_PERIOD_INIT_VALUE               ( ( DWORD )CALC_PERIOD_VALUE_FROM_FREQ( PWM_INIT_FREQUENCY ) )
#define PWM_DUTY_CYCLE_VALUE                ( ( DWORD )PWM_PERIOD_INIT_VALUE >> 1 ) // 50% duty cycle
// Deadtime register count values are in terms of the IPBus clock regardless of PWM prescalor and clock selection
#define PWM_DEAD_TIME_COUNT0_VALUE          COIL_DEADTIME_MIN_COUNTS_0
#define PWM_DEAD_TIME_COUNT1_VALUE          COIL_DEADTIME_MIN_COUNTS_1


//-------------------
// PRIVATE VARIABLES
//-------------------
DWORD dwDeadTimeCounts;


//-------------------
// PRIVATE FUNCTIONS
//-------------------

// NOTE: The deadtime is counted in IP_BUS clocks regardless of the PWM module prescaler.

#if PWM0_ENABLED
//---------------------------------------------------------------
static void Pwm0Init( WORD wDeadTimeNs )
{
 	//clock is enabled by SIM module

    // Disable the DPWM channel
    PWMA_MCTRL = 0;

#ifdef PWM_A_0_CTRL2_INIT
  PWMA_SM0CTRL2    = PWM_A_0_CTRL2_INIT;
#endif
#ifdef PWM_A_0_CTRL_INIT
  PWMA_SM0CTRL     = PWM_A_0_CTRL_INIT;
#endif
#ifdef PWM_A_0_FRACVAL1_INIT
  PWMA_SM0FRCTRL   = PWM_A_0_FRACVAL1_INIT;
#endif
#ifdef PWM_A_0_OCTRL_INIT
  PWMA_SM0OCTRL    = PWM_A_0_OCTRL_INIT;
#endif
#ifdef PWM_A_0_DISMAP0_INIT
  PWMA_SM0DISMAP0  = PWM_A_0_DISMAP0_INIT;
#endif
#ifdef PWM_A_0_DISMAP1_INIT
    PWMA_SM0DISMAP1  = PWM_A_0_DISMAP1_INIT;
#endif
#ifdef PWM_A_0_TCTRL_INIT
    PWMA_SM0TCTRL     = PWM_A_0_TCTRL_INIT;
#endif

    // Initialize PWM Value registers
#ifdef PWM_A_0_INIT_INIT
  PWMA_SM0INIT     = PWM_A_0_INIT_INIT;
#endif
#ifdef PWM_A_0_VAL0_INIT
  PWMA_SM0VAL0     = PWM_A_0_VAL0_INIT;
#endif
#ifdef PWM_A_0_VAL1_INIT
  PWMA_SM0VAL1     = PWM_A_0_VAL1_INIT;
#endif
#ifdef PWM_A_0_VAL2_INIT
  PWMA_SM0VAL2     = PWM_A_0_VAL2_INIT;
#endif
#ifdef PWM_A_0_VAL3_INIT
    PWMA_SM0VAL3     = PWM_A_0_VAL3_INIT;
#endif
#ifdef PWM_A_0_VAL4_INIT
    PWMA_SM0VAL4     = PWM_A_0_VAL4_INIT;
#endif
#ifdef PWM_A_0_VAL5_INIT
    PWMA_SM0VAL5     = PWM_A_0_VAL5_INIT;
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
    PWMA_SM0DTCNT0   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
  PWMA_SM0DTCNT1   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif

}
#endif


#if PWM1_ENABLED || PWM1_USED_FOR_SOUND_CONTROL
//---------------------------------------------------------------
void Pwm1Init( WORD wDeadTimeNs )
{
 	//clock is enabled by SIM module

    // Disable the DPWM channel
    PWMA_MCTRL = 0;

#ifdef PWM_A_1_CTRL2_INIT
  PWMA_SM1CTRL2    = PWM_A_1_CTRL2_INIT;
#endif
#ifdef PWM_A_1_CTRL_INIT
  PWMA_SM1CTRL     = PWM_A_1_CTRL_INIT;
#endif
#ifdef PWM_A_1_FRACVAL1_INIT
  PWMA_SM1FRCTRL   = PWM_A_1_FRACVAL1_INIT;
#endif
#ifdef PWM_A_1_OCTRL_INIT
  PWMA_SM1OCTRL    = PWM_A_1_OCTRL_INIT;
#endif
#ifdef PWM_A_1_DISMAP0_INIT
  PWMA_SM1DISMAP0  = PWM_A_1_DISMAP0_INIT;
#endif
#ifdef PWM_A_1_DISMAP1_INIT
    PWMA_SM1DISMAP1  = PWM_A_1_DISMAP1_INIT;
#endif
#ifdef PWM_A_1_TCTRL_INIT
    PWMA_SM1TCTRL     = PWM_A_1_TCTRL_INIT;
#endif

  // Initialize PWM Value registers
#ifdef PWM_A_1_INIT_INIT
  PWMA_SM1INIT     = PWM_A_1_INIT_INIT;
#endif
#ifdef PWM_A_1_VAL0_INIT
  PWMA_SM1VAL0     = PWM_A_1_VAL0_INIT;
#endif
#ifdef PWM_A_1_VAL1_INIT
  PWMA_SM1VAL1     = PWM_A_1_VAL1_INIT;
#endif
#ifdef PWM_A_1_VAL2_INIT
  PWMA_SM1VAL2     = PWM_A_1_VAL2_INIT;
#endif
#ifdef PWM_A_1_VAL3_INIT
    PWMA_SM1VAL3     = PWM_A_1_VAL3_INIT;
#endif
#ifdef PWM_A_1_VAL4_INIT
    PWMA_SM1VAL4     = PWM_A_1_VAL4_INIT;
#endif
#ifdef PWM_A_1_VAL5_INIT
    PWMA_SM1VAL5     = PWM_A_1_VAL5_INIT;
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
    PWMA_SM1DTCNT0   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
    PWMA_SM1DTCNT1   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif

}
#endif


#if PWM2_ENABLED || PWM2_USED_FOR_SOUND_CONTROL
//---------------------------------------------------------------
void Pwm2Init( WORD wDeadTimeNs )
{
 	//clock is enabled by SIM module

    // Disable the DPWM channel
    PWMA_MCTRL = 0;

#ifdef PWM_A_2_CTRL2_INIT
  PWMA_SM2CTRL2    = PWM_A_2_CTRL2_INIT;
#endif
#ifdef PWM_A_2_CTRL_INIT
  PWMA_SM2CTRL     = PWM_A_2_CTRL_INIT;
#endif
#ifdef PWM_A_2_FRACVAL1_INIT
  PWMA_SM2FRCTRL   = PWM_A_2_FRACVAL1_INIT;
#endif
#ifdef PWM_A_2_OCTRL_INIT
  PWMA_SM2OCTRL    = PWM_A_2_OCTRL_INIT;
#endif
#ifdef PWM_A_2_DISMAP0_INIT
  PWMA_SM2DISMAP0  = PWM_A_2_DISMAP0_INIT;
#endif
#ifdef PWM_A_2_DISMAP1_INIT
    PWMA_SM2DISMAP1  = PWM_A_2_DISMAP1_INIT;
#endif
#ifdef PWM_A_2_TCTRL_INIT
    PWMA_SM2TCTRL     = PWM_A_2_TCTRL_INIT;
#endif

  // Initialize PWM Value registers
#ifdef PWM_A_2_INIT_INIT
  PWMA_SM2INIT     = PWM_A_2_INIT_INIT;
#endif
#ifdef PWM_A_2_VAL0_INIT
  PWMA_SM2VAL0     = PWM_A_2_VAL0_INIT;
#endif
#ifdef PWM_A_2_VAL1_INIT
  PWMA_SM2VAL1     = PWM_A_2_VAL1_INIT;
#endif
#ifdef PWM_A_2_VAL2_INIT
    PWMA_SM2VAL2     = PWM_A_2_VAL2_INIT;
#endif
#ifdef PWM_A_2_VAL3_INIT
    PWMA_SM2VAL3     = PWM_A_2_VAL3_INIT;
#endif
#ifdef PWM_A_2_VAL4_INIT
    PWMA_SM2VAL4     = PWM_A_2_VAL4_INIT;
#endif
#ifdef PWM_A_2_VAL5_INIT
    PWMA_SM2VAL5     = PWM_A_2_VAL5_INIT;
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
  PWMA_SM2DTCNT0   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
  PWMA_SM2DTCNT1   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif

 }
#endif


#if PWM3_ENABLED || PWM3_USED_FOR_SOUND_CONTROL
//---------------------------------------------------------------
void Pwm3Init( WORD wDeadTimeNs )
{
 	//clock is enabled by SIM module

    // Disable the DPWM channel
    PWMA_MCTRL = 0;

#ifdef PWM_A_3_CTRL2_INIT
  PWMA_SM3CTRL2    = PWM_A_3_CTRL2_INIT;
#endif
#ifdef PWM_A_3_CTRL_INIT
  PWMA_SM3CTRL     = PWM_A_3_CTRL_INIT;
#endif
#ifdef PWM_A_3_FRACVAL1_INIT
  PWMA_SM3FRCTRL   = PWM_A_3_FRACVAL1_INIT;
#endif
#ifdef PWM_A_3_OCTRL_INIT
  PWMA_SM3OCTRL    = PWM_A_3_OCTRL_INIT;
#endif
#ifdef PWM_A_3_DISMAP0_INIT
  PWMA_SM3DISMAP0  = PWM_A_3_DISMAP0_INIT;
#endif
#ifdef PWM_A_3_DISMAP1_INIT
  PWMA_SM3DISMAP1  = PWM_A_3_DISMAP1_INIT;
#endif
#ifdef PWM_A_3_TCTRL_INIT
    PWMA_SM3TCTRL     = PWM_A_3_TCTRL_INIT;
#endif

  // Initialize PWM Value registers
#ifdef PWM_A_3_INIT_INIT
  PWMA_SM3INIT     = PWM_A_3_INIT_INIT;
#endif
#ifdef PWM_A_3_VAL0_INIT
  PWMA_SM3VAL0     = PWM_A_3_VAL0_INIT;
#endif
#ifdef PWM_A_3_VAL1_INIT
  PWMA_SM3VAL1     = PWM_A_3_VAL1_INIT;
#endif
#ifdef PWM_A_3_VAL2_INIT
  PWMA_SM3VAL2     = PWM_A_3_VAL2_INIT;
#endif
#ifdef PWM_A_3_VAL3_INIT
  PWMA_SM3VAL3     = PWM_A_3_VAL3_INIT;
#endif
#ifdef PWM_A_3_VAL4_INIT
    PWMA_SM3VAL4     = PWM_A_3_VAL4_INIT;
#endif
#ifdef PWM_A_3_VAL5_INIT
    PWMA_SM3VAL5     = PWM_A_3_VAL5_INIT;
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
  PWMA_SM3DTCNT0   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif
#ifdef CPU_CLOCK_PERIOD_NS_INTEGER
  PWMA_SM3DTCNT1   = ( wDeadTimeNs / CPU_CLOCK_PERIOD_NS_INTEGER );
#endif

}
#endif


//------------------
// PUBLIC VARIABLES
//------------------


//------------------
// PUBLIC FUNCTIONS
//------------------


//---------------------------------------------------------------
void PwmInit( WORD wDeadTimeNs )
{
    WORD wPwmOutputEnable = 0;

    // Configure ALL channels even if not used to
    //  insure proper initialization.

    // Initialize per definitions - set in mapping.h file
    #if PWM0_ENABLED
    Pwm0Init( wDeadTimeNs );
    wPwmOutputEnable |= ( PWMA_OUTEN_PWMA_EN_0 + PWMA_OUTEN_PWMB_EN_0 );
    #endif
    #if PWM1_ENABLED
    Pwm1Init( wDeadTimeNs );
    wPwmOutputEnable |= ( PWMA_OUTEN_PWMA_EN_1 + PWMA_OUTEN_PWMB_EN_1 );
    #endif
    #if PWM2_ENABLED
    Pwm2Init( wDeadTimeNs );
    wPwmOutputEnable |= ( PWMA_OUTEN_PWMA_EN_2 + PWMA_OUTEN_PWMB_EN_2 );
    #endif
    #if PWM3_ENABLED
    Pwm3Init( wDeadTimeNs );
    wPwmOutputEnable |= ( PWMA_OUTEN_PWMA_EN_3 + PWMA_OUTEN_PWMB_EN_3 );
    #endif

    // Configure the PWM fault control registers
    PWMA_OUTEN    = wPwmOutputEnable;   
#ifdef PWM_A_MASK_INIT
  PWMA_MASK     = PWM_A_MASK_INIT;
#endif
#ifdef PWM_A_SWCOUT_INIT
  PWMA_SWCOUT   = PWM_A_SWCOUT_INIT;
#endif
#ifdef PWM_A_DTSRCSEL_INIT
  PWMA_DTSRCSEL = PWM_A_DTSRCSEL_INIT;
#endif
#ifdef PWM_A_FCTRL0_INIT
  PWMA_FCTRL0    = PWM_A_FCTRL0_INIT;
#endif
#ifdef PWM_A_FCTRL1_INIT
  PWMA_FCTRL1    = PWM_A_FCTRL1_INIT;
#endif
#ifdef PWM_A_FSTS_INIT
  PWMA_FSTS0     = PWM_A_FSTS_INIT;
#endif
#ifdef PWM_A_FSTS1_INIT
  PWMA_FSTS1     = PWM_A_FSTS1_INIT;
#endif
#ifdef PWM_A_FFILT_INIT
  PWMA_FFILT0    = PWM_A_FFILT_INIT;
#endif
#ifdef PWM_A_FFILT1_INIT
  PWMA_FFILT1    = PWM_A_FFILT1_INIT;
#endif
}

//---------------------------------------------------------------
void PwmFrequencyUpdate( BYTE byChannel, DWORD dwNewFrequency, WORD wDutyCyclePercent )
{

    DWORD dwPWMVal1;
    DWORD dwPWMVal3;


    // NOTE 1: This function assumes the PWM is operating in independent mode.
    //  The function calculates the register values required to achieve the specified
    //  frequency output at the specified Duty Cycle.

    // NOTE 2: The PWM module will not accept changes to registers while the LDOK bit
    //  is set.  The bit MUST be cleared prior to attempting register updates, and then
    //  set once more to force the new value to get loaded on the next cycle.

    // Calculate the normal PWM Modulus/period value required for the specified frequency
    dwPWMVal1    = CALC_PERIOD_VALUE_FROM_FREQ( dwNewFrequency );
    dwPWMVal3    = ( dwPWMVal1 * wDutyCyclePercent ) / MAX_DUTY_CYCLE_PERCENT;   // Duty cycle is 10,000s of a percent

    // Update the specified channel
    switch( byChannel )
    {
            #if PWM0_ENABLED
            case 0:
                // Ensure LDOK bit is clear before updating registers
            	if(PWMA_MCTRL & PWMA_MCTRL_RUN_0)
				{
            	   	while(PWMA_MCTRL&PWMA_MCTRL_LDOK_0) {};
				}
				else
				{
					PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;
				}
            	
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;

                PWMA_SM0INIT = 0;
                PWMA_SM0VAL1 = ( WORD )dwPWMVal1;
                PWMA_SM0VAL2 = 0;
                PWMA_SM0VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL4&5 for independent PWM for bulk driver */
                PWMA_SM0VAL4 = ( WORD )dwPWMVal1>>1;
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM0VAL5=PWMA_SM0VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM0VAL5 = PWMA_SM0VAL4+PWMA_SM0VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_0;

                break;
            #endif

            #if PWM1_ENABLED
            case 1:
                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_1;

                PWMA_SM1INIT = 0;
                PWMA_SM1VAL1 = ( WORD )dwPWMVal1;
                PWMA_SM1VAL2 = 0;
                PWMA_SM1VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL4&5 for independent PWM for bulk driver */
                PWMA_SM1VAL4 = ( WORD )dwPWMVal1>>1;
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM1VAL5=PWMA_SM1VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM1VAL5 = PWMA_SM1VAL4+PWMA_SM1VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_1;

                break;
            #endif

            #if PWM2_ENABLED
            case 2:
                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_2;

                PWMA_SM2INIT = 0;
                PWMA_SM2VAL1 = ( WORD )dwPWMVal1;
                PWMA_SM2VAL2 = 0;
                PWMA_SM2VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL4&5 for independent PWM for bulk driver */
                PWMA_SM2VAL4 = ( WORD )dwPWMVal1>>1;
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM2VAL5=PWMA_SM2VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM2VAL5 = PWMA_SM2VAL4+PWMA_SM2VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_2;

                break;
            #endif

            #if PWM3_ENABLED
            case 3:
                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_3;

                PWMA_SM3INIT = 0;
                PWMA_SM3VAL1 = ( WORD )dwPWMVal1;
                PWMA_SM3VAL2 = 0;
                PWMA_SM3VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL4&5 for independent PWM for bulk driver */
                PWMA_SM3VAL4 = ( WORD )dwPWMVal1>>1;
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM3VAL5=PWMA_SM3VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM3VAL5 = PWMA_SM3VAL4+PWMA_SM3VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_3;

                break;
            #endif

            default:
                // Illegal channel!
                break;
            break;
    }
}


//---------------------------------------------------------------
void PwmDutyCycleUpdate( BYTE byChannel, WORD wDutyCyclePercent )
{

    DWORD dwPWMVal3;

    // NOTE 1: This function assumes the PWM is operating in independent mode.
    //  The function calculates the register values required to achieve the specified
    //  Duty Cycle output at the current frequency of operation.

    // NOTE 2: The PWM module will not accept changes to registers while the LDOK bit
    //  is set.  The bit MUST be cleared prior to attempting register updates, and then
    //  set once more to force the new value to get loaded on the next cycle.

    // Update the specified channel
    switch( byChannel )
    {
            #if PWM0_ENABLED
            case 0:
            	if(PWMA_MCTRL & PWMA_MCTRL_RUN_0)
				{
            	   	while(PWMA_MCTRL&PWMA_MCTRL_LDOK_0) {};
				}
				else
				{
					PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;
				}

            	
                dwPWMVal3  = ( ( DWORD )wDutyCyclePercent * PWMA_SM0VAL1 ) / MAX_DUTY_CYCLE_PERCENT;   // Duty cycle is 10,000s of a percent;

                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;

                // Only need to update VAL3 for new duty cycle
                PWMA_SM0VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL5 for independent PWM for bulk driver */
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM0VAL5=PWMA_SM0VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM0VAL5 = PWMA_SM0VAL4+PWMA_SM0VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_0;

                break;
            #endif

            #if PWM1_ENABLED
            case 1:

                dwPWMVal3  = ( ( DWORD )wDutyCyclePercent * PWMA_SM1VAL1 ) / MAX_DUTY_CYCLE_PERCENT;   // Duty cycle is 10,000s of a percent;

                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_1;

                // Only need to update VAL3 for new duty cycle
                PWMA_SM1VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL5 for independent PWM for bulk driver */
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM1VAL5=PWMA_SM1VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM1VAL5 = PWMA_SM1VAL4+PWMA_SM1VAL3;

                PWMA_MCTRL |= PWMA_MCTRL_LDOK_1;

                break;
            #endif

            #if PWM2_ENABLED
            case 2:

                dwPWMVal3  = ( ( DWORD )wDutyCyclePercent * PWMA_SM2VAL1 ) / MAX_DUTY_CYCLE_PERCENT;   // Duty cycle is 10,000s of a percent;

                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_2;

                // Only need to update VAL3 for new duty cycle
                PWMA_SM2VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL5 for independent PWM for bulk driver */
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM2VAL5=PWMA_SM2VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM2VAL5 = PWMA_SM2VAL4+PWMA_SM2VAL3;

                PWMA_MCTRL   |= PWMA_MCTRL_LDOK_2;

                break;
            #endif

            #if PWM3_ENABLED
            case 3:

                dwPWMVal3  = ( ( DWORD )wDutyCyclePercent * PWMA_SM3VAL1 ) / MAX_DUTY_CYCLE_PERCENT;   // Duty cycle is 10,000s of a percent;

                // Ensure LDOK bit is clear before updating registers
                PWMA_MCTRL |= PWMA_MCTRL_CLDOK_3;

                // Only need to update VAL3 for new duty cycle
                PWMA_SM3VAL3 = ( WORD )dwPWMVal3;
                /* Set VAL5 for independent PWM for bulk driver */
                if (wDutyCyclePercent==PWM_INIT_DUTY_CYCLE)
                    PWMA_SM3VAL5=PWMA_SM3VAL2;  /* For 50% duty, let PWMA and PWMB transition happen at the same time. */
                else
                    PWMA_SM3VAL5 = PWMA_SM3VAL4+PWMA_SM3VAL3;

                PWMA_MCTRL   |= PWMA_MCTRL_LDOK_3;

                break;
            #endif

            default:
                // Illegal channel!
            break;
    }

}

