/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
#ifndef __WCT_LIBSTRUCT_HEADER_H__
#define __WCT_LIBSTRUCT_HEADER_H__

//lib configuration
//coil configuration
#if _WCT_MPTX4
#define NUM_DEVICES                             4
#define COMM_CHANNELS_PER_DEVICE                2
#define NUM_COILS_PER_DEVICE                    1
#elif _WCT_MPTX1
#define NUM_DEVICES                             1
#define COMM_CHANNELS_PER_DEVICE                4
#define NUM_COILS_PER_DEVICE                    1
#endif
#define NUM_COILS                               ( NUM_DEVICES * NUM_COILS_PER_DEVICE )

#define COIL_ANALOG_PING TRUE

//-------------------------- systemData defines -------------------------
#define NUM_DEVICES_IN_EEPROM                   NUM_DEVICES


// Module operating definitions
#define COMM_DETECT_USING_CAPTURE_REGISTER      FALSE
#define DIGITAL_DATA_INVERTED                   TRUE
#define COMM_DETECTED_ON_FALLING_EDGE           TRUE
#define DUAL_EDGE_COMM_ENABLED                  TRUE
#define INITIAL_CHECKSUM_VALUE                  0           // zero for normal production

//------------------------------------------------------------------------------
//  Digital IO
//------------------------------------------------------------------------------
#define ENABLE_FET_DRIVER               GPIO_SETHIGH(digitalIo.fetDriver)
#define DISABLE_FET_DRIVER              GPIO_SETLOW(digitalIo.fetDriver)




/**************************************************************************/ /*!
 * @brief Structure containing Gpio Pin configuration
 *
 * This structure is used to define GPIO pins for library by application.
 ******************************************************************************/
typedef struct
{
	WORD * gpio_cpu16Port;
	BYTE gpio_cu8Pin;
}WCH_GPIO;

/**************************************************************************/ /*!
 * @brief Structure containing list of all GPIO used by the application and library
 *
 * This structure is defined statically and provides pin definition to application
 * and library. To change state of the pin you have to use @ref GPIO_SETHIGH(), 
 * @ref GPIO_SETLOW() or @ref GPIO_TOGGLE();
 ******************************************************************************/
typedef struct
{
	WCH_GPIO fetDriver;
}WCH_PINS;

//------------------------------------------------------------------------------
// RF Data Rx States

typedef enum
{
    RF_DATA_RX_STATE_IDLE = 0,
    RF_DATA_RX_STATE_PREAMBLE_WAIT,  // 1
    RF_DATA_RX_STATE_START_BIT_WAIT,  // 2
    RF_DATA_RX_STATE_DATA_RECEIVE,      // 3
    RF_DATA_RX_STATE_STOP_BIT_WAIT,     // 4
    RF_DATA_RX_STATE_BYTE_RECEIVED,     // 5
    RF_DATA_RX_STATE_PACKET_RECEIVED, // 6
    RF_DATA_RX_STATE_ERROR                   // 7

} PACKING_ATTRIBUTE RF_DATA_RX_STATE;


//------------------------------------------------------------------------------
// Comm Flags
typedef struct
{
    BYTE bRfDataRxBitTimeCentered   : 1;
    BYTE bLookingForFirstOddBitTimeOccurrence    : 1;
    BYTE bUnused1                   : 1;
    BYTE bUnused2                   : 1;
    BYTE bUnused3                   : 1;
    BYTE bUnused4                   : 1;
    BYTE bUnused5                   : 1;
    BYTE bPacketErrorQueued         : 1;

} PACKING_ATTRIBUTE COMM_FLAGS;


//------------------------------------------------------------------------------
// Packet Status Flags
typedef struct
{
    BYTE bDataPacketError               : 4;
    BYTE bPacketReadyForParsing         : 1;
    BYTE bReceivedControlUpdate         : 1;
    BYTE bCommChannelUsed               : 2;

} PACKING_ATTRIBUTE PACKET_STATUS;


//------------------------------------------------------------------------------
// RF Data Parameters

typedef struct
{
    RF_DATA_RX_STATE            eRfDataRxState;

    WORD                        wRfDataRxTransitionLastCounter;
    WORD                        wRfDataTimerCalibrationValue;
    WORD                        wStartBitReferenceCount;
    WORD                        wStopBitReferenceCount;

    WORD                        wRfDataRxShiftRegister;
    BYTE                        byRfDataRxShiftCount;

    BYTE                        byRfDataRxTransitionBitAvail;
    BYTE                        byRfDataRxTransitionBitAvailCount;

    BYTE                        byRfPreambleCount;

    COMM_FLAGS                  btCommFlags;

    BYTE                        byNumBytesInPacket;
    BYTE                        byDataPacketBufferIndex;
    BYTE                        byDataPacketBuffer[ MAX_PACKET_SIZE ];
    PACKET_STATUS               btPacketStatus;

} PACKING_ATTRIBUTE RF_DATA_PARAMS;




//------------------------------------------------------------------------------
// Device Control Flags
typedef struct
{
    BYTE bPowering              : 1;    // are we supplying power?
    BYTE bCharged               : 1;    // indicates a battery charge is complete
    BYTE bDevicePresent         : 1;    // is the feedback signal present?
    BYTE bReadyForPower         : 1;    // Indicates device is ready for power
    BYTE bPowerXferTerminated   : 1;    // has there been a request to terminate power xfer?
    BYTE bCollectInputCurrent   : 1;
    BYTE bInitialBeepIssued     : 1;
    BYTE bFodSupported          : 1;

} PACKING_ATTRIBUTE DEVICE_CONTROL_FLAGS;


//------------------------------------------------------------------------------
// Device Status Flags
typedef struct
{
    // This set of flags indicates when new status has been received
    WORD bSignalStrengthUpdated : 1;
    WORD bDeviceVersionUpdated  : 1;
    WORD bMfgIdUpdated          : 1;
    WORD bDeviceIdMswUpdated    : 1;
    WORD bDeviceIdLswUpdated    : 1;
    WORD bPwrCtrlHoldoffUpdated : 1;
    WORD bMaxPowerUpdated       : 1;
    WORD bReportedPowerUpdated  : 1;
    WORD bInputPowerUpdated     : 1;
    WORD bChargeStatusUpdated   : 1;
    WORD bEPTReasonUpdated      : 1;
    WORD bReserved              : 5;

} PACKING_ATTRIBUTE DEVICE_STATUS_FLAGS;


//------------------------------------------------------------------------------
// Device Identification Parameters
typedef struct
{
    BYTE                        MajorVersion : 4;
    BYTE                        MinorVersion : 4;
    BYTE                        byUsesExtendedId;
    WORD                        wManufacturerId;
    WORD                        wDeviceIdMsw;
    WORD                        wDeviceIdLsw;
    BYTE                        ExtDeviceId[ 8 ];
    
} PACKING_ATTRIBUTE DEVICE_IDENTIFICATION;


typedef enum
{
    INITIALIZING = 0,
    DIAGNOSTIC_MODE,
    NO_DEVICE_PRESENT,
    DEVICE_CHARGING,
    DEVICE_CHARGED,
    DEVICE_FOD_FAULT,
    DEVICE_FAULT,
    SYSTEM_FAULT,
    NVM_FAULT,
    POWER_LIMIT,
    UNCHANGED

} DEVICE_STATE;

//------------------------------------------------------------------------------
// Control Algorithm States
typedef enum
{
    FREQUENCY_CONTROL_ALGORITHM = 0,
    RAIL_CONTROL_ALGORITHM,
    DUTY_CYCLE_CONTROL_ALGORITHM

} CONTROL_ALGORITHM;

//------------------------------------------------------------------------------
// Device Operating States
typedef enum
{
    OP_STATE_OFF = 0,           // 0
    OP_STATE_PING_WAIT,     // 1
    OP_STATE_PING_INIT,     // 2
    OP_STATE_PING_STOP,     // 3
    OP_STATE_ANALOG_PING_ACTIVE,    // 4
    OP_STATE_RAPID_DIGITAL_PING_ACTIVE, //5
    OP_STATE_DIGITAL_PING_ACTIVE,   // 6
    OP_STATE_PING_READ,     // 7
    OP_STATE_IDENTIFICATION,    // 8
    OP_STATE_EXT_IDENTIFICATION,    // 9
    OP_STATE_CONFIGURATION,     // 10 
    OP_STATE_POWER_XFER_INIT,   // 11
    OP_STATE_POWER_XFER,        // 12
    OP_STATE_FAULT      // 13

} PACKING_ATTRIBUTE DEVICE_OP_STATE;

#if ADC_SAMPLE_AVERAGE_API
typedef struct {
    WORD wAdcSample;
    WORD weight;
}T_ADC_SAMPLE_AVERAGE;
#endif

//------------------------------------------------------------------------------
// Run-Time Parameters
typedef struct
{
    BYTE                        byDeviceId;
    BYTE                        bySignalStrength;
    DEVICE_IDENTIFICATION       DeviceIdentification;

    union
    {
        WORD                    all;
        DEVICE_STATUS_FLAGS     bit;

    } DeviceStatusFlags;

    DEVICE_STATE                DeviceStatus;
    WORD                        wPingStartRefCount;
    WORD                        wStartBitReferenceCount;
    WORD                        wLastPacketRefCount;
    WORD                        wElapsedTimeFromStartBit;
    PACKET_STATUS               btPacketStatus;
    RF_DATA_PARAMS*             pRfDataParams;
    BYTE                        byCommPacketTypeReceived;
    BYTE                        byEptReasonCode;

    // Coil params
    BYTE                        byCoilIds[ NUM_COILS_PER_DEVICE ];
    BYTE                        byActiveCoilId;
    BYTE                        byCoilIdIndex;
    WORD                        wPreviousAnalogPingAdcCounts[ NUM_COILS_PER_DEVICE ];

    // Control params
    DWORD                       dwActiveFreq;
    WORD                        wAnalogPingDutyCycle;
    WORD                        wActiveDutyCycle;
    WORD                        wMaxDutyCycle;
    WORD                        wMinDutyCycle;
    CONTROL_ALGORITHM           eControlAlgorithm;

    // PID params
    WORD                        wFreqStep;
    WORD                        wCoilCurrent;
    WORD                        wSetPoint;
    WORD                        wActiveTimeCount;
    WORD                        wActiveTimeThreshold;
    int                         iReportedError;
    int                         iPreviousError;
    int                         iCurrentError;
    int                         iProportionalTerm;
    int                         iIntegralTerm;
    int                         iDerivativeTerm;
    int                         iPidOutput;
    DWORD                       dwPrevActiveFreq;
    WORD                        wPrevActiveDutyCycle;
    WORD                        wPrevSetPoint;
    WORD                        wStartingCoilCurrent;
    BYTE                        byIterationNum;

    // Device Config / Status params
    WORD                        wMaxHalfWatts;
    WORD                        wLastMsgHeard;
    WORD                        wLastCepHeaderHeard;
    WORD                        wOpStateTimeAccumMs;
    WORD                        wSafetyInputVoltage;
    BYTE                        byConfigPacketCount;
    BYTE                        byPidDelayTime;
    BYTE                        byPercentCharged;
    BYTE                        byPercentPowerUsed;
    DEVICE_OP_STATE             eOpState;
    DEVICE_CONTROL_FLAGS        btDeviceControlFlags;
    WORD                        wDeviceFaultStatus;
    DWORD                       dwPowerXferRetryTimerMs;

    // Analog I/O params
    DWORD                       dwCoilCurrentDiodeOffset;
    WORD                        wInputVoltageCalibration;       // NOTE: These calibration values exist in RunTime params
    WORD                        wInputCurrentCalibration;       //       to provide us with the ability to override them
    WORD                        wCoilCurrentCalibration;        //       with "nominal" values during calibration.
    WORD                        wOverCurrentLimit;

    //Power Loss params
    WORD                        wPowerLossDetectionThreshold;
    WORD                        wQuadraticFinalDivisorExponent;
    DWORD                       dwPowerLossFaultTimer;
    DWORD                       dwRxReportedPower;
    WORD                        wLastReceivedPowerMessage;
    WORD                        wInputCurrent;
    WORD                        wFodBufferIndex;
    WORD                        wFodSampleCount;
    WORD                        wCoilCurrentHistory[ MAX_POWER_LOSS_SAMPLE_HISTORY ];
    WORD                        wInputCurrentHistory[ MAX_POWER_LOSS_SAMPLE_HISTORY ];
    WORD                        wInputVoltageHistory[ MAX_POWER_LOSS_SAMPLE_HISTORY ];
    WORD                        wMeasuredInputVoltage;
    WORD                        wMeasuredCoilCurrent;
    DWORD                       dwMeasuredInputCurrent;
    DWORD                       dwCalculatedInputPower;
    SWORD                       swQuadTerm;
    SWORD                       swLinearTerm;
    SWORD                       swTxInternalLoss;
    SWORD                       swParasiticLoss;
    SWORD                       swNormalizationLoss;
    SWORD                       swNormalizedParasiticLoss;
    BYTE                        byFodWindowSize;
    BYTE                        byFodWindowOffset;
    BYTE                        byPowerLossFaultCount;
    BYTE                        byDDMReStartNeeded;
    BYTE                        byDDMTimeOutCnt;        
    BYTE						byDDMTryCnt;            //device ping count, will increased by one at every ping
    WORD                        wDDMLastPktTimerMs;
    WORD                        wDDMReStartTimerCntMs;

    DWORD                       dwPowerLossIndicationToPwrCessationMsSelected;
    WORD                        wPowerLossDetectionThresholdSelected;
    BYTE                        byNumFodTripsToIndicationSelected;
    
    //Resonance shift FOD params
    WORD                        wRSFODThreshold[NUM_COILS_PER_DEVICE];

    //Temperature params
    SBYTE bOverTemperatureLimit;

    #if ADC_SAMPLE_AVERAGE_API
    WORD                        wAveragedInputVoltage;
    WORD                        wAveragedInputCurrent;
    WORD                        wAveragedCoilCurrent;
    SWORD                       wAveragedTempe;
    DWORD                       dwAveragedInputPower;
    #endif
    WORD wInCurrDynamicLimit;
    WORD wInCurrDynamicLimitRetDiff;

    
    #if MUTE_FOR_SHORT_RESET
    WORD wLastPwrOffTime;
    #endif
} RUNTIME_PARAMS;

//---------------NVM PARAMETER STRUCTURE DEFINITIONS----------------------------

//------------------------------------------------------------------------------
// Diagnostic NVM Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    // Frequency Sweep Operational Parameters
    DWORD   dwSweepStartFreq;
    DWORD   dwSweepEndFreq;
    WORD    wSweepStepFreq;
    WORD    wSweepFrequencyDurationTimeMs;

    // Duty Cycle Sweep Operational Parameters
    WORD    wSweepStartDutyCycle;
    WORD    wSweepEndDutyCycle;
    WORD    wSweepStepDutyCycle;
    WORD    wSweepDutyCycleDurationTimeMs;

} DIAGNOSTIC_PARAMS;

//------------------------------------------------------------------------------
// Charging Frequency Breakpoint Table
typedef struct
{
    DWORD   dwDeltaFreq;               // delta frequency to use over this breakpoint
    DWORD   dwFreqBreakPoint;          // frequency breakpoint

} CHARGING_FREQ_BREAKPOINT;

//------------------------------------------------------------------------------
// System Level WPC Parameter Sets
// OpState NVM System Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
#if BIG_ENDIAN
    // NOTE: Bitfield is organized MSB down to LSB
    WORD    btEnableStartPowerXferTone: 1;
    WORD    btEnableStopPowerXferTone: 1;
    WORD    btReserved: 2;
    WORD    btReserved2: 2;
    WORD    btDebugAnalogPing: 1;
    WORD    btDebugPmdOperation: 1;
    WORD    btDebugAutobaud: 1;
    WORD    btDebugMultiChannelComm: 1;
    WORD    btDebugCommStatus: 1;
    WORD    btDebugOperationalState: 1;
    WORD    btDebugVerboseOperation: 1;
    WORD    btDebugGeneralOperation: 1;
    WORD    btDebugVerbosePidOperation: 1;
    WORD    btDebugPidOperation: 1;
#else
    // NOTE: Bitfield is organized LSB down to MSB
    WORD    btDebugPidOperation: 1;
    WORD    btDebugVerbosePidOperation: 1;
    WORD    btDebugGeneralOperation: 1;
    WORD    btDebugVerboseOperation: 1;
    WORD    btDebugOperationalState: 1;
    WORD    btDebugCommStatus: 1;
    WORD    btDebugMultiChannelComm: 1;
    WORD    btDebugAutobaud: 1;
    WORD    btDebugPmdOperation: 1;
    WORD    btDebugAnalogPing: 1;
    WORD    btReserved2: 2;
    WORD    btReserved: 2;
    WORD    btEnableStopPowerXferTone: 1;
    WORD    btEnableStartPowerXferTone: 1;
#endif
} WPC_DIAGNOSTICS;

typedef struct
{
#if BIG_ENDIAN
    // NOTE: Bitfield is organized MSB down to LSB
    //WORD    btEptOVFaultLedIndicationDisabled: 1;
    WORD    btEptUnknownFaultLedIndicationDisabled: 1;
    WORD    btReserved: 7;
    WORD    btFodOverrideBinSelection: 4;
    WORD    btFodHardwareBinOverride: 1;
    WORD    btAnalogPingDisable: 1;
    WORD    btPowerTimeoutShutdownEnabled: 1;
    WORD    btShutdownOnVersionMismatch: 1;
#else
    // NOTE: Bitfield is organized LSB down to MSB
    WORD    btShutdownOnVersionMismatch: 1;
    WORD    btPowerTimeoutShutdownEnabled: 1;
    WORD    btAnalogPingDisable: 1;
    WORD    btFodHardwareBinOverride: 1;
    WORD    btFodOverrideBinSelection: 4;
    //WORD    btEptOVFaultLedIndicationDisabled: 1;
    WORD    btEptUnknownFaultLedIndicationDisabled: 1;
    WORD    btReserved: 7;
#endif
} WPC_PROTECTIONS;


//------------------------------------------------------------------------------
// OpState NVM System Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    // PWM drive
    WORD                    wPwmDeadTimeNs;
    WORD                    wReserved;

    // Option bitfield declarations
    WPC_DIAGNOSTICS         WpcDiagnostics;
    WPC_PROTECTIONS         WpcProtections;

} OPSTATE_SYSTEM_PARAMS;

//------------------------------------------------------------------------------
// OpState NVM Operating Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    // Digital Ping Operational Parameters
    DWORD                   dwPingFrequency;
    WORD                    wPingPulseDurationTimeMs;
    WORD                    wPingIntervalMs;

    // Analog Ping Operational Parameters
    DWORD                   dwAnalogPingFrequency;
    WORD                    wAnalogPingMinCoilCurrentThreshold;
    WORD                    wAnalogPingCoilCurrentThreshold;
    BYTE                    byAnalogPingDutyCycle;
    BYTE                    byAnalogPingPulseDuration;
    BYTE                    byAnalogPingAdcSampleTime;
    BYTE                    byDigitalPingRetryIntervalSeconds;

    // Signal Detection Parameters
    WORD                    wOverCurrentThreshold;
    WORD                    wSafetyInputThreshold;
    DWORD                   dwInputPowerThreshold;

    // Operating Frequencies
    DWORD                   dwMinFreq;
    DWORD                   dwMaxFreq;

    // PID Parameters
    WORD                    wIntegralUpdateInterval;
    WORD                    wDerivativeUpdateInterval;
    SWORD                   iIntegralUpperLimit;
    SWORD                   iIntegralLowerLimit;
    SWORD                   iPidUpperLimit;
    SWORD                   iPidLowerLimit;
    WORD                    wPidScaleFactor;
    BYTE                    byKp;
    BYTE                    byKi;
    BYTE                    byKd;
    BYTE                    byDelayTimeMs;
    BYTE                    byActiveTimeMs;
    BYTE                    bySettleTimeMs;
    BYTE                    byNumPidAdjustmentsPerActiveWindow;

    // Duty Cycle Control Parameters
    BYTE                    byMaxDutyCycle;             // Measured in whole %
    BYTE                    byMinDutyCycle;             // Measured in whole %
    BYTE                    byDCStep;                   // Measured in hundredths of a %
    BYTE                    byDCPidScaleFactor;
    BYTE                    byDCKp;
    BYTE                    byDCKi;
    BYTE                    byDCKd;
    #if BYTE_ACCESS_NOT_SUPPORTED
    BYTE                    byReservedPadding;
    #endif

    // Charging Frequency Breakpoint Table
    CHARGING_FREQ_BREAKPOINT FreqBreakPointTable[ NUM_FREQ_BREAKPOINTS ];
    DWORD                   dwDeltaFreq5;

} OPSTATE_OP_PARAMS;

//------------------------------------------------------------------------------
// Quadratic Parameters
typedef struct
{
    SWORD                   swQuadCoefficient;
    WORD                    wQuadExponent;
    SWORD                   swLinearCoefficient;
    WORD                    wLinearExponent;
    SWORD                   swConstantCoefficient;
    WORD                    wReserved;

} QUADRATIC_PARAMS;

//------------------------------------------------------------------------------
// FOD Characterization Parameters
typedef struct
{
    SWORD                   swQuadCoefficient;
    WORD                    wQuadExponent;
    SWORD                   swLinearCoefficient;
    WORD                    wLinearExponent;
    SWORD                   swConstantCoefficient;
    SWORD                   swPowerLossCalibrationOffset;

} FOD_CHARACTERIZATION_PARAMS;

//------------------------------------------------------------------------------
// FOD Normalization Parameters
typedef struct
{
    QUADRATIC_PARAMS        QuadraticParams[ NUM_NORM_REGIONS ];
    DWORD                   dwNormalizationBreakpoint[ ( NUM_NORM_REGIONS - 1 ) ];

} FOD_NORMALIZATION_PARAMS;

//----------------------------------------------------------------------------
//Resonance shift FOD operation params	

typedef struct
{
    WORD                   wRSFODdetectionOffset;
    WORD                   wReserved;

} RSFOD_OP_PARAMS;

//Resonance shift FOD calibration params
typedef struct
{
    SWORD                   swLinearCoeff;
    SWORD                   swConstantCoeff;
} RSFOD_CHARACTERIZATION_PARAMS;

//------------------------------------------------------------------------------
// Power Loss NVM Operating Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    DWORD                   dwPowerLossIndicationToPwrCessationMs;
    DWORD                   dwPowerLossFaultRetryTimeMs;
    WORD                    wPowerLossBaseThreshold;
    WORD                    wPowerLossIncrementalThreshold;
    BYTE                    byNumFodTripsToIndication;
    BYTE                    byDefaultWindowOffset;
    BYTE                    byDumpPldResultsForLegacyDevices;
    BYTE                    byReserved;

} POWERLOSS_OP_PARAMS;

//------------------------------------------------------------------------------
// Power Loss NVM Calibration Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    // FOD Characterization Params
    FOD_CHARACTERIZATION_PARAMS FodCharacterizationParams[ NUM_COILS_PER_DEVICE ];

    // FOD Normalization Params
    FOD_NORMALIZATION_PARAMS    FodNormalizationParams[ NUM_COILS_PER_DEVICE ];
    
    //RSFOD calibration Params
    RSFOD_CHARACTERIZATION_PARAMS RSFodCharacterizationParams[NUM_COILS_PER_DEVICE];

} POWERLOSS_CAL_PARAMS;

//------------------------------------------------------------------------------
// Analog NVM Calibration Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    WORD                    wInputVoltageCalibration;  // 100% = 1024
    WORD                    wInputCurrentCalibration;  // 100% = 1024
    WORD                    wCoilCurrentCalibration;   // 100% = 1024
    WORD                    wCoilCurrentDiodeDrop;

} ANALOG_CAL_PARAMS;

//------------------------------------------------------------------------------
// Display
//todo move this to appl.

#if BIG_ENDIAN
//------------------------------------------------------------------------------
// LED Parameters
typedef struct
{
    // NOTE: Bitfield is organized MSB down to LSB
    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    WORD    btLedInvertChargeStateDisplay: 1;
    WORD    btLedDisplayChargeState: 1;
    WORD    btLedReserved: 4;
    #else
    WORD    btLedReserved: 6;
    #endif
    WORD    btLedDiagnosticLedOffState: 1;
    WORD    btLedDiagnosticLedOnState: 1;
    WORD    btLedPowerLimitState: 1;
    WORD    btLedNvmFaultState: 1;
    WORD    btLedSystemFaultState: 1;
    WORD    btLedDeviceFaultState: 1;
    WORD    btLedModFaultState: 1;
    WORD    btLedDeviceChargedState: 1;
    WORD    btLedPowerXferState: 1;
    WORD    btLedStandbyState: 1;
    WORD    btLedInitializationState: 1;
    
} LED_STATE_BITS;
#else
//------------------------------------------------------------------------------
// LED Parameters
typedef struct
{
    // NOTE: Bitfield is organized LSB up to MSB
    WORD    btLedInitializationState: 1;
    WORD    btLedStandbyState: 1;
    WORD    btLedPowerXferState: 1;
    WORD    btLedDeviceChargedState: 1;
    WORD    btLedModFaultState: 1;
    WORD    btLedDeviceFaultState: 1;
    WORD    btLedSystemFaultState: 1;
    WORD    btLedNvmFaultState: 1;
    WORD    btLedPowerLimitState: 1;
    WORD    btLedDiagnosticLedOnState: 1;
    WORD    btLedDiagnosticLedOffState: 1;
    #if CHARGE_STATUS_VIA_DUTY_CYCLE_ENABLED
    WORD    btLedReserved: 3;
    WORD    btLedDisplayChargeState: 1;
    WORD    btLedInvertChargeStateDisplay: 1;
    #else
    WORD    btLedReserved: 5;
    #endif
    
} LED_STATE_BITS;
#endif

typedef union
{
    WORD            all;
    LED_STATE_BITS  bit;

} LED_STATE_PARAMS;

typedef struct
{
    LED_STATE_PARAMS    wLedOnOffStateBitfield;     // Bit-map where '1' indicates LED is ON for the corresponding state
    LED_STATE_PARAMS    wLedBlinkStateBitfield;     // Bit-map where '1' indicates LED is BLINKING for the corresponding state

} LED_PARAMS;

typedef struct
{
    LED_PARAMS  LedParams[ NUM_LEDS_PER_CHANNEL ];
    WORD        wFaultBlinkRateMs;
    WORD        wModFaultBlinkRateMs;
    WORD        wOpStateBlinkRateMs;
    WORD        wDelayAtPowerUpMs;

} LED_OPERATION;

//------------------------------------------------------------------------------
// Sound NVM System Parameters
// NOTE: This structure definition MUST reside on a 32-bit address boundary!
typedef struct
{
    WORD        wSoundLowFrequency;
    WORD        wSoundHighFrequency;
    WORD        wSoundFirstToneDuration;
    WORD        wSoundFirstPauseDuration;
    WORD        wSoundSecondToneDuration;
    WORD        wSoundSecondPauseDuration;
    WORD        wDcBeeperDuration;
    //WORD        wSoundReserved;
} SOUND_PARAMS;

typedef struct{
    BYTE bMajorVersion;
    BYTE bMinorVersion;
    BYTE bSubVersion;
}Firmware_Build_Version;

#if RESONANCE_SHIFT_FOD
typedef struct
{
	BYTE byAnalogDetectionCounter;
	BYTE byAnalogReleaseCounter;
	BYTE byAnalogDetectionFlag;
	BYTE byRSFODetectionCounter;
	BYTE byRSFODetectionFlag;
}RESONANCE_SHIFT_FOD_PARAMS;
#endif    

#endif /* __WCTTX_LIBSTRUCT_HEADER_H__ */
