/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __WCT_HAL_CFG_H
#define __WCT_HAL_CFG_H     1

//temporary, will be moved completely to appl
#include "application_cfg.h"

//This WCT configuration uses the initialisation macros from the QuickStart GCT
#include "peripheral_cfg.h"

//Include Freemaster config
#include "freemaster.h"

//This WCT configuration uses the OCCS HAL for calculating the IP_BUS_CLK from OCCS initialization
#include "occs.h"
#include "qsci.h"

//----------------------- Pin assignment -----------------------
#define WAKEUP_PIN_PORT                         GPIOE_DR
#define WAKEUP_PIN_BITMASK                      0x0020
#define WAKEUP_PIN_BIT                          5
#define TP1_PIN_PORT                            GPIOB_DR
#define TP1_PIN_BITMASK                         0x0001
#define DIAG_DISABLE_PIN_PORT                   GPIOB_DR
#define DIAG_DISABLE_PIN_BITMASK                0x0010

#define SWITCH_AUX_POWER_ON         (GPIOC_DR|=BIT10)
#define SWITCH_AUX_POWER_OFF         (GPIOC_DR&=~BIT10)

#define SWITCH_AUX_VCCH_ON         (GPIOC_DR|=BIT9)
#define SWITCH_AUX_VCCH_OFF         (GPIOC_DR&=~BIT9)

//----------------------- System Clock -----------------------
//At normal mode, clk of core:ip = 1:1
//At fast mode, clk of core:ip = 2:1
#define IP_BUS_CLK                              ( OCCS_CORE_FREQ / 2 )
#define IP_BUS_CLK_OF_LOW_POWER_MODE (OCCS_MSTROSC_FREQ / 2 / 2)    // Low power mode, PLL off, and fast mode is enabled, so IP bus=8M/2/2=2M.

//----------------------- Tick Timer defines -----------------------
#define TICK_INTERRUPT_TIME                     0.000200            // tick interrupt interval for normal operation
#define SYSTEM_TICK_TIME                        0.001               // 1ms system tick
#if TICK_TIMER_USE_PIT
#define TICK_TIMER_INTERRUPT                        void MWCT10xx_ISR_PIT0_ROLLOVER( void )
#define TICK_TIMER_INT_CLEAR_FLAG               (PIT0_CTRL&=~PIT0_CTRL_PRF)
#define PIT0_IRQ_PRIORITY_LEVEL              ( INTC_IPR10 = ( INTC_IPR10 & ~INTC_IPR10_PIT0_ROLLOVR ) | ( 2 << 12 ) ) //IRQ_INT_PRIORITY_1
#define TICK_TIMER_INTERRUPT_ENABLE             (PIT0_CTRL|=PIT0_CTRL_PRIE)
#define TICK_TIMER_INTERRUPT_DISABLE            (PIT0_CTRL&=~PIT0_CTRL_PRIE)
#define PIT_CLOCK_SOURCE_SELECTION              2  //ROSC_8M
#define TICK_TIMER_INIT \
     /* todo: change to gct settings */\
    PIT0_CTRL = PIT_0_CTRL_INIT;\
    PIT0_MOD = PIT_0_MOD_INIT;\
    /* Set PIT0 interrupt priority */\
    PIT0_IRQ_PRIORITY_LEVEL;\
    /* Run PIT */\
    PIT0_CTRL |= PIT0_CTRL_CNT_EN+PIT0_CTRL_PRIE;
#define PIT_0_PRESCALER                  (1<<((PIT_0_CTRL_INIT&PIT0_CTRL_PRESCALER)>>3)) //1
#else //TICK_TIMER_USE_PIT
#define TICK_TIMER_INTERRUPT                    void MWCT10xx_ISR_TMRA_1( void )
#define TICK_TIMER_INT_CLEAR_FLAG               ( TMR_1_SCTRL &= ~TMR_1_SCTRL_TCF )
#define TIMERA1_IRQ_PRIORITY_LEVEL              ( INTC_IPR2 = ( INTC_IPR2 & ~INTC_IPR2_TMRA_1 ) | ( 2 << 4 ) ) //IRQ_INT_PRIORITY_1
#define TICK_TIMER_INTERRUPT_ENABLE             ( TMR_1_SCTRL |= 0x4000)
#define TICK_TIMER_INTERRUPT_DISABLE            ( TMR_1_SCTRL &= ~0x4000 )
#define TICK_TIMER_INIT \
    /* Configure control register - mask off source to keep timer from starting */\
    TMR_1_CTRL      = ( QT_A1_CTRL_INIT & ~TMR_1_CTRL_CM );\
    TMR_1_SCTRL     = QT_A1_SCR_INIT;\
    /* Configure the timer compare values */\
    TMR_1_COMP1     = QT_A1_CMP1_INIT;\
    TMR_1_COMP2     = QT_A1_CMP2_INIT;\
    /* Set the initial timer value */\
    TMR_1_LOAD      = QT_A1_LOAD_INIT;\
    TMR_1_CSCTRL    = QT_A1_COMSCR_INIT;\
    /* Set timerA1 interrupt priority */\
    TIMERA1_IRQ_PRIORITY_LEVEL;\
    /* Turn on the timer peripheral */\
    TMR_0_ENBL |= QT_A0_ENBL_INIT&TMR_0_ENBL_ENBL_1;\
    /* start the timer*/\
    TMR_1_CTRL      |= ( QT_A1_CTRL_INIT & TMR_1_CTRL_CM )
#endif

//------------------------------ watchdog -------------------------------
#define DISABLE_WATCHDOG()                                ( COP_CTRL &= ~COP_CTRL_CEN )
#define ENABLE_WATCHDOG()                                 ( COP_CTRL |= (COP_CTRL_CEN) )

#define KICK_THE_WATCHDOG()                               do{ COP_CNTR = 0x5555; COP_CNTR = 0xAAAA; }while(0)

//---------------------- Non-Volatile Memory defines ---------------------
// NOTE: Flash Sector 0 is reserved for interrupt vectors and Flash Sector 1 contains flash
// configuration data (including security)
// We'll configure Data flash to be last flash sectors
// Each sector is 1 kB
#define NVM_STORAGE_IN_FLASH                    TRUE

#define DATA_FLASH_BASE_SECTOR_NUMBER             31  
#define NUM_DATA_FLASH_SECTORS                    1
//----------------------- digiComm module defines -----------------------

//----------------------------------------------------------------------
// COMM input timing relies on TMRA3 as a reference when interrupt driven
// NOTE: Prescaler should be set to '32' in peripheral_cfg.h
// - sets up 640ns resolution when ICLK = 50MHz
// - timer rolls over at 0xFFFF, or every 41.94ms
//      (Minimizes probability of "false" preamble detection: calculation
//      of bit times on first edge following a state machine reset can
//      generate the scenario due to timer roll-over.  The effect is more
//      prominent when communications signals are noisy.)
//----------------------------------------------------------------------

// Verify valid clock calculation... note that this syntax does not work on all compilers!
//#if TMRA3_TIMER_CLOCK == 0
//#error TMRA3_TIMER_CLOCK not valid!
//#endif

#define DDM_TMR_IRQ_PRIORITY_LEVEL                2   // Highest available priority for GPIO interrupts
#define SET_DDM_TMR_INTERRUPT_PRIORITY           ( INTC_IPR2 = ( INTC_IPR2 & ~INTC_IPR2_TMRA_0 ) | ( ((DDM_TMR_IRQ_PRIORITY_LEVEL) + 1) << 6 ) )

#define QI_COMM_INPUT_TIMER_CLOCK_FREQ             ( IP_BUS_CLK / (1<<((QT_A3_CTRL_INIT>>9)&0x7)) ) // get the timer divider from configuration
#define QI_COMM_INPUT_TIMER_COUNT                  (WORD*)(&TMR_3_CNTR)
#define QI_COMM_INPUT_TIMER_MAXIMUM_COUNT          65536

// Timer Register initialization
// NOTE: The timer PRESCALE value MUST be set according to the value defined in clock_cfg.h!
/*    #define TMRA3_CTRL_CONFIGURATION                ( CNT_RISING_EDGE_PRIMARY_SOURCE_1 + \
                                                      PRIMARY_SOURCE_IS_IP_BUS_CLK_DIV_32 + \
                                                      COUNT_REPEATEDLY + COUNT_UNTIL_ROLLOVER + COUNT_UP )
 */

//----------------------------------------------------------------------
// COMM input timeout calculations rely on TIMERA2 as a reference
// NOTE: Prescaler should be set to '128' in peripheral_cfg.h
// - sets up 2.56us resolution when ICLK = 50MHz
// - timer rolls over at 0xFFFF, or every 167.8ms
//----------------------------------------------------------------------

// Verify valid clock calculation... note that this syntax does not work on all compilers!
//#if TMRA2_TIMER_CLOCK == 0
//#error TMRA2_TIMER_CLOCK not valid!
//#endif

#define QI_COMM_REFERENCE_TIMER_CLOCK_FREQ      ( IP_BUS_CLK / (1<<((QT_A2_CTRL_INIT>>9)&0x7)) ) // get the timer divider from configuration
#define QI_COMM_REFERENCE_TIMER_COUNT              (WORD*)(&TMR_2_CNTR)
#define QI_COMM_REFERENCE_TIMER_MAXIMUM_COUNT      65536


// COMM input configuration and mapping (GPIO edge interrupts)
//  Interrupt status is reported in CPIOC_IPR (0xF167)
//  Interrupt flags are set and cleared in GPIOC_IESR / GPIOC_IEDGE (0xF168)
//  Interrupt polarity is selected in GPIOC_IPOLR (0xF166)
//  Pin state is reported in GPIOC_DR (0xF161)
//  Interrupts are enabled in the GPIOC_IENR register
// Channel 1 COMM 1     GPIOC7
// Channel 1 COMM 2     GPIOC8
// Channel 2 COMM 1     GPIOC5
// Channel 2 COMM 2     GPIOC6
// Channel 3 COMM 1     GPIOC11
// Channel 3 COMM 2     GPIOC12
// Channel 4 COMM 1     GPIOC14
// Channel 4 COMM 2     GPIOC15

// COMM Input Register Initialization
// NOTE: We are implementing a single-device platform utilizing hardware CH4

#if 0
#define COMM_INTERRUPT_STATUS_PORT              (WORD*) &GPIOC_IESR
#define COMM_IRQ_BITMASK                        ( BIT4 + BIT6 )
#define QI_COMM_INTERRUPT                        void MWCT10xx_ISR_GPIOC( void )

#define ENABLE_COMM_INTERRUPTS                  ( GPIOC_IENR |= COMM_IRQ_BITMASK )
#define DISABLE_COMM_INTERRUPTS                 ( GPIOC_IENR &= ~COMM_IRQ_BITMASK )
#define CLEAR_COMM_INT_FLAG                     // Interrupts are cleared by clearing individual status flags
#define GPIOC_IRQ_PRIORITY_LEVEL                2   // Highest available priority for GPIO interrupts
#define SET_COMM_INTERRUPT_PRIORITY()           ( INTC_IPR12 = ( INTC_IPR12 & ~INTC_IPR12_GPIOC ) | ( ((GPIOC_IRQ_PRIORITY_LEVEL) + 1) << 0 ) )

#define COMM_PORT_BITMASKS                      { { BIT4, BIT6 } }
#define COMM_INPUT_PORTS                        { { (WORD*) &GPIOC_DR, (WORD*) &GPIOC_DR } }
#define COMM_INPUT_BIT_MASKS                    COMM_PORT_BITMASKS
#define COMM_INT_POL_REGISTER                   { { (WORD*) &GPIOC_IPOLR, (WORD*) &GPIOC_IPOLR } }
#define COMM_INT_POL_BIT_MASKS                  COMM_PORT_BITMASKS
#endif

#define DDM_DMA_INTERRUPT                        void MWCT10xx_ISR_DMA0( void )
#define DDM_TMRA0_INTERRUPT                        void MWCT10xx_ISR_TMRA_0( void )


//coil
//------------------------- Coil module defines -------------------------
#define PWM_INIT_FREQUENCY                      175000  // This is the PING frequency or start frequency
#define PWM_INIT_DUTY_CYCLE                     5000    // 50%


// Overcurrent condition definitions
#define HARDWARE_OVERCURRENT_THRESHOLD_MA       4000    // 4 Amp current limit

// Overcurrent condition definitions
#define CURRENT_LIMIT_THRESHOLD                 4       // Trip current-limit if over-current detected on 4 consecutive periods
#define PWM_OVERCURRENT_FLAG_REGISTER           PWMA_FSTS0
// Map the macros to clear the overcurrent flags
#define CLEAR_COIL0_OVERCURRENT_FLAG            CLEAR_PWM0_FAULT_FLAG
#define CLEAR_COIL1_OVERCURRENT_FLAG            CLEAR_PWM1_FAULT_FLAG
#define CLEAR_COIL2_OVERCURRENT_FLAG            CLEAR_PWM2_FAULT_FLAG
#define CLEAR_COIL3_OVERCURRENT_FLAG            CLEAR_PWM3_FAULT_FLAG

#define OTHER_PWM_CHANNEL_ENABLED               ( PWM_MCTRL & ( PWM0_GENERATOR_ENABLED | PWM1_GENERATOR_ENABLED | PWM2_GENERATOR_ENABLED | PWM3_GENERATOR_ENABLED ) )

// NOTE: We are implementing a single-device platform utilizing hardware CH4
#define PWM0_USED_FOR_COIL_CONTROL              TRUE
#define PWM_CHANNEL_MAPPING                     { 0 }
// This definition has only been used for analog ping feedback, currently not implenting analog ping
#define COIL_CURRENT_ADC_CHANNEL_MAPPING        { COIL_CURR_SENSE_C1 }

// Coil Discharge is not supported by this hardware
#define COIL_DISCHARGE_CONTROL_PORTS            { (WORD*)&GPIOC_DR }
#define COIL_DISCHARGE_CONTROL_BIT_MASKS        { BIT8 }


#define COIL_DUTYCYCLE_PERCENT_DEFAULT          5000            // This represents 50% duty cycle output
// General definitions
//#define MAX_PWM_CHANNELS                        0
#define MAX_DUTY_CYCLE_PERCENT                  10000       // Depicts 



//---------------- Diagnostics and Debug module defines -------------------
#if FREEMASTER_SUPPORTED
#define WCT_DEBUG_FREEMASTER                    TRUE
#else
#define WCT_DEBUG_FREEMASTER                    FALSE
#endif

// SCI0 configuration
#if (FREEMASTER_SUPPORTED) && (FMSTR_USE_SCI) && (FMSTR_SCI_BASE == 0xE080)
// Use QSCI0 for FreeMASTER debug
#define DEBUG_CONSOLE_QSCI0                     FALSE    // We are using Peripheral QSCI0 for diagnostics
#define QSCI0_TX_INTERRUPT_DRIVEN             FALSE
#else
// Use QSCI0 (UART module) for debug console print
#define DEBUG_CONSOLE_QSCI0                     TRUE    // We are using Peripheral QSCI0 for diagnostics
#define QSCI0_TX_INTERRUPT_DRIVEN             TRUE
#endif
#define QSCI0_TX_INTERRUPT                      void MWCT10xx_ISR_QSCI0_TDRE( void )
#define DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL  1
#define DEBUG_CONSOLE_INTERRUPT_PRIORITY        ( INTC_IPR5 = ( INTC_IPR5 & ~INTC_IPR5_QSCI0_TDRE ) | ( ( (DEBUG_CONSOLE_INTERRUPT_PRIORITY_LEVEL) + 1) << 10 ) )
#define QSCI0_TX_BUFFER_SIZE                    80     // define size of TX buffer
#define DEBUG_CONSOLE_HEADER_FILE               "qsci.h"
#define DEBUG_SW_VERSION_IN_BOOT    	    FALSE  // Print SW version, system cfg during boot




//------------------ PowerLossDetection module defines -----------------
#define INPUT_POWER_16BIT_SHIFT_VALUE           15
#define INPUT_POWER_16BIT_MULTIPLIER            32768
#define COIL_CURRENT_SQUARED_16BIT_SHIFT_VALUE  6
#define COIL_CURRENT_SQUARED_16BIT_MULTIPLIER   64



//------------------------- Comparator module defines -------------------------

#define COMP_DAC_REFERENCE_VOLTAGE              3.3  // 3.3 Volts

// Set up comparator C to be used for hardware overcurrent sensing
#define COMPARATORA_ENABLED                     FALSE
#define COMPC_DAC_SETTING_V                     ( HARDWARE_OVERCURRENT_THRESHOLD_MA / COIL_CURRENT_GAIN_MA_PER_VOLT )

#define COMPARATORB_ENABLED                     FALSE

#define COMPARATORC_ENABLED                     FALSE


#define COMPARATORD_ENABLED                     FALSE


//------------------------- ADC module defines ---------------------------

// ADC Channel IDs
// This list defines the order of the channels in the local buffer
// This list must correspond to ADC sequence of analog inputs defined by ADLSTx registers
typedef enum
{
        ADC_CHANNEL_VIN_SENSE = 0,
        ADC_CHANNEL_COIL_CURR_SENSE_C1,
        ADC_CHANNEL_INPUT_CURR_SENSE_C1,
        ADC_CHANNEL_BOARD_TEMPERATURE,
        ADC_CHANNEL_MAX
//	ADC_CHANNEL_COIL_CURR_SENSE_C1 = 0,
//	ADC_CHANNEL_INPUT_CURR_SENSE_C1,
//	ADC_CHANNEL_INPUT_CURR_OFFSET_SENSE,
//	ADC_CHANNEL_VIN_SENSE,
//	ADC_CHANNEL_MAX,
} ADC_CHANNEL_ID;

#define ADC_CONVERSION_COMPLETE_INTERRUPT       void MWCT10xx_ISR_ADC_CC0( void )
#define ADC_TRIGGER_CONVERSION                  ( ADC_CTRL1 |= 0x2000 )

#define ADC_REFERENCE_VOLTAGE               3.3     //3.0V

// Configuration for digital filtering
// NOTE: This filter rate is critical for proper operation of the PID control
//  The PID loop runs in 1ms increments and the ADC sample rate interval is 200us
#define ADC_ROLLING_AVG_SAMPLE_CONFIG       ADC_ROLLING_AVG_EQUAL_4_SAMPLES       // Maximum oversample for Analog COMM is 4X



// Input Current Sense Circuits
#define INPUT_CURRENT_SHUNT_R_VALUE             0.010
#define INPUT_ISENSE_GAIN                       100.0
#define INPUT_CURRENT_GAIN_MA_PER_VOLT          ( 1000.0 * ( 1.0 / ( INPUT_CURRENT_SHUNT_R_VALUE * INPUT_ISENSE_GAIN ) ) )

// Configuration for Analog Input Channels
// Vsense Circuit
#define VIN_SENSE_DIVIDER_TOP_R_VALUE           154000.0
#define VIN_SENSE_DIVIDER_BOTTOM_R_VALUE        20000.0
#define VIN_SENSE_DIVISOR                       ( VIN_SENSE_DIVIDER_BOTTOM_R_VALUE / ( VIN_SENSE_DIVIDER_TOP_R_VALUE + VIN_SENSE_DIVIDER_BOTTOM_R_VALUE ) )
#define VIN_SENSE_MICROVOLTS_PER_ADC_COUNT      ( DWORD )( MICROVOLTS_PER_ADC_COUNT_FLOAT / VIN_SENSE_DIVISOR )
#define INPUT_VOLTAGE_GAIN_MV_PER_VOLT          ( ( 1.0 / VIN_SENSE_DIVISOR ) * 1000.0 )        // Used in the PowerLossDetectionModule

// Coil Current Sense Circuit
// This circuit is capacitively coupled - no current transformer
//  Isense = ( Itotal * Zresonance caps ) / ( Zresonance caps + Zsense cap + Rsense )
//   therefore
//  Itotal = ( Isense * ( Zresonance caps + Zsense cap + Rsense ) ) / Zresonance caps
//   and if 
//  Isense = Vrsense / Rsense
//   substitution yields:
//  Itotal = ( ( ( Vrsense / Rsense ) * ( Zresonance caps + Zsense cap + Rsense ) ) / ( Zresonance caps ) )
//   solving for a constant yielding mA/V, we substitute '1' for Vrsense
#define PI                                      3.141592654
#define ISENSE_C_VALUE                          ( 4700.0 * 1E-12 )   // 4700pF
#define CRESONANCE_VALUE                        ( 400.0 * 1E-9 )    // 400nF
#define NOMINAL_OPERATING_FREQUENCY             120000.0            // Typical 5W power transfer at ~120KHz
                                                                    //TODO: update this frequency value


#define ISENSE_C_IMPEDANCE                      ( 1.0 / ( 2.0 * PI * NOMINAL_OPERATING_FREQUENCY * ISENSE_C_VALUE ) )
#define CRESONANCE_IMPEDANCE                    ( 1.0 / ( 2.0 * PI * NOMINAL_OPERATING_FREQUENCY * CRESONANCE_VALUE ) )

#define ISENSE_R_VALUE                          56110.0

// Defines for Coil current sensing by resister network
#define VSENSE_TOP_R_VALUE               7550.0
#define VSENSE_BOTTOM_R_VALUE         5110.0
#define VSENSE_R_VALUE_RECIP_SUM    ((1.0/VSENSE_TOP_R_VALUE)+(1.0/VSENSE_BOTTOM_R_VALUE))
#define ISENSE_SCALE                           ((CRESONANCE_IMPEDANCE+ISENSE_C_IMPEDANCE+ISENSE_R_VALUE)/CRESONANCE_IMPEDANCE)
#define ISENSE_CONSTANT_FLOAT          ((ADC_REFERENCE_VOLTAGE*1000.0/VSENSE_TOP_R_VALUE)*ISENSE_SCALE)
#define COIL_CURRENT_CALIBRATION_COEFF 0.75
#define ISENSE_CONSTANT_RMS_FLOAT          (ISENSE_CONSTANT_FLOAT * 0.707 * COIL_CURRENT_CALIBRATION_COEFF)
#define ISENSE_CONSTANT                     ( DWORD )( ISENSE_CONSTANT_FLOAT + 0.5 )
#define ISENSE_CONSTANT_RMS                     ( DWORD )( ISENSE_CONSTANT_RMS_FLOAT + 0.5 )

#define COIL_CURRENT_GAIN_MA_PER_VOLT    (VSENSE_R_VALUE_RECIP_SUM*ISENSE_SCALE*1000.0*COIL_CURRENT_CALIBRATION_COEFF)

#define ISENSE_SCALE_INT (DWORD)(ISENSE_SCALE)

// Define a look-up table for coil current and safety input assignments
#define ADC_COIL_CURRENT_LIST                   { ADC_CHANNEL_COIL_CURR_SENSE_C1 }
#define ADC_INPUT_CURRENT_LIST                  { ADC_CHANNEL_INPUT_CURR_SENSE_C1 }

    // Analog pin configuration
    /*#define INPUT_CURR_OFFSET_SENSE                             SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define INPUT_CURR_SENSE_C1                                 SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS
    #define INPUT_CURR_SENSE_C2                                 SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS
    #define INPUT_CURR_SENSE_C3                                 SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS
    #define INPUT_CURR_SENSE_C4                                 SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS
    #define COIL_CURR_SENSE_C1                                  SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define COIL_CURR_SENSE_C2                                  SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define COIL_CURR_SENSE_C3                                  SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define COIL_CURR_SENSE_C4                                  SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define VIN_SENSE                                           SINGLE_ENDED_ANA0_DIFF_ANA0PLUS_ANA1MINUS*/
    #define INPUT_CURR_SENSE_C1                                 SINGLE_ENDED_ANA2_DIFF_ANA2PLUS_ANA3MINUS
    #define COIL_CURR_SENSE_C1                                  SINGLE_ENDED_ANA1_DIFF_ANA0PLUS_ANA1MINUS
    #define VIN_SENSE                                           SINGLE_ENDED_ANA0_DIFF_ANA0PLUS_ANA1MINUS


#define ADC_NUM_CHANS                                       ADC_CHANNEL_MAX
#define ADC_SET_INT_PRIORITY(x) INTC_IPR2 |= (((x+1)&0x3)<<10) //INTC_IPR0_ADC_CC0_IPL_0




//----------------------- Adc sample average defines -------------------------
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_0 0 // sample weight = 1/128
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_1 1 // sample weight = 2/128
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_2 2 // sample weight = 4/128
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_3 3 // sample weight = 8/128
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_4 4 // sample weight = 16/128
#define ADC_SAMPLE_AVERAGE_WEIGHT_MAX_LEVEL 7

#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_INPUT_VOLTAGE ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_4
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_COIL_CURRENT ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_2
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_INPUT_CURRENT ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_2
#define ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_TEMPERATURE ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_4

#define ADC_SAMPLE_AVERAGE_WEIGHT_MAX (WORD)(1<<ADC_SAMPLE_AVERAGE_WEIGHT_MAX_LEVEL)  //128

#define ADC_SAMPLE_AVERAGE_INPUT_VOLTAGE_INIT_VALUE (WORD)((5000*1000.0)/VIN_SENSE_MICROVOLTS_PER_ADC_COUNT)
#define ADC_SAMPLE_AVERAGE_INPUT_CURRENT_INIT_VALUE (WORD)((2400*1000.0)/INPUT_I_MICROAMPS_PER_ADC_COUNT)



#define PACKING_ATTRIBUTE


//----------------------- debug module defines -------------------------
#define CLEAR_TP1 (GPIOF_DR&= ~BIT2)
#define SET_TP1     (GPIOF_DR|= BIT2)
#define TOGGLE_TP1 (GPIOF_DR^= BIT2)
#define CLEAR_TP2 (GPIOF_DR&= ~BIT3)
#define SET_TP2     (GPIOF_DR|= BIT3)
#define TOGGLE_TP2 (GPIOF_DR^= BIT3)
#define CLEAR_TP3 (GPIOC_DR&= ~BIT4)
#define SET_TP3     (GPIOC_DR|= BIT4)
#define TOGGLE_TP3 (GPIOC_DR^= BIT4)


#endif  //__WCT_HAL_CFG_H
