/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"

#include "adc.h"
#include "pwm.h"
#include "flash.h"
#include "freemaster.h"
#include "display.h"
#include DEBUG_CONSOLE_HEADER_FILE
#include "wct_debug.h"
#include "sound.h"
#include "lowPowerMode.h"


//-----------------
// PRIVATE DEFINES
//-----------------

//-------------------
// PRIVATE VARIABLES
//-------------------
#if LOW_POWER_MODE_SUPPORTED
static WORD wLowPowerModeTimeMs = 0;
BYTE bLowPowerModeStatus = FALSE;
WORD wTxIdleTime=0; //Tx idle time, unit: second
#endif

WORD wInVolAdc;



//------------------------------------------------------------------------------
void WCT_Timer1msTickHandler( WORD wNumTicks )
{
    #if ADC_SAMPLE_AVERAGE_API
    WORD i;
    WORD sample;
    T_ADC_SAMPLE_AVERAGE *pAdcAverage;
    #endif
    
#if LOW_POWER_MODE_SUPPORTED
    WORD wDisplayLowPowerTimeMs;
#if SOUND_SUPPORTED
    WORD wSoundLowPowerTimeMs;
#endif
#endif
    
#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_Timer1msTickHandler)
        SET_TP1;
    else if (dbg2_select_flag==MEASURE_Timer1msTickHandler)
        SET_TP2;
#endif

#if LOW_POWER_MODE_SUPPORTED
    wLowPowerModeTimeMs = WCT_OpStateProcessTimeSlice( wNumTicks );
    wDisplayLowPowerTimeMs = DisplayProcessTimeSlice( wNumTicks );
#if SOUND_SUPPORTED
    wSoundLowPowerTimeMs = SoundProcessTimeSlice( wNumTicks );
    
    // Assess Low Power time
	if( wSoundLowPowerTimeMs < wDisplayLowPowerTimeMs )
	{
		wDisplayLowPowerTimeMs = wSoundLowPowerTimeMs;
	}
#endif

    if( wDisplayLowPowerTimeMs < wLowPowerModeTimeMs )
	{
		// This handler needs to run sooner than 1ms handler
		wLowPowerModeTimeMs = wDisplayLowPowerTimeMs;
	}
#else
    WCT_OpStateProcessTimeSlice( wNumTicks );
    DisplayProcessTimeSlice( wNumTicks );
#if SOUND_SUPPORTED
    SoundProcessTimeSlice( wNumTicks );
#endif
#endif

    #if ADC_SAMPLE_AVERAGE_API
    for(i=0;i<NUM_DEVICES;i++)
    {
        if( (RunTimeParams[i].btDeviceControlFlags.bPowering==FALSE) || (RunTimeParams[i].btDeviceControlFlags.bPowering==TRUE && RunTimeParams[i].btDeviceControlFlags.bCollectInputCurrent==TRUE))
        {   // Average input current, only when powering and no communication.
            sample = WCT_AnalogIoGetInputCurrent(i, CURRENT_REPORT_ADC_COUNTS);
            pAdcAverage = &AdcAverage[ADC_CHANNEL_INPUT_CURR_SENSE_C1];
            WCT_AdcSampleAverage(pAdcAverage, sample);
            RunTimeParams[i].wAveragedInputCurrent = WCT_ConvertInCurrAdcToRmsMilliAmps(pAdcAverage->wAdcSample); 

            wInVolAdc = WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT_ADC_COUNTS );
            pAdcAverage = &AdcAverage[ADC_CHANNEL_VIN_SENSE];
            WCT_AdcSampleAverage(pAdcAverage, wInVolAdc);
            RunTimeParams[i].wAveragedInputVoltage = WCT_ConvertInVolAdcToMilliVolts(pAdcAverage->wAdcSample);
        }
    }
    #endif

#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_Timer1msTickHandler)
        CLEAR_TP1;
    else if (dbg2_select_flag==MEASURE_Timer1msTickHandler)
        CLEAR_TP2;
#endif

#if LOW_POWER_MODE_SUPPORTED
    // Check for opportunity to enter low power mode
    // IMPORTANT: Do this following 10ms service loop to allow LED updates
    // AC Sound is stopped if low power mode entered. So only when AC sound off, enable low power
#if SOUND_SUPPORTED
    if( wLowPowerModeTimeMs && wSoundLowPowerTimeMs>=LOW_POWER_MODE_TIME_INFINITE )   
#else
    if( wLowPowerModeTimeMs )   
#endif
    {
        // Firmware is indicating we can save power
        if( wLowPowerModeTimeMs == LOW_POWER_MODE_TIME_INFINITE )
        {
            // System is indicating a permanent fault
            PRE_LOW_POWER_HANDLER;
            #if DEBUG_LOW_POWER_MODE
            SET_LOW_POWER_MODE_DEBUG_PIN_HIGH;
            #endif
            ENTER_SHUTDOWN_MODE;
        }
        else
        {
#if LOW_POWER_DISPLAY_MODE
			DisplayUpdateLedsInLowPowerMode(wLowPowerModeTimeMs,wNumTicks);
#endif
        	PRE_LOW_POWER_HANDLER;
		    #if DEBUG_LOW_POWER_MODE
		    SET_LOW_POWER_MODE_DEBUG_PIN_HIGH;
		    #endif
		    bLowPowerModeStatus=TRUE; 
		    ENTER_LOW_POWER_MODE_MS( wLowPowerModeTimeMs );
		    bLowPowerModeStatus=FALSE;
		    #if DEBUG_LOW_POWER_MODE
		    SET_LOW_POWER_MODE_DEBUG_PIN_LOW;
		    #endif
		    POST_LOW_POWER_HANDLER;
		    #if (USE_STOP_OR_DELAY_FOR_LOW_POWER==STOP_FOR_LP)    
		    // Update our TICK count according to the amount of time we were in low power mode
		    //TimerAddTimerTicks( wLowPowerModeTimeMs );
		    #endif
        }
    }
#endif

}

//------------------------------------------------------------------------------
void WCT_Timer10msTickHandler( WORD wNumTicks )
{
#if WCT_DEBUG_FREEMASTER
    WctDebugProcess();
#endif

}


//------------------------------------------------------------------------------
void WCT_Timer100msTickHandler( void )
{
    // Kick the internal WDT
    KICK_THE_WATCHDOG();

    #if EXT_WATCHDOG_SUPPORTED
    // Toggle the external WDT kick
    if( PIN_EXT_WD_KICK )
    {
        SET_PIN_EXT_WD_KICK_LOW;
    }
    else
    {
        SET_PIN_EXT_WD_KICK_HIGH;
    }
    #endif
}

//------------------------------------------------------------------------------
void WCT_Timer1SecTickHandler( void )
{
    WORD i;
    #if ADC_SAMPLE_AVERAGE_API
    WORD sample;
    T_ADC_SAMPLE_AVERAGE *pAdcAverage;
    #endif
    
    #ifdef DEBUG_ADC
    WORD wValue;

    // for debug, dump the ADC voltages to the debug port...
    // NOTE: Use DebugPutDateCodeString() to force printing of leading zeroes.
    wValue = AdcRead( ADC_CHANNEL_INPUT_CURR_SENSE_C4 );
    WCT_DebugPutString( "~I=0x" );
    DebugPutDateCodeString( wValue );
/*    wValue = AdcRead( ADC_CHANNEL_INPUT_CURR_OFFSET_SENSE );
    WCT_DebugPutString( "~Ioffset=0x" );
    DebugPutDateCodeString( wValue );
    wValue = AdcRead( ADC_CHANNEL_COIL_CURR_SENSE_C4 );
    WCT_DebugPutString( "~Icoil=0x" );
    DebugPutDateCodeString( wValue );*/

    wValue = AdcRead( ADC_CHANNEL_VIN_SENSE );
//    sprintf(msg, ", V=0x%03X   \r", value);
    WCT_DebugPutString( "~V=0x" );
    DebugPutDateCodeString( wValue );
    WCT_DebugPutString( "   \r" );
    #endif

#if LOW_POWER_MODE_SUPPORTED
    {
        BYTE idle;

        /* If system is not in power transfer state for a certain time, stop system to save power. 
        * If system is in fault, do not stop system.*/
        idle=TRUE;
        for(i=0;i<NUM_DEVICES;i++)
        {
            if(RunTimeParams[i].eOpState>=OP_STATE_POWER_XFER || RunTimeParams[i].dwPowerXferRetryTimerMs>0)
            {
                idle = FALSE;
                wTxIdleTime=0;
                break;
            }
        }

        if(idle)
            wTxIdleTime++;
    }
#endif

#if 0
    for(i=0;i<NUM_DEVICES;i++)
    {
        WCT_GetAverageBoardTemperature(&RunTimeParams[i]);  // Average board temperature.
    }
#endif

    #if ADC_SAMPLE_AVERAGE_API
    for(i=0;i<NUM_DEVICES;i++)
    {
        sample = WCT_AnalogIoGetBoardTemperature(TEMPERATURE_REPORT_ADC_COUNTS);
        pAdcAverage = &AdcAverage[ADC_CHANNEL_BOARD_TEMPERATURE];
        WCT_AdcSampleAverage(pAdcAverage, sample);
        RunTimeParams[i].wAveragedTempe = (SWORD) WCT_ConvertAdcToTempe(pAdcAverage->wAdcSample); 
    }
    #endif

#if PRINT_CUSTOMER_MESSAGE && QSCI0_ENABLED
    for(i=0;i<NUM_DEVICES;i++)
    {
        message_print(i, RunTimeParams[i].eOpState);
    }
#endif
    
}


//------------------------------------------------------------------------------
void WCT_Timer1MinTickHandler( void )
{
    // Perform temperature calibration of internal oscillator
}
