/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
#ifndef __WCT_CALLBACKS_H__
#define __WCT_CALLBACKS_H__

//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_hal_cfg.h"
#include "wct_lib.h"

extern RF_DATA_PARAMS eRfDataParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
extern RF_DATA_INTERRUPT_PARAMS eRfDataInterruptParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];

extern RUNTIME_PARAMS          RunTimeParams[ NUM_DEVICES ];       // Parameters used during normal operation

extern HW_CONFIG_OPTIONS       HwConfigOptions;
extern NVM_PARAMS              NvmParams;
extern BYTE bLowPowerModeStatus;


//------------------------------------------------------------------------------
//  Analog callbacks
//------------------------------------------------------------------------------
// Typically we want to keep the scale factor sized to 16 bits in order to guarantee the calculations can be
// done within a double-word variable.  Since the scaling factor can represent values in excess of 100%, we
// will declare the nominal calibration scale factor size as 15 bits (allows maximum scale factor of 200%).
// NOTE: We will include this declaration only if not already defined to allow the use of custom values.
#ifndef NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS
#define NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS       15
#endif

#define NOMINAL_CALIBRATION_SCALE_FACTOR        ( WORD )( 1 << NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS )


#define VOLTS_PER_ADC_COUNT                     ( MICROVOLTS_PER_ADC_COUNT_FLOAT / 1000000.0 )

// NOTE: The mapping file holds the hardware-specific definitions for V/V conversions
//  ***We will multiply the specified scale factor to eliminate floating point math during run-time
#define LOW_GAIN_SCALE_FACTOR_MODIFIED          ( WORD )( LOW_GAIN_SCALE_FACTOR * 1024 )

#define LOW_GAIN_SWITCH_POINT_COUNTS            ( WORD )( LOW_GAIN_SWITCH_POINT_VOLTS / VOLTS_PER_ADC_COUNT )
#define NORMAL_GAIN_SWITCH_POINT_COUNTS         ( WORD )( NORMAL_GAIN_SWITCH_POINT_VOLTS / VOLTS_PER_ADC_COUNT )


// NOTE: The current sense circuit reports the PEAK current, convert to RMS:
#define COIL_I_PEAK_MICROAMPS_PER_ADC_COUNT     ( WORD )( 1000 * COIL_CURRENT_GAIN_MA_PER_VOLT * VOLTS_PER_ADC_COUNT )
#define COIL_I_RMS_MICROAMPS_PER_ADC_COUNT      ( WORD )( 1000 * COIL_CURRENT_GAIN_MA_PER_VOLT * VOLTS_PER_ADC_COUNT * 0.707 )

#define INPUT_I_MICROAMPS_PER_ADC_COUNT         ( WORD )( 1000 * INPUT_CURRENT_GAIN_MA_PER_VOLT * VOLTS_PER_ADC_COUNT )

//added for calculation of Input Power
// NOTE: The 16-bit shift value is a constant used to insure the max counts squared is represented within a 16-bit variable.
//  For a 12-bit ADC, the max counts squared is 16769025.  Representing this as a 16-bit variable, we are required to shift right by '8'.
// NOTE: Cast ADC_COUNTS_SQUARED_MAX_VALUE definition to DWORD value to prevent compilers from calculating (0xFFF)^2 as negative number!
#define ADC_COUNTS_SQUARED_MAX_VALUE                ( DWORD )( 1.0 * MAX_ADC_COUNTS * MAX_ADC_COUNTS )
#define ADC_16BIT_SHIFT_VALUE                       ( WORD )( ( 2 * ADC_BITS ) - 16 )
#define ADC_16BIT_DIVISOR_VALUE                     ( WORD )( 1U << ADC_16BIT_SHIFT_VALUE )
#define ADC_COUNTS_SQUARED_16BIT_MAX_COUNTS         ( WORD )( ADC_COUNTS_SQUARED_MAX_VALUE / ADC_16BIT_DIVISOR_VALUE )    //( WORD )( ADC_COUNTS_SQUARED_MAX_VALUE / ( WORD )( 1U << ADC_16BIT_SHIFT_VALUE ) )

// Calculate the MAX ranges of ADC measurements supported by the hardware
#define INPUT_CURRENT_MAX_RANGE_FLOAT               ( INPUT_CURRENT_GAIN_MA_PER_VOLT * ADC_REFERENCE_VOLTAGE )
#define INPUT_VOLTAGE_MAX_RANGE_FLOAT               ( INPUT_VOLTAGE_GAIN_MV_PER_VOLT * ADC_REFERENCE_VOLTAGE )
#define COIL_CURRENT_MAX_RANGE_FLOAT                ( COIL_CURRENT_GAIN_MA_PER_VOLT * ADC_REFERENCE_VOLTAGE )
#define COIL_CURRENT_MAX_RANGE_RMS_FLOAT                ( COIL_CURRENT_MAX_RANGE_FLOAT * 0.707)
#define COIL_CURRENT_SQUARED_MAX_RANGE_FLOAT        ( COIL_CURRENT_MAX_RANGE_FLOAT * COIL_CURRENT_MAX_RANGE_FLOAT )
#define INPUT_POWER_MAX_RANGE_FLOAT                 ( ( INPUT_CURRENT_MAX_RANGE_FLOAT * INPUT_VOLTAGE_MAX_RANGE_FLOAT ) / 1000.0 )

#define INPUT_CURRENT_MAX_RANGE                     ( DWORD )( INPUT_CURRENT_MAX_RANGE_FLOAT + 0.5 )
#define INPUT_VOLTAGE_MAX_RANGE                     ( DWORD )( INPUT_VOLTAGE_MAX_RANGE_FLOAT + 0.5 )
#define COIL_CURRENT_MAX_RANGE                      ( DWORD )( COIL_CURRENT_MAX_RANGE_FLOAT + 0.5 )
#define COIL_CURRENT_RMS_MAX_RANGE                      ( DWORD )( COIL_CURRENT_MAX_RANGE_RMS_FLOAT + 0.5 ) 
#define COIL_CURRENT_SQUARED_MAX_RANGE              ( DWORD )( COIL_CURRENT_SQUARED_MAX_RANGE_FLOAT + 0.5 )
#define INPUT_POWER_MAX_RANGE                       ( DWORD )( INPUT_POWER_MAX_RANGE_FLOAT + 0.5 )


// Calculate the terms used for Input Power assessment
#define POWER_PER_ADC_COUNTS_16BIT                  ( INPUT_POWER_MAX_RANGE_FLOAT / ADC_COUNTS_SQUARED_16BIT_MAX_COUNTS )
//#define SCALED_POWER_PER_ADC_COUNTS_16BIT           ( WORD )( ( POWER_PER_ADC_COUNTS_16BIT << INPUT_POWER_16BIT_SHIFT_VALUE ) + 0.5 )
#define SCALED_POWER_PER_ADC_COUNTS_16BIT           ( WORD )( ( POWER_PER_ADC_COUNTS_16BIT * INPUT_POWER_16BIT_MULTIPLIER ) + 0.5 )

// Calculate the terms used for Coil Current^2 assessment
#define COIL_CURRENT_SQUARED_PER_ADC_COUNTS_16BIT   ( COIL_CURRENT_SQUARED_MAX_RANGE_FLOAT / ADC_COUNTS_SQUARED_16BIT_MAX_COUNTS )
//#define SCALED_COIL_CURRENT_SQUARED_PER_ADC_COUNTS_16BIT    ( DWORD )( ( COIL_CURRENT_SQUARED_PER_ADC_COUNTS_16BIT << COIL_CURRENT_SQUARED_16BIT_SHIFT_VALUE ) + 0.5 )
#define SCALED_COIL_CURRENT_SQUARED_PER_ADC_COUNTS_16BIT    ( WORD )( ( COIL_CURRENT_SQUARED_PER_ADC_COUNTS_16BIT * COIL_CURRENT_SQUARED_16BIT_MULTIPLIER ) + 0.5 )

// Calculate the terms used to calculate the Transmitter losses
#define QUADRATIC_PREDIVISOR_EXPONENT               ( WORD )( 16 - COIL_CURRENT_SQUARED_16BIT_SHIFT_VALUE )


#endif
