/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_callbacks.h"
#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "peripheralInit.h"

#include "adc.h"
#include "pwm.h"
#include "flash.h"
#include "freemaster.h"
#include "display.h"
#include DEBUG_CONSOLE_HEADER_FILE
#include "wct_debug.h"
#include "sound.h"
#include "lowPowerMode.h"
#include "dma.h"


//-----------------
// PRIVATE DEFINES
//-----------------
TICK_TIMER_INTERRUPT;
DDM_TMRA0_INTERRUPT;

RUNTIME_PARAMS          RunTimeParams[ NUM_DEVICES ];       // Parameters used during normal operation

HW_CONFIG_OPTIONS       HwConfigOptions;
NVM_PARAMS              NvmParams;

/*FMSTR_TSA_TABLE_BEGIN(runtimeParams)
    FMSTR_TSA_RW_VAR(RunTimeParams,      FMSTR_TSA_USERTYPE(RUNTIME_PARAMS))
    FMSTR_TSA_STRUCT(RUNTIME_PARAMS)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, byDeviceId,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, bySignalStrength,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, dwActiveFreq,   FMSTR_TSA_UINT32)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wActiveDutyCycle,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wFreqStep,   FMSTR_TSA_UINT16)

    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, eOpState,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wDeviceFaultStatus,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, byEptReasonCode,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, byPercentCharged,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, byPercentPowerUsed,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, iReportedError,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wMaxHalfWatts,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, byPidDelayTime,   FMSTR_TSA_UINT8)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wCoilCurrent,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wOverCurrentLimit,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wInputCurrent,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wInputVoltageHistory,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wInputCurrentHistory,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wCoilCurrentHistory,   FMSTR_TSA_UINT16)
//    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wCalculatedInputPower,   FMSTR_TSA_UINT16)
    FMSTR_TSA_MEMBER(RUNTIME_PARAMS, wSetPoint,   FMSTR_TSA_UINT16)
    
FMSTR_TSA_TABLE_END()
*/

#if DEBUG_GPIO
BYTE dbg1_select_flag;
BYTE dbg2_select_flag;
#endif

WORD gADCInitValue[NUM_DEVICES];


//-------------------
// PRIVATE VARIABLES
//-------------------


//-------------------
// PUBLIC VARIABLES
//-------------------
volatile WORD SR_lock = 0U;            /* Lock */
volatile WORD SR_reg;                  /* Current value of the SR register */
/*lint -esym(765,CpuMode) Disable MISRA rule (8.10) checking for symbols (CpuMode). */
BYTE CpuMode;                   /* Current speed mode */

#pragma define_section DMA_DATA ".dma_data"
#pragma section DMA_DATA begin
SWORD gDDMRecordBuf[NUM_DEVICES][RECORD_BUF_LEN/2];
#pragma section DMA_DATA end

Firmware_Build_Version gFWBuildVersion;

//------------------------------------------------------------------------------
//  Qi Communication callbacks
//------------------------------------------------------------------------------
RF_DATA_PARAMS eRfDataParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
RF_DATA_INTERRUPT_PARAMS eRfDataInterruptParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];

#if ADC_SAMPLE_AVERAGE_API
T_ADC_SAMPLE_AVERAGE AdcAverage[ADC_CHANNEL_MAX];

/* Adc sample average by weight function, i.e. adc = old*0.99 + new*0.01 
 *  Here it's simplified by shift, adc = (old*((1<<7)-(1<<n))+new<<n)>>7
 */
void WCT_AdcSampleAverage(T_ADC_SAMPLE_AVERAGE *pAdcAverage, WORD sample)
{
    DWORD average;
    WORD weight;

    /* new*x% */
    weight = pAdcAverage->weight;
    average = (DWORD)sample<<weight;
    
    /* old*(100-x)% */
    average += (DWORD) pAdcAverage->wAdcSample*(ADC_SAMPLE_AVERAGE_WEIGHT_MAX-(1<<weight));

    /* old*(100-x)% + new*x% */
    average >>= 7; // Div 128
    average &= 0xfff;   //12 bits ADC
    pAdcAverage->wAdcSample = (WORD)average;   
}
#endif


//TMRA0 is allocated to DEVICE 0
#if CODE_WARRIOR
#pragma interrupt alignsp saveall
#endif
DDM_TMRA0_INTERRUPT
{
	TMR_0_CSCTRL &= ~TMR_0_CSCTRL_TCF1;  //Clear TMRA0 interrupt flag
#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_DDM_FILTER)
        SET_TP1;
    else if (dbg2_select_flag==MEASURE_DDM_FILTER)
        SET_TP2;
#endif
    
  	WCT_DDMDataAnalyze(0);
  	
#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_DDM_FILTER)
        CLEAR_TP1;
    else if (dbg2_select_flag==MEASURE_DDM_FILTER)
        CLEAR_TP2;
#endif
}

//------------------------------------------------------------------------------
//  Tick Timer callbacks
//------------------------------------------------------------------------------
//implementation of the application
#if CODE_WARRIOR
#pragma interrupt saveall
#endif
TICK_TIMER_INTERRUPT
{
#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_TICK_ISR)
        SET_TP1;
    else if (dbg2_select_flag==MEASURE_TICK_ISR)
        SET_TP2;
#endif

	#if TICK_TIMER_INTERRUPT_PRIORITY_IS_HIGHER_THAN_COMM_INTERRUPT_PRIORITY
	volatile STANDARD_REGISTER_SIZE OriginalInterruptConfig;
	
	TICK_TIMER_ENABLE_NESTED_HIGHER_PRIORITY_INTERRUPTS;
	#endif
	//Call the system tick timer
	WCT_TickTimerUpdate();
#if LOW_POWER_MODE_SUPPORTED
      if (!bLowPowerModeStatus) //In low power mode, core clock is low, so skip the long time handler.
#endif
	WCT_TickTimerHandler();

	TICK_TIMER_INT_CLEAR_FLAG;          // Clear the Interrupt flag

	#if TICK_TIMER_INTERRUPT_PRIORITY_IS_HIGHER_THAN_COMM_INTERRUPT_PRIORITY
	TICK_TIMER_DISABLE_SOFTWARE_INTERRUPT_NESTING;
	#endif
	
	#if PROCESSOR_INTERRUPTS_REQUIRE_ACKNOWLEDGE
	TICK_TIMER_INTERRUPT_ACKNOWLEDGE;
	#endif

#if DEBUG_GPIO
    if(dbg1_select_flag==MEASURE_TICK_ISR)
        CLEAR_TP1;
    else if (dbg2_select_flag==MEASURE_TICK_ISR)
        CLEAR_TP2;
#endif
}
#if CODE_WARRIOR
#pragma interrupt off
#endif

void WCT_TickTimerInit(void)
{
	//clock is enabled by SIM module
	do { TICK_TIMER_INIT; }while(0);

    // Enable interrupts in the application	
}

void WCT_TickTimerInt( BYTE state )
{
    if (state == WCT_ENABLE)
        TICK_TIMER_INTERRUPT_ENABLE;
    else
        TICK_TIMER_INTERRUPT_DISABLE;
}
//implementation of library callbacks
void WCT_KickTheWatchdog( void )
{
	KICK_THE_WATCHDOG();
}


//------------------------------------------------------------------------------
//  PWM callbacks for analog ping
//------------------------------------------------------------------------------

const STANDARD_REGISTER_SIZE PwmChannelMapping[ NUM_COILS ] = PWM_CHANNEL_MAPPING;
const STANDARD_REGISTER_SIZE CoilCurrentAdcChannelMapping[ NUM_COILS ] = COIL_CURRENT_ADC_CHANNEL_MAPPING;
STANDARD_REGISTER_SIZE * const ptrCoilDischargeControlPort[ NUM_COILS ] = COIL_DISCHARGE_CONTROL_PORTS;
const STANDARD_REGISTER_SIZE CoilDischargeControlBitMask[ NUM_COILS ] = COIL_DISCHARGE_CONTROL_BIT_MASKS;
volatile WORD wCoilDutyCyclePercent[ NUM_COILS ];

#if COIL_ANALOG_PING
//volatile BYTE byLoopInterruptCount = 0;
#if PWM0_ENABLED
#pragma interrupt saveall
//------------------------------------------------------------------------------
void MWCT10xx_ISR_PWM_RELOAD0( void )
{
    // Loop 1 interrupt
    //byLoopInterruptCount++;

    // Clear the Interrupt flag
    CLEAR_PWM0_RELOAD_INTERRUPT_STATUS;
}
#pragma interrupt off
#endif

#if PWM1_ENABLED
#pragma interrupt saveall
//------------------------------------------------------------------------------
void MWCT10xx_ISR_PWM_RELOAD1( void )
{
    // Loop 1 interrupt
    //byLoopInterruptCount++;

    // Clear the Interrupt flag
    CLEAR_PWM1_RELOAD_INTERRUPT_STATUS;
}
#pragma interrupt off
#endif

#if PWM2_ENABLED
#pragma interrupt saveall
//------------------------------------------------------------------------------
void MWCT10xx_ISR_PWM_RELOAD2( void )
{
    // Loop 1 interrupt
    //byLoopInterruptCount++;

    // Clear the Interrupt flag
    CLEAR_PWM2_RELOAD_INTERRUPT_STATUS;
}
#pragma interrupt off
#endif

#if PWM3_ENABLED
#pragma interrupt saveall
//------------------------------------------------------------------------------
void MWCT10xx_ISR_PWM_RELOAD3( void )
{
    // Loop 1 interrupt
    //byLoopInterruptCount++;

    // Clear the Interrupt flag
    CLEAR_PWM3_RELOAD_INTERRUPT_STATUS;
}
#pragma interrupt off
#endif
#endif

//------------------------------------------------------------------------------
//  Coil callbacks
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
void WCT_CoilInit( void )
{
    BYTE byCoilNum;

    // Initialize the Duty Cycle table
    for( byCoilNum = 0; byCoilNum < NUM_COILS; byCoilNum++ )
    {
        wCoilDutyCyclePercent[ byCoilNum ] = COIL_DUTYCYCLE_PERCENT_DEFAULT;
    }

    // Initialize the Pwm channels
    PwmInit( NvmParams.SystemParams.OpStateParams.wPwmDeadTimeNs );

    // Reload interrupt is replaced by while loop
#if COIL_ANALOG_PING    
    // Set the interrupts to highest priority for any active channels
    #if PWM0_ENABLED
    //INTC_IPR5 = ( ( INTC_IPR5 & ~INTC_IPR5_PWM_RELOAD0 ) | ( IRQ_INT_PRIORITY_2 << 6 ) );
    #endif
    #if PWM1_ENABLED
    //INTC_IPR5 = ( ( INTC_IPR5 & ~INTC_IPR5_PWM_RELOAD1 ) | ( IRQ_INT_PRIORITY_2 << 2 ) );
    #endif
    #if PWM2_ENABLED
    //INTC_IPR4 = ( ( INTC_IPR4 & ~INTC_IPR4_PWM_RELOAD2 ) | ( IRQ_INT_PRIORITY_2 << 14 ) );
    #endif
    #if PWM3_ENABLED
    //todo, define by macro and move it to appl cfg
    //INTC_IPR4 = ( ( INTC_IPR4 & ~INTC_IPR4_PWM_RELOAD3 ) | ( IRQ_INT_PRIORITY_2 << 8 ) );
    #endif
#endif
}


//------------------------------------------------------------------------------
BYTE WCT_CoilFreqSet( BYTE byCoilId, DWORD dwFreqValue )
{
    if ( NUM_COILS > byCoilId )
    {
        PwmFrequencyUpdate( PwmChannelMapping[ byCoilId ], dwFreqValue, wCoilDutyCyclePercent[ byCoilId ] );
        return RET_VALUE_GOOD;
    }
    else
    {
        return RET_VALUE_INVALID_PARAM;
    }
}


//------------------------------------------------------------------------------
BYTE WCT_CoilDutyCycleSet( BYTE byCoilId, WORD wDutyCyclePercent )
{
    if ( NUM_COILS > byCoilId )
    {
        wCoilDutyCyclePercent[ byCoilId ] = wDutyCyclePercent;
        PwmDutyCycleUpdate( PwmChannelMapping[ byCoilId ], wDutyCyclePercent );
        return RET_VALUE_GOOD;
    }
    else
    {
        return RET_VALUE_INVALID_PARAM;
    }

}

//------------------------------------------------------------------------------
BYTE WCT_CoilControl( BYTE byCoilId, BOOL bState )
{
    BYTE byReturnValue = RET_VALUE_GOOD;

    // NOTE: The overcurrent flag must be cleared AFTER the PWM module is turned off

    switch( PwmChannelMapping[ byCoilId ] )
    {
        #if ( PWM0_ENABLED && PWM0_USED_FOR_COIL_CONTROL )
        case 0:
        {
            if ( bState == WCT_ENABLE )
            {
                // Clear overcurrent Fault if one exists
                CLEAR_COIL0_OVERCURRENT_FLAG;
                PwmStartupControl(byCoilId);
            }
            else
            {
                ENABLE_PWM0_OUTPUT_OVERRIDE;
                // Disable the Pwm channel
                DISABLE_PWM0;
            }

            //if( !OTHER_PWM_CHANNEL_ENABLED )
            //{
            //    DISABLE_FET_DRIVER;
            //}
            break;
        }
        #endif
        #if ( PWM1_ENABLED && PWM1_USED_FOR_COIL_CONTROL )
        case 1:
        {
            if ( bState == WCT_ENABLE )
            {
                // Clear overcurrent Fault if one exists
                CLEAR_COIL1_OVERCURRENT_FLAG;
                PwmStartupControl(byCoilId);
            }
            else
            {
                ENABLE_PWM1_OUTPUT_OVERRIDE;
                // Disable the Pwm channel
                DISABLE_PWM1;
            }

            //if( !OTHER_PWM_CHANNEL_ENABLED )
            //{
            //    DISABLE_FET_DRIVER;
            //}
            break;
        }
        #endif
        #if ( PWM2_ENABLED && PWM2_USED_FOR_COIL_CONTROL )
        case 2:
        {
            if ( bState == WCT_ENABLE )
            {
                // Clear overcurrent Fault if one exists
                CLEAR_COIL2_OVERCURRENT_FLAG;
                PwmStartupControl(byCoilId);
            }
            else
            {
                ENABLE_PWM2_OUTPUT_OVERRIDE;
                // Disable the Pwm channel
                DISABLE_PWM2;
            }


            //if( !OTHER_PWM_CHANNEL_ENABLED )
            //{
            //    DISABLE_FET_DRIVER;
            //}
            break;
        }
        #endif
        #if ( PWM3_ENABLED && PWM3_USED_FOR_COIL_CONTROL )
        case 3:
        {
            if ( bState == WCT_ENABLE )
            {
                // Clear overcurrent Fault if one exists
                CLEAR_COIL3_OVERCURRENT_FLAG;
                PwmStartupControl(byCoilId);
            }
            else
            {
                ENABLE_PWM3_OUTPUT_OVERRIDE;
                // Disable the Pwm channel
                DISABLE_PWM3;
            }

            //if( !OTHER_PWM_CHANNEL_ENABLED )
            //{
            //    DISABLE_FET_DRIVER;
            //}
            break;
        }
        #endif
        default:
            // Illegal coil!!!
            byReturnValue = RET_VALUE_INVALID_PARAM;
            break;
    }

    return byReturnValue;
}

//------------------------------------------------------------------------------
BOOL WCT_CoilCheckHardwareOvercurrentStatus( BYTE byCoilId )
{
    BOOL bReturnValue = FALSE;

    if( byCoilId > NUM_COILS )
    {
        // Illegal coil!!!
        bReturnValue = TRUE;
    }
    else if( PWM_OVERCURRENT_FLAG_REGISTER & ( 0x01 << PwmChannelMapping[ byCoilId ] ) )
    {
        // Fault flag is set, we have a fault (PWM outputs already disabled)
        bReturnValue = TRUE;
        // Clear the fault flag (by writing a one to the flag)
        PWM_OVERCURRENT_FLAG_REGISTER |= ( 0x01 << PwmChannelMapping[ byCoilId ] );
    }
    else
    {
        bReturnValue = FALSE;
    }

    return bReturnValue;
}

// PWM start up control to get expected frequency and duty on the first pulse.
void PwmStartupControl(BYTE byCoilId)
{
    WORD VAL3;
    
    switch( PwmChannelMapping[ byCoilId ] )
    {
        #if ( PWM0_ENABLED && PWM0_USED_FOR_COIL_CONTROL )
        case 0:
            //Step-1, let PWM runs at zero output.
            PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;
            VAL3=PWMA_SM0VAL3;
            //PWMA_SM0VAL2 = 0;
            PWMA_SM0VAL3 = 0;
            PWMA_SM0VAL4 = 0;
            PWMA_SM0VAL5 = 0;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_0;
            
            ENABLE_PWM0;
            CLEAR_PWM0_RELOAD_INTERRUPT_STATUS;
            while(!PWM0_RELOAD_INTERRUPT_STATUS);
            DISABLE_PWM0_OUTPUT_OVERRIDE;
            
            //Step-2, at next PWM pulse, let PWM runs normally.
            while(PWMA_MCTRL&PWMA_MCTRL_LDOK_0);
            //PWMA_SM0VAL2 = 0;
            PWMA_SM0VAL3 = VAL3;
            PWMA_SM0VAL4 = ( WORD )PWMA_SM0VAL1>>1;
            PWMA_SM0VAL5=PWMA_SM0VAL2;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_0;

            break;
        #endif

        #if ( PWM1_ENABLED && PWM1_USED_FOR_COIL_CONTROL )
        case 1:
            //Step-1, let PWM runs at zero output.
            PWMA_MCTRL |= PWMA_MCTRL_CLDOK_1;
            VAL3=PWMA_SM1VAL3;
            //PWMA_SM0VAL2 = 0;
            PWMA_SM1VAL3 = 0;
            PWMA_SM1VAL4 = 0;
            PWMA_SM1VAL5 = 0;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_1;
            
            ENABLE_PWM1;
            CLEAR_PWM1_RELOAD_INTERRUPT_STATUS;
            while(!PWM1_RELOAD_INTERRUPT_STATUS);
            DISABLE_PWM1_OUTPUT_OVERRIDE;
            
            //Step-2, at next PWM pulse, let PWM runs normally.
            while(PWMA_MCTRL&PWMA_MCTRL_LDOK_1);
            //PWMA_SM1VAL2 = 0;
            PWMA_SM1VAL3 = VAL3;
            PWMA_SM1VAL4 = ( WORD )PWMA_SM1VAL1>>1;
            PWMA_SM1VAL5=PWMA_SM1VAL2;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_1;

            break;
        #endif

        #if ( PWM2_ENABLED && PWM2_USED_FOR_COIL_CONTROL )
        case 2:
            //Step-1, let PWM runs at zero output.
            PWMA_MCTRL |= PWMA_MCTRL_CLDOK_2;
            VAL3=PWMA_SM1VAL3;
            //PWMA_SM2VAL2 = 0;
            PWMA_SM2VAL3 = 0;
            PWMA_SM2VAL4 = 0;
            PWMA_SM2VAL5 = 0;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_2;
            
            ENABLE_PWM2;
            CLEAR_PWM2_RELOAD_INTERRUPT_STATUS;
            while(!PWM2_RELOAD_INTERRUPT_STATUS);
            DISABLE_PWM2_OUTPUT_OVERRIDE;
            
            //Step-2, at next PWM pulse, let PWM runs normally.
            while(PWMA_MCTRL&PWMA_MCTRL_LDOK_2);
            //PWMA_SM2VAL2 = 0;
            PWMA_SM2VAL3 = VAL3;
            PWMA_SM2VAL4 = ( WORD )PWMA_SM2VAL1>>1;
            PWMA_SM2VAL5=PWMA_SM2VAL2;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_2;

            break;
        #endif

        #if ( PWM3_ENABLED && PWM3_USED_FOR_COIL_CONTROL )
        case 3:
            //Step-1, let PWM runs at zero output.
            PWMA_MCTRL |= PWMA_MCTRL_CLDOK_3;
            VAL3=PWMA_SM3VAL3;
            //PWMA_SM3VAL2 = 0;
            PWMA_SM3VAL3 = 0;
            PWMA_SM3VAL4 = 0;
            PWMA_SM3VAL5 = 0;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_3;
            
            ENABLE_PWM3;
            CLEAR_PWM3_RELOAD_INTERRUPT_STATUS;
            while(!PWM3_RELOAD_INTERRUPT_STATUS);
            DISABLE_PWM3_OUTPUT_OVERRIDE;
            
            //Step-2, at next PWM pulse, let PWM runs normally.
            while(PWMA_MCTRL&PWMA_MCTRL_LDOK_3);
            //PWMA_SM3VAL2 = 0;
            PWMA_SM3VAL3 = VAL3;
            PWMA_SM3VAL4 = ( WORD )PWMA_SM3VAL1>>1;
            PWMA_SM3VAL5=PWMA_SM3VAL2;
            PWMA_MCTRL |= PWMA_MCTRL_LDOK_3;

            break;
        #endif

        default:
            // Illegal coil!!!
            break;
    }
}

//------------------------------------------------------------------------------
WORD WCT_CoilAnalogPing( BYTE byCoilId, BYTE byPulseDuration, BYTE byAdcSampleTime )
{
#if COIL_ANALOG_PING
    BYTE byLoopInterruptCount;
    WORD wAdcSample = 0;
    BYTE byHighSideDuration = byPulseDuration;
    BYTE byPulseDurationTemp,byAdcSampleTimeTemp;
    WORD *recBuf;
    
    // NOTE: This function assumes the associated PWM channel has been configured for
    //       the proper frequency and duty cycle of operation by the calling function.

    byPulseDurationTemp = byPulseDuration+1;      // PWM pulse should add one for the 175KHz analog ping workaround
    byAdcSampleTimeTemp = byAdcSampleTime+1;   // with opt-size4,. the ++ operation on the stack data, byPulseDuration and  byPulseDuration is abnormal. 

    if ( NUM_COILS > byCoilId )
    {
        // Disable all interrupts but ours to insure accurate control
        // NOTE: Disable COMM interrupts (highest priority) and set the interrupt mask for level 2
        DISABLE_CPU_INTERRUPTS;
        //DISABLE_COMM_INTERRUPTS;
        __EI2();
        ENABLE_CPU_INTERRUPTS;

        // Trigger a burst of output pulses on the appropriate channel
        switch( PwmChannelMapping[ byCoilId ] )
        {
            #if PWM0_ENABLED
            case 0:
                // Enable the GPIO
               CLEAR_PWM0_FAULT_FLAG;
               recBuf = (WORD *)gDDMRecordBuf[0];

                // Enable the PWM channel
                // NOTE: The reference timer will always start at zero!
                PWM_SetTriggerPos(0, 25);   // PWM trig ADC sampling at 1/4 cycle
                DMA_Start_ForAdcRead(0);  // DMA start for ADC sample read

                PwmStartupControl(byCoilId);

                CLEAR_PWM0_RELOAD_INTERRUPT_STATUS;  // Clearing RF must be behind of PWM start
                byLoopInterruptCount = 0;

                while(  (byLoopInterruptCount <= byPulseDurationTemp) ||
                           ( byLoopInterruptCount <= byAdcSampleTimeTemp ))
                {
                    if(PWM0_RELOAD_INTERRUPT_STATUS)
                    {
                        byLoopInterruptCount++;
                        CLEAR_PWM0_RELOAD_INTERRUPT_STATUS;
                    }
                    if(PWMA_SM0STS&PWMA_SM0STS_CMPF_3)
                    {
                        PWMA_SM0STS|=PWMA_SM0STS_CMPF_3;    // Clear val3 compare flag.
                        if( byLoopInterruptCount == byHighSideDuration )
                        {
                            // Turn off the high-side output
                            ENABLE_PWM0A_OUTPUT_OVERRIDE;
                        }
                    }
                    if( byLoopInterruptCount == byPulseDurationTemp )
                    {
                        // Turn off the low-side output
                        ENABLE_PWM0_OUTPUT_OVERRIDE;
                    }
                    if( byLoopInterruptCount == byAdcSampleTimeTemp )
                    {
                        // Trigger manual ADC sample
                        break;
                    }
                }
                // Disable the DPWM channel
                CLEAR_PWM0_RELOAD_INTERRUPT_STATUS;  // Clearing RF must be behind of PWM start
                
                DISABLE_PWM0;
                DMA_Stop(0);
                // Clear any over-current indication
                CLEAR_PWM0_FAULT_FLAG;
                wAdcSample=gADCInitValue[0]-(recBuf[byLoopInterruptCount]>>3);   //Use the last sample for analog ping
                break;
            #endif

            #if PWM1_ENABLED
            case 1:
                // Enable the GPIO
               CLEAR_PWM1_FAULT_FLAG;
               recBuf = (WORD *)gDDMRecordBuf[1];

                // Enable the PWM channel
                // NOTE: The reference timer will always start at zero!
                PWM_SetTriggerPos(1, 25);   // PWM trig ADC sampling at 1/4 cycle
                DMA_Start_ForAdcRead(1);  // DMA start for ADC sample read
                PwmStartupControl(byCoilId);
                CLEAR_PWM1_RELOAD_INTERRUPT_STATUS;  // Clearing RF must be behind of PWM start
                byLoopInterruptCount = 0;

                while(  byLoopInterruptCount <= byPulseDurationTemp )
                {
                    if(PWM1_RELOAD_INTERRUPT_STATUS)
                    {
                        byLoopInterruptCount++;
                        CLEAR_PWM1_RELOAD_INTERRUPT_STATUS;
                    }
                    if( byLoopInterruptCount == byPulseDurationTemp )
                    {
                        // Turn off the low-side output
                        ENABLE_PWM1_OUTPUT_OVERRIDE;
                    }
                }
                // Disable the DPWM channel
                DISABLE_PWM1;
                // Clear any over-current indication
                CLEAR_PWM1_FAULT_FLAG;
                DMA_Stop(1);
                wAdcSample=gADCInitValue[1]-(recBuf[byPulseDurationTemp]>>3);   //Use the last sample for analog ping
                break;
            #endif

            #if PWM2_ENABLED
            case 2:
                // Enable the GPIO
               CLEAR_PWM2_FAULT_FLAG;
               recBuf = (WORD *)gDDMRecordBuf[2];

                // Enable the PWM channel
                // NOTE: The reference timer will always start at zero!
                PWM_SetTriggerPos(2, 25);   // PWM trig ADC sampling at 1/4 cycle
                DMA_Start_ForAdcRead(2);  // DMA start for ADC sample read
                PwmStartupControl(byCoilId);
                CLEAR_PWM2_RELOAD_INTERRUPT_STATUS;  // Clearing RF must be behind of PWM start
                byLoopInterruptCount = 0;

                while(  byLoopInterruptCount <= byPulseDurationTemp )
                {
                    if(PWM2_RELOAD_INTERRUPT_STATUS)
                    {
                        byLoopInterruptCount++;
                        CLEAR_PWM2_RELOAD_INTERRUPT_STATUS;
                    }
                    if( byLoopInterruptCount == byPulseDurationTemp )
                    {
                        // Turn off the low-side output
                        ENABLE_PWM2_OUTPUT_OVERRIDE;
                    }
                }
                // Disable the DPWM channel
                DISABLE_PWM2;
                // Clear any over-current indication
                CLEAR_PWM2_FAULT_FLAG;
                DMA_Stop(2);
                wAdcSample=gADCInitValue[2]-(recBuf[byPulseDurationTemp]>>3);   //Use the last sample for analog ping
                break;
            #endif

            #if PWM3_ENABLED
            case 3:
                // Enable the GPIO
               CLEAR_PWM3_FAULT_FLAG;
               recBuf = (WORD *)gDDMRecordBuf[3];

                // Enable the PWM channel
                // NOTE: The reference timer will always start at zero!
                PWM_SetTriggerPos(3, 25);   // PWM trig ADC sampling at 1/4 cycle
                DMA_Start_ForAdcRead(3);  // DMA start for ADC sample read
                PwmStartupControl(byCoilId);
                CLEAR_PWM3_RELOAD_INTERRUPT_STATUS;  // Clearing RF must be behind of PWM start
                byLoopInterruptCount = 0;

                while(  byLoopInterruptCount <= byPulseDurationTemp )
                {
                    if(PWM3_RELOAD_INTERRUPT_STATUS)
                    {
                        byLoopInterruptCount++;
                        CLEAR_PWM3_RELOAD_INTERRUPT_STATUS;
                    }
                    if( byLoopInterruptCount == byPulseDurationTemp )
                    {
                        // Turn off the low-side output
                        ENABLE_PWM3_OUTPUT_OVERRIDE;
                    }
                }
                // Disable the DPWM channel
                DISABLE_PWM3;
                // Clear any over-current indication
                CLEAR_PWM3_FAULT_FLAG;
                DMA_Stop(3);
                wAdcSample=gADCInitValue[3]-(recBuf[byPulseDurationTemp]>>3);   //Use the last sample for analog ping
                break;
            #endif

            default:
                // Error
                break;
        }

        // Clear INT flag
        ADC_STAT  |= 0x1F00;     // clear interrupt flags by writting a 1 to them
        __EI0();
    }

    if(wAdcSample < 10)
    	wAdcSample = 10;
    
    return wAdcSample;
#else
    return 0;
#endif
}


//------------------------------------------------------------------------------
BYTE WCT_CoilDischarge( BYTE byCoilId, BOOL bState )
{
    BYTE byReturnValue = RET_VALUE_GOOD;

    if ( NUM_COILS > byCoilId )
    {
        if( bState )
        {
            *( ptrCoilDischargeControlPort[ byCoilId ] ) |= CoilDischargeControlBitMask[ byCoilId ];
        }
        else
        {
            *( ptrCoilDischargeControlPort[ byCoilId ] ) &= ~CoilDischargeControlBitMask[ byCoilId ];
        }
    }
    else
    {
        byReturnValue = RET_VALUE_INVALID_PARAM;
    }

    return ( byReturnValue );
}


//------------------------------------------------------------------------------
//  Analog callbacks
//------------------------------------------------------------------------------

const WORD wAdcCoilCurrentIndex[ NUM_COILS ] = ADC_COIL_CURRENT_LIST;
const WORD wAdcInputCurrentIndex[ NUM_COILS ] = ADC_INPUT_CURRENT_LIST;
WORD wAdcInputCurrentOffset[ NUM_COILS ];

RUNTIME_PARAMS* pCh0RunTimeParams = &RunTimeParams[0];

void WCT_EnableAdcConversions(void)
{
	ADC_TRIGGER_CONVERSION;
}


//------------------------------------------------------------------------------
void WCT_AnalogIoInit( RUNTIME_PARAMS* pRunTimeParams )
{
    DWORD dwTemp;
    OP_PARAMS* pOpParams = &NvmParams.OpParams[ pRunTimeParams->byDeviceId ];
    ANALOG_CAL_PARAMS* pCalParams = &NvmParams.CalParams.AnalogParams[ pRunTimeParams->byDeviceId ];

    dwTemp = pOpParams->OpStateParams.wOverCurrentThreshold;
    dwTemp = ISENSE_CONSTANT_RMS - dwTemp;
    dwTemp <<= ADC_BITS;
    dwTemp /= COIL_CURRENT_RMS_MAX_RANGE;   //dwTemp is coil current low value
    if( dwTemp >= (DWORD)gADCInitValue[pRunTimeParams->byDeviceId] )
    	dwTemp = 0;
    dwTemp = (DWORD)gADCInitValue[pRunTimeParams->byDeviceId] - dwTemp;
    pRunTimeParams->wOverCurrentLimit = dwTemp;

    // Load ADC calibration parameters for runtime use
    // NOTE: The redundant storage of these parameters is required because they 
    //       may be over-written by diagnostics during system calibration.
    pRunTimeParams->wInputVoltageCalibration = pCalParams->wInputVoltageCalibration;
    pRunTimeParams->wInputCurrentCalibration = pCalParams->wInputCurrentCalibration;
    pRunTimeParams->wCoilCurrentCalibration = pCalParams->wCoilCurrentCalibration;

    //  Diode drop stored in mV, convert to ADC counts
    //  NOTE: Multiply by 1049, then shift right 10 to prevent div-by-1000
    pRunTimeParams->dwCoilCurrentDiodeOffset = pCalParams->wCoilCurrentDiodeDrop * 1049;
    pRunTimeParams->dwCoilCurrentDiodeOffset >>= 10;                            // Scales to 1.024 x offset value
    pRunTimeParams->dwCoilCurrentDiodeOffset *= ADC_COUNTS_PER_VOLT;
    pRunTimeParams->dwCoilCurrentDiodeOffset >>= 10;                            // Div-by 1024 to generate ADC counts

    pRunTimeParams->bOverTemperatureLimit = BOARD_TEMPERATURE_CENTIGRADE_LIMIT;
    //pRunTimeParams->wAveragedTempe = ROOM_TEMPERATURE_CENTIGRADE;
    pRunTimeParams->wInCurrDynamicLimit = IN_CURR_HIGH_TO_LIMIT;

#if ADC_SAMPLE_AVERAGE_API
    AdcAverage[ADC_CHANNEL_VIN_SENSE].weight                  = ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_INPUT_VOLTAGE;
    AdcAverage[ADC_CHANNEL_COIL_CURR_SENSE_C1].weight   = ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_COIL_CURRENT;
    AdcAverage[ADC_CHANNEL_INPUT_CURR_SENSE_C1].weight = ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_INPUT_CURRENT;
    AdcAverage[ADC_CHANNEL_BOARD_TEMPERATURE].weight   = ADC_SAMPLE_AVERAGE_WEIGHT_LEVEL_TEMPERATURE;

    AdcAverage[ADC_CHANNEL_VIN_SENSE].wAdcSample          = WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT_ADC_COUNTS );
    AdcAverage[ADC_CHANNEL_INPUT_CURR_SENSE_C1].wAdcSample          = ADC_SAMPLE_AVERAGE_INPUT_CURRENT_INIT_VALUE;  //Init adc average for input current. 
#endif
}

//------------------------------------------------------------------------------
void WCT_PerformInputCurrentCalibration( void )
{
    // NOTE: This function should be called when the PWM output is INACTIVE.
    // The function will store the average input current reading as a calibration constant.
    // The function assumes zero current flow through the coil.

    BYTE byCoilId;

    for( byCoilId = 0; byCoilId < NUM_COILS; byCoilId++ )
    {
        // Clear the input offset value
        // NOTE: This must be done first to insure we get the correct value from WCT_AnalogIoGetInputCurrent
        wAdcInputCurrentOffset[ byCoilId ] = 0;

        // Get the new offset value
        wAdcInputCurrentOffset[ byCoilId ] = WCT_AnalogIoGetInputCurrent( byCoilId, CURRENT_REPORT_RAW_ADC_COUNTS );
    }
}


//------------------------------------------------------------------------------
void ADC_CoilCurrentInit(BYTE byDevice)
{
	gADCInitValue[byDevice] = ADC_ManualConversion(byDevice, 8, 0);
}

//------------------------------------------------------------------------------
void WCT_PerformCoilCurrentCalibration( void )
{
	BYTE byDevice;
    
       // Initialize the Duty Cycle table
	for( byDevice = 0; byDevice < NUM_DEVICES; byDevice++ )
	{
		ADC_CoilCurrentInit(byDevice);
	}

}

//------------------------------------------------------------------------------
WORD WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT eVoltageReport )
{
    // Read the board power supply voltage

    DWORD dwPowerSupplyVoltage;

    dwPowerSupplyVoltage = AdcRead( ADC_CHANNEL_VIN_SENSE );

    if( eVoltageReport != VOLTAGE_REPORT_RAW_ADC_COUNTS )
    {
        // Calibrate the ADC reading
        dwPowerSupplyVoltage *= pCh0RunTimeParams->wInputVoltageCalibration;
        dwPowerSupplyVoltage >>= NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS;
    }

    if( eVoltageReport == VOLTAGE_REPORT_ADC_MILLIVOLTS )
    {
        // ( Calculate in two steps for best resolution )
        dwPowerSupplyVoltage *= MICROVOLTS_PER_ADC_COUNT;
        dwPowerSupplyVoltage /= 1000;
    }
    else if( eVoltageReport == VOLTAGE_REPORT_ACTUAL_MILLIVOLTS )
    {
        // ( Calculate in two steps for best resolution )
        dwPowerSupplyVoltage *= VIN_SENSE_MICROVOLTS_PER_ADC_COUNT;
        dwPowerSupplyVoltage /= 1000;
    }

    return( ( WORD ) dwPowerSupplyVoltage );
}

//------------------------------------------------------------------------------
/* Convert input voltage Adc raw data to milli-volts.
 * Input: 
 *      adc: adc raw data.
 * Output:
 *      voltage in mV.      
 */
WORD WCT_ConvertInVolAdcToMilliVolts(WORD adc)
{
    DWORD dwActual;

    dwActual = (DWORD) adc;
    dwActual *= VIN_SENSE_MICROVOLTS_PER_ADC_COUNT;
    dwActual /= 1000;
    return (WORD) dwActual;
}

//------------------------------------------------------------------------------
/* Convert input current adc raw data to RMS milli-amps.
 * Input: 
 *      adc: adc raw data.
 * Output:
 *      current in mA.      
 */
WORD WCT_ConvertInCurrAdcToRmsMilliAmps(WORD adc)
{
    DWORD dwActual;

    dwActual = (DWORD) adc;
    dwActual *= INPUT_I_MICROAMPS_PER_ADC_COUNT;
    dwActual /= 1000;
    return (WORD) dwActual;
}

//------------------------------------------------------------------------------
WORD WCT_AnalogIoGetCoilCurrent( BYTE byCoilId, CURRENT_REPORT eCurrentReport )
{
    DWORD dwRtn = 0;

    if( byCoilId < NUM_COILS )
    {
        // Valid coil request
        dwRtn = AdcRead( (ADC_CHANNEL_ID) ( wAdcCoilCurrentIndex[ byCoilId ] ) );

        if( eCurrentReport != CURRENT_REPORT_RAW_ADC_COUNTS )
        {
            // Calibrate the ADC reading
            dwRtn *= pCh0RunTimeParams->wCoilCurrentCalibration;
            dwRtn >>= NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS;
        }
    }

    if( eCurrentReport == CURRENT_REPORT_ADC_MILLIVOLTS )
    {
        // Report the mV reading at the ADC pin
        // ( Calculate in two steps for best resolution )
        dwRtn *= MICROVOLTS_PER_ADC_COUNT; 
        dwRtn /= 1000; 
    }
    else if( eCurrentReport == CURRENT_REPORT_PEAK_MILLIAMPS )
    {
        // Report the calculated RMS current in mA
        // ( Calculate in two steps for best resolution )
        dwRtn *= COIL_I_PEAK_MICROAMPS_PER_ADC_COUNT;
        dwRtn /= 1000;
    }
    else if( eCurrentReport == CURRENT_REPORT_RMS_MILLIAMPS )
    {
        // Report the calculated RMS current in mA
        // ( Calculate in two steps for best resolution )
        dwRtn *= COIL_I_RMS_MICROAMPS_PER_ADC_COUNT;
        dwRtn /= 1000;
    }
 
    return ( ( WORD ) dwRtn );
}

/* Get the valley value of AC coil current samples for the RMS value calculation. 
 * wCoilCurrent = (DC-min)*calibrarionCOeff, so calibrated min value = DC*coeff-wCoilCurrent
 */
WORD AnalogIoGetCoilCurrentLowValue (BYTE byCoilId, WORD wCoilCurrent)
{
    DWORD dwRtn = 0;

    if( byCoilId < NUM_COILS )
    {
        dwRtn = (DWORD) gADCInitValue[byCoilId];
        dwRtn *= pCh0RunTimeParams->wCoilCurrentCalibration;
        dwRtn >>= NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS;
        dwRtn -= wCoilCurrent;
    }

    return ( ( WORD ) dwRtn );    
}

//------------------------------------------------------------------------------
WORD WCT_AnalogIoGetInputCurrent( BYTE byCoilId, CURRENT_REPORT eCurrentReport )
{
    DWORD dwRtn = 0;

    if( byCoilId < NUM_COILS )
    {
        // Valid coil request
        dwRtn = AdcRead( (ADC_CHANNEL_ID) ( wAdcInputCurrentIndex[ byCoilId ] ) );

        if( eCurrentReport != CURRENT_REPORT_ADC_MILLIVOLTS )
        {
            // Remove calibration offset
            if( dwRtn > wAdcInputCurrentOffset[ byCoilId ] )
                dwRtn -= wAdcInputCurrentOffset[ byCoilId ];
            else
                dwRtn = 0;
        }

        if( eCurrentReport != CURRENT_REPORT_RAW_ADC_COUNTS )
        {
            // Calibrate the ADC reading
            dwRtn *= pCh0RunTimeParams->wInputCurrentCalibration;
            dwRtn >>= NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS;
        }
    }

    if( eCurrentReport == CURRENT_REPORT_ADC_COUNTS )
    {
        // Just report raw counts
    }
    else if( eCurrentReport == CURRENT_REPORT_ADC_MILLIVOLTS )
    {
        // Report the mV reading at the ADC pin
        // ( Calculate in two steps for best resolution )
        dwRtn *= MICROVOLTS_PER_ADC_COUNT; 
        dwRtn /= 1000; 
    }
    else if( eCurrentReport == CURRENT_REPORT_PEAK_MILLIAMPS )
    {
        // DC Channel - Report the current in mA
        // ( Calculate in two steps for best resolution )
        dwRtn *= INPUT_I_MICROAMPS_PER_ADC_COUNT;
        dwRtn /= 1000;
    }
    else if( eCurrentReport == CURRENT_REPORT_RMS_MILLIAMPS )
    {
        // DC Channel - Report the current in mA
        // ( Calculate in two steps for best resolution )
        dwRtn *= INPUT_I_MICROAMPS_PER_ADC_COUNT;
        dwRtn /= 1000;
    }

    return ( ( WORD ) dwRtn );
}

WORD wTemperatureCalibration=0x8000;    // TODO: move to eeprom params
//------------------------------------------------------------------------------
WORD WCT_AnalogIoGetBoardTemperature( TEMPERATURE_REPORT eTemperatureReport )
{
    // Read the board power supply voltage

    DWORD dwRtn;
    SWORD Tempe;

    dwRtn = AdcRead( ADC_CHANNEL_BOARD_TEMPERATURE );

    if( eTemperatureReport != TEMPERATURE_REPORT_RAW_ADC_COUNTS )
    {
        // Calibrate the ADC reading
        dwRtn *= wTemperatureCalibration;
        dwRtn >>= NOM_CAL_SCALE_FACTOR_SIZE_IN_BITS;
    }


    if( eTemperatureReport == TEMPERATURE_REPORT_ADC_MILLIVOLTS )
    {
        // ( Calculate in two steps for best resolution )
        dwRtn *= MICROVOLTS_PER_ADC_COUNT;
        dwRtn /= 1000;
    }
    else if( eTemperatureReport == TEMPERATURE_REPORT_ACTUAL_CENTIGRADE)
    {
        dwRtn = (DWORD)WCT_ConvertAdcToTempe(dwRtn);
    }


    return (WORD) dwRtn;
}

void WCT_CalcInputPower(RUNTIME_PARAMS *pRunTimeParams, DWORD dwSampledInputVoltageCounts, DWORD dwSampledInputCurrentCounts, DWORD dwSampledCoilCurrentCounts)
{
	FOD_CHARACTERIZATION_PARAMS* pPwrLossCharParams = &NvmParams.CalParams.PowerLossParams[ pRunTimeParams->byDeviceId ].FodCharacterizationParams[ pRunTimeParams->byCoilIdIndex ];
    SDWORD sdwTemp;
    DWORD dwTemp;
    DWORD dwCoilCurrSqr;
    WORD wQuadraticFinalDivisorExponent;

    // Calculate the input voltage (in mV)
    dwTemp = dwSampledInputVoltageCounts;
    dwTemp *= INPUT_VOLTAGE_MAX_RANGE;
    dwTemp >>= ADC_BITS;
    pRunTimeParams->wMeasuredInputVoltage = ( WORD ) dwTemp;

    // Calculate the input current (in mA)
    dwTemp = dwSampledInputCurrentCounts;
    dwTemp *= INPUT_CURRENT_MAX_RANGE;
    dwTemp >>= ADC_BITS;
    pRunTimeParams->dwMeasuredInputCurrent = dwTemp;

    // Calculate coil current (in mA)
    dwTemp = dwSampledCoilCurrentCounts;
    dwTemp *= COIL_CURRENT_RMS_MAX_RANGE; //COIL_CURRENT_MAX_RANGE;
    dwTemp >>= ADC_BITS;
    dwTemp = ISENSE_CONSTANT_RMS-dwTemp;
    pRunTimeParams->wMeasuredCoilCurrent = ( WORD ) dwTemp;
    dwCoilCurrSqr= dwTemp*dwTemp;
    dwCoilCurrSqr>>= QUADRATIC_PREDIVISOR_EXPONENT;

    // Calculate Input Power (in mW)
    dwTemp = dwSampledInputVoltageCounts;
    dwTemp *= dwSampledInputCurrentCounts;
    dwTemp >>= ADC_16BIT_SHIFT_VALUE;
    dwTemp *= SCALED_POWER_PER_ADC_COUNTS_16BIT;
    dwTemp >>= INPUT_POWER_16BIT_SHIFT_VALUE;
    pRunTimeParams->dwCalculatedInputPower = dwTemp;


    // Calculate the power losses in the Transmitter
    // Use the following equation that defines the Transmitter Losses:
    //
    //  Ptxlosses = ( C5*ip^2 + C6*ip + C7 )
    //    where:
    //           ip = peak coil current
    //

    // Calculate the final divisor exponent for the quadratic equation
    wQuadraticFinalDivisorExponent = ( pPwrLossCharParams->wQuadExponent - QUADRATIC_PREDIVISOR_EXPONENT );

    // Calculate the first term of the TxLosses: (C5)*(Itxcoil)^2
    sdwTemp = dwCoilCurrSqr;
    sdwTemp *= pPwrLossCharParams->swQuadCoefficient;
    sdwTemp >>= wQuadraticFinalDivisorExponent;                     // Yields final quadratic loss in mW
    pRunTimeParams->swQuadTerm = (SWORD)sdwTemp;
    pRunTimeParams->swTxInternalLoss = (SWORD)sdwTemp;              // Save first part of TX losses
	
}

void WCT_GetAveragedInputPower(RUNTIME_PARAMS *pRunTimeParams)
{
    DWORD dwTemp;

    dwTemp = pRunTimeParams->wAveragedInputVoltage;
    dwTemp *= pRunTimeParams->wAveragedInputCurrent;
    dwTemp /= 1000;
    
    pRunTimeParams->dwAveragedInputPower = dwTemp;
}

//------------------------------------------------------------------------
WORD WCT_GetSampledPowerSupplyVoltage( WORD wRealVoltage )
{
	DWORD dwTemp;
	dwTemp = (DWORD)wRealVoltage;
	dwTemp <<= ADC_BITS;
	dwTemp /= INPUT_VOLTAGE_MAX_RANGE;
	return( DWORD ) dwTemp;
}

void TMRA_Disable(BYTE byDevice)
{	
	switch(byDevice)
	{
	case 0:
		EnterCritical();
		TMR_0_ENBL &= ~0x01U;    /* Disable TMRA0 */ 
		TMR_0_CNTR = 0;
		TMR_0_CSCTRL &= ~TMR_0_CSCTRL_TCF1;  //Clear TMRA0 interrupt flag
		ExitCritical();
		break;
	}
}

void TMRA_Enable(BYTE byDevice)
{
	switch(byDevice)
	{
	case 0:
		TMR_0_CNTR = 0;;        /* Set the Counter register */ 
		TMR_0_ENBL |= 0x01U;    /* Enable timer channel */
		break;
	}
}

void TMRA_SetCompareValue(BYTE byDevice, WORD wValue)
{
	switch(byDevice)
	{
	case 0:
		TMR_0_COMP1 = wValue;       /* Set the Compare register 1 */ 
		TMR_0_CMPLD1 = wValue;      /* Set the Compare register 1 */ 
		break;
	}
}

/*
 * PWM0 is allocated to DEVICE 0
 * byPos is a percent value for whole PWM period
 */
void PWM_SetTriggerPos(BYTE byDevice, BYTE byPos)
{
	DWORD tmp;

    switch(byDevice)
	{
	case 0:
        // Ensure LDOK bit is clear before updating registers
		if(PWMA_MCTRL & PWMA_MCTRL_RUN_0)
		{
	    	while(PWMA_MCTRL&PWMA_MCTRL_LDOK_0) {};
		}
		else
		{
			PWMA_MCTRL |= PWMA_MCTRL_CLDOK_0;
		}
		tmp = PWMA_SM0VAL1;
		tmp = tmp * byPos / 100;
		PWMA_SM0VAL0 = tmp;
		PWMA_MCTRL |= PWMA_MCTRL_LDOK_0;
		break;
	}
}

/*
 * ADC Input B2 is allocated to sample coil current waveform, B2 is allocated to sample8
 * When the ADC sample is triggered by PWM, we need to jump the first ADC sample result,
 * since the PWM trigger signal position maybe need one PWM cycle to update.
 * When the ADC sample is not triggered by PWM, we need to start ADC sample manually,
 */
WORD ADC_ManualConversion(BYTE byDevice, WORD wAvgCnt, BYTE byIsTrigByPwm)
{
	WORD i=0;
	DWORD sum=0;
	switch(byDevice)
	{
	case 0:		
		if(byIsTrigByPwm)
		{
			ADC_STAT |= ADC_STAT_EOSI1;
			while(!(ADC_STAT&ADC_STAT_EOSI1)) {};
		}
		ADC_STAT |= ADC_STAT_EOSI1;
		
		while(i < wAvgCnt)
		{
			if(!byIsTrigByPwm)
				ADC_CTRL2 |= ADC_CTRL2_START1;
			while(!(ADC_STAT&ADC_STAT_EOSI1)) {};
			ADC_STAT |= ADC_STAT_EOSI1;
			sum += (ADC_RSLT8 >> 3);
			i++;
		}
		break;
	}
	
	return sum/wAvgCnt;
}

/*
 * To find the valley of the coil current waveform, we assume the valley will not change when we locating its position
 */

WORD DDM_SetBestTriggerPos(BYTE byDevice, BYTE byDiv,RUNTIME_PARAMS *pRunTimeParams)
{
	BYTE i, pos = 25;
	WORD result,  min = 65535;
	BYTE triggernum;
	WORD adcdelayduty;

	adcdelayduty=(25*(pRunTimeParams->dwActiveFreq/1000))/10;		//ADC delay  2.5*100ns/(1/175000)
	adcdelayduty=(adcdelayduty+50)/100;		//round

	//if duty cycle is small, change the trigger number based on current duty
	if(pRunTimeParams->wActiveDutyCycle==pRunTimeParams->wMaxDutyCycle)
		triggernum=30;
	else
	{
		triggernum=(pRunTimeParams->wActiveDutyCycle/100)-1;
		triggernum-=(BYTE)adcdelayduty;
		if(triggernum>=30) triggernum=30;
	}
	//The valley will be located in 0~25 percent of PWM period actually, we will find it from 0~30 percent for safety.
	for(i=0; i<triggernum; i++)
	{
		PWM_SetTriggerPos(byDevice, i);
		result = ADC_ManualConversion(byDevice, 3, 1); 
		if(min > result)
		{
			min = result;
			pos = i;
		}
	}

	//Make sue the byDiv is a valid value for next compute
	if((byDiv == 0) || (byDiv > 2))
	{
		byDiv = 1;
	}
	
	if(byDiv == 2)
	{
		pos = 25+(pos>>1);
	}
	
	//The final trigger position will be valley value divide byDiv. 
	PWM_SetTriggerPos(byDevice, pos);

	/*
	 * gADCInitValue is an DC offset value, initialised at system bring up
	 */
	return gADCInitValue[byDevice] - min;
}


/*
 * Stop DMA module, stop the DMA interrupt
 */
void DDM_Stop(BYTE byDevice)
{	
	TMRA_Disable(byDevice);
	DMA_Stop(byDevice);
}

/*
 * Re-config and start DMA module, we will get an interrupt after every RECORD_BUF_LEN/2/2 DMA transfer.
 * Since the DMA module itself can't issue an interrupt after every RECORD_BUF_LEN/2/2 DMA transfer, 
 * we use a timer which use same PWM trigger pulse with ADC module, the timer will issue an interrupt after
 * every RECORD_BUF_LEN/2/2 PWM trigger pulse.
 * PWM trigger pulse -> ADC sample -> DMA transfer
 * PWM trigger pulse -> Timer count
 */
void DDM_Start(BYTE byDevice)
{
    DMA_Config(byDevice);
	TMRA_SetCompareValue(byDevice, RECORD_BUF_LEN/2/2 - 1);
	//it should not be interrupted here
	EnterCritical();
	DMA_Start(byDevice);
	TMRA_Enable(byDevice);
	ExitCritical();
}

//------------------------------------------------------------------------------
//  Fault protection callbacks
//------------------------------------------------------------------------------
/* Over temperature protection.
 * Input: 
 *      adc: adc raw data.
 * Output:
 * Return: 
 *      TRUE: temperatue exceeds the limit, FALSE: temperatue withim the limit.
 */
BOOL WCT_OverTemperatureProtection(RUNTIME_PARAMS *pRunTimeParams)
{
#if OVER_TEMPERATURE_PROTECTION_SUPPORT
    if( pRunTimeParams->wAveragedTempe >= pRunTimeParams->bOverTemperatureLimit)
        return TRUE;
    else
        return FALSE;
#else
        return FALSE;
#endif
}

//------------------------------------------------------------------------------
//  Debug Console callbacks
//------------------------------------------------------------------------------

void WCT_DebugPutChar( BYTE byChar )
{
#if DEBUG_CONSOLE_QSCI0 && QSCI0_ENABLED
	Qsci0PutChar( byChar );
#elif DEBUG_CONSOLE_QSCI1 && QSCI1_ENABLED
	Qsci1PutChar( byChar );
#endif
}

void WCT_DebugPutString( char *pbyStr )
{
    // Write out the string to the QSCI0

    while( ( *pbyStr ) != 0x00 )
    {
        WCT_DebugPutChar( ( BYTE ) *pbyStr );
        pbyStr++;
    }
}

void WCT_DebugFlushBuffer( void )
{
#if DEBUG_CONSOLE_QSCI0 && QSCI0_ENABLED
	Qsci0FlushBuffer();
#elif DEBUG_CONSOLE_QSCI1 && QSCI1_ENABLED
	Qsci1FlushBuffer();
#endif

}

#if _DEBUG
//----------------------- debug module defines -------------------------
void clear_TP1(void)
{   
    CLEAR_TP1;
}
void clear_TP2(void)
{   
    CLEAR_TP2;
}
void set_TP1(void)
{   
    SET_TP1;
}
void set_TP2(void)
{   
    SET_TP2;
}
void toggle_TP1(void)
{   
    TOGGLE_TP1;
}
void toggle_TP2(void)
{   
    TOGGLE_TP2;
}

void toggle_TP3(void)
{   
    TOGGLE_TP3;
}

#endif
