/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
#ifndef __WCT_HAL_H__
#define __WCT_HAL_H__

//----------
// INCLUDES
//----------
#include "defines.h"
#include "wct_lib.h"
#include "wct_libconst.h"
#include "wct_libstruct.h"
#include "wct_systemDataWPC.h"
//-----------
// FUNCTIONS
//-----------


/*****************************************************************************/
/*! @addtogroup MPTX_LibTimerCallback MP TX Lib Timer Callbacks
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        Clean the Watchdog
 * @details      This function is implemented in the application to clean 
 *               the Watchog counter by the library. 
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_KickTheWatchdog( void );

/*****************************************************************************
 *//*!
 * @brief        Tick Timer initialization
 * @details      This function is implemented in the application to initialize the Tick Timer.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_TickTimerInit(void);
/*****************************************************************************
 *//*!
 * @brief        Enable or disables the Tick Timer Isr 
 * @details      This function must be implemented in the application and is called 
 *               to enable or disable the interrupt of the Tick Timer.
 * @param[in]    state  WCT_ENABLE - enable Tick Timer interrupt.
 *                      WCT_DISABLE - disable Tick Timer interrupt.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_TickTimerInt( BYTE state );
/*! @} End of MPTX_LibTimerCallback */



/*****************************************************************************/
/*! @addtogroup MPTX_LibCoilCallback MP TX Lib Coil Callbacks
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        Coil initialization.
 * @details      This function is implemented in the application to initialize the coils.
 *               This callback must initialize also the PWM outputs.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_CoilInit( void );
/*****************************************************************************
 *//*!
 * @brief        Set Coil Frequency.
 * @details      This function is implemented in the application to set the frequency 
 *               of the coils switch. 
 * @param[in]    byCoilId  ID of coil.
 * @param[in]    dwFreqValue  Desired frequency.
 * 
 * @return       Result of callback:
 *                - RET_VALUE_GOOD
 *                - RET_VALUE_BAD
 *                - RET_VALUE_INVALID_PARAM
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
BYTE WCT_CoilFreqSet( BYTE byCoilId, DWORD dwFreqValue );
/*****************************************************************************
 *//*!
 * @brief        Set a Coil switch duty cycle and Frequency.
 * @details      This function is implemented in the application to set the duty cycle 
 *               of the coils switch. 
 * @param[in]    byCoilId  ID of coil.
 * @param[in]    wDutyCyclePercent  Desired duty cycle.
 * 
 * @return       Result of callback:
 *                - RET_VALUE_GOOD
 *                - RET_VALUE_BAD
 *                - RET_VALUE_INVALID_PARAM
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
BYTE WCT_CoilDutyCycleSet( BYTE byCoilId, WORD wDutyCyclePercent );
/*****************************************************************************
 *//*!
 * @brief        Connect or disconnect the coil.
 * @details      This function is implemented in the application to connect or disconnect 
 *               the coils switch.
 * @param[in]    byCoilId  ID of coil.
 * @param[in]    bState    WCT_ENABLE - Connect the coil.
 *                         WCT_DISABLE - disconnect the coil.
 * 
 * @return       Result of callback:
 *                - RET_VALUE_GOOD
 *                - RET_VALUE_BAD
 *                - RET_VALUE_INVALID_PARAM
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
BYTE WCT_CoilControl( BYTE byCoilId, BOOL bState );
/*****************************************************************************
 *//*!
 * @brief        Send Analog Ping
 * @details      This function is implemented in the application to send the analog ping. 
 * @param[in]    byCoilId  ID of coil
 * @param[in]    byPulseDuration  Analog ping duration
 * @param[in]    byAdcSampleTime  ADC sample time
 * 
 * @return       Result of callback:
 *                - RET_VALUE_GOOD
 *                - RET_VALUE_BAD
 *                - RET_VALUE_INVALID_PARAM
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_CoilAnalogPing( BYTE byCoilId, BYTE byPulseDuration, BYTE byAdcSampleTime );
/*****************************************************************************
 *//*!
 * @brief        Discharge the coil
 * @details      This function is implemented in the application to discharge the coil. 
 * @param[in]    byCoilId  ID of coil
 * @param[in]    bState  bState
 * 
 * @return       Result of callback:
 *                - RET_VALUE_GOOD
 *                - RET_VALUE_BAD
 *                - RET_VALUE_INVALID_PARAM
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
BYTE WCT_CoilDischarge( BYTE byCoilId, BOOL bState );
/*****************************************************************************
 *//*!
 * @brief        Check of HW overcurrent status.
 * @details      This function is implemented in the application to check HW Overcurrent status.
 * @param[in]    byCoilId  ID of coil
 * 
 * @return       Result of callback:
 *                - TRUE
 *                - FALSE
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      31-Jan-2012
 ******************************************************************************/
BOOL WCT_CoilCheckHardwareOvercurrentStatus( BYTE byCoilId );
/*! @} End of MPTX_LibCoilCallback */



/*****************************************************************************/
/*! @addtogroup MPTX_LibConsoleCallback MP TX Lib Console Callbacks
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        Put char to console.
 * @details      This function is implemented in the application to put the char to the console. 
 * @param[in]    byChar  Character to send
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_DebugPutChar( BYTE byChar );
/*****************************************************************************
 *//*!
 * @brief        Put string to console.
 * @details      This function is implemented in the application to put the string to the console. 
 * @param[in]    pbyStr  String to send
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_DebugPutString( char *pbyStr );
/*****************************************************************************
 *//*!
 * @brief        Flush characters in TX buffer.
 * @details      This function is implemented in the application to flush the TX buffer. 
 * 
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Feb-2013
 ******************************************************************************/
void WCT_DebugFlushBuffer( void );

/*! @} End of MPTX_LibConsoleCallback */
/*****************************************************************************/
/*! @addtogroup MPTX_LibADCCallback MP TX Lib ADC Callbacks
 * @{*/
typedef enum
{
    VOLTAGE_REPORT_ADC_COUNTS = 0,
    VOLTAGE_REPORT_ADC_MILLIVOLTS,
    VOLTAGE_REPORT_ACTUAL_MILLIVOLTS,
    VOLTAGE_REPORT_RAW_ADC_COUNTS

} VOLTAGE_REPORT;

typedef enum
{
    CURRENT_REPORT_ADC_COUNTS = 0,
    CURRENT_REPORT_ADC_MILLIVOLTS,
    CURRENT_REPORT_PEAK_MILLIAMPS,
    CURRENT_REPORT_RMS_MILLIAMPS,
    CURRENT_REPORT_RAW_ADC_COUNTS

} CURRENT_REPORT;

typedef enum
{
    TEMPERATURE_REPORT_ADC_COUNTS = 0,
    TEMPERATURE_REPORT_ADC_MILLIVOLTS,
    TEMPERATURE_REPORT_TEMPE_RESISTANCE,
    TEMPERATURE_REPORT_ACTUAL_CENTIGRADE,
    TEMPERATURE_REPORT_RAW_ADC_COUNTS

} TEMPERATURE_REPORT;

/*****************************************************************************
 *//*!
 * @brief        Initialize Analog layer.
 * @details      This function is implemented in the application to initialize the calibration constants.
 * @param[in]    pRunTimeParams Run Time Parameters for desired coil.
 * @return       Power supply voltage.

 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Jan-2013
 ******************************************************************************/
void WCT_AnalogIoInit( RUNTIME_PARAMS* pRunTimeParams );            // Read the board power supply voltage
/*****************************************************************************
 *//*!
 * @brief        Get Power Supply Voltage.
 * @details      This function is implemented in the application to read the power supply voltage. 
 * @param[in]    eVoltageReport  Output format. see more in VOLTAGE_REPORT_xxx.
 * @return       Power supply voltage.
 *
 * @note         This function is required by the MP 30W library
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_AnalogIoGetPowerSupplyVoltage( VOLTAGE_REPORT eVoltageReport );            // Read the board power supply voltage
/*****************************************************************************
 *//*!
 * @brief        Get Coil Current.
 * @details      This function is implemented in the application to read the coil current. 
 * @param[in]    byCoilId        ID of coil
 * @param[in]    eCurrentReport  Output format. see more in CURRENT_REPORT_xxx.
 * @return       Coil current.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_GetSampledPowerSupplyVoltage( WORD wRealVoltage );                          // convert to sampled board power supply voltage
/*****************************************************************************
 *//*!
 * @brief        Get sampled power supply voltage.
 * @details      This function is to convert a real voltage to ADC value. 
 * @param[in]    wRealvoltage
 * @return       ADC power supply voltage value.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      28-Feb-2014
 ******************************************************************************/
WORD WCT_AnalogIoGetCoilCurrent( BYTE byCoilId, CURRENT_REPORT eCurrentReport );    // Read the designated coil current
/*****************************************************************************
 *//*!
 * @brief        Get Coil Current low value.
 * @details      This function is implemented in the application to read the coil current. 
 * @param[in]    byCoilId        ID of coil
 * @param[in]    eCurrentReport  Output format. see more in CURRENT_REPORT_xxx.
 * @return       Coil current.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD AnalogIoGetCoilCurrentLowValue (BYTE byCoilId, WORD wCoilCurrent);
/*****************************************************************************
 *//*!
 * @brief        Get Input Current.
 * @details      This function is implemented in the application to read the input current. 
 * @param[in]    byCoilId        ID of coil
 * @param[in]    eCurrentReport  Output format. see more in CURRENT_REPORT_xxx.
 * @return       Input current.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_AnalogIoGetInputCurrent( BYTE byCoilId, CURRENT_REPORT eCurrentReport );   // Read the designated input current

/*****************************************************************************
 *//*!
 * @brief        Get Board Temperature.
 * @details      This function is implemented in the application to read the board temperature. 
 * @param[in]    eTemperatureReport  Output format. see more in TEMPERATURE_REPORT.
 * @return       Board temperature.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      10-Sep-2013
 ******************************************************************************/
WORD WCT_AnalogIoGetBoardTemperature( TEMPERATURE_REPORT eTemperatureReport );

/*****************************************************************************
 *//*!
 * @brief        Get temperature value from the resistance of thermistor
 * @details      This function is implemented in the application to read board temperature. 
 * @param[in]  adc value.
 * @return       Actual temperature.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      10-Sep-2013
 ******************************************************************************/
SBYTE WCT_ConvertAdcToTempe(DWORD adc);

/*****************************************************************************
 *//*!
 * @brief        Input Current Calibration.
 * @details      This function is implemented in the application to perform the input current calibration. 
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_PerformInputCurrentCalibration( void );

/*****************************************************************************
 *//*!
 * @brief        Coil Current Calibration.
 * @details      This function is implemented in the application to perform the coil current calibration. 
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_PerformCoilCurrentCalibration( void );

/*****************************************************************************
 *//*!
 * @brief        Trigger ADC conversion
 * @details      This function is implemented in the application to trigger 
 *               the ADC conversion by the library.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_EnableAdcConversions( void );

/*****************************************************************************
 *//*!
 * @brief        Calculate Input Power.
 * @details      This function is implemented in the application to calculate 
 *               the actual input power.
 * @param[in]    pRunTimeParams Run Time Parameters for desired coil.
 * @param[in]    dwSampledInputVoltageCounts Sampled Input Voltage in raw format.
 * @param[in]    dwSampledInputCurrentCounts Sampled Input Current in raw format.
 * @param[in]    dwSampledCoilCurrentCounts Sampled Coil Current in raw format.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      31-Jan-2012
 ******************************************************************************/
void WCT_CalcInputPower(RUNTIME_PARAMS *pRunTimeParams, DWORD dwSampledInputVoltageCounts, DWORD dwSampledInputCurrentCounts, DWORD dwSampledCoilCurrentCounts);

void WCT_GetAveragedInputPower(RUNTIME_PARAMS *pRunTimeParams);

void DDM_Stop(BYTE byCoilId);
void DDM_Start(BYTE byCoilId);
WORD DDM_SetBestTriggerPos(BYTE byDevice, BYTE byDiv,RUNTIME_PARAMS *pRunTimeParams);

void AdcWriteChannel(BYTE byChannel, WORD wData);

void PWM_SetTriggerPos(BYTE byDevice, BYTE byPos);
WORD ADC_ManualConversion(BYTE byDevice, WORD wAvgCnt, BYTE byIsTrigByPwm);

void PwmStartupControl(BYTE byCoilId);

WORD WCT_ConvertInVolAdcToMilliVolts(WORD adc);
WORD WCT_ConvertInCurrAdcToRmsMilliAmps(WORD adc);
#if ADC_SAMPLE_AVERAGE_API
void WCT_AdcSampleAverage(T_ADC_SAMPLE_AVERAGE *pAdcAverage, WORD sample);
#endif

BOOL WCT_OverTemperatureProtection(RUNTIME_PARAMS *pRunTimeParams);

#if PRINT_CUSTOMER_MESSAGE && QSCI0_ENABLED
void message_print(BYTE byDevice, DEVICE_OP_STATE state);
#endif

void clear_TP1(void);
void clear_TP2(void);
void set_TP1(void);
void set_TP2(void);
void toggle_TP1(void);
void toggle_TP2(void);
void toggle_TP3(void);

/*! @} End of MPTX_LibADCCallback */

/*! @} End of MPTX_LibDisplayCallback */
//----------
// TYPEDEFS
//----------



#endif  //__WCT_HAL_H__
