/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
#ifndef __WCT_COMM_PACKETS_WPC_H__

    #define __WCT_COMM_PACKETS_WPC_H__

    //----------
    // INCLUDES
    //----------


    //---------
    // DEFINES
    //---------

    // Specification Revision Definitions
    #define SPECIFICATION_MAJOR_VERSION             1
    #define SPECIFICATION_MINOR_VERSION             1

    // Communication definitions
    #define MAX_PACKET_SIZE                         30          // packet type 0xFF has header, payload of 27 bytes and checksum
    #define SIGNAL_STRENGTH_NUM_BYTES               3           // header + 1 payload byte + checksum
    #define ID_PACKET_NUM_BYTES                     9           // header + 7 payload bytes + checksum
    #define EXT_ID_PACKET_NUM_BYTES                 10          // header + 8 payload bytes + checksum
    #define POWER_CONTROL_HOLDOFF_PACKET_NUM_BYTES  3           // header + 1 payload bytes + checksum
    #define CONFIG_PACKET_NUM_BYTES                 7           // header + 5 payload bytes + checksum

    #define NOMINAL_COMMUNICATIONS_RATE             2000        // 2Kbps communications rate
    #define PERMISSIBLE_CLOCK_TOLERANCE_PERCENT     20          // +/-20% clock tolerance between primary and secondary devices
    #define NUM_BITS_PER_BYTE_TRANSMITTED           11          // 8 data bits, 1 Start, 1 Stop and 1 Parity bit
    #define MIN_PREAMBLE_LENGTH                     11
    #define MAX_PREAMBLE_LENGTH                     25
    #define MIN_PREAMBLE_DETECTION_COUNT            4

    // Calculate nominal packet transmission times
    #define NOMINAL_DATA_BIT_TIME                   ( 1.0 / NOMINAL_COMMUNICATIONS_RATE )
    #define SIGNAL_STRENGTH_TRANSMIT_TIME_MS        ( SIGNAL_STRENGTH_NUM_BYTES * NUM_BITS_PER_BYTE_TRANSMITTED * NOMINAL_DATA_BIT_TIME * 1000 )
    #define ID_PACKET_TRANSMIT_TIME_MS              ( ID_PACKET_NUM_BYTES * NUM_BITS_PER_BYTE_TRANSMITTED * NOMINAL_DATA_BIT_TIME * 1000 )
    #define EXT_ID_PACKET_TRANSMIT_TIME_MS          ( EXT_ID_PACKET_NUM_BYTES * NUM_BITS_PER_BYTE_TRANSMITTED * NOMINAL_DATA_BIT_TIME * 1000 )
    #define POWER_CONTROL_HOLDOFF_PACKET_TRANSMIT_TIME_MS   ( POWER_CONTROL_HOLDOFF_PACKET_NUM_BYTES * NUM_BITS_PER_BYTE_TRANSMITTED * NOMINAL_DATA_BIT_TIME * 1000 )
    #define CONFIG_PACKET_TRANSMIT_TIME_MS          ( CONFIG_PACKET_NUM_BYTES * NUM_BITS_PER_BYTE_TRANSMITTED * NOMINAL_DATA_BIT_TIME * 1000 )




    // -------------- Timing definitions from specification ---------------
    // NOTE: These definitions are provided as both floating point numbers
    //       (to accomodate calculations) and integer values representing
    //       milliseconds to accomodate precompile checks not capable of
    //       using logic containing equations.

    // Table 5-1
    #define MIN_DIGITAL_PING_WINDOW_MS              65          // Transmitter must detect START bit of first packet within this timeframe from start of ping
    #define MAX_DIGITAL_PING_WINDOW_MS              70
    #define MAX_POWER_SIGNAL_TERMINATION_TIME_MS    28          // Transmitter must remove power within this timeframe following a FIRST_PACKET_TIMEOUT
    #define MAX_FIRST_PACKET_TIMEOUT_MS             20          // Signal Strength packet should conclude within this time from START bit detection
    #define MAX_POWER_SIGNAL_EXPIRATION_TIME_MS     90          // Transmitter must remove power within this timeframe if not progressing to Identification & Configuration

    // Table 5-2
    #define MAX_NEXT_PACKET_TIMEOUT_MS              21 			
    #define MAX_PACKET_LENGTH_MS                    170

    // Table 5-3
    #define MIN_POWER_CONTROL_HOLDOFF_TIME_MS       5           // This is the MINIMUM hold-off time that may be reported by a Receiver
    #define MAX_POWER_CONTROL_HOLDOFF_TIME_MS       205         // This is the MAXIMUM hold-off time that may be reported by a Receiver

    // Table 5-4
    #define NOM_CONTROL_ERROR_PACKET_TIMEOUT_MS     1500
    #define MAX_CONTROL_ERROR_PACKET_TIMEOUT_MS     1800        // Transmitter must remove power if a CEP packet is not received before this timeout
    #define NOM_POWER_CONTROL_ACTIVE_TIME_MS        20
    #define MAX_POWER_CONTROL_ACTIVE_TIME_MS        21          // Represents the MAX amount of time the Transmitter may execute the PID loop
    #define MIN_POWER_CONTROL_SETTLING_TIME_MS      3
    #define NOM_POWER_CONTROL_SETTLING_TIME_MS      5           // Represents the amount of time the system uses for stabilization following PID adjustments
    #define MAX_POWER_CONTROL_SETTLING_TIME_MS      7
    #define NOM_RECEIVED_POWER_PACKET_TIMEOUT_MS    23000       // Transmitter must remove power if a Received Power packet is not seen before this timeout
    #define MAX_RECEIVED_POWER_PACKET_TIMEOUT_MS    24000
    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define NOM_RECTIFIED_POWER_PACKET_TIMEOUT_MS   35000       // Transmitter must remove power if a Rectified Power packet is not seen before this timeout
    #define MAX_RECTIFIED_POWER_PACKET_TIMEOUT_MS   36000

    // Table 5-5
    #define NOM_POWER_RECEIVER_RESET_TIME_MS        25
    #define MAX_POWER_RECEIVER_RESET_TIME_MS        28          // The Receiver must reset its operating state (return to Selection state) within this time

    // Table 5-6
    #define MIN_WAKEUP_TIME_MS                      19
    #define NOM_WAKEUP_TIME_MS                      40          // The Receiver must start the first packet within this much time after power application
    #define MAX_WAKEUP_TIME_MS                      64

    // Table 5-7
    #define MIN_SILENT_TIME_MS                      6           // The Receiver must leave at least this much time between packet transmissions
    #define NOM_SILENT_TIME_MS                      7

    // Table 5-8
    #define NOM_CEP_INTERVAL_MS                     250
    #define MAX_CEP_INTERVAL_MS                     350         // The Receiver must have no more than this delay between CEP packet transmissions (measured START bit to START bit)
    #define MIN_CONTROLLER_TIME_MS                  24          // The Receiver must leave at least this much time for Transmitter to make adjustments to the operating point
    #define NOM_CONTROLLER_TIME_MS                  25
    #define NOM_RX_RECEIVED_POWER_PACKET_TIME_MS    1500
    #define MAX_RX_RECEIVED_POWER_PACKET_TIME_MS    4000
    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define NOM_RX_RECTIFIED_POWER_PACKET_TIME_MS   5000
    #define MAX_RX_RECTIFIED_POWER_PACKET_TIME_MS   6000        // The Receiver must have no more than this delay between Rectified Power packet transmissions (measured START bit to START bit)


    // ------------- Conversion of timing definitions to floating point ---------------

    #define MIN_DIGITAL_PING_WINDOW                 ( MIN_DIGITAL_PING_WINDOW_MS / 1000.0 )
    #define MAX_DIGITAL_PING_WINDOW                 ( MAX_DIGITAL_PING_WINDOW_MS / 1000.0 )
    #define MAX_POWER_SIGNAL_TERMINATION_TIME       ( MAX_POWER_SIGNAL_TERMINATION_TIME_MS / 1000.0 ) 
    #define MAX_FIRST_PACKET_TIMEOUT                ( MAX_FIRST_PACKET_TIMEOUT_MS / 1000.0 ) 
    #define MAX_POWER_SIGNAL_EXPIRATION_TIME        ( MAX_POWER_SIGNAL_EXPIRATION_TIME_MS / 1000.0 ) 

    #define MAX_NEXT_PACKET_TIMEOUT                 ( MAX_NEXT_PACKET_TIMEOUT_MS / 1000.0 )
    #define MAX_PACKET_LENGTH                       ( MAX_PACKET_LENGTH_MS / 1000.0 )

    #define MIN_POWER_CONTROL_HOLDOFF_TIME          ( MIN_POWER_CONTROL_HOLDOFF_TIME_MS / 1000.0 )
    #define MAX_POWER_CONTROL_HOLDOFF_TIME          ( MAX_POWER_CONTROL_HOLDOFF_TIME_MS / 1000.0 )

    #define NOM_CONTROL_ERROR_PACKET_TIMEOUT        ( NOM_CONTROL_ERROR_PACKET_TIMEOUT_MS / 1000.0 )
    #define MAX_CONTROL_ERROR_PACKET_TIMEOUT        ( MAX_CONTROL_ERROR_PACKET_TIMEOUT_MS / 1000.0 )
    #define NOM_POWER_CONTROL_ACTIVE_TIME           ( NOM_POWER_CONTROL_ACTIVE_TIME_MS / 1000.0 )
    #define MAX_POWER_CONTROL_ACTIVE_TIME           ( MAX_POWER_CONTROL_ACTIVE_TIME_MS / 1000.0 )
    #define MIN_POWER_CONTROL_SETTLING_TIME         ( MIN_POWER_CONTROL_SETTLING_TIME_MS / 1000.0 )
    #define NOM_POWER_CONTROL_SETTLING_TIME         ( NOM_POWER_CONTROL_SETTLING_TIME_MS / 1000.0 )
    #define MAX_POWER_CONTROL_SETTLING_TIME         ( MAX_POWER_CONTROL_SETTLING_TIME_MS / 1000.0 )
    #define NOM_RECEIVED_POWER_PACKET_TIMEOUT       ( NOM_RECEIVED_POWER_PACKET_TIMEOUT_MS / 1000.0 )
    #define MAX_RECEIVED_POWER_PACKET_TIMEOUT       ( MAX_RECEIVED_POWER_PACKET_TIMEOUT_MS / 1000.0 )
    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define NOM_RECTIFIED_POWER_PACKET_TIMEOUT      ( NOM_RECTIFIED_POWER_PACKET_TIMEOUT_MS / 1000.0 )
    #define MAX_RECTIFIED_POWER_PACKET_TIMEOUT      ( MAX_RECTIFIED_POWER_PACKET_TIMEOUT_MS / 1000.0 )

    #define NOM_POWER_RECEIVER_RESET_TIME           ( NOM_POWER_RECEIVER_RESET_TIME_MS / 1000.0 )
    #define MAX_POWER_RECEIVER_RESET_TIME           ( MAX_POWER_RECEIVER_RESET_TIME_MS / 1000.0 )

    #define MIN_WAKEUP_TIME                         ( MIN_WAKEUP_TIME_MS / 1000.0 )
    #define NOM_WAKEUP_TIME                         ( NOM_WAKEUP_TIME_MS / 1000.0 )
    #define MAX_WAKEUP_TIME                         ( MAX_WAKEUP_TIME_MS / 1000.0 )

    #define MIN_SILENT_TIME                         ( MIN_SILENT_TIME_MS / 1000.0 )
    #define NOM_SILENT_TIME                         ( NOM_SILENT_TIME_MS / 1000.0 )

    #define NOM_CEP_INTERVAL                        ( NOM_CEP_INTERVAL_MS / 1000.0 )
    #define MAX_CEP_INTERVAL                        ( MAX_CEP_INTERVAL_MS / 1000.0 )
    #define MIN_CONTROLLER_TIME                     ( MIN_CONTROLLER_TIME_MS / 1000.0 )
    #define NOM_CONTROLLER_TIME                     ( NOM_CONTROLLER_TIME_MS / 1000.0 )
    #define NOM_RX_RECEIVED_POWER_PACKET_TIME       ( NOM_RX_RECEIVED_POWER_PACKET_TIME_MS / 1000.0 )
    #define MAX_RX_RECEIVED_POWER_PACKET_TIME       ( MAX_RX_RECEIVED_POWER_PACKET_TIME_MS / 1000.0 )
    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define NOM_RX_RECTIFIED_POWER_PACKET_TIME      ( NOM_RX_RECTIFIED_POWER_PACKET_TIME_MS / 1000.0 )
    #define MAX_RX_RECTIFIED_POWER_PACKET_TIME      ( MAX_RX_RECTIFIED_POWER_PACKET_TIME_MS / 1000.0 )



    // ------------------------ Common timing definitions -----------------------------

    #define PID_ACTIVE_TIME                         NOM_POWER_CONTROL_ACTIVE_TIME
    #define PID_ACTIVE_TIME_MS                      NOM_POWER_CONTROL_ACTIVE_TIME_MS
    #define PID_SETTLE_TIME                         NOM_POWER_CONTROL_SETTLING_TIME
    #define PID_SETTLE_TIME_MS                      NOM_POWER_CONTROL_SETTLING_TIME_MS

    #define DEFAULT_PID_DELAY_TIME_MS               MIN_POWER_CONTROL_HOLDOFF_TIME_MS


    // ---------------------- Transmitter timing definitions --------------------------

    #define MAX_DIGITAL_PING_TIME                   ( ( ( MAX_DIGITAL_PING_WINDOW - MIN_DIGITAL_PING_WINDOW ) /  2 ) + MIN_DIGITAL_PING_WINDOW )
    #define MAX_DIGITAL_PING_TIME_MS                ( MAX_DIGITAL_PING_TIME * 1000 )

    #define FIRST_PACKET_TIMEOUT                    MAX_FIRST_PACKET_TIMEOUT
    #define FIRST_PACKET_TIMEOUT_MS                 MAX_FIRST_PACKET_TIMEOUT_MS

    #define NEXT_PACKET_TIMEOUT                     MAX_NEXT_PACKET_TIMEOUT
    #define NEXT_PACKET_TIMEOUT_MS                  MAX_NEXT_PACKET_TIMEOUT_MS

    #define MAX_PACKET_LENGTH_TIMEOUT               MAX_PACKET_LENGTH
    #define MAX_PACKET_LENGTH_TIMEOUT_MS            MAX_PACKET_LENGTH_MS

    #define DEVICE_REMOVED_TIMEOUT                  NOM_CONTROL_ERROR_PACKET_TIMEOUT
    #define DEVICE_REMOVED_TIMEOUT_MS               NOM_CONTROL_ERROR_PACKET_TIMEOUT_MS

    #define RECEIVED_POWER_PACKET_TIMEOUT           NOM_RECEIVED_POWER_PACKET_TIMEOUT
    #define RECEIVED_POWER_PACKET_TIMEOUT_MS        NOM_RECEIVED_POWER_PACKET_TIMEOUT_MS
    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define RECTIFIED_POWER_PACKET_TIMEOUT          NOM_RECTIFIED_POWER_PACKET_TIMEOUT
    #define RECTIFIED_POWER_PACKET_TIMEOUT_MS       NOM_RECTIFIED_POWER_PACKET_TIMEOUT_MS




    // ---------------------- Receiver timing definitions --------------------------

    // NOTE: True Control Error inter-packet timing is determined by the selected PID DELAY TIME
    #define CONTROL_ERROR_PACKET_LATE               NOM_CEP_INTERVAL
    #define CONTROL_ERROR_PACKET_LATE_MS            NOM_CEP_INTERVAL_MS

    #define CONTROL_ERROR_CONTROLLER_TIME_MS        ( PID_ACTIVE_TIME_MS + PID_SETTLE_TIME_MS )

    #define RECEIVER_WAKE_TIME_EARLY                MIN_WAKEUP_TIME
    #define RECEIVER_WAKE_TIME_EARLY_MS             MIN_WAKEUP_TIME_MS
    #define RECEIVER_WAKE_TIME_LATE                 MAX_WAKEUP_TIME
    #define RECEIVER_WAKE_TIME_LATE_MS              MAX_WAKEUP_TIME_MS
    #define POWER_RECEIVER_RESET_TIME               MAX_POWER_RECEIVER_RESET_TIME
    #define POWER_RECEIVER_RESET_TIME_MS            MAX_POWER_RECEIVER_RESET_TIME_MS

    #define MAX_PID_DELAY_TIME_MS                   MAX_POWER_CONTROL_HOLDOFF_TIME_MS
    #define SILENT_TIME                             NOM_SILENT_TIME_MS

    // NOTE: The following definitions are used for backward compatibility with v1.0 compliant devices:
    #define RECTIFIED_POWER_INTERPACKET_TIME_MS     NOM_RX_RECTIFIED_POWER_PACKET_TIME_MS





    // WCT-specific definitions for General Use
    #define INTERPACKET_BUFFER_TIME_MS              2           // This 2ms accounts for possible differences in clock accuracies between primary and secondary sides

    // Calculated Timeout Limits Based on Specification
    #define MAX_PING_TIME_MS                        ( MAX_WAKEUP_TIME_MS + MAX_FIRST_PACKET_TIMEOUT_MS )
    #define ID_PACKET_TIMEOUT_MS                    ( MAX_NEXT_PACKET_TIMEOUT_MS + ( ID_PACKET_TRANSMIT_TIME_MS * ( ( 100 + PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100 ) ) )
    #define EXTENDED_ID_PACKET_TIMEOUT_MS           ( MAX_NEXT_PACKET_TIMEOUT_MS + ( EXT_ID_PACKET_TRANSMIT_TIME_MS * ( ( 100 + PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100 ) ) )
    #define POWER_CONTROL_HOLDOFF_PACKET_TIMEOUT_MS ( MAX_NEXT_PACKET_TIMEOUT_MS + ( POWER_CONTROL_HOLDOFF_PACKET_TRANSMIT_TIME_MS * ( ( 100 + PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100 ) ) )
    #define CONFIG_PACKET_TIMEOUT_MS                ( MAX_NEXT_PACKET_TIMEOUT_MS + ( CONFIG_PACKET_TRANSMIT_TIME_MS * ( ( 100 + PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100 ) ) )
    #define CONTROL_ERROR_PACKET_EARLY_MIN_MS       ( MIN_POWER_CONTROL_HOLDOFF_TIME_MS + PID_ACTIVE_TIME_MS + PID_SETTLE_TIME_MS + INTERPACKET_BUFFER_TIME_MS )
    #define CONTROL_ERROR_PACKET_EARLY_MAX_MS       ( MAX_PID_DELAY_TIME_MS + PID_ACTIVE_TIME_MS + PID_SETTLE_TIME_MS + INTERPACKET_BUFFER_TIME_MS )

    #define IDE_PK_BYTE_OFFSET_VERSION 0
    #define IDE_PK_BYTE_OFFSET_MANU_CODE 1
    #define IDE_PK_BYTE_OFFSET_DEV_ID 3

    #define IDE_PK_DEV_ID_3_MASK 0x7F
    #define IDE_PK_EXT_IDE_MASK 0x80
    #define IDE_PK_EXT_IDE_BIT_POSITION 7
    
    #define CFG_PK_BYTE_OFFSET_POWER 0
    #define CFG_PK_BYTE_OFFSET_COUNT 2
    #define CFG_PK_BYTE_OFFSET_WINDOW 3

    #define CFG_PK_MAX_PWR_MASK 0x3F
    #define CFG_PK_PWR_CLASS_MASK 0xC0
    #define CFG_PK_PWR_CLASS_BIT_POSITION 6

    #define CFG_PK_WIN_OFFSET_MASK 0x07
    #define CFG_PK_WIN_SIZE_MASK 0xF8
    #define CFG_PK_WIN_SIZE_BIT_POSITION 3

    // Rx code
    //------------------------------------------------------------------------------

    //----------
    // TYPEDEFS
    //----------

    //------------------------------------------------------------------------------
    // WARNING - Please keep in mind that bitmapped enums need to have a
    // fixed length of 1 byte.  This is why the packed attribute is
    // applied - which results in the 1-byte length.
    //
    // Enums provide the benefit of automatic decode in the debugger.
    //
    // NOTE: When declaring bit-mapped structures, the members representing
    // the LSB of a BYTE must be declared FIRST for the Microchip platform.
    //------------------------------------------------------------------------------
    typedef enum
    {
        // Ping Phase
        RF_PACKET_TYPE_SIGNAL_STRENGTH          = 0x01,

        // Identification and Configuration Phase
        RF_PACKET_TYPE_IDENTIFICATION           = 0x71,
        RF_PACKET_TYPE_EXT_IDENTIFICATION       = 0x81,
        RF_PACKET_TYPE_POWER_CONTROL_HOLDOFF    = 0x06,
        RF_PACKET_TYPE_CONFIGURATION            = 0x51,

        // Power Transfer Phase
        RF_PACKET_TYPE_END_POWER_XFER           = 0x02,
        RF_PACKET_TYPE_CONTROL_ERROR            = 0x03,
        RF_PACKET_TYPE_RECEIVED_POWER           = 0x04,
        RF_PACKET_TYPE_CHARGE_STATUS            = 0x05,

        // Proprietary Packets
        RF_PACKET_TYPE_PROPRIETARY1             = 0x18,
        RF_PACKET_TYPE_PROPRIETARY2             = 0x19,
        RF_PACKET_TYPE_PROPRIETARY3             = 0x28,
        RF_PACKET_TYPE_PROPRIETARY4             = 0x29,
        RF_PACKET_TYPE_PROPRIETARY5             = 0x38,
        RF_PACKET_TYPE_PROPRIETARY6             = 0x48,
        RF_PACKET_TYPE_PROPRIETARY7             = 0x58,
        RF_PACKET_TYPE_PROPRIETARY8             = 0x68,
        RF_PACKET_TYPE_PROPRIETARY9             = 0x78,
        RF_PACKET_TYPE_PROPRIETARY10            = 0x84,
        RF_PACKET_TYPE_PROPRIETARY11            = 0xA4,
        RF_PACKET_TYPE_PROPRIETARY12            = 0xC4,
        RF_PACKET_TYPE_PROPRIETARY13            = 0xE2,
        RF_PACKET_TYPE_PROPRIETARY14            = 0xF2

    } PACKING_ATTRIBUTE   RF_PACKET_TYPE;


    //------------------------------------------------------------------------------
    typedef enum
    {
        REASON_UNKNOWN                      = 0x00,
        REASON_CHARGE_COMPLETE              = 0x01,
        REASON_INTERNAL_FAULT               = 0x02,
        REASON_OVER_TEMP                    = 0x03,
        REASON_OVER_VOLT                    = 0x04,
        REASON_OVER_CURRENT                 = 0x05,
        REASON_BATTERY_FAILURE              = 0x06,
        REASON_RECONFIGURE                  = 0x07,
        REASON_NO_RESPONSE                  = 0x08

    } PACKING_ATTRIBUTE   END_REASON_CODE;


    //------------------------------------------------------------------------------
    // Signal Strength packet format - Table 6-4
    // NOTE: The Signal Strength is a value calculated from the equation: ( (U/Umax) * 256 )
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                bySignalStrength;
        #else
        BYTE                                bySignalStrength;
        #endif
    } PACKING_ATTRIBUTE   SIGNAL_STRENGTH_PAYLOAD;


    //------------------------------------------------------------------------------
    // End Power Transfer packet format - Table 6-5
    // NOTE: This value consists solely of a "Reason Code" as defined above and in
    //       Table 6-6
    typedef struct
    {
        END_REASON_CODE                     ReasonCode;

    } PACKING_ATTRIBUTE   END_POWER_XFER_PAYLOAD;


    //------------------------------------------------------------------------------
    // Control Error packet format - Table 6-7
    // NOTE: This is a two's complement number in the range -128 to +127 (inclusive)
    //       where positive numbers represent a request for more power and negative
    //       numbers represent a request for less power.
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                byError;
        #else
        BYTE                                byError;
        #endif
    } PACKING_ATTRIBUTE   CONTROL_ERROR_PAYLOAD;


    //------------------------------------------------------------------------------
    // Received Power packet format - Table 6-8  (was Rectified Power in v1.0)
    // NOTE: For a v1.0 compliant device, this unsigned value represents the power
    //       that the Receiver is providing at the rectifier as a percentage of the
    //       reported Maximum Power.
    //       For a v1.1 compliant device, this unsigned value represents the power
    //       that the Receiver detects at its coil.  This value is calculated based
    //       upon known operating parameters and coefficients developed from 
    //       calibration:  byReceivedPowerValue = ( Preceived * 128 ) / (Reported Maximum Power)
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                byReceivedPowerValue;
        #else
        BYTE                                byReceivedPowerValue;
        #endif
    } PACKING_ATTRIBUTE   POWER_USAGE_PAYLOAD;


    //------------------------------------------------------------------------------
    // Charge Status packet format - Table 6-9
    // NOTE: This unsigned value represents the charge state of a battery, where
    //       '0' corresponds to empty and '100' corresponds to full.
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                byPercentCharged;
        #else
        BYTE                                byPercentCharged;
        #endif
    } PACKING_ATTRIBUTE   CHARGE_STATUS_PAYLOAD;


    //------------------------------------------------------------------------------
    // Power Control Holdoff packet format - Table 6-10
    // NOTE: This unsigned value indicates the amount of time (in ms) which the 
    //       Transmitter shall wait prior to making PID adjustments after receiving
    //       a CEP packet.
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                byPowerControlHoldoffTimeMs;
        #else
        BYTE                                byPowerControlHoldoffTimeMs;
        #endif
    } PACKING_ATTRIBUTE   POWER_CONTROL_HOLDOFF_PAYLOAD;


    //------------------------------------------------------------------------------
    // Configuration packet format - Table 6-11
    typedef struct
    {
#if BYTE_ACCESS_NOT_SUPPORTED
    #if BIG_ENDIAN
        WORD                                Padding1 : 8;
        WORD                                PowerClass : 2;         /* PowerClass treated as an exponent, e.g. 10 ^ x */
        WORD                                MaximumPower : 6;       /* Used as the significand in the maxpwr calc */
        WORD                                ReservedByte1;          /* Not used */
        WORD                                Padding2 : 8;
        WORD                                Prop : 1;               /* Proprietary power transfer control (0 = normal) */
        WORD                                ReservedByte2 : 4;      /* Not used */
        WORD                                Count : 3;              /* Indicates # of optional packets sent (e.g. PID params) */
        WORD                                Padding3 : 8;
        WORD                                WindowSize : 5;          /* Power averaging window size in units of 4ms */
        WORD                                WindowOffset : 3;        /* Time between power averaging window and packet transmission in units of 4ms */
        WORD                                ReservedByte3;          /* Not used */
    #else
        WORD                                MaximumPower : 6;       /* Used as the significand in the maxpwr calc */
        WORD                                PowerClass : 2;         /* PoewrClass treated as an exponent, e.g. 10 ^ x */
        WORD                                Padding1 : 8;
        WORD                                ReservedByte1;          /* Not used */
        WORD                                Count : 3;              /* Indicates # of optional packets sent (e.g. PID params) */
        WORD                                ReservedByte2 : 4;      /* Not used */
        WORD                                Prop : 1;               /* Proprietary power transfer control (0 = normal) */
        WORD                                Padding2 : 8;
        WORD                                WindowOffset: 3;        /* Time between power averaging window and packet transmission in units of 4ms */
        WORD                                WindowSize: 5;          /* Power averaging window size in units of 4ms */
        WORD                                Padding3 : 8;
        WORD                                ReservedByte3;          /* Not used */
    #endif
#else
    #if BIG_ENDIAN
        BYTE                                PowerClass : 2;         // PowerClass treated as an exponent, e.g. 10 ^ x
        BYTE                                MaximumPower : 6;       // Used as the significand in the maxpwr calc
        BYTE                                ReservedByte1;          // Not used
        BYTE                                Prop : 1;               // Proprietary power transfer control (0 = normal)
        BYTE                                ReservedByte2 : 4;      // Not used
        BYTE                                Count : 3;              // Indicates # of optional packets sent (e.g. PID params)
        BYTE                                WindowSize: 5;          // Power averaging window size in units of 4ms
        BYTE                                WindowOffset: 3;        // Time between power averaging window and packet transmission in units of 4ms
        BYTE                                ReservedByte3;          // Not used
    #else
        BYTE                                MaximumPower : 6;       // Used as the significand in the maxpwr calc
        BYTE                                PowerClass : 2;         // PoewrClass treated as an exponent, e.g. 10 ^ x
        BYTE                                ReservedByte1;          // Not used
        BYTE                                Count : 3;              // Indicates # of optional packets sent (e.g. PID params)
        BYTE                                ReservedByte2 : 4;      // Not used
        BYTE                                Prop : 1;               // Proprietary power transfer control (0 = normal)
        BYTE                                WindowOffset: 3;        // Time between power averaging window and packet transmission in units of 4ms
        BYTE                                WindowSize: 5;          // Power averaging window size in units of 4ms
        BYTE                                ReservedByte3;          // Not used
    #endif
#endif
    } PACKING_ATTRIBUTE   CONFIGURATION_PAYLOAD;


    //------------------------------------------------------------------------------
    // Identification packet format - Table 6-12
    typedef struct
    {
#if BYTE_ACCESS_NOT_SUPPORTED
    #if BIG_ENDIAN
        WORD                                Padding1 : 8;
        WORD                                MajorVersion : 4;
        WORD                                MinorVersion : 4;
        WORD                                ManufacturerCodeMsb;
        WORD                                ManufacturerCodeLsb;
        WORD                                Padding2 : 8;
        WORD                                ExtendedIdIncluded : 1;
        WORD                                BasicDeviceId3 : 7;
        WORD                                BasicDeviceId2;
        WORD                                BasicDeviceId1;
        WORD                                BasicDeviceId0;
    #else
        WORD                                MinorVersion : 4;
        WORD                                MajorVersion : 4;
        WORD                                Padding1 : 8;
        WORD                                ManufacturerCodeMsb;
        WORD                                ManufacturerCodeLsb;
        WORD                                BasicDeviceId3 : 7;
        WORD                                ExtendedIdIncluded : 1;
        WORD                                Padding2 : 8;
        WORD                                BasicDeviceId2;
        WORD                                BasicDeviceId1;
        WORD                                BasicDeviceId0;
    #endif
#else
    #if BIG_ENDIAN
        BYTE                                MajorVersion : 4;
        BYTE                                MinorVersion : 4;
        BYTE                                ManufacturerCodeMsb;
        BYTE                                ManufacturerCodeLsb;
        BYTE                                ExtendedIdIncluded : 1;
        BYTE                                BasicDeviceId3 : 7;
        BYTE                                BasicDeviceId2;
        BYTE                                BasicDeviceId1;
        BYTE                                BasicDeviceId0;
    #else
        BYTE                                MinorVersion : 4;
        BYTE                                MajorVersion : 4;
        BYTE                                ManufacturerCodeMsb;
        BYTE                                ManufacturerCodeLsb;
        BYTE                                BasicDeviceId3 : 7;
        BYTE                                ExtendedIdIncluded : 1;
        BYTE                                BasicDeviceId2;
        BYTE                                BasicDeviceId1;
        BYTE                                BasicDeviceId0;
    #endif
#endif
    } PACKING_ATTRIBUTE   IDENTIFICATION_PAYLOAD;


    //------------------------------------------------------------------------------
    // Extended Identification packet format - Table 6-13
    typedef struct
    {
#if BYTE_ACCESS_NOT_SUPPORTED
        WORD                                ExtDeviceId7;
        WORD                                ExtDeviceId6;
        WORD                                ExtDeviceId5;
        WORD                                ExtDeviceId4;
        WORD                                ExtDeviceId3;
        WORD                                ExtDeviceId2;
        WORD                                ExtDeviceId1;
        WORD                                ExtDeviceId0;
#else
        BYTE                                ExtDeviceId7;
        BYTE                                ExtDeviceId6;
        BYTE                                ExtDeviceId5;
        BYTE                                ExtDeviceId4;
        BYTE                                ExtDeviceId3;
        BYTE                                ExtDeviceId2;
        BYTE                                ExtDeviceId1;
        BYTE                                ExtDeviceId0;
#endif
    } PACKING_ATTRIBUTE   EXT_IDENTIFICATION_PAYLOAD;



    //-----------
    // VARIABLES
    //-----------


    //-----------
    // FUNCTIONS
    //-----------



#endif
