/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __WCT_LIB_H__
#define __WCT_LIB_H__

//----------
// INCLUDES
//----------
#include "wct_libconst.h"
#include "wct_libstruct.h"
#include "wct_systemDataWPC.h"
#include "wcttx_lib_cfg.h"

//---------
// DEFINES
//---------

//----------
// TYPEDEFS
//----------

//------------------------------------------------------------------------------
// RF Data Interrupt Parameters

typedef struct
{
    STANDARD_REGISTER_SIZE      LastPortState;
    WORD                        wLastActiveEdgeTimerValue;
    WORD                        wTimeBetweenActiveEdges[ COMM_EDGE_BUFFER_SIZE ];
    WORD                        wRefTimeBetweenActiveEdges[ COMM_EDGE_BUFFER_SIZE ];

    PACKET_STATUS*              pRunTimePacketStatus;

    WORD                        wLastInactiveEdgeTimerValue;
    WORD                        wTimeBetweenOppositeEdges[ COMM_EDGE_BUFFER_SIZE ];
//    #if !COMM_DETECT_USING_CAPTURE_REGISTER
    STANDARD_REGISTER_SIZE *    pCommInputPort;
    STANDARD_REGISTER_SIZE      CommInputBitMask;

//    #if !( PROCESSOR_SUPPORTS_CONFIG_OF_GPIO_INTERRUPTS_ON_BOTH_EDGES | PROCESSOR_ALWAYS_GENERATES_GPIO_INTERRUPTS_ON_BOTH_EDGES )
    STANDARD_REGISTER_SIZE *    pCommIntPolRegister;
    STANDARD_REGISTER_SIZE      CommIntPolBitMask;
/*    #endif
    #endif
*/
    BYTE                        byBufferReadPointer;
    BYTE                        byBufferWritePointer;

} PACKING_ATTRIBUTE RF_DATA_INTERRUPT_PARAMS;

/**************************************************************************/ /*!
 * @brief Structure containing static configuration of the TX library
 *
 * This structure is used to configure the TX library. The configuration is 
 * connected to TX charger timeouts. 
 ******************************************************************************/
typedef struct
{
	SDWORD sdTickInterruptsPerSystem;
	BYTE bNumDevices;
	BYTE bCommChannelsPerDevice;
	BYTE byNumCoilsPerDevice;
	WORD* pwCommReferenceTimerCount;
	DWORD dwCommReferenceTimerMaximumCount;
	DWORD dwCommReferenceTimerFreq;
	WORD wMaxDigitalPingTimeRefCounts;
	WORD wFirstPacketTimeoutRefCounts;
	WORD wNextPacketTimeoutRefCounts;
	WORD wMaxPacketLengthTimeoutMs;
	WORD wNomControlErrorPacketTimeoutMs;
	WORD wNomRectifiedPowerPacketTimeoutMs;
	WORD wDefaultPidDelayTimeMs;
	WORD wMaxPidDelayTimeMs;
	WORD wReturnToChargingTimeDelayMs;
	BYTE byNumFreqBreakpoints;
//	DWORD pDataFlashBaseAddress;
//	WORD wSizeCalParams; // can be replaced by "sizeof( ANALOG_CAL_PARAMS ))*2*gStaticConf.bNumDevices+sizeof(CHECKSUM_PARAMS)", reason is independenci on bNumDevices
	WORD wCoilIPeakMicroampsPerAdcCount;
	BYTE byMaxPowerLossSampleHistory;
      WORD wTimeOutForSleep;
}WCHTX_STATIC_CONFIG;

/**************************************************************************/ /*!
 * @brief Structure containing time constants of communication layer
 *
 * This structure is used to define communication timeouts and pointers to timer 
 * counters which are used by library.
 ******************************************************************************/
typedef struct
{
	WORD wRfDataRxTimeoutCounts;
	BYTE bCommEdgeBuffer;
	//WORD* pwCommInterruptStatusPort;
	//WORD* pwCommInputTimerCount;
	DWORD dwCommInputTimerMaximumCount;
	WORD wRfDataMinInitialRx1BitTimeRefCounts;
	WORD wRfDataMaxInitialRx1BitTimeRefCounts;
	WORD wRfDataIdealRx1BitTimeRefCounts;
	WORD wRfDataRxTimeoutRefCounts;
	WORD wRfDataMinRx1BitTimeRefCounts;
	WORD wRfDataMaxRx1BitTimeRefCounts;
	WORD wRfDataMinRx2BitTimeRefCounts;
	BYTE bMinPreambuleDetectionCount;
#if (TXCOMM_DDM == 1)
      DWORD dwCommInputTimerClock;
#endif
}WCHTX_COMMUNICATION_CONST;

typedef struct
{
	WORD* pwCommInterruptStatusPort;
	DWORD dwPowermatMinPowerLowBitTimeRefCounts;
	DWORD dwPowermatMaxPowerLowBitTimeRefCounts;
	DWORD dwPowermatMinPowerHighBitTimeRefCounts;
	DWORD dwPowermatMaxPowerHighBitTimeRefCounts;
	DWORD dwPowermatMinPowerOkBitTimeRefCounts;
	DWORD dwPowermatMaxPowerOkBitTimeRefCounts;
	DWORD dwPowermatMinChargedBitTimeRefCounts;
	DWORD dwPowermatMaxChargedBitTimeRefCounts;
	DWORD dwPowermatTimeoutRefCounts;
	DWORD dwRailVSenseMicrovoltsPerAdcCount;
	DWORD dwInputIMicroampsPerAdcCount;
	
}WCHTX_POWERMAT_CONST;

typedef struct
{
    #if DEBUG_RETRY_ISSUE_IN_EDGE_SIDE
    //Constants for retry issue at edge side or when phone charging
    WORD wCtrlErrValueToForcePid;               // The ctrlErr value of forcing PID.
    WORD wCommTOTimeLimitToForcePid;    //The time limit of comm time out happen to enable force pid
    #endif
    #if DEBUG_FOD_RETRY
    //Constants for FOD retry issue
    WORD wPingFailCntLimitToExitFault;            // Ping fail count limit to exit fault protection.
    WORD wPingCurrentDeltaForFaultHoldOff;  // The delta of input current change in ping phase, used for fault state exit.
    #endif
    #if REPEAT_RETRY_FAULT
    //Repeated retry issue at edge side
    WORD wTimeLimitForRetryHoldOn;   // If the time from last retry is very short ( less than the limit), we record it for retry hold on judgement.
    WORD wTimeLimitForRetryHoldOff;  // If TX is stable for some time after last retry, clear the retry counter.
    WORD wShortRetrysLimitToHoldOn; // If continuous retry due to time out, let TX enter wait hold on state.
    #endif
    BYTE  bSigStrenLowTimitForRetryHoldOn;  // When continuous retry, judge the signal strength to count retry.
    BYTE  bSigStrenLimitToPower;
}WCHTX_RETRY_ISSUE_CONST;

typedef struct
{
    WORD wInVolLowToIndicate; //mV, the voltage level for power limit indication
    WORD wInVolLowToLimit;   //mV, the voltage level for power limit
    #if UNDER_VOLTAGE_PROTECTION
    WORD wInVolLowToShutOff;    // mV, the voltage low level to shut off
    #endif
    WORD wInVolLowTimeToIndicate; //millisecond, how long the voltage is lower than the indication level, then LED indication on.
    WORD wInVolLowTimeToLimit; //millisecond, how long the voltage is lower than the limit level, then power limit working.
    #if UNDER_VOLTAGE_PROTECTION
    WORD wInVolLowTimeToShutOff;    // millisecond, how long the voltage is lower than the limit level, then power shut off.
    #endif

    WORD wInVolHighToFallback;   //mV, the voltage level for power limit
    
    WORD wInCurrHighToIndicate; //mA, the current level for power limit indication
    WORD wInCurrHighToLimit;   //mA, the current level for power limit
    WORD wInCurrHighTimeToIndicate; //millisecond, how long the current is higher than the indication level, then LED indication on.
    WORD wInCurrHighTimeToLimit; //millisecond, how long the current is higher than the limit level, then power limit working.
    WORD wInPowerHighTimeToShutOff; // millisecond, how long the input power is over the limit , then fault happen.
    
    WORD wForceFreqStep;  //Hz, the frequency step for percentage
    WORD wForceFreqLimitation;   //Hz, the frequency increment limitation
}DYNAMIC_POWER_LIMIT_CONST;

typedef struct
{
	WORD wManufacturerId;
	WORD wPowerLossIndicationToPwrCessationMs;
    WORD wPowerLossBaseThreshold;
    BYTE byNumFodTripsToIndication;
    BYTE byReserved;
}FOD_PARAMS_CONST;

typedef struct
{
	BYTE byAnalogDetectionNum;
	BYTE byAnalogReleaseNum;
	BYTE byRSFODetectionNum;
}RESONANCE_SHIFT_FOD_PARAMS_CONST;

#define GPIO_SETHIGH(pin) do{ \
		WORD* pPort = (WORD*)pin.gpio_cpu16Port;\
		/*if (pPort)*/\
		*pPort |= 1<<(pin.gpio_cu8Pin);\
}while(0)
#define GPIO_SETLOW(pin) do{ \
		BYTE* pPort = (BYTE*)pin.gpio_cpu16Port;\
		/*if (pPort)*/\
		*pPort &= ~(1<<(pin.gpio_cu8Pin));\
}while(0)
#define GPIO_TOGGLE(pin) do{ \
		WORD* pPort = (WORD*)pin.gpio_cpu16Port;\
		/*if (pPort)*/\
		*pPort ^= 1<<(pin.gpio_cu8Pin);\
}while(0)

#define WCT_ENABLE  (1)
#define WCT_DISABLE (0)

//-----------
// VARIABLES
//-----------
extern const WCH_PINS digitalIo;


extern const WCHTX_STATIC_CONFIG gStaticConf;
#if 1
extern const WCHTX_COMMUNICATION_CONST gCommunicationConsts;
extern const WCHTX_POWERMAT_CONST gPowermatCommConst;
extern const WCHTX_RETRY_ISSUE_CONST gRetryIssueConsts;
extern const DYNAMIC_POWER_LIMIT_CONST gDplConsts;
extern const DWORD gRetryTimerConsts[];
extern const FOD_PARAMS_CONST gSecondaryFodParams[];
extern const RESONANCE_SHIFT_FOD_PARAMS_CONST gRSFodParams;
extern STANDARD_REGISTER_SIZE * const ptrCommInputPort[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
extern const STANDARD_REGISTER_SIZE CommInputBitMask[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
extern STANDARD_REGISTER_SIZE * const ptrCommIntPolRegister[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
extern const STANDARD_REGISTER_SIZE CommIntPolBitMask[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
#if SOUND_SUPPORTED
extern SOUND_PARAMS gSoundParams;
#endif

extern RF_DATA_PARAMS eRfDataParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];
extern RF_DATA_INTERRUPT_PARAMS eRfDataInterruptParams[ NUM_DEVICES ][ COMM_CHANNELS_PER_DEVICE ];

//powermat communication
#if POWERMAT_COMM_ENABLED
extern POWERMAT_INTERRUPT_PARAMS ePowerMatInterruptParams[ NUM_DEVICES ];

#if POWERMAT_DETECT_USING_CAPTURE_REGISTER
    // The following variables are used to map the parameters to the correct COMM input
    extern POWERMAT_INTERRUPT_PARAMS* const PowerMatCaptureInterruptParamsPointers[] = POWERMAT_IC_INTERRUPT_PARAMS_POINTERS;
#else
    extern STANDARD_REGISTER_SIZE * const ptrPowerMatInputPort[ NUM_DEVICES ];
    extern const STANDARD_REGISTER_SIZE PowerMatInputBitMask[ NUM_DEVICES ];
#endif
#endif
    
#endif

#if _DEBUG
extern BYTE dbg1_select_flag;
extern BYTE dbg2_select_flag;
#endif

extern WORD gADCInitValue[NUM_DEVICES];

#if LOW_POWER_MODE_SUPPORTED
extern WORD wTxIdleTime;
#endif

#define RECORD_LEN_POWER_OF_TWO			10
#define RECORD_BUF_LEN   				(1024)
#define RECODE_LEN_MASK					(RECORD_BUF_LEN/2-1)
extern SWORD gDDMRecordBuf[NUM_DEVICES][RECORD_BUF_LEN/2];

/**************************************************************************/ /*!
 * @brief Structure containing list of all GPIO used by the application and library
 *
 * This structure is defined statically and provides pin definition to application
 * and library. To change state of the pin you have to use @ref GPIO_SETHIGH(), 
 * @ref GPIO_SETLOW() or @ref GPIO_TOGGLE();
 ******************************************************************************/
extern RUNTIME_PARAMS                   RunTimeParams[ NUM_DEVICES ];

//extern const WORD wAdcCoilCurrentIndex[ NUM_COILS ];
extern const WORD wAdcCoilCurrentIndex[  ];

#if ADC_SAMPLE_AVERAGE_API
extern T_ADC_SAMPLE_AVERAGE AdcAverage[];
#endif

extern Firmware_Build_Version gFWBuildVersion;


//-----------
// FUNCTIONS
//-----------

extern volatile WORD SR_lock;          /* Lock */
extern volatile WORD SR_reg;           /* Current value of the SR register */
/*lint -esym(765,CpuMode) Disable MISRA rule (8.10) checking for symbols (CpuMode). */
extern BYTE CpuMode;                   /* Current speed mode */

/* Save status register and disable interrupts */
#define EnterCritical()     do {\
                              /*lint -save  -e530 Disable MISRA rule (9.1) checking. */\
                              volatile WORD StatusReg;\
                              asm(move.w SR, StatusReg);\
                              asm(bfset  #0x0300,SR);\
                              asm(nop);\
                              asm(nop);\
                              asm(nop);\
                              asm(nop);\
                              asm(nop);\
                              asm(nop);\
                              if (SR_lock++ == 0U) {\
                                SR_reg = StatusReg;\
                              }\
                              /*lint -restore Enable MISRA rule (9.1) checking. */\
                            } while (0)

/* Restore status register  */
#define ExitCritical()      do {\
                              if (--SR_lock == 0U) {\
                                asm(moveu.w SR_reg, SR);\
                                asm(nop);\
                                asm(nop);\
                              }\
                            } while (0)

//-----------
// FUNCTIONS
//-----------

/*****************************************************************************
 *//*!
 * @brief        Set debug mode.
 * @details      This function is used only for debug purpose for FreeMASTER tool.
 * @param[in]    debugMode Debug mode parameter
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
BYTE WCT_OpStateDebugMode(BYTE* debugMode);

/*****************************************************************************/
/*! @addtogroup MPTX_LibInit MP TX Lib initialization
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        Initialization and start of the WCT Library state machine.
 * @details      This function initializes and starts the WCT state machine. 
 *               It must be called after the NVM structure is initialized 
 *               by WCT_InitSystemParams(), WCT_InitOpParams() 
 *               and WCT_InitCalParams() functions.
 *
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Run( void );

/*****************************************************************************
 *//*!
 * @brief        TX Charger Status Initialization.
 * @details      This function initializes the internal status of the TX charger.
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_InitError( void );

/*****************************************************************************
 *//*!
 * @brief        Initialize System Diagnostics Variables.
 * @details      This function copies NVM configuration from the FLASH to the System structure.
 * @param[in]    pNvmSystemParams  Pointer to System parameters in Flash.
 * @param[in]    pRamSystemParams  Pointer to System parameters in NVM structure.
 * @param[in]    wNumParamBytes  Size of System parameters structure.
 *
 * @return       Value of error code.
 *               -  ERROR_CODE_CORRUPT_E2_SYSTEM_PARAMS
 *               -  ERROR_CODE_SUCCESS
 * @note         This timer Isr function is handled by predefined library callbacks.
 * @internal
 * @version      23-Jan-2012
 ******************************************************************************/
BYTE WCT_InitSystemParams( WORD *pNvmSystemParams, WORD *pRamSystemParams,  WORD wNumParamBytes);

/*****************************************************************************
 *//*!
 * @brief        Initialize Operational Diagnostics Variables.
 * @details      This function copies NVM configuration from the FLASH to the Calibration structure.
 * @param[in]    pNvmpOperatingParams  Pointer to Operating parameters in Flash.
 * @param[in]    pRamOperatingParams   Pointer to Operating parameters in NVM structure.
 * @param[in]    wNumParamBytes  Size of Operating parameters structure.
 *
 * @return       Value of error code.
 *               -  ERROR_CODE_CORRUPT_E2_OP_PARAMS
 *               -  ERROR_CODE_SUCCESS
 * @note         This timer Isr function is handled by predefined library callbacks.
 * @internal
 * @version      23-Jan-2012
 ******************************************************************************/
BYTE WCT_InitOpParams( WORD *pNvmOperatingParams, WORD *pRamOperatingSystemParams,  WORD wNumParamBytes );

/*****************************************************************************
 *//*!
 * @brief        Initialize System Calibration Variables.
 * @details      This function copies NVM configuration from the FLASH to the Calibration structure.
 * @param[in]    pNvmCalibratingParams  Pointer to Calibration parameters in Flash.
 * @param[in]    pRamCalibratingParams  Pointer to Calibration parameters in NVM structure.
 * @param[in]    wNumParamBytes  Size of Calibration parameters structure.
 *
 * @return       Value of error code.
 *               -  ERROR_CODE_CORRUPT_E2_CAL_PARAMS
 *               -  ERROR_CODE_SUCCESS
 * @note         This timer Isr function is handled by predefined library callbacks.
 * @internal
 * @version      23-Jan-2012
 ******************************************************************************/
BYTE WCT_InitCalParams( WORD *pNvmCalibratingParams, WORD *pRamCalibratingParams,  WORD wNumParamBytes );

/*! @} End of MPTX_LibInit */



/*****************************************************************************/
/*! @addtogroup MPTX_LibRuntime MP TX Runtime Lib API
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        TX charger handler.
 * @details      This function handles the TX charger. This function must 
 *               be called periodically from the main application loop or task.
 *
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Poll( void );

/*****************************************************************************
 *//*!
 * @brief        Qi Communication Decoder
 * @details      This function handles the low-level Qi communication. This function 
 *               must be called in the GPIO ISR handler for the pin which is used
 *               as the Qi communication input.
 *                
 * @note         This function is handled by predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_CommQiDecodeHandler( void );

/*****************************************************************************
 *//*!
 * @brief        PowerMat Communication Decoder.
 * @details      This function handles the low-level PowerMat communication. 
 *               This function must be called in the GPIO ISR handler for the pin, 
 *               or PWM HW fault interrupt routine in case of AUTO_A13, 
 *               which is used as the PowerMat communication input.
 *                
 * @note         This function is handled by predefined library callbacks.
 * @internal
 * @version      31-Jan-2012
 ******************************************************************************/
//void WCT_CommPowermatHandler( void );

// The following are error codes related to the primary
#define ERROR_CODE_SUCCESS                          0x00
#define ERROR_CODE_COMM_FAILURE                     0x01
#define ERROR_CODE_INCORRECT_POWER_ADAPTER          0x02
#define ERROR_CODE_FAN_FAILURE                      0x04
#define ERROR_CODE_HARDWARE_FAILURE                 0x08
#define ERROR_CODE_CORRUPT_E2_DIAGNOSTIC_PARAMS     0x10
#define ERROR_CODE_CORRUPT_E2_SYSTEM_PARAMS         0x20
#define ERROR_CODE_CORRUPT_E2_OP_PARAMS             0x40
#define ERROR_CODE_CORRUPT_E2_CAL_PARAMS            0x80
#define ERROR_CODE_INVALID                          0x100
#define ERROR_CODE_DEMO                             0x200
#define ERROR_CODE_DEMO_EXPIRED                     0x400
#define ERROR_CODE_OVER_VOLTAGE       0x800
#define ERROR_CODE_OVER_TEMPERATURE       0x1000

// The following are error codes related to the secondary devices
#define ERROR_CODE_SECONDARY_DEVICE_NOT_PRESENT     0x0001
#define ERROR_CODE_SECONDARY_INTERNAL_FAULT         0x0002
#define ERROR_CODE_SECONDARY_OVERTEMP               0x0004
#define ERROR_CODE_SECONDARY_OVERVOLT               0x0008
#define ERROR_CODE_SECONDARY_OVERCURRENT            0x0010
#define ERROR_CODE_SECONDARY_BATTERY_FAULT          0x0020
#define ERROR_CODE_SECONDARY_NO_RESPONSE            0x0080          // Indicates secondary detects no response to error packets
#define ERROR_CODE_SECONDARY_UNKNOWN                  0x0100
#define ERROR_CODE_SECONDARY_NOT_COMPLIANT          0x0200
#define ERROR_CODE_SECONDARY_FOD_FAULT              0x0400
#define ERROR_CODE_SECONDARY_EXCESS_POWER           0x0800
#define ERROR_CODE_SECONDARY_SAFETY_TRIP            0x1000
#define ERROR_CODE_SECONDARY_UNDER_VOLTAGE         0x2000
#define ERROR_CODE_SECONDARY_REPEAT_RETRY            0x4000
#define ERROR_CODE_SECONDARY_RSFOD_FAULT             0x8000

typedef enum
{
    RETRY_OF_EPT_UNKNOWN = 0,
    RETRY_OF_EPT_CHARGE_COMPLETE,
    RETRY_OF_EPT_INTERNAL_FAULT,
    RETRY_OF_EPT_OVER_VOLT,
    RETRY_OF_EPT_OVER_TEMPERATURE,
    RETRY_OF_EPT_OVER_CURRENT,
    RETRY_OF_EPT_BATTERY_FAILURE,
    RETRY_OF_EPT_NO_RESPONSE,
    RETRY_OF_INPUT_POWER_OVER_LIMIT,
    RETRY_OF_TEMPERATURE_OVER_LIMIT,
    RETRY_OF_REPEAT_RETRY,
    RETRY_OF_INPUT_UNDER_VOLTAGE,
    RETRY_OF_RS_FOD,
    RETRY_TYPES
}DEVICE_RETRY;

/*****************************************************************************
 *//*!
 * @brief        Get TX Charger Status
 * @details      This function returns the internal status of the TX charger. 
 *
 * @return       The last error code. See list of error codes formatted as ERROR_CODE_XXX.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_GetSystemErrorCode ( void );
/*****************************************************************************
 *//*!
 * @brief        Set TX Charger Status
 * @details      This function sets the internal status of the TX charger. 
 * @param[in]    wErrorCode  Error code to be set. See list of Error code ERROR_CODE_XXX
 *  
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_SetSystemErrorCode ( WORD wErrorCode );
/*! @} End of MPTX_LibRuntime */

/*****************************************************************************
 *//*!
 * @brief        Tick Timer Update
 * @details      This function handles the TX charger. This function 
 *               has to be called regularly every 200us. 
 *               The timer has to be initialized by the @ref WCT_TickTimerInit(); "WCT_TickTimerInit()" function. 
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_TickTimerUpdate(void);

/*****************************************************************************
 *//*!
 * @brief        Tick Timer Process
 * @details      This function handles the TX charger. This function 
 *               has to be called regularly every 200us. 
 *               The timer has to be initialized by the @ref WCT_TickTimerInit(); "WCT_TickTimerInit()" function. 
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_TickTimerHandler( void );

/*****************************************************************************
 *//*!
 * @brief        Get system timer ticks
 * @details     This function returns system timer ticks in ms.
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_TimerGetTimerTicks( void );

/*****************************************************************************
 *//*!
 * @brief        Delay code execution
 * @details      This function uses the system tick timer to delay the code execution for a desired time.
 * @param[in]    wNumMsToDelay  Desired time to delay in ms.
 *
 * @note         This function is handled by predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_TimerDelayMs( WORD wNumMsToDelay );

/*****************************************************************************
 *//*!
 * @brief        Get elapsed time
 * @details      This function uses the system tick timer to get elapsed time.
 * @param[in]    wLastTimePoint  last time point in ms.
 *
 * @note         This function is handled by predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_TimerGetElapsedTime( WORD wLastTimePoint );

/*****************************************************************************
 *//*!
 * @brief        1ms Timer handler 
 * @details      This function must be implemented in the application and is called 
 *                by the library every 1 millisecond. The implementation is application-specific.
 * @param[in]    wNumTicks  Actual count of ticks
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Timer1msTickHandler( WORD wNumTicks );
/*****************************************************************************
 *//*!
 * @brief        10ms Timer handler 
 * @details      This function must be implemented in the application and is called 
 *               by the library every 10 milliseconds. The implementation is application-specific.
 * @param[in]    wNumTicks  Actual count of ticks
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Timer10msTickHandler( WORD wNumTicks );
/*****************************************************************************
 *//*!
 * @brief        100ms Timer handler 
 * @details      This function must be implemented in the application and is called 
 *               by the library every 100 milliseconds. The implementation is application-specific.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Timer100msTickHandler( void );
/*****************************************************************************
 *//*!
 * @brief        1s Timer handler 
 * @details      This function must be implemented in the application and is called 
 *               by the library every 1 second. The implementation is application-specific.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Timer1SecTickHandler( void );
/*****************************************************************************
 *//*!
 * @brief        1Min Timer handler 
 * @details      This function must be implemented in the application and is called 
 *               by the library every 1 minute. The implementation is application-specific.
 *
 * @note         This function is implemented in predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_Timer1MinTickHandler( void );
/*****************************************************************************/
/*! @addtogroup MPTX_LibDisplayCallback WXT TX Lib Display Callbacks
 * @{*/
/*****************************************************************************
 *//*!
 * @brief        Update device status.
 * @details      This function is implemented in the application to visualize the new state of the charger.
 * @param[in]    byDeviceId  ID of device.
 * @param[in]    eDeviceState  New device state.
 *
 * @internal
 * @version      31-Jan-2012
 ******************************************************************************/
void WCT_OnUpdateDeviceStatus( BYTE byDeviceId, DEVICE_STATE eDeviceState );

/*****************************************************************************
 *//*!
 * @brief        Communication Decoder.
 * @details      This function handles the communication. This function 
 *               must be called in the 1ms timer handler.
 * @param[in]    wTimePassedMs  Number of millisecond till last call of WCT_OpStateProcessTimeSlice().
 *                
 * @note         This function is handled by predefined library callbacks.
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
WORD WCT_OpStateProcessTimeSlice( WORD wTimePassedMs );

/*****************************************************************************
 *//*!
 * @brief        Calculate Input Power.
 * @details      This function is used only for debug purpose for FreeMASTER tool.
 * @param[in]    pRunTimeParams pointer to runtime parameters.
 * @param[in]    btDebugPowerLossOperation Power Loss trigger
 *                
 * @internal
 * @version      18-Dec-2012
 ******************************************************************************/
void WCT_CalculateInputPower( RUNTIME_PARAMS *pRunTimeParams, BOOL btDebugPowerLossOperation );

/*****************************************************************************
 *//*!
 * @brief        Qi Communication Digital Demodulation Decoder.
 * @details      This function handles the communication. 
 * @param[in]    byDeviceId  ID of device.
 *                
 * @internal
 * @version      23-Aug-2013
 ******************************************************************************/
void WCT_DDMDataAnalyze(BYTE byDevice);

#if _DEBUG
#define DBG_LG_RETRY_ISSUE TRUE
#define DBG_T_NEXT_ISSUE TRUE
#endif

void DebugPutHexChar( BYTE byChar );                            // Put out a byte as a Hex value
void DebugPutHexBuffer( BYTE * buf, unsigned int size );        // Write out a buffer in Hex
void DebugPutDecChar( BYTE byChar );                            // Put out a byte as a decimal value
void DebugPutSignedDecChar( BYTE byChar );                      // Put out a byte as a signed decimal value
void DebugPutSignedDecWord( WORD wValue );                      // Put out a WORD as a signed decimal value
void DebugPutDoubleWordValue( DWORD dwValue );                  // used since the printf used does not support long numbers
BYTE DebugReceiveHexValue( void );                              // Receive a hex value on the debug port
DWORD DebugReceiveDword( BYTE byLengthMax );


#endif

