/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/
#include "defines.h"
#include "wct_callbacks.h"
#include "peripheralInit.h"

#include "wct_hal.h"
#include "wct_hal_cfg.h"
#include "freemaster.h"
#include DEBUG_CONSOLE_HEADER_FILE
#include "wct_debug.h"
#include "adc.h"
#include "wct_libconst.h"
#if SOUND_SUPPORTED
#include "sound.h"
#endif

const WCH_PINS digitalIo=
{
	{(WORD *)&(WAKEUP_PIN_PORT), (WAKEUP_PIN_BIT)}
};


const WCHTX_STATIC_CONFIG gStaticConf = {
		TICK_INTERRUPTS_PER_SYSTEM_TICK,
		NUM_DEVICES,
		COMM_CHANNELS_PER_DEVICE,
		NUM_COILS_PER_DEVICE,
		QI_COMM_REFERENCE_TIMER_COUNT,
		QI_COMM_REFERENCE_TIMER_MAXIMUM_COUNT,
		QI_COMM_REFERENCE_TIMER_CLOCK_FREQ,
		MAX_DIGITAL_PING_TIME_REF_COUNTS,
		FIRST_PACKET_TIMEOUT_REF_COUNTS,
		NEXT_PACKET_TIMEOUT_REF_COUNTS,
		MAX_PACKET_LENGTH_TIMEOUT_MS,
		NOM_CONTROL_ERROR_PACKET_TIMEOUT_MS,
		NOM_RECTIFIED_POWER_PACKET_TIMEOUT_MS,
		DEFAULT_PID_DELAY_TIME_MS,
		MAX_PID_DELAY_TIME_MS,
		RETURN_TO_CHARGING_TIME_DELAY_MS,
		NUM_FREQ_BREAKPOINTS,
//		DATA_FLASH_BASE_ADDRESS,
//		sizeof(CAL_PARAMS),
		COIL_I_PEAK_MICROAMPS_PER_ADC_COUNT,
		MAX_POWER_LOSS_SAMPLE_HISTORY,
		TIME_OUT_SECOND_FROM_IDLE_TO_SLEEP
};

const WCHTX_COMMUNICATION_CONST gCommunicationConsts = {
		4,
		COMM_EDGE_BUFFER_SIZE,
		//COMM_INTERRUPT_STATUS_PORT,
		//QI_COMM_INPUT_TIMER_COUNT,
		QI_COMM_INPUT_TIMER_MAXIMUM_COUNT,
		RF_DATA_MIN_INITIAL_RX_1BIT_TIME_REF_COUNTS,
		RF_DATA_MAX_INITIAL_RX_1BIT_TIME_REF_COUNTS,
		RF_DATA_IDEAL_RX_1BIT_TIME_REF_COUNTS,
		RF_DATA_RX_TIMEOUT_REF_COUNTS,
		RF_DATA_MIN_RX_1BIT_TIME_REF_COUNTS,
		RF_DATA_MAX_RX_1BIT_TIME_REF_COUNTS,
		RF_DATA_MIN_RX_2BIT_TIME_REF_COUNTS,
		MIN_PREAMBLE_DETECTION_COUNT,
		QI_COMM_INPUT_TIMER_CLOCK_FREQ
};

const WCHTX_RETRY_ISSUE_CONST gRetryIssueConsts = {
            #if DEBUG_RETRY_ISSUE_IN_EDGE_SIDE
            CTRL_ERR_VALUE_TO_FORCE_PID,
            CEP_TIME_OUT_LIMIT_TO_FORCE_PID,
            #endif
            #if DEBUG_FOD_RETRY
            PING_FAIL_COUNT_LIMIT_TO_EXIT_FAULT_HOLD_ON,
            PING_CURRENT_DELTA_TO_EXIT_FAULT_HOLD_ON,
            #endif
            #if REPEAT_RETRY_FAULT
            REPEATED_RETRY_TIME_LIMIT_TO_HOLD_ON,
            NO_RETRY_TIME_LIMIT_TO_HOLD_OFF,
            REPEATED_RETRYS_LIMIT_TO_HOLD_ON,
            #endif
            SIGNAL_STRENGTH_LOW_LIMIT_FOR_RETRY_ISSUE,
            SIGNAL_STRENGTH_LIMIT_TO_POWER,
};

const DYNAMIC_POWER_LIMIT_CONST gDplConsts = {
            IN_VOL_LOW_TO_INDICATE,       
            IN_VOL_LOW_TO_LIMIT,          
            #if UNDER_VOLTAGE_PROTECTION
            IN_VOL_LOW_TO_SHUT_OFF,
            #endif
            IN_VOL_LOW_TIME_TO_INDICATE,  
            IN_VOL_LOW_TIME_TO_LIMIT,     
            #if UNDER_VOLTAGE_PROTECTION
            IN_VOL_LOW_TIME_TO_SHUT_OFF,
            #endif
            
            IN_VOL_HIGH_TO_FALLBACK,
            
            IN_CURR_HIGH_TO_INDICATE,     
            IN_CURR_HIGH_TO_LIMIT,        
            IN_CURR_HIGH_TIME_TO_INDICATE,
            IN_CURR_HIGH_TIME_TO_LIMIT,   
            IN_POWER_HIGH_TIME_TO_SHUT_OFF,
            
            FORCE_FREQ_STEP,
            FORCE_FREQ_LIMITATION
};

const DWORD gRetryTimerConsts[RETRY_TYPES] = {
            RETRY_TIMER_EPT_UNKNOWN,           
            RETRY_TIMER_EPT_CHARGE_COMPLETE,   
            RETRY_TIMER_EPT_INTERNAL_FAULT,    
            RETRY_TIMER_EPT_OVER_VOLT,
            RETRY_TIMER_EPT_OVER_TEMPERATURE,
            RETRY_TIMER_EPT_OVER_CURRENT,      
            RETRY_TIMER_EPT_BATTERY_FAILURE,   
            RETRY_TIMER_EPT_NO_RESPONSE,       
            RETRY_TIMER_INPUT_POWER_OVER_LIMIT,
            RETRY_TIMER_TEMPERATURE_OVER_LIMIT,
            RETRY_TIMER_REPEAT_RETRY,          
            RETRY_TIMER_INPUT_UNDER_VOLTAGE,
            RETRY_TIMER_RS_FOD
};

//This table must end with {0, 0, 0, 0, 0}
const FOD_PARAMS_CONST gSecondaryFodParams[] = {
		{0x0017,  1000,  300, 1, 0}, //Avid
		{0x0024,  1000, 300, 1, 0}, //AUDIODEV
		//{0x0010,  5000,  1500, 2, 0}, //LG Gpro
		//{0x0010,  5000,  1500, 4, 0}, //LG F200L
		//{0x0010,  5000,  1500, 3, 0}, //LG Nexus4
		{0, 0, 0, 0, 0} //End Mark
};

const RESONANCE_SHIFT_FOD_PARAMS_CONST gRSFodParams = {
		ANALOG_DETECTION_NUM,
		ANALOG_RELEASE_NUM,
		RS_FO_DETECTION_NUM
};



#if SOUND_SUPPORTED
SOUND_PARAMS gSoundParams = {
            400,  //wSoundLowFrequency;
            600, //wSoundHighFrequency;
            50,   //wSoundFirstToneDuration;
            500,  //wSoundFirstPauseDuration;
            50,   //wSoundSecondToneDuration;
            10,  //wSoundSecondPauseDuration;
            50,   //wDcBeeperDuration;
};
#endif

