/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __WCT_CONST_HEADER_H__
#define __WCT_CONST_HEADER_H__

	//----------
    // INCLUDES
    //----------
#include "defines.h"
#include "wct_commPacketsWPC.h"

    //----------
    // TYPEDEFS
    //----------


    //---------
    // DEFINES
    //---------


// Calculate final frequency and period values derived from clock
//#define CPU_CLOCK_FREQ                          ( IP_BUS_CLK ) // Normal mode, core:ip = 1:1
#define CPU_CLOCK_FREQ                          ( OCCS_CORE_FREQ ) // Fast mode, core:ip = 2:1

#define CPU_CLOCK_FREQ_MHZ                      ( ( CPU_CLOCK_FREQ ) / 1000000UL )
#define CPU_CLOCK_PERIOD_NS                     ( 1.0E+9 / ( CPU_CLOCK_FREQ ) ) 
#define CPU_CLOCK_PERIOD_NS_INTEGER             ( WORD )( CPU_CLOCK_PERIOD_NS + 0.5 )

//system timer
#define TICK_INTERRUPTS_PER_SEC                 ( 1 / TICK_INTERRUPT_TIME )
#define SYSTEM_TICKS_PER_SEC                    ( 1 / SYSTEM_TICK_TIME )
#define TICK_INTERRUPTS_PER_SYSTEM_TICK         ( WORD ) ( TICK_INTERRUPTS_PER_SEC / SYSTEM_TICKS_PER_SEC )

//PowerLossDetection.h
#define NUM_NORM_REGIONS                3

///////////////////////////////////////////////////////////////////////////////
//from digiCommWPC.c
//low level communication decoder
// NOTE: Communication timing parameters should be defined in the commPacketsWPC header file!
// Calculate the bit times we can expect due to clock accuracy
#define MIN_COMMUNICATIONS_RATE                 ( NOMINAL_COMMUNICATIONS_RATE * ( ( 100.0 - PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100.0 ) )
#define MAX_COMMUNICATIONS_RATE                 ( NOMINAL_COMMUNICATIONS_RATE * ( ( 100.0 + PERMISSIBLE_CLOCK_TOLERANCE_PERCENT ) / 100.0 ) )
#define MIN_INITIAL_COMM_BIT_TIME               ( 1.0 / MAX_COMMUNICATIONS_RATE )
#define MAX_INITIAL_COMM_BIT_TIME               ( 1.0 / MIN_COMMUNICATIONS_RATE )

// Calculate the bit time limits that the state machine is capable of discerning
#define RF_DATA_MIN_RX_1BIT_TIME                ( 0.75 * NOMINAL_DATA_BIT_TIME )        // 3/4 bit time
#define RF_DATA_MAX_RX_1BIT_TIME                ( 1.25 * NOMINAL_DATA_BIT_TIME )        // 1-1/4 bit time
#define RF_DATA_MIN_RX_2BIT_TIME                ( 1.75 * NOMINAL_DATA_BIT_TIME )        // 1-3/4 bit time
#define RF_DATA_MAX_RX_2BIT_TIME                ( 2.25 * NOMINAL_DATA_BIT_TIME )        // 2-1/4 bit time
#define RF_DATA_RX_TIMEOUT                      ( 2.50 * MAX_INITIAL_COMM_BIT_TIME )    // timeout after 2.5 bit times

#define RF_DATA_MIN_RX_1BIT_TIME_IN_US          ( WORD ) ( ( RF_DATA_MIN_RX_1BIT_TIME * 1000000 ) + 0.5 )
#define RF_DATA_MAX_RX_1BIT_TIME_IN_US          ( WORD ) ( ( RF_DATA_MAX_RX_1BIT_TIME * 1000000 ) + 0.5 )
#define RF_DATA_MIN_RX_2BIT_TIME_IN_US          ( WORD ) ( ( RF_DATA_MIN_RX_2BIT_TIME * 1000000 ) + 0.5 )
#define RF_DATA_MAX_RX_2BIT_TIME_IN_US          ( WORD ) ( ( RF_DATA_MAX_RX_2BIT_TIME * 1000000 ) + 0.5 )
#define RF_DATA_RX_TIMEOUT_IN_US                ( WORD ) ( ( RF_DATA_RX_TIMEOUT * 1000000 ) + 0.5 )

// Bit time counts calculated based on reference timer resolution
#define COMM_INPUT_TIMER_RESOLUTION             ( 1.0 / QI_COMM_INPUT_TIMER_CLOCK_FREQ )
#define RF_DATA_MIN_RX_1BIT_TIME_REF_COUNTS     ( WORD ) ( ( RF_DATA_MIN_RX_1BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_MAX_RX_1BIT_TIME_REF_COUNTS     ( WORD ) ( ( RF_DATA_MAX_RX_1BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_MIN_RX_2BIT_TIME_REF_COUNTS     ( WORD ) ( ( RF_DATA_MIN_RX_2BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_MAX_RX_2BIT_TIME_REF_COUNTS     ( WORD ) ( ( RF_DATA_MAX_RX_2BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_RX_TIMEOUT_REF_COUNTS           ( WORD ) ( ( RF_DATA_RX_TIMEOUT / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )

#define RF_DATA_MIN_INITIAL_RX_1BIT_TIME_REF_COUNTS ( WORD ) ( ( MIN_INITIAL_COMM_BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_MAX_INITIAL_RX_1BIT_TIME_REF_COUNTS ( WORD ) ( ( MAX_INITIAL_COMM_BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define RF_DATA_IDEAL_RX_1BIT_TIME_REF_COUNTS   ( WORD ) ( ( NOMINAL_DATA_BIT_TIME / COMM_INPUT_TIMER_RESOLUTION ) + 0.5 )

// Bit time counts calculated based on TICK timer interrupt resolution
#define RF_DATA_MIN_RX_1BIT_TIME_COUNTS         ( WORD ) ( ( RF_DATA_MIN_RX_1BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_MAX_RX_1BIT_TIME_COUNTS         ( WORD ) ( ( RF_DATA_MAX_RX_1BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_MIN_RX_2BIT_TIME_COUNTS         ( WORD ) ( ( RF_DATA_MIN_RX_2BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_MAX_RX_2BIT_TIME_COUNTS         ( WORD ) ( ( RF_DATA_MAX_RX_2BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_RX_TIMEOUT_COUNTS               ( WORD ) ( ( RF_DATA_RX_TIMEOUT / TICK_INTERRUPT_TIME ) + 0.5 )

#define RF_DATA_MIN_INITIAL_RX_1BIT_TIME_COUNTS ( WORD ) ( ( MIN_INITIAL_COMM_BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_MAX_INITIAL_RX_1BIT_TIME_COUNTS ( WORD ) ( ( MAX_INITIAL_COMM_BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )
#define RF_DATA_IDEAL_RX_1BIT_TIME_COUNTS       ( WORD ) ( ( NOMINAL_DATA_BIT_TIME / TICK_INTERRUPT_TIME ) + 0.5 )

///////////////////////////////////////////////////////////////////////////////
    // RF Interrupt Defines
    #define COMM_EDGE_BUFFER_SIZE                   20
	// NOTE: These definitions MUST be specified in user code for proper initialization
    #ifndef QI_COMM_INPUT_TIMER_CLOCK_FREQ
    #warning Definition of QI_COMM_INPUT_TIMER_CLOCK_FREQ not found!
    #endif

//!!cannot be changed!! 
//communication
//SystemData
//systemDataWPCtypeA1Freq.h
#define NUM_FREQ_BREAKPOINTS                            4

//TX application
//from opstateWPCtypeA1freq.h
#define MAX_POWER_LOSS_SAMPLE_HISTORY           40

//from opstateWPCtypeAfreq.c
#define RETURN_TO_CHARGING_TIME_DELAY_MS        3000        // Clear CHARGED flag and return to charging after 3sec in PowerXfer state

#define COMM_REFERENCE_TIMER_RESOLUTION         ( 1.0 / QI_COMM_REFERENCE_TIMER_CLOCK_FREQ )
#define MAX_DIGITAL_PING_TIME_REF_COUNTS        ( WORD )( ( MAX_DIGITAL_PING_TIME / COMM_REFERENCE_TIMER_RESOLUTION ) + 0.5 )
// NOTE: Extending First packet timeout by 2ms to account for asynchronous loop operation to pass Compliance Test #5
#define FIRST_PACKET_TIMEOUT_REF_COUNTS         ( WORD )( ( ( FIRST_PACKET_TIMEOUT + 0.002 ) / COMM_REFERENCE_TIMER_RESOLUTION ) + 0.5 )
#define NEXT_PACKET_TIMEOUT_REF_COUNTS          ( WORD )( ( MAX_NEXT_PACKET_TIMEOUT / COMM_REFERENCE_TIMER_RESOLUTION ) + 0.5 )
#define MAX_PACKET_LENGTH_TIMEOUT_REF_COUNTS    ( WORD )( ( MAX_PACKET_LENGTH_TIMEOUT / COMM_REFERENCE_TIMER_RESOLUTION ) + 0.5 )


//powermat
// Calculate the bit time limits that the state machine is capable of discerning
#define POWERMAT_NOMINAL_CHARGED_BIT_TIME       0.004           // 4ms = 250Hz
#define POWERMAT_MIN_CHARGED_BIT_TIME           ( 0.75 * POWERMAT_NOMINAL_CHARGED_BIT_TIME )    // 3/4 bit time
#define POWERMAT_MAX_CHARGED_BIT_TIME           ( 1.25 * POWERMAT_NOMINAL_CHARGED_BIT_TIME )    // 1-1/4 bit time

#define POWERMAT_NOMINAL_POWER_OK_BIT_TIME      0.002           // 2ms = 500Hz
#define POWERMAT_MIN_POWER_OK_BIT_TIME          ( 0.75 * POWERMAT_NOMINAL_POWER_OK_BIT_TIME )   // 3/4 bit time
#define POWERMAT_MAX_POWER_OK_BIT_TIME          ( 1.25 * POWERMAT_NOMINAL_POWER_OK_BIT_TIME )   // 1-1/4 bit time

#define POWERMAT_NOMINAL_POWER_HIGH_BIT_TIME    0.001           // 1ms = 1000Hz
#define POWERMAT_MIN_POWER_HIGH_BIT_TIME        ( 0.75 * POWERMAT_NOMINAL_POWER_HIGH_BIT_TIME ) // 3/4 bit time
#define POWERMAT_MAX_POWER_HIGH_BIT_TIME        ( 1.25 * POWERMAT_NOMINAL_POWER_HIGH_BIT_TIME ) // 1-1/4 bit time

#define POWERMAT_NOMINAL_POWER_LOW_BIT_TIME     0.000125        // 125us = 8000Hz
#define POWERMAT_MIN_POWER_LOW_BIT_TIME         ( 0.75 * POWERMAT_NOMINAL_POWER_LOW_BIT_TIME )  // 3/4 bit time
#define POWERMAT_MAX_POWER_LOW_BIT_TIME         ( 1.25 * POWERMAT_NOMINAL_POWER_LOW_BIT_TIME )  // 1-1/4 bit time

#define POWERMAT_TIMEOUT_TIME                   0.005           // 5ms

// Bit time counts calculated based on reference timer resolution
#define POWERMAT_INPUT_TIMER_RESOLUTION             ( 1.0 / POWERMAT_INPUT_TIMER_CLOCK_FREQ )
#define POWERMAT_MIN_CHARGED_BIT_TIME_REF_COUNTS    ( DWORD ) ( ( POWERMAT_MIN_CHARGED_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MAX_CHARGED_BIT_TIME_REF_COUNTS    ( DWORD ) ( ( POWERMAT_MAX_CHARGED_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MIN_POWER_OK_BIT_TIME_REF_COUNTS   ( DWORD ) ( ( POWERMAT_MIN_POWER_OK_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MAX_POWER_OK_BIT_TIME_REF_COUNTS   ( DWORD ) ( ( POWERMAT_MAX_POWER_OK_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MIN_POWER_HIGH_BIT_TIME_REF_COUNTS ( DWORD ) ( ( POWERMAT_MIN_POWER_HIGH_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MAX_POWER_HIGH_BIT_TIME_REF_COUNTS ( DWORD ) ( ( POWERMAT_MAX_POWER_HIGH_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MIN_POWER_LOW_BIT_TIME_REF_COUNTS  ( DWORD ) ( ( POWERMAT_MIN_POWER_LOW_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_MAX_POWER_LOW_BIT_TIME_REF_COUNTS  ( DWORD ) ( ( POWERMAT_MAX_POWER_LOW_BIT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )
#define POWERMAT_TIMEOUT_REF_COUNTS                 ( DWORD ) ( ( POWERMAT_TIMEOUT_TIME / POWERMAT_INPUT_TIMER_RESOLUTION ) + 0.5 )


//NVM
//#define SYSTEM_PARAMS_FLASH_BASE_ADDRESS               (DATA_FLASH_BASE_ADDRESS)
//#define OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE      (SYSTEM_PARAMS_FLASH_BASE_ADDRESS + sizeof( SYSTEM_PARAMS ))
//#define CAL_PARAMS_FLASH_BASE_ADDRESS                  (OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE + ( sizeof( OP_PARAMS ) * (NUM_DEVICES) ))
#define SYSTEM_PARAMS_FLASH_BASE_ADDRESS               (DATA_FLASH_BASE_WORD_ADDRESS)
#define OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE      (SYSTEM_PARAMS_FLASH_BASE_ADDRESS + sizeof( SYSTEM_PARAMS )/2)
#define CAL_PARAMS_FLASH_BASE_ADDRESS                  (OP_PARAMS_FLASH_BASE_ADDRESS_FIRST_DEVICE + ( sizeof( OP_PARAMS ) * (NUM_DEVICES)/2 ))

//Low power mode
// NOTE: A function can request permanent shutdown by returning the following value for sleep time
#define LOW_POWER_MODE_TIME_INFINITE    0xFFFF

// Temperature protection
#define MAX_TEMPERATURE_SAMPLE_HISTORY           10

// Input voltage average
#define MAX_INPUT_VOLTAGE_SAMPLE_HISTORY           10

// Mute time (ms) for short charging reset
#define MUTE_TIME_FOR_SHORT_RESET           2000 //millisecond

#define RETRY_ISSUE_WORKAROUND_FOR_PHONE_CHARGING TRUE

// Dynamic power limit
#define DYNAMIC_POWER_LIMIT TRUE // Dynamic power limit.

#define IN_VOL_LOW_TO_INDICATE         4500   //mV
#define IN_VOL_LOW_TO_LIMIT            4500   //mV
#define IN_VOL_LOW_TIME_TO_INDICATE    2000    //millisecond
#define IN_VOL_LOW_TIME_TO_LIMIT       4     //millisecond
#define IN_VOL_HIGH_TO_FALLBACK 	   4800  //mV

#define FORCE_FREQ_STEP 	   	   	   10  //Hz
#define FORCE_FREQ_LIMITATION 	   	   300  //Hz

#define IN_CURR_HIGH_TO_INDICATE       2200   //mA
#define IN_CURR_HIGH_TO_LIMIT          2400   //mA
#define IN_CURR_HIGH_TIME_TO_INDICATE  1    //millisecond
#define IN_CURR_HIGH_TIME_TO_LIMIT     1     //millisecond
#define IN_POWER_HIGH_TIME_TO_SHUT_OFF 1000 //millisecond

#define POWER_LIMIT_INDICATION_BY_PWM_AND_CEP FALSE

//Resonance shift FO detection.
#define RESONANCE_SHIFT_FOD 			FALSE
#define	ANALOG_DETECTION_NUM			3
#define	ANALOG_RELEASE_NUM				3
#define	RS_FO_DETECTION_NUM				2

// Over temperature protection
#define OTP_PROTECTION_SHUT_DOWN_INFINITE FALSE // Shut down MCU infinitely for over temperature protection

// Sound
#define MUTE_FOR_SHORT_RESET TRUE

// Repeat retry fault
#define REPEAT_RETRY_FAULT FALSE
#define REPEATED_RETRY_TIME_LIMIT_TO_HOLD_ON 6000
#define NO_RETRY_TIME_LIMIT_TO_HOLD_OFF 20000
#define REPEATED_RETRYS_LIMIT_TO_HOLD_ON 2

//Retry issue at coil edge side
#define DEBUG_RETRY_ISSUE_IN_EDGE_SIDE FALSE
#define CEP_TIME_OUT_LIMIT_TO_FORCE_PID	500 //millisecond
#define CTRL_ERR_VALUE_TO_FORCE_PID	30

//FOD issue of Qi test #25c,#25d
#define DEBUG_FOD_RETRY TRUE
#define PING_FAIL_COUNT_LIMIT_TO_EXIT_FAULT_HOLD_ON 3
#define PING_CURRENT_DELTA_TO_EXIT_FAULT_HOLD_ON 20 // %

#define SIGNAL_STRENGTH_LOW_LIMIT_FOR_RETRY_ISSUE 60
#define SIGNAL_STRENGTH_LIMIT_TO_POWER 40

// Fault retry timer
#define NUM_MS_PER_MINUTE                       60000
#define FIVE_MINUTE_RETRY_TIME_MS               ( NUM_MS_PER_MINUTE * 5 )
#define FATAL_FAULT_TIMEOUT                     ( NUM_MS_PER_MINUTE * 255 )     // Report of '255' indicates fatal fault - don't retry

#define RETRY_TIMER_EPT_UNKNOWN             0
#define RETRY_TIMER_EPT_CHARGE_COMPLETE     FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_EPT_INTERNAL_FAULT      FATAL_FAULT_TIMEOUT
#define RETRY_TIMER_EPT_OVER_VOLT           FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_EPT_OVER_TEMPERATURE    FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_EPT_OVER_CURRENT        0
#define RETRY_TIMER_EPT_BATTERY_FAILURE     FATAL_FAULT_TIMEOUT
#define RETRY_TIMER_EPT_NO_RESPONSE         0
#define RETRY_TIMER_INPUT_POWER_OVER_LIMIT  FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_TEMPERATURE_OVER_LIMIT  FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_REPEAT_RETRY            FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_INPUT_UNDER_VOLTAGE     FIVE_MINUTE_RETRY_TIME_MS
#define RETRY_TIMER_RS_FOD					FATAL_FAULT_TIMEOUT

//Reconfigure issue fix of Qi #22a
#define RECONFIGURE_ISSUE TRUE

//----------------------- Adc Sample Average -------------------------
#define ADC_SAMPLE_AVERAGE_API TRUE

//----------------------- Under voltage protection -------------------------
#define UNDER_VOLTAGE_PROTECTION FALSE

#define IN_VOL_LOW_TO_SHUT_OFF            4200   //mV
#define IN_VOL_LOW_TIME_TO_SHUT_OFF   2000   //millisecond

//----------------------- FOD related -------------------------
#define DISABLE_WINDOW_OFFSET_CHECK TRUE

//----------------------- Device fault -------------------------
#define DISABLE_NOT_COMPLIANT_FAULT_DISPLAY TRUE

#endif
