/*******************************************************************************
*
* Copyright 2004-2014 Freescale Semiconductor, Inc.
* Copyright 2018 NXP.
* All rights reserved.
*******************************************************************************/

#ifndef __WCT_SYSDATA_H__

    #define __WCT_SYSDATA_H__

    //----------
    // INCLUDES
    //----------

    //---------
    // DEFINES
    //---------
    #define SYSTEM_PARAMS_BASE_ADDRESS                      0
    #define OP_PARAMS_BASE_ADDRESS_FIRST_DEVICE             SYSTEM_PARAMS_BASE_ADDRESS              + sizeof( SYSTEM_PARAMS )
    #define CAL_PARAMS_BASE_ADDRESS                         OP_PARAMS_BASE_ADDRESS_FIRST_DEVICE     + ( sizeof( OP_PARAMS ) * gStaticConf.bNumDevices )
    #define NEXT_AVAILABLE_BASE_ADDRESS                     CAL_PARAMS_BASE_ADDRESS                 + sizeof( CAL_PARAMS )

    #define FOD_SUPPORTED                                   TRUE


    //----------
    // TYPEDEFS
    //----------

    //------------------------------------------------------------------------------
    // Hardware Configuration Parameters
    typedef struct
    {
        BYTE    byLedSelection;
        BYTE    byPowerLossDetectionSelection;
        BYTE    bSupervisoryMicroPresent: 1;
        BYTE    bUseAdaptiveDeadtimeFetDriver: 1;
        BYTE    bPowerLossDetectionActive: 1;
        BYTE    bDiagnosticsEnabled: 1;
        BYTE    bAcBeeperEnabled: 1;
        BYTE    bIrdaClockEnabled: 1;
        BYTE    bActiveHighLedDrive: 1;
        BYTE    bSupervisorCompatible: 1;

    } HW_CONFIG_OPTIONS;


    //------------------------------------------------------------------------------
    // Checksum Parameters
    typedef struct
    {
        #if BYTE_ACCESS_NOT_SUPPORTED
        WORD                    wPaddingWord1;
        WORD                    byChecksum;
        #else
        BYTE                    byPaddingByte1;
        BYTE                    byPaddingByte2;
        BYTE                    byPaddingByte3;
        BYTE                    byChecksum;
        #endif

    } CHECKSUM_PARAMS;

    //------------------------------------------------------------------------------
    // System Parameters
    typedef struct
    {
        LED_OPERATION           LedOperation;

        #if SOUND_SUPPORTED
        //SOUND_PARAMS            SoundParams;
        #endif
        
        #if PROCESSOR_REQUIRES_NVM_STORAGE_SPACE
        MICROCONTROLLER_PARAMS  MicroControllerParams;
        #endif

        OPSTATE_SYSTEM_PARAMS   OpStateParams;

        CHECKSUM_PARAMS         ChecksumParams;

    } SYSTEM_PARAMS;


    //------------------------------------------------------------------------------
    // Operating Parameters
    typedef struct
    {
        OPSTATE_OP_PARAMS       OpStateParams;

        POWERLOSS_OP_PARAMS     PowerLossParams;
        
        RSFOD_OP_PARAMS         RSFodParams;                 //--------------Added by WangLi

        CHECKSUM_PARAMS         ChecksumParams;

    } OP_PARAMS;


    //------------------------------------------------------------------------------
    // Calibration Parameters
    typedef struct
    {
        ANALOG_CAL_PARAMS       AnalogParams[ NUM_DEVICES ];

        POWERLOSS_CAL_PARAMS    PowerLossParams[ NUM_DEVICES ];

        CHECKSUM_PARAMS         ChecksumParams;

    } CAL_PARAMS;



    //------------------------------------------------------------------------------
    // Define layout of NVM
    typedef struct
    {
        SYSTEM_PARAMS           SystemParams;
        OP_PARAMS               OpParams[ NUM_DEVICES ];
        CAL_PARAMS              CalParams;

    } NVM_PARAMS;



    //-----------
    // VARIABLES
    //-----------
    extern HW_CONFIG_OPTIONS        HwConfigOptions;
    extern NVM_PARAMS               NvmParams;



    //-----------
    // FUNCTIONS
    //-----------
    BYTE SystemComputeDataBlockChecksum( BYTE* pbyDataBlock, WORD wSizeOfDataBlock );
    void SystemInitializeVariables( void );
    void SystemInitializeDiagnosticParams( void );

#endif
